Package: biomartr
Title: Genomic Data Retrieval
Version: 1.0.7
Authors@R: c(person("Hajk-Georg", "Drost",
                role = c("aut", "cre"),
                email = "hajk-georg.drost@tuebingen.mpg.de",
                comment = c(ORCID = "http://orcid.org/0000-0002-1567-306X")),
                person("Haakon", "Tjeldnes", 
                email = "hauken_heyken@hotmail.com", 
                role = c("aut", "ctb"))
                )
Description: Perform large scale genomic data retrieval and functional annotation retrieval. This package aims to provide users with a standardized
                way to automate genome, proteome, 'RNA', coding sequence ('CDS'), 'GFF', and metagenome
                retrieval from 'NCBI RefSeq', 'NCBI Genbank', 'ENSEMBL', 
                and 'UniProt' databases. Furthermore, an interface to the 'BioMart' database
                (Smedley et al. (2009) <doi:10.1186/1471-2164-10-22>) allows users to retrieve
                functional annotation for genomic loci. In addition, users can download entire databases such
                as 'NCBI RefSeq' (Pruitt et al. (2007) <doi:10.1093/nar/gkl842>), 'NCBI nr',
                'NCBI nt', 'NCBI Genbank' (Benson et al. (2013) <doi:10.1093/nar/gks1195>), etc. with only one command.
VignetteBuilder: knitr
NeedsCompilation: yes
Depends: R (>= 3.1.1)
Imports: biomaRt, Biostrings, curl, tibble, jsonlite, data.table (>=
        1.9.4), dplyr (>= 0.3.0), readr (>= 1.4.0), downloader (>=
        0.3), RCurl (>= 1.95-4.5), XML (>= 3.98-1.1), httr (>= 0.6.1),
        stringr (>= 0.6.2), purrr, R.utils, philentropy, withr, fs
Suggests: knitr (>= 1.6), rmarkdown (>= 0.3.3), devtools (>= 1.6.1),
        testthat, seqinr, magrittr
License: GPL-2
URL: https://docs.ropensci.org/biomartr/,
        https://github.com/ropensci/biomartr
BugReports: https://github.com/ropensci/biomartr/issues
RoxygenNote: 7.2.3
Encoding: UTF-8
X-schema.org-keywords: BioMart, genomic-data-retrieval,
        annotation-retrieval, database-retrieval, NCBI, ENSEMBL,
        biological-data-retrieval
X-schema.org-applicationCategory: Data Access
X-schema.org-isPartof: "ropensci.org"
Packaged: 2023-12-02 14:46:32 UTC; hdrost
Author: Hajk-Georg Drost [aut, cre] (<https://orcid.org/0000-0002-1567-306X>),
  Haakon Tjeldnes [aut, ctb]
Maintainer: Hajk-Georg Drost <hajk-georg.drost@tuebingen.mpg.de>
Repository: CRAN
Date/Publication: 2023-12-02 17:40:07 UTC
Built: R 4.6.0; ; 2026-04-29 12:17:06 UTC; unix

# Generated by roxygen2: do not edit by hand

export(biomart)
export(cachedir)
export(cachedir_set)
export(check_annotation_biomartr)
export(download.database)
export(download.database.all)
export(ensembl_divisions)
export(get.ensembl.info)
export(getAssemblyStats)
export(getAttributes)
export(getBioSet)
export(getCDS)
export(getCDSSet)
export(getCollection)
export(getCollectionSet)
export(getDatasets)
export(getENSEMBLGENOMESInfo)
export(getENSEMBLInfo)
export(getFilters)
export(getGENOMEREPORT)
export(getGFF)
export(getGFFSet)
export(getGO)
export(getGTF)
export(getGenome)
export(getGenomeSet)
export(getGroups)
export(getKingdomAssemblySummary)
export(getKingdoms)
export(getMarts)
export(getMetaGenomeAnnotations)
export(getMetaGenomeSummary)
export(getMetaGenomes)
export(getProteome)
export(getProteomeSet)
export(getRNA)
export(getRNASet)
export(getReleases)
export(getRepeatMasker)
export(getSummaryFile)
export(getUniProtInfo)
export(getUniProtSTATS)
export(is.genome.available)
export(listDatabases)
export(listGenomes)
export(listGroups)
export(listKingdoms)
export(listMetaGenomes)
export(listNCBIDatabases)
export(meta.retrieval)
export(meta.retrieval.all)
export(organismAttributes)
export(organismBM)
export(organismFilters)
export(read_assemblystats)
export(read_cds)
export(read_genome)
export(read_gff)
export(read_proteome)
export(read_rm)
export(read_rna)
export(refseqOrganisms)
export(summary_cds)
export(summary_genome)
import(Biostrings)
import(RCurl)
import(biomaRt)
import(curl)
importFrom(RCurl,url.exists)
importFrom(XML,getHTMLLinks)
importFrom(data.table,":=")
importFrom(data.table,.BY)
importFrom(data.table,.EACHI)
importFrom(data.table,.GRP)
importFrom(data.table,.I)
importFrom(data.table,.N)
importFrom(data.table,.NGRP)
importFrom(data.table,.SD)
importFrom(data.table,chmatch)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,fwrite)
importFrom(data.table,rbindlist)

biomartr
========

<!-- badges: start -->
[![](https://badges.ropensci.org/93_status.svg)](https://github.com/ropensci/software-review/issues/93)
[![rstudio mirror downloads](https://cranlogs.r-pkg.org/badges/biomartr)](https://github.com/r-hub/cranlogs.app)
[![rstudio mirror downloads](https://cranlogs.r-pkg.org/badges/grand-total/biomartr)](https://github.com/r-hub/cranlogs.app)
[![Paper link](https://img.shields.io/badge/Published%20in-Bioinformatics-126888.svg)](https://academic.oup.com/bioinformatics/article/33/8/1216/2931816)
[![install with bioconda](https://img.shields.io/badge/install%20with-bioconda-brightgreen.svg?style=flat)](https://bioconda.github.io/recipes/r-biomartr/README.html)
![Visitors](https://api.visitorbadge.io/api/visitors?path=https%3A%2F%2Fgithub.com%2Fropensci%2Fbiomartr&label=VISITORS&countColor=%23263759&style=flat)

<!-- badges: end -->
 
## Genomic Data Retrieval with R

### Motivation:

This package is born out of my own frustration to automate the genomic data retrieval process to create computationally reproducible scripts for large-scale genomics studies. Since I couldn't find easy-to-use and fully reproducible software libraries I sat down and tried to implement a framework that would enable anyone to automate and standardize the genomic data retrieval process. I hope that this package is useful to others as well and that it helps to promote reproducible research in genomics studies.

I happily welcome anyone who wishes to contribute to this project :) Just drop me an email.

Please find a detailed [documentation here](https://docs.ropensci.org/biomartr/articles/).


### Citation

__Please cite `biomartr` if it was helpful for your research. This will allow me to
continue maintaining this project in the future.__

> Drost HG, Paszkowski J. __Biomartr: genomic data retrieval with R__. *Bioinformatics* (2017) 33(8): 1216-1217. [doi:10.1093/bioinformatics/btw821](https://academic.oup.com/bioinformatics/article/doi/10.1093/bioinformatics/btw821/2931816/Biomartr-genomic-data-retrieval-with-R).


### Short package description:

The vastly growing number of sequenced genomes allows us to perform a new type of biological research.
Using a comparative approach these genomes provide us with new insights on how biological information is encoded 
on the molecular level and how this information changes over evolutionary time.

The first step, however, of any genome based study is to retrieve genomes and their annotation from databases. To automate the
retrieval process of this information on a meta-genomic scale, the `biomartr` package provides interface functions for genomic sequence retrieval and functional annotation retrieval. The major aim of `biomartr` is to facilitate computational reproducibility and large-scale handling of genomic data for (meta-)genomic analyses.
In addition, `biomartr` aims to address the `genome version crisis`. With `biomartr` users can now control and be informed 
about the genome versions they retrieve automatically. Many large scale genomics studies lack this information
and thus, reproducibility and data interpretation become nearly impossible when documentation of genome version information
gets neglected.

In detail, `biomartr` automates genome, proteome, CDS, RNA, Repeats, GFF/GTF (annotation), genome assembly quality, and metagenome project data retrieval from the major biological databases such as

- [NCBI RefSeq](https://www.ncbi.nlm.nih.gov/refseq/)
- [NCBI Genbank](https://www.ncbi.nlm.nih.gov/genbank/)
- [ENSEMBL](https://www.ensembl.org/index.html)
- [ENSEMBLGENOMES](http://ensemblgenomes.org) (as of April 2019 - `ENSEMBL` and `ENSEMBLGENOMES` were joined - see [details here](https://www.ensembl.info/2019/03/08/joint-rest-server-for-ensembl-and-ensembl-genomes-in-ensembl-96/))
- [UniProt](https://www.uniprot.org)

Furthermore, an interface to the `Ensembl Biomart` database allows users to retrieve functional annotation for genomic loci using a novel and organism centric search strategy. In addition, users can [download entire databases](https://docs.ropensci.org/biomartr/articles/Database_Retrieval.html) such as 

- `NCBI RefSeq` 
- `NCBI nr` 
- `NCBI nt`
- `NCBI Genbank`
- `ENSEMBL` 

with only one command.

### Similar Work

The main difference between the [BiomaRt](https://www.bioconductor.org/packages/release/bioc/html/biomaRt.html) package and the [biomartr](https://docs.ropensci.org/biomartr/) package is that `biomartr` extends the `functional annotation retrieval` procedure of `BiomaRt` and __in addition__ provides useful retrieval functions for genomes, proteomes, coding sequences, gff files, RNA sequences, Repeat Masker annotations files, and functions for the retrieval of entire databases such as `NCBI nr` etc.

Please consult the [Tutorials section](https://docs.ropensci.org/biomartr/#tutorials) for more details.

`In the context of functional annotation retrieval` the `biomartr` package allows users to screen available marts using only the scientific name of an organism of interest instead of first searching for marts and datasets which support a particular organism of interest (which is required when using the `BiomaRt` package). Furthermore, `biomartr` allows you to search for particular topics when searching for attributes and filters. I am aware that the similar naming of the packages is unfortunate, but it arose due to historical reasons (please find a detailed explanation here: https://github.com/ropensci/biomartr/blob/master/FAQs.md and here [#11](https://github.com/ropensci/biomartr/issues/11)).

I also dedicated [an entire vignette to compare](https://docs.ropensci.org/biomartr/articles/Functional_Annotation.html) the `BiomaRt` and `biomartr` package functionality in the context of `Functional Annotation` (where their functionality overlaps which comprises about only 20% of the overall functionality of the biomartr package).

### Feedback
>__I truly value your opinion and improvement suggestions. Hence, I would be extremely grateful if you could take this 1 minute and 3 question survey (https://goo.gl/forms/Qaoxxjb1EnNSLpM02) so that I can learn how to improve `biomartr` in the best possible way. Many many thanks in advance.__


## Installation

The `biomartr` package relies on some [Bioconductor](https://www.bioconductor.org/install/) tools and thus requires
installation of the following packages:

```r
# Install core Bioconductor packages
if (!requireNamespace("BiocManager"))
    install.packages("BiocManager")
BiocManager::install()
# Install package dependencies
BiocManager::install("Biostrings")
BiocManager::install("biomaRt")
```

Now users can install `biomartr` from CRAN:

```r
# install biomartr 1.0.7 from CRAN
install.packages("biomartr", dependencies = TRUE)

# install the developer version containing the newest features
BiocManager::install("ropensci/biomartr")
```

## Installation with Bioconda

With an activated Bioconda channel (see 2. Set up channels), install with:

```
conda install r-biomartr
```

and update with:

```
conda update r-biomartr
```

or use the docker container:
```
docker pull quay.io/biocontainers/r-biomartr:<tag>
```
(check [r-biomartr/tags](https://quay.io/repository/biocontainers/r-biomartr?tab=tags) for valid values for <tag>)


## Example

### Collection Retrieval

The automated retrieval of collections (= Genome, Proteome, CDS, RNA, GFF, Repeat Masker, AssemblyStats files)
will make sure that the genome file of an organism will match the CDS, proteome, RNA, GFF, etc file
and was generated using the same genome assembly version. One aspect of why genomics studies
fail in computational and biological reproducibility is that it is not clear whether CDS, proteome, RNA, GFF, etc files
used in a proposed analysis were generated using the same genome assembly file denoting the same genome assembly version.
To avoid this seemingly trivial mistake we encourage users to retrieve
genome file collections using the `biomartr` function `getCollection()`
and attach the corresponding output as Supplementary Data
to the respective genomics study to ensure computational and biological reproducibility.


```r
# download collection for Saccharomyces cerevisiae
biomartr::getCollection( db = "refseq", organism = "Saccharomyces cerevisiae")
```

Internally, the `getCollection()` function will now generate a folder named `refseq/Collection/Saccharomyces_cerevisiae`
and will store all genome and annotation files for `Saccharomyces cerevisiae` in the same folder.
In addition, the exact genoem and annotation version will be logged in the `doc` folder.

Internally, a text file named `doc_Saccharomyces_cerevisiae_db_refseq.txt` is generated. The information stored in this log file is structured as follows:

```
File Name: Saccharomyces_cerevisiae_assembly_stats_refseq.txt
Organism Name: Saccharomyces_cerevisiae
Database: NCBI refseq
URL: ftp://ftp.ncbi.nlm.nih.gov/genomes/all/GCF/000/146/045/GCF_000146045.2_R64/GCF_000146045.2_R64_assembly_stats.txt
Download_Date: Wed Jun 27 15:21:51 2018
refseq_category: reference genome
assembly_accession: GCF_000146045.2
bioproject: PRJNA128
biosample: NA
taxid: 559292
infraspecific_name: strain=S288C
version_status: latest
release_type: Major
genome_rep: Full
seq_rel_date: 2014-12-17
submitter: Saccharomyces Genome Database
```

In an ideal world this reference file could then be included as supplementary information in any
life science publication that relies on genomic information so that
reproducibility of experiments and analyses becomes achievable.


### Genome retrieval of hundreds of genomes using only one command

Download all mammalian vertebrate genomes from `NCBI RefSeq` via:

```r
# download all vertebrate genomes
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "genome")
```
All geneomes are stored in the folder named according to the kingdom.
In this case `vertebrate_mammalian`. Alternatively, users can specify
the `out.folder` argument to define a custom output folder path.

### Frequently Asked Questions (FAQs)

Please find [all FAQs here](https://github.com/ropensci/biomartr/blob/master/FAQs.md).

### Discussions and Bug Reports

I would be very happy to learn more about potential improvements of the concepts and functions
provided in this package.

Furthermore, in case you find some bugs or need additional (more flexible) functionality of parts
of this package, please let me know:

https://github.com/HajkD/biomartr/issues


## Tutorials

Getting Started with `biomartr`:

- [NCBI Database Retrieval](https://docs.ropensci.org/biomartr/articles/Database_Retrieval.html)
- [Genomic Sequence Retrieval](https://docs.ropensci.org/biomartr/articles/Sequence_Retrieval.html)
- [Meta-Genome Retrieval](https://docs.ropensci.org/biomartr/articles/MetaGenome_Retrieval.html)
- [Functional Annotation](https://docs.ropensci.org/biomartr/articles/Functional_Annotation.html)
- [BioMart Examples](https://docs.ropensci.org/biomartr/articles/BioMart_Examples.html)


Users can also read the tutorials within ([Posit (former RStudio)](https://posit.co/)) :

```r
# source the biomartr package
library(biomartr)

# look for all tutorials (vignettes) available in the biomartr package
# this will open your web browser
browseVignettes("biomartr")
```

## NEWS
The current status of the package as well as a detailed history of the functionality of each version of `biomartr` can be found in the [NEWS](https://docs.ropensci.org/biomartr/news/index.html) section.


## Install Developer Version
Some bug fixes or new functionality will not be available on CRAN yet, but in the developer version here on GitHub. To download and install the most recent version of `biomartr` run:

```r
# install the current version of biomartr on your system
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("ropensci/biomartr")
```

## Genomic Data Retrieval

#### Meta-Genome Retrieval

* `meta.retrieval()` : Perform Meta-Genome Retieval from NCBI of species belonging to the same kingdom of life or to the same taxonomic subgroup
* `meta.retrieval.all()` : Perform Meta-Genome Retieval from NCBI of the entire kingdom of life
* `getMetaGenomes()` : Retrieve metagenomes from NCBI Genbank
* `getMetaGenomeAnnotations()` : Retrieve annotation *.gff files for metagenomes from NCBI Genbank
* `listMetaGenomes()` : List available metagenomes on NCBI Genbank
* `getMetaGenomeSummary()` : Helper function to retrieve the assembly_summary.txt file from NCBI genbank metagenomes
* `clean.retrieval()`: Format meta.retrieval output

#### Genome Retrieval

* `listGenomes()` : List all genomes available on NCBI and ENSEMBL servers
* `listKingdoms()` : list the number of available species per kingdom of life on NCBI and ENSEMBL servers
* `listGroups()` : list the number of available species per group on NCBI and ENSEMBL servers
* `getKingdoms()` : Retrieve available kingdoms of life
* `getGroups()` : Retrieve available groups for a kingdom of life
* `is.genome.available()` : Check Genome Availability  NCBI and ENSEMBL servers
* `getCollection()` : Retrieve a Collection: Genome, Proteome, CDS, RNA, GFF, Repeat Masker, AssemblyStats
* `getGenome()` : Download a specific genome stored on NCBI and ENSEMBL servers
* `getGenomeSet()` : Genome Retrieval of multiple species
* `getProteome()` : Download a specific proteome stored on NCBI and ENSEMBL servers
* `getProteomeSet()` : Proteome Retrieval of multiple species
* `getCDS()` : Download a specific CDS file (genome) stored on NCBI and ENSEMBL servers
* `getCDSSet()` : CDS Retrieval of multiple species
* `getRNA()` : Download a specific RNA file stored on NCBI and ENSEMBL servers
* `getRNASet()` : RNA Retrieval of multiple species
* `getGFF()` : Genome Annotation Retrieval from NCBI (`*.gff`) and ENSEMBL (`*.gff3`) servers
* `getGTF()` : Genome Annotation Retrieval (`*.gtf`) from ENSEMBL servers
* `getRepeatMasker() :` Repeat Masker TE Annotation Retrieval
* `getAssemblyStats()` : Genome Assembly Stats Retrieval from NCBI
* `getKingdomAssemblySummary()` : Helper function to retrieve the assembly_summary.txt files from NCBI for all kingdoms
* `getMetaGenomeSummary()` : Helper function to retrieve the assembly_summary.txt files from NCBI genbank metagenomes
* `getSummaryFile()` : Helper function to retrieve the assembly_summary.txt file from NCBI for a specific kingdom
* `getENSEMBLInfo()` : Retrieve ENSEMBL info file
* `getGENOMEREPORT()` : Retrieve GENOME_REPORTS file from NCBI

#### Import Downloaded Files
* `read_genome()` : Import genomes as Biostrings or data.table object
* `read_proteome()` : Import proteome as Biostrings or data.table object
* `read_cds()` : Import CDS as Biostrings or data.table object
* `read_gff()` : Import GFF file
* `read_rna()` : Import RNA file
* `read_rm()` : Import Repeat Masker output file
* `read_assemblystats()` : Import Genome Assembly Stats File

#### Database Retrieval

* `listNCBIDatabases()` : Retrieve a List of Available NCBI Databases for Download
* `download.database()` : Download a NCBI database to your local hard drive
* `download.database.all()` : Download a complete NCBI Database such as e.g. `NCBI nr` to your local hard drive

### BioMart Queries

* `biomart()` : Main function to query the BioMart database
* `getMarts()` : Retrieve All Available BioMart Databases
* `getDatasets()` : Retrieve All Available Datasets for a BioMart Database
* `getAttributes()` : Retrieve All Available Attributes for a Specific Dataset
* `getFilters()` : Retrieve All Available Filters for a Specific Dataset
* `organismBM()` : Function for organism specific retrieval of available BioMart marts and datasets
* `organismAttributes()` : Function for organism specific retrieval of available BioMart attributes
* `organismFilters()` : Function for organism specific retrieval of available BioMart filters

### Performing Gene Ontology queries

#### Gene Ontology

* `getGO()` : Function to retrieve GO terms for a given set of genes


## Download Developer Version On Windows Systems

```r
# On Windows, this won't work - see ?build_github_devtools
install_github("HajkD/biomartr", build_vignettes = TRUE, dependencies = TRUE)

# When working with Windows, first you need to install the
# R package: rtools -> install.packages("rtools")

# Afterwards you can install devtools -> install.packages("devtools")
# and then you can run:

devtools::install_github("HajkD/biomartr", build_vignettes = TRUE, dependencies = TRUE)

# and then call it from the library
library("biomartr", lib.loc = "C:/Program Files/R/R-3.1.1/library")
```

# Code of conduct

Please note that this project is released with a [Contributor Code of Conduct](https://github.com/ropensci/biomartr/blob/master/CONDUCT.md). By participating in this project you agree to abide by its terms.

#' @title Helper function for retrieving biological sequence files from ENSEMBL
#' @description This function downloads gff files of query
#' organisms from ENSEMBL.
#' @inheritParams getENSEMBL
#' @author Hajk-Georg Drost
#' @return either a character path to downloaded file, or a logical FALSE, specifying failure.
getENSEMBL.Seq <- function(organism, type = "dna", id.type = "toplevel", release = NULL, path) {

  getENSEMBL(organism, type, id.type, release, path, format = "fasta")
}

#' @title Helper function for retrieving gff/gtf files from ENSEMBL
#' @description This function downloads gff/gtff
#' files of query organisms from ENSEMBL.
#' @inheritParams getENSEMBL
#' @author Hajk-Georg Drost
#' @noRd
getENSEMBL.Annotation <- function(organism, type = "dna", release = NULL,
                                  path, format = "gff3") {
  getENSEMBL(organism, type, id.type = "none", release, path, format)
}

#' @title Helper function for retrieving gtf files from ENSEMBL
#' @description This function downloads gff
#' files of query organisms from ENSEMBL.
#' @inheritParams getENSEMBL
#' @import curl RCurl
#' @author Hajk-Georg Drost
#' @return character filepath to download file, returns FALSE if failed.
getENSEMBL.gtf <- function(organism, type = "dna",
                           path, release = NULL) {
  getENSEMBL.Annotation(organism, type, release, path, format = "gtf")[1]
}

#' @title Genome Assembly Stats Retrieval
#' @description  Main genome assembly stats retrieval function for an organism
#' of interest. By specifying the scientific name of an organism of interest the
#' corresponding  genome assembly stats file storing the assembly statistics of
#' the organism of interest can be downloaded and stored locally.
#' Genome assembly stats files can be retrieved from several databases.
#' @param db a character string specifying the database from which the genome
#' shall be retrieved:
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' \item \code{db = "ensembl"}
#' }
#' @param organism a character string specifying the scientific name of the
#' organism of interest, e.g. \code{organism = "Homo sapiens"}.
#' @param type shall only the file be retrieved (default)
#' \code{type = "download"} or should the corresponding file be downloaded and
#' subsequently be imported \code{type = "import"}.
#' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.
#' @param skip_bacteria Due to its enormous dataset size (> 700MB as of July 2023),
#' the bacterial summary file will not be loaded by default anymore. If users
#' wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
#' bacterial summary file will be downloaded.
#' @param path a character string specifying the location (a folder) in
#' which the corresponding file shall be stored. Default is
#' \code{path} = \code{file.path("_ncbi_downloads","genomeassembly_stats")}.
#' @param mute_citation logical value indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @details Internally this function loads the the overview.txt file from NCBI:
#'
#'  refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/
#'
#'  genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/
#'
#' to retrieve available scientific names of organisms and creates a directory
#' '_ncbi_downloads/genomeassembly_stats' to store
#' the Genome Assembly Stats of interest as text file for future processing.
#' In case the corresponding fasta file already exists within the
#' '_ncbi_downloads/genomeassembly_stats' folder and is
#' accessible within the workspace, no download process will be performed.
#'
#' An example genome assembly stats file can be found here:
#' ftp://ftp.ncbi.nlm.nih.gov/genomes/all/GCF/000/001/405/
#' GCF_000001405.36_GRCh38.p10/GCF_000001405.36_GRCh38.p10_assembly_stats.txt.
#'
#' @return File path to downloaded genome assembly stats file.
#' @examples \dontrun{
#' # download the genome assembly stats file of Saccharomyces cerevisiae
#' # from NCBI RefSeq
#' # and store the corresponding genome file in
#' # '_ncbi_downloads/genomeassembly_stats'
#' file_path <- getAssemblyStats( db = "refseq",
#'                  organism = "Saccharomyces cerevisiae",
#'                  path = file.path("_ncbi_downloads","genomeassembly_stats"))
#' # import the raw file as it is downloaded
#' Scerevisiae.stats <- read_assemblystats(file_path, type = "raw")
#'
#' # download the genome assembly stats file of Saccharomyces cerevisiae
#' # from NCBI RefSeq
#' # and import overall statistics of the genome assembly
#' Scerevisiae.stats.import <- getAssemblyStats( db = "refseq",
#'                  organism = "Saccharomyces cerevisiae",
#'                  type = "import",
#'                  path = file.path("_ncbi_downloads","genomeassembly_stats"))
#' }
#'
#' @seealso \code{\link{getGenome}}, \code{\link{getProteome}}, \code{\link{getCDS}},
#' \code{\link{getGFF}}, \code{\link{getRNA}}, \code{\link{getCollection}}, \code{\link{meta.retrieval}},
#' \code{\link{read_assemblystats}}
#' @export

getAssemblyStats <-
  function(db = "refseq",
           organism,
           reference = FALSE,
           skip_bacteria = TRUE,
           type = "download",
           path = file.path("_ncbi_downloads", "genomeassembly_stats"),
           mute_citation = FALSE) {
    if (!is.element(db, c("refseq", "genbank")))
      stop(
        paste0(
          "Please select one of the available data bases: ",
          "'refseq' and 'genbank'."
        ),
        call. = FALSE
      )
    
    if (!is.element(type, c("download", "import")))
      stop(
        paste0(
          "Please choose either type = 'download' ",
          "(if you would like to ",
          "download the genome assembly stats file) or ",
          "type = 'import' (if you would like to download and import the ",
          "genome assembly stats file).",
          collapse = ""
        ),
        call. = FALSE
      )
    
    message("-> Starting assembly quality stats retrieval of '",
            organism,
            "' from ",
            db,
            " ...")
    message("\n")
    
    # get Kingdom Assembly Summary file
    AssemblyFilesAllKingdoms <-
      getKingdomAssemblySummary(db = db, skip_bacteria = skip_bacteria)
    
    species <- organism
    # test whether or not genome is available
    suppressMessages(is.genome.available(
      organism = organism,
      db = db,
      skip_bacteria = skip_bacteria
    ))
    
    if (!file.exists(path)) {
      dir.create(path, recursive = TRUE)
    }
    
    organism_name <-
      refseq_category <- version_status <- NULL
    
    organism <- stringr::str_replace_all(organism, "\\(", "")
    organism <- stringr::str_replace_all(organism, "\\)", "")
    
    assembly_accession <- taxid <- ftp_path <- NULL
    
    if (reference) {
      if (!is.taxid(organism)) {
        FoundOrganism <-
          dplyr::filter(
            AssemblyFilesAllKingdoms,
            stringr::str_detect(organism_name, organism) |
              stringr::str_detect(assembly_accession, organism),
            ((refseq_category == "representative genome") |
               (refseq_category == "reference genome")
            ),
            (version_status == "latest"),
            !is.na(ftp_path)
          )
      } else {
        FoundOrganism <-
          dplyr::filter(
            AssemblyFilesAllKingdoms,
            taxid == as.integer(organism),
            ((refseq_category == "representative genome") |
               (refseq_category == "reference genome")
            ),
            (version_status == "latest"),
            !is.na(ftp_path)
          )
      }
    } else {
      if (!is.taxid(organism)) {
        FoundOrganism <-
          dplyr::filter(
            AssemblyFilesAllKingdoms,
            stringr::str_detect(organism_name, organism) |
              stringr::str_detect(assembly_accession, organism),
            (version_status == "latest"),
            !is.na(ftp_path)
          )
      } else {
        FoundOrganism <-
          dplyr::filter(
            AssemblyFilesAllKingdoms,
            taxid == as.integer(organism),
            (version_status == "latest"),
            !is.na(ftp_path)
          )
      }
    }
    
    if (nrow(FoundOrganism) == 0) {
      message(
        paste0(
          "----------> No reference genome or representative genome was found for '",
          organism,
          "'. Thus, download for this species has been omitted.",
          " Have you tried to specify 'reference = FALSE' ?"
        )
      )
      return("Not available")
    } else {
      if (nrow(FoundOrganism) > 1) {
        warnings(
          "More than one entry has been found for '",
          organism,
          "'. Only the first entry '",
          FoundOrganism[1, 1],
          "' has been used for subsequent genome retrieval."
        )
        FoundOrganism <- FoundOrganism[1,]
      }
      
      organism <-
        stringr::str_replace_all(organism, " ", "_")
      
      download_url <-
        paste0(FoundOrganism$ftp_path,
               "/",
               paste0(basename(FoundOrganism$ftp_path),
                      "_assembly_stats.txt"))
      
      local.org <-
        stringr::str_replace_all(organism, "-", "_")
      local.org <-
        stringr::str_replace_all(organism, "\\/", "_")
      
      # if (!exists.ftp.file(url = paste0(FoundOrganism$ftp_path, "/"),
      #                      file.path = download_url)) {
      #     message(
      #         "Unfortunately no assembly stats file could be
      #         found for organism '",
      #         organism,
      #         "'. Thus, the download of this organism has been omitted."
      #     )
      #     return(FALSE)
      # }
      
      if (nrow(FoundOrganism) == 1) {
        if (file.exists(file.path(path,
                                  paste0(
                                    local.org, "_assembly_stats_", db, ".txt"
                                  )))) {
          message(
            "File ",
            file.path(
              path,
              paste0(local.org, "_assembly_stats_", db, ".txt")
            ),
            " exists already. Thus, download has been skipped."
          )
        } else {
          tryCatch({
            utils::capture.output(custom_download(
              download_url,
              destfile = file.path(
                path,
                paste0(local.org, "_assembly_stats_", db,
                       ".txt")
              ),
              mode = "wb"
            ))
          }, error = function(e) {
            message(
              "The download session seems to have timed out at the FTP site '",
              download_url,
              "'. This could be due to an overload of queries to the databases.",
              " Please restart this function to continue the data retrieval process or wait ",
              "for a while before restarting this function in case your IP address was logged due to an query overload on the server side."
            )
            return("Not available")
          })
        }
        
        message("-> Genome assembly quality stats file download completed!")
        
        # download md5checksum file for organism of interest
        custom_download(
          paste0(FoundOrganism$ftp_path, "/md5checksums.txt"),
          file.path(path,
                    paste0(local.org, "_md5checksums.txt")),
          mode = "wb"
        )
        
        # test check sum
        md5_file_path <- file.path(path,
                                   paste0(local.org,
                                          "_md5checksums.txt"))
        md5_file <-
          read_md5file(md5_file_path)
        
        file_name <- NULL
        
        md5_sum <- dplyr::filter(md5_file,
                                 file_name == paste0("./", paste0(
                                   basename(FoundOrganism$ftp_path),
                                   "_assembly_stats.txt"
                                 )))$md5
        # check if md5 info is available
        if (!is.character(md5_sum)){
        message(
          "-> Checking md5 hash of file: ",
          file.path(path,
                    paste0(
                      local.org, "_assembly_stats_", db,
                      ".txt"
                    )),
          " (md5: ",
          md5_sum,
          ")",
          " ..."
        )
        
        if (!(tools::md5sum(file.path(
          path,
          paste0(local.org, "_assembly_stats_", db,
                 ".txt")
        )) == md5_sum))
          stop(
            paste0(
              "Please download the file '",
              md5_file_path,
              "' again. The md5 hash between the downloaded file and the file ",
              "stored at NCBI do not match.",
              collapse = ""
            )
          )
        unlink(md5_file_path)
        message("-> The md5 hash of file '", md5_file_path, "' matches!")
        }
        
        docFile(
          file.name =
            paste0(local.org, "_assembly_stats_", db, ".txt"),
          organism  = organism,
          url       = download_url,
          database  = db,
          path      = path,
          refseq_category = FoundOrganism$refseq_category,
          assembly_accession = FoundOrganism$assembly_accession,
          bioproject = FoundOrganism$bioproject,
          biosample = FoundOrganism$biosample,
          taxid = FoundOrganism$taxid,
          infraspecific_name = FoundOrganism$infraspecific_name,
          version_status = FoundOrganism$version_status,
          release_type = FoundOrganism$release_type,
          genome_rep = FoundOrganism$genome_rep,
          seq_rel_date = FoundOrganism$seq_rel_date,
          submitter = FoundOrganism$submitter
        )
        
        doc <- tibble::tibble(
          file_name = paste0(
            ifelse(
              is.taxid(organism),
              paste0("taxid_", local.org),
              local.org
            ),
            "_genomic_",
            db,
            ".fna.gz"
          ),
          organism  = organism,
          url       = download_url,
          database  = db,
          path      = path,
          refseq_category = FoundOrganism$refseq_category,
          assembly_accession = FoundOrganism$assembly_accession,
          bioproject = FoundOrganism$bioproject,
          biosample = FoundOrganism$biosample,
          taxid = FoundOrganism$taxid,
          infraspecific_name = FoundOrganism$infraspecific_name,
          version_status = FoundOrganism$version_status,
          release_type = FoundOrganism$release_type,
          genome_rep = FoundOrganism$genome_rep,
          seq_rel_date = FoundOrganism$seq_rel_date,
          submitter = FoundOrganism$submitter
          
        )
        
        readr::write_tsv(doc, file = file.path(path, paste0(
          "doc_", local.org, "_db_", db, ".tsv"
        )))
        
        message(
          paste0(
            "-> The assembly statistics file of '",
            organism,
            "' has been downloaded to '",
            path,
            "' and has been named '",
            paste0(local.org, "_assembly_stats_", db, ".txt"),
            "'."
          )
        )
        
        if (type == "download"){
          please_cite_biomartr(mute_citation = mute_citation)
        
          return(file.path(path,
                           paste0(
                             local.org, "_assembly_stats_", db, ".txt"
                           )))
        }
        if (type == "import") {
          assembly_stats_file <- read_assemblystats(file.path(
            path,
            paste0(local.org, "_assembly_stats_", db, ".txt")
          ), type = "stats")
          
          assembly_stats_file <-
            dplyr::bind_cols(tibble::tibble(species = species),
                             assembly_stats_file)
          please_cite_biomartr(mute_citation = mute_citation)
          
          return(assembly_stats_file)
        }
      } else {
        message(
          "File: ",
          download_url,
          " could not be loaded properly...
                     Something went wrong with the internet connection."
        )
      }
    }
  }

context("Test: getAssemblyStats()")


test_that("The getAssemblyStats() downloads assembly stats file and reads
          raw input: NCBI RefSeq ..",{

    skip_on_cran()
    skip_on_travis()

    library(magrittr)

    Scerevisiae.stats <- getAssemblyStats(
        db = "refseq",
        organism = "Saccharomyces cerevisiae",
        path = tempdir(),
        mute_citation = TRUE
    )


    raw <- Scerevisiae.stats %>%
        read_assemblystats(type = "raw")

    stats <- Scerevisiae.stats %>%
        read_assemblystats(type = "stats")

    expect_false(nrow(raw) == nrow(stats))
})


test_that("The getAssemblyStats() downloads assembly stats file and reads raw
          input: NCBI Genbank ..",{

    skip_on_cran()
    skip_on_travis()

    Scerevisiae.stats <- getAssemblyStats(
        db = "genbank",
        organism = "Saccharomyces cerevisiae",
        path = tempdir(),
        mute_citation = TRUE
    )

    raw <- read_assemblystats(Scerevisiae.stats, type = "raw")

    stats <- read_assemblystats(Scerevisiae.stats, type = "stats")
    expect_false(nrow(raw) == nrow(stats))
})


test_that("The getAssemblyStats() downloads assembly stats file and imports file
          ..",{

    skip_on_cran()
    skip_on_travis()
    Scerevisiae.stats <- getAssemblyStats(
        db = "refseq",
        organism = "Saccharomyces cerevisiae",
        type = "import",
        path = tempdir(),
        mute_citation = TRUE
    )
    expect_false(nrow(Scerevisiae.stats) == 0)

})

test_that("The getAssemblyStats() throws error when undefined 'type' is selected
          ..",{

              skip_on_cran()
              skip_on_travis()
              expect_error(getAssemblyStats(
                  db = "refseq",
                  organism = "Saccharomyces cerevisiae",
                  type = "somethingelse",
                  path = tempdir(),
                  mute_citation = TRUE))
})


test_that("The getAssemblyStats() throws error when undefined 'db' is selected
          ..",{

              skip_on_cran()
              skip_on_travis()
              expect_error(getAssemblyStats(
                  db = "somethingelse",
                  organism = "Saccharomyces cerevisiae",
                  type = "download",
                  path = tempdir(),
                  mute_citation = TRUE))
})


test_that("The getAssemblyStats() throws error when undefined 'organism' is selected
          ..",{
              skip_on_cran()
              skip_on_travis()
              expect_message(getAssemblyStats(
                  db = "refseq",
                  organism = "somethingelse",
                  type = "download",
                  path = tempdir(),
                  mute_citation = TRUE))
})

#' Get uniprot info from organism
#'
#' @param organism character, name of organism
#' @param path path at which the info file shall be stored locally.
#' @param update shall the internal \code{\link{cachedir}} file be deleted and the info
#' file freshly downloaded from the UniProt API?
#' @export
getUniProtInfo <- function(organism, path = cachedir(), update = TRUE) {
  organism_new <- stringr::str_replace_all(organism, " ", "%20")
  organism_name_path <- stringr::str_replace_all(organism, " ", "_")

  file <- file.path(cachedir(), paste0(organism_name_path, "_uniprot_info.tsv"))
  ebi_url <- "https://www.ebi.ac.uk/"
  ebi_url_proteome <- paste0(ebi_url, "proteins/api/proteomes")
  if (file.exists(file) && !update) {
    suppressWarnings(uniprot_species_info <- read_uniprot_info(file))

  } else {
    tryCatch({
      uniprot_species_info <-
        tibble::as_tibble(jsonlite::fromJSON(
          paste0(ebi_url_proteome, "?offset=0&size=-1&name=", organism_new)
        ))
      uniprot_species_info <- write_uniprot_info(file, uniprot_species_info,
                                                 organism)
    }, error = function(e)
      message(
        "Something went wrong when trying to access the API '", paste0(ebi_url_proteome, "?offset=0&size=-1&name=", organism_new), "'",
        " Sometimes the internet connection isn't stable and re-running the function might help. Is it possible to access the homepage '",
        ebi_url, "' through your browser?"
      ))}

    return(uniprot_species_info)
}

read_uniprot_info <- function(file) {
  readr::read_tsv(
    file,
    col_names = TRUE,
    col_types = readr::cols(
      name = readr::col_character(),
      upid = readr::col_integer(),
      taxonomy = readr::col_integer(),
      isReferenceProteome = readr::col_logical(),
      isRepresentativeProteome = readr::col_logical()
    )
  )
}

write_uniprot_info <- function(file, uniprot_species_info, organism) {
  name <- upid <- taxonomy <- isReferenceProteome <-
    isRepresentativeProteome <- superregnum <- NULL

  uniprot_species_info <-
    dplyr::filter(uniprot_species_info, stringr::str_detect(name, organism))

  if (nrow(uniprot_species_info) > 1) {
    # message("There are more than one entry for '",organism,"'.",
    #         " Please select the one below that you prefer and re-run this function using the full name you chose.")
    # message("\n")
    # message("Options are: ", paste0("organism = '",uniprot_species_info$name,"', "),".")
    # Subset to valid reference
    is_reference <- uniprot_species_info$isReferenceProteome
    if (any(is_reference)) {
      uniprot_species_info <- uniprot_species_info[is_reference, ]
    }

    most_expansive_assembly <- which.max(sapply(uniprot_species_info$dbReference, nrow))
    uniprot_species_info <- uniprot_species_info[most_expansive_assembly,]
  }

  if (nrow(uniprot_species_info) == 0) {
    warning("Could not find a valid reference assembly id for uniprot for this species!")
    return(uniprot_species_info)
  }

  uniprot_species_info <-
    dplyr::select(
      uniprot_species_info,
      name,
      upid,
      taxonomy,
      isReferenceProteome,
      isRepresentativeProteome,
      superregnum
    )

  readr::write_tsv(uniprot_species_info, file)
  return(uniprot_species_info)
}

#' @title Retrieve the assembly_summary.txt file from NCBI genbank metagenomes
#' @description Retrieval function of the assembly_summary.txt file
#' from NCBI genbank metagenomes.
#' This files stores all available metagenome projects on NCBI Genbank.
#' @param local_file where to store this backend file, default:
#' file.path(cachedir(), "assembly_summary_metagenomes_genbank.txt")
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' meta.summary <- getMetaGenomeSummary()
#' meta.summary
#' }
#' @seealso \code{\link{getKingdomAssemblySummary}},
#' \code{\link{getSummaryFile}}
#' @export

getMetaGenomeSummary <- function(local_file = file.path(cachedir(), "assembly_summary_metagenomes_genbank.txt")) {
    url <- "ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/metagenomes/assembly_summary.txt"
    custom_download_check_local(url, local_file, NULL, db = "genbank")
    suppressWarnings(summary.file <-
                         tibble::as_tibble(
                             readr::read_delim(
                                 local_file,
                                 comment = "#",
                                 delim = "\t",
                                 quote = "\"",
                                 escape_backslash = FALSE,
                                 col_names = c(
                                     "assembly_accession",
                                     "bioproject",
                                     "biosample",
                                     "wgs_master",
                                     "refseq_category",
                                     "taxid",
                                     "species_taxid",
                                     "organism_name",
                                     "infraspecific_name",
                                     "isolate",
                                     "version_status",
                                     "assembly_level",
                                     "release_type",
                                     "genome_rep",
                                     "seq_rel_date",
                                     "asm_name",
                                     "submitter",
                                     "gbrs_paired_asm",
                                     "paired_asm_comp",
                                     "ftp_path",
                                     "excluded_from_refseq"
                                 ),
                                 col_types = readr::cols(
                                    assembly_accession = readr::col_character(),
                                     bioproject = readr::col_character(),
                                     biosample = readr::col_character(),
                                     wgs_master = readr::col_character(),
                                     refseq_category = readr::col_character(),
                                     taxid = readr::col_integer(),
                                     species_taxid = readr::col_integer(),
                                     organism_name = readr::col_character(),
                                    infraspecific_name = readr::col_character(),
                                     isolate = readr::col_character(),
                                     version_status = readr::col_character(),
                                     assembly_level = readr::col_character(),
                                     release_type = readr::col_character(),
                                     genome_rep = readr::col_character(),
                                     seq_rel_date = readr::col_date(),
                                     asm_name = readr::col_character(),
                                     submitter = readr::col_character(),
                                     gbrs_paired_asm = readr::col_character(),
                                     paired_asm_comp = readr::col_character(),
                                     ftp_path = readr::col_character(),
                                excluded_from_refseq = readr::col_character()
                                 )
                             )
                         ))

    return(summary.file)
}

context("Test: getMetaGenomeSummary()")

test_that("The getMetaGenomeSummary() interface works properly..",{
    skip_on_cran()
    skip_on_travis()
    expect_false(nrow(getMetaGenomeSummary()) == 0)
})

#' @title Retrieve All Available Datasets for a BioMart Database
#' @description This funcion queries the BioMart API and returns a table
#' storing all available datasets for a selected BioMart databases.
#' @param mart a character string specifying the database (mart) for 
#' which datasets shall be listed.
#' @param mute_citation logical value indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # search for available datasets
#' # getMarts()
#' # choose database: "ENSEMBL_MART_ENSEMBL"
#' head(getDatasets("ENSEMBL_MART_ENSEMBL"), 10)
#' }
#' @seealso \code{\link{getMarts}}, \code{\link{getAttributes}}, 
#' \code{\link{getFilters}}, \code{\link{organismBM}}, 
#' \code{\link{organismFilters}}, \code{\link{organismAttributes}}
#' @export

getDatasets <- function(mart, mute_citation = FALSE) {
    if (!is.character(mart))
        stop("Please use a character string as mart.", call. = FALSE)
    
    if (stringr::str_detect(mart, "ENSEMBL"))
        # connect to BioMart API
        datasetPage <-
            httr::handle(
                paste0(
paste0("http://www.ensembl.org/biomart/martservice?",
       "type=datasets&requestid=biomart&mart=", collapse = ""),
                    mart
                )
            )
    
    if (stringr::str_detect(mart, "plants"))
        # connect to BioMart API
        datasetPage <-
            httr::handle(
                paste0(
paste0("http://plants.ensembl.org/biomart/martservice?",
"type=datasets&requestid=biomart&mart=", collapse = ""),
                    mart
                )
            )
    
    if (stringr::str_detect(mart, "fung"))
        # connect to BioMart API
        datasetPage <-
            httr::handle(
                paste0(
                    paste0("http://fungi.ensembl.org/biomart/martservice?",
"type=datasets&requestid=biomart&mart=", collapse = ""),
                    mart
                )
            )
    
    if (stringr::str_detect(mart, "protist"))
        # connect to BioMart API
        datasetPage <-
            httr::handle(
                paste0(
paste0("http://protists.ensembl.org/biomart/martservice?",
"type=datasets&requestid=biomart&mart=", collapse = ""),
                    mart
                )
            )
    
    if (stringr::str_detect(mart, "metazoa"))
        # connect to BioMart API
        datasetPage <-
            httr::handle(
                paste0(
paste0("http://metazoa.ensembl.org:80/biomart/martservice?",
"type=datasets&requestid=biomart&mart=", collapse = ""),
                    mart
                )
            )
    
    xmlContentDatasets <- httr::GET(handle = datasetPage)
    
    tryCatch({
        httr::stop_for_status(xmlContentDatasets)
        
        # extract dataset name, description, and version, etc.
        rawDF <-
            do.call("rbind", apply(as.data.frame(stringr::str_split(
                httr::content(
                    xmlContentDatasets,
                    as = "text",
                    encoding = "UTF-8"
                ),
                "\n"
            )), 1, function(x)
                unlist(strsplit(x, "\t"))))
        
        colnames(rawDF) <- paste0("V", seq_len(ncol(rawDF)))
        
        if (dim(rawDF)[1] > 2)
            # store available datasets
            dsBioMart <-
            as.data.frame(
                rawDF[-seq(1, nrow(rawDF), 2), c("V2", "V3", "V5")],
                stringsAsFactors = FALSE,
                colClasses = rep("character", 3)
            )
        
        if (dim(rawDF)[1] <= 2)
            dsBioMart <- data.frame(V2 = "",
                                    V3 = "",
                                    V5 = "")
        
        colnames(dsBioMart) <- c("dataset", "description", "version")
        please_cite_biomartr(mute_citation = mute_citation)
        
        return(tibble::as_tibble(dsBioMart))
    }, error = function(e)
        stop(
            "Your input mart '",
            mart,
            "' could not be found. Please use getMarts() 
            to choose from available marts.", call. = FALSE
        ))
}

context("Test: getDatasets()")


equal_lists <- function(df1, df2) {
        isTRUE(all.equal(df1, df2))
}


test_that("The getDatasets() interface works properly..",{

    skip_on_cran()
    skip_on_travis()

    expect_identical(getMarts()[1 , "mart"][[1]], "ENSEMBL_MART_ENSEMBL")
    expect_output(getDatasets(mart = "ENSEMBL_MART_ENSEMBL"))

})

#' @title Import GFF File
#' @description This function reads an organism specific CDS stored
#' in a defined file format.
#' @param file a character string specifying the path to the file
#' storing the CDS.
#' @author Hajk-Georg Drost
#' @details This function takes a string specifying the path to the GFF file
#' of interest (e.g. the path returned by \code{\link{getGFF}}).
#' @return Either a \code{Biostrings} or \code{data.table} object.
#' @family gff
#' @family readers
#' @export
read_gff <- function(file) {
    # read gff file content
        gff.input <-
            readr::read_delim(
                file = file,
                delim = "\t",
                col_names = FALSE,
                comment = "#",
                col_types = readr::cols(
                  "X1" = readr::col_character(),
                  "X2" = readr::col_character(),
                  "X3" = readr::col_character(),
                  "X4" = readr::col_integer(),
                  "X5" = readr::col_integer(),
                  "X6" = readr::col_character(),
                  "X7" = readr::col_character(),
                  "X8" = readr::col_character(),
                  "X9" = readr::col_character()
                )
            )

    if (!file.exists(file))
        stop("The file path you specified does not seem to exist: '", file,"'.", call. = FALSE)

    if (ncol(gff.input) > 9)
        stop("The gff file format can not store more than 9 columns!", call. = FALSE)

    # name standardized columns
    gffNames <- c("seqid",
                  "source",
                  "type",
                  "start",
                  "end",
                  "score",
                  "strand",
                  "phase",
                  "attribute")

    names(gff.input) <- gffNames

    return(gff.input)
}

#' @title Import CDS as Biostrings or data.table object
#' @description This function reads an organism specific CDS stored in a
#' defined file format.
#' @param file a character string specifying the path to the file storing
#' the CDS.
#' @param format a character string specifying the file format used to store
#' the genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.
#' @param obj.type a character string specifying the object stype in which the
#' genomic sequence shall be represented.
#' Either as \code{obj.type = "Biostrings"} (default) or as
#' \code{obj.type = "data.table"}.
#' @param delete_corrupt a logical value specifying whether potential CDS
#' sequences that cannot be divided by 3 shall be
#' be excluded from the the dataset. Default is \code{delete_corrupt = FALSE}.
#' @param ... additional arguments that are used by
#' \code{\link[seqinr]{read.fasta}}.
#' @author Hajk-Georg Drost
#' @details The \code{read.cds} function takes a string specifying the path
#' to the cds file of interest as first argument.
#'
#' It is possible to read in different proteome file standards such as
#' \emph{fasta} or \emph{genebank}.
#'
#' CDS stored in fasta files can be downloaded from
#' http://www.ensembl.org/info/data/ftp/index.html.
#' @return A data.table storing the gene id in the first column and the
#' corresponding sequence as string in the second column.
#' @family cds
#' @family readers
#' @import Biostrings
#' @export
read_cds <-
    function(file,
             format = "fasta",
             obj.type = "Biostrings",
             delete_corrupt = FALSE,
             ...) {
        if (!is.element(format, c("fasta", "gbk")))
            stop("Please choose a file format that is
                 supported by this function.",
                 call. = FALSE)

        if (!is.element(obj.type, c("Biostrings", "data.table")))
            stop(
                "Please specify a valid object type: obj.type = 'Biostrings'
                (default) or obj.type = 'data.table'.",
                call. = FALSE
            )

        if (!file.exists(file))
            stop("The file path you specified does not seem to exist: '", file,"'.", call. = FALSE)

        geneids <- seqs <- NULL

        if (obj.type == "Biostrings") {
            tryCatch({
                cds_file <-
                    Biostrings::readBStringSet(filepath = file,
                                               format = format, ...)
            }, error = function(e) {
                stop(
                    paste0(
                        "File ",
                        file,
                        " could not be read properly. \n",
                        "Please make sure that ",
                        file,
                        " contains only CDS sequences and is in ",
                        format,
                        " format."
                    ),
                    call. = FALSE
                )
            })

            return(cds_file)
        }

        if (obj.type == "data.table") {
            geneids <- seqs <- NULL

            tryCatch({
                cds_file <-
                    Biostrings::readDNAStringSet(filepath = file, format = format, ...)

                cds_names <-
                    as.vector(unlist(sapply(cds_file@ranges@NAMES, function(x) {
                        return(strsplit(x, " ")[[1]][1])
                    })))

                cds.dt <-
                    data.table::data.table(geneids = cds_names ,
                                           seqs = tolower(as.character(cds_file)))


                data.table::setkey(cds.dt, geneids)

                mod3 <-
                    function(x) {
                        return((nchar(x) %% 3) == 0)
                    }

                all_triplets <- as.logical(cds.dt[ , mod3(seqs)])

                n_seqs <- nrow(cds.dt)

            }, error = function(e) {
                stop(
                    "File ",
                    file,
                    " could not be read properly.",
                    "\n",
                    "Please make sure that ",
                    file,
                    " contains only CDS sequences and is in ",
                    format,
                    " format."
                )
            })

            if (!all(all_triplets)) {
                message(
                    "There seem to be ",
                    length(which(!all_triplets)),
                    " coding sequences in your input dataset which cannot be properly divided in base triplets, because their sequence length cannot be divided by 3."
                )
                corrupted_file <-
                    paste0(basename(file), "_corrupted_cds_seqs.fasta")

                message(
                    "A fasta file storing all corrupted coding sequences for inspection was generated and stored at '",
                    file.path(getwd(), corrupted_file),
                    "'."
                )
                message("\n")
                corrupted_seqs <- as.data.frame(cds.dt[which(!all_triplets)])
                seq_vector <- corrupted_seqs$seqs
                names(seq_vector) <- corrupted_seqs$geneids
                corrupted_seqs_biostrings <- Biostrings::DNAStringSet(seq_vector, use.names = TRUE)
                Biostrings::writeXStringSet(corrupted_seqs_biostrings, filepath = corrupted_file)

                if (delete_corrupt) {
                    message(
                        "You chose option 'delete_corrupt = TRUE', thus corrupted coding sequences were removed.",
                        "If after consulting the file '",
                        corrupted_file,
                        "' you still wish to retain all coding sequences please specify the argument 'delete_corrupt = FALSE'."
                    )
                    message("\n")
                    return(cds.dt[-which(!all_triplets) , list(geneids, seqs)])
                }

                if (!delete_corrupt) {
                    message(
                        "You chose option 'delete_corrupt = FALSE', thus corrupted coding sequences were retained for subsequent analyses.")
                    message(
                        "The following modifications were made to the CDS sequences that were not divisible by 3:")
                    message(
                        "- If the sequence had 1 residue nucleotide then the last nucleotide of the sequence was removed.")
                    message(
                        "- If the sequence had 2 residue nucleotides then the last two nucleotides of the sequence were removed.")
                    message(
                        "If after consulting the file '",
                        corrupted_file,
                        "' you wish to remove all corrupted coding sequences please specify the argument 'delete_corrupt = TRUE'."
                    )

                    mod3_residue_1 <-
                        function(x) {
                            return((nchar(x) %% 3) == 1)
                        }

                    mod3_residue_2 <-
                        function(x) {
                            return((nchar(x) %% 3) == 2)
                        }

                    residue_1 <- cds.dt[ , mod3_residue_1(seqs)]
                    residue_2 <- cds.dt[ , mod3_residue_2(seqs)]

                    residue_1_seqs <- as.character(cds.dt[which(residue_1) , seqs])
                    residue_2_seqs <- as.character(cds.dt[which(residue_2) , seqs])

                    residue_1_seqs_vec <- as.character(sapply(residue_1_seqs, function(x) {
                        stringr::str_sub(x, 1, nchar(x) - 1)
                    }))

                    residue_2_seqs_vec <- as.character(sapply(residue_2_seqs, function(x) {
                        stringr::str_sub(x, 1, nchar(x) - 2)
                    }))

                    cds.dt[which(residue_1) , seqs := residue_1_seqs_vec]
                    cds.dt[which(residue_2) , seqs := residue_2_seqs_vec]

                    all_triplets_new <- cds.dt[ , mod3(seqs)]

                    if (any(!all_triplets_new)) {
                        stop("Something went wring during the trimming process. Not all sequences were trimmed properly.", call. = FALSE)
                    } else {
                        message("All corrupted CDS were trimmed.")
                    }

                    n_seqs_new <- nrow(cds.dt)

                    if(!(n_seqs == n_seqs_new))

                         stop("After trimming corrupted CDS some sequences seem to be lost. Please check what might have gone wrong with the sequence trimming.", call. = FALSE)
                    return(cds.dt)
                }
            } else {
                return(cds.dt)
            }
        }
    }

context("Test: getCDS()")


test_that("The getCDS() interface to NCBI RefSeq works properly and skips download when file already exists..",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "refseq",
                     organism = "Saccharomyces cerevisiae",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "refseq",
                     organism = "Saccharomyces cerevisiae",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})


test_that("The getCDS() interface to NCBI RefSeq works properly with taxid (including repeating the command)..",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "refseq",
                     organism = "559292",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "refseq",
                     organism = "559292",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})


test_that("The getCDS() interface to NCBI RefSeq works properly with assembly id (including repeating the command)..",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "refseq",
                     organism = "GCF_000146045.2",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "refseq",
                     organism = "GCF_000146045.2",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})

test_that("The getCDS() interface to NCBI Genbank works properly and skips download when file already exists..",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "genbank",
                     organism = "Saccharomyces cerevisiae",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "genbank",
                     organism = "Saccharomyces cerevisiae",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})


test_that("The getCDS() interface to NCBI Genbank works properly when using taxids (including repeating the command)..",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "genbank",
                     organism = "559292",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "genbank",
                     organism = "559292",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})


test_that("The getCDS() interface to NCBI Genbank works properly when using assembly id (including repeating the command) ...",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "genbank",
                     organism = "GCA_000146045.2",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "genbank",
                     organism = "GCA_000146045.2",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})

test_that("The getCDS() throws error when wrong 'db' is selected..",{
    skip_on_cran()
    skip_on_travis()
    expect_error(getCDS( db       = "somethingelse",
                           organism = "Saccharomyces cerevisi",
                           path     = tempdir(),
                         mute_citation = TRUE))
})

test_that("The getCDS() throws error when wrong 'organism' is selected..",{
    skip_on_cran()
    skip_on_travis()
    expect_equal(getCDS( db       = "refseq",
                         organism = "somethingelse",
                         path     = tempdir(),
                         mute_citation = TRUE), "Not available")
})


test_that("The getCDS() interface to Ensembl skips download when file already exists..",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "ensembl",
                     organism = "Saccharomyces cerevisiae",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "ensembl",
                     organism = "Saccharomyces cerevisiae",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})

test_that("getCDS() interface to Ensembl works with taxid and skips download when file already exists..",{

        skip_on_cran()
        skip_on_travis()
        out1 <- read_cds(getCDS( db       = "ensembl",
                         organism = "6239",
                         path     = tempdir(),
                         mute_citation = TRUE), format = "fasta")

        out2 <- read_cds(getCDS( db       = "ensembl",
                         organism = "6239",
                         path     = tempdir(),
                         mute_citation = TRUE), format = "fasta")
        expect_equal(out1, out2)
})

test_that("getCDS() interface to Ensembl works with accession id and skips download when file already exists....",{

    skip_on_cran()
    skip_on_travis()
    out1 <- read_cds(getCDS( db       = "ensembl",
                     organism = "GCA_000002985.3",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")

    out2 <- read_cds(getCDS( db       = "ensembl",
                     organism = "GCA_000002985.3",
                     path     = tempdir(),
                     mute_citation = TRUE), format = "fasta")
    expect_equal(out1, out2)
})

#' @title Check Genome Availability
#' @description This function checks the availability of a given genome on the
#' NBCI servers specified as scientific name.
#' @param db a character string specifying the database from which the genome
#' shall be retrieved:
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' \item \code{db = "ensembl"}
#' \item \code{db = "uniprot"}
#' }
#' @param organism there are three options to characterize an organism:
#' \itemize{
#' \item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
#' \item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
#' \item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
#' }
#' @param skip_bacteria Due to its enormous dataset size (> 700MB as of July 2023),
#' the bacterial summary file will not be loaded by default anymore. If users
#' wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
#' bacterial summary file will be downloaded.
#' @param details a logical value specifying whether or not details on genome
#' size, kingdom, etc. shall be printed to the console intead of a
#' boolean value.
#' @details
#' Internally this function calls the \code{\link{listGenomes}} function to
#' detect all available genomes and checks whether or not the specified organism
#' is available for download.
#' @return a logical value specifing whether or not the genome of the input
#' organism is available. In case \code{details} = \code{TRUE} only a character
#' string specifying the genome details is being returned.
#' @author Hajk-Georg Drost
#' @examples \dontrun{
#' # checking whether the Homo sapiens genome is stored on NCBI
#' is.genome.available(organism = "Homo sapiens", db = "refseq")
#'
#' # and printing details
#' is.genome.available(organism = "Homo sapiens", db = "refseq", details = TRUE)
#'
#' # checking whether the Homo sapiens genome is stored on ENSEMBL
#' is.genome.available(organism = "Homo sapiens", db = "ensembl")
#'
#' # and printing details
#' is.genome.available(organism = "Homo sapiens",
#'                     details = TRUE,
#'                     db = "ensembl")
#' }
#' @export

is.genome.available <- function(db = "refseq", organism,
                                skip_bacteria = "TRUE", details = FALSE) {
        all_db <- c(
          "refseq",
          "genbank",
          "ensembl",
          "ensemblgenomes",
          "uniprot"
        )

        if (!is.element(db, all_db))
            stop(
                "Please select one of the available data bases:\n",
                paste(all_db, collapse = ", "),
                call. = FALSE
            )

        if (is.element(db, c("refseq", "genbank"))) {
          return(is.genome.available.refseq.genbank(db = db, organism = organism, details = details))
        }

        if (db %in% c("ensembl", "ensemblgenomes")) {
          return(is.genome.available.ensembl(db, organism, details))
        }

        if (db == "uniprot") {

            if (is.taxid(organism)) {
                uniprot_rest_url <- paste0(
                    "https://www.ebi.ac.uk/proteins/api/proteomes?offset=0&size=-1&taxid=",
                    as.integer(organism)
                )

                rest_status_test <- curl_fetch_memory(uniprot_rest_url)

                if (rest_status_test$status_code != 200) {
                    message(
                        "Something went wrong when trying to access the API 'https://www.ebi.ac.uk/proteins/api/proteomes'.",
                        " Sometimes the internet connection isn't stable and re-running the function might help. Otherwise, could there be an issue with the firewall? ", "Is it possbile to access the homepage 'https://www.ebi.ac.uk/' through your browser?"
                    )
                }
                uniprot_species_info <-
                    tibble::as_tibble(jsonlite::fromJSON(
                        uniprot_rest_url
                    ))

            } else {

                organism_new <- stringr::str_replace_all(organism, " ", "%20")

                uniprot_rest_url_name <- paste0(
                    "https://www.ebi.ac.uk/proteins/api/proteomes?offset=0&size=-1&name=",
                    organism_new
                )

                rest_status_test_name <- curl_fetch_memory(uniprot_rest_url_name)

                if ((rest_status_test_name$status_code != 200)) {
                    message(
                        "Something went wrong when trying to access the API 'https://www.ebi.ac.uk/proteins/api/proteomes'.",
                        " Sometimes the internet connection isn't stable and re-running the function might help. Otherwise, could there be an issue with the firewall? ", "Is it possbile to access the homepage 'https://www.ebi.ac.uk/' through your browser?"
                    )
                }

                if (rest_status_test_name$status_code == 200) {
                    uniprot_species_info <-
                        tibble::as_tibble(jsonlite::fromJSON(
                            uniprot_rest_url_name
                        ))
                }
            }

            if (!details) {
                if (nrow(uniprot_species_info) == 0) {
                    message(
                        "Unfortunatey, no entry for '",
                        organism,
                        "' was found in the '",
                        db,
                        "' database. ",
                        "Please consider specifying ",
                        paste0("'db = ", dplyr::setdiff(
                            all_db,
                            db
                        ), collapse = "' or "),
                        "' to check whether '",
                        organism,
                        "' is available in these databases."
                    )
                    return(FALSE)
                }

                if (nrow(uniprot_species_info) > 0) {
                    message(
                        "A reference or representative genome assembly is available for '",
                        organism,
                        "'."
                    )
                    if (nrow(uniprot_species_info) > 1) {
                        message(
                            "More than one entry was found for '",
                            organism,
                            "'.",
                            " Please consider to run the function 'is.genome.available()' and specify 'is.genome.available(organism = ",
                            organism,
                            ", db = ",
                            db,
                            ", details = TRUE)'.",
                            " This will allow you to select the 'assembly_accession' identifier that can then be ",
                            "specified in all get*() functions."
                        )
                    }
                    return(TRUE)
                }

            }
          if (details)
            return(uniprot_species_info)
        }
}


is.genome.available.refseq.genbank <- function(db = "refseq",
                                               organism,
                                               skip_bacteria = TRUE,
                                               details = FALSE) {

  organism_name <- assembly_accession <- taxid <- NULL

  AssemblyFilesAllKingdoms <- getKingdomAssemblySummary(db = db, skip_bacteria = skip_bacteria)

  orgs <-
    stringr::str_replace_all(AssemblyFilesAllKingdoms$organism_name,
                             "\\(", "")
  orgs <- stringr::str_replace_all(orgs, "\\)", "")

  AssemblyFilesAllKingdoms <-
    dplyr::mutate(AssemblyFilesAllKingdoms, organism_name = orgs)

  organism <-
    stringr::str_replace_all(organism, "\\(", "")
  organism <-
    stringr::str_replace_all(organism, "\\)", "")

  # test if organism specification could be a taxid
  if (!is.taxid(organism)) {
    FoundOrganism <-
      dplyr::filter(AssemblyFilesAllKingdoms,
                    stringr::str_detect(organism_name, organism) |
                      assembly_accession == organism)

  } else {
    FoundOrganism <-
      dplyr::filter(AssemblyFilesAllKingdoms,
                    taxid == as.integer(organism))
  }

  if (nrow(FoundOrganism) == 0) {
    organism_no_hit_message_zero(organism, db)
    return(FALSE)
  }

  if (nrow(FoundOrganism) > 0) {
    if (!details) {

      if (all(FoundOrganism$refseq_category == "na")) {
        message("Only a non-reference genome assembly is available for '", organism, "'.",
                " Please make sure to specify the argument 'reference = FALSE' when running any get*() function.")
      } else {
        message("A reference or representative genome assembly is available for '", organism, "'.")
      }

      if (nrow(FoundOrganism) > 1) {
        organism_no_hit_message_more_than_one(organism, db)
      }
      return(TRUE)
    }


    if (details) {
      if (all(FoundOrganism$refseq_category == "na")) {
        message("Only a non-reference genome assembly is available for '", organism, "'.",
                " Please make sure to specify the argument 'reference = FALSE' when running any get*() function.")
      }
      return(FoundOrganism)
    }
  }
}

#' Check if genome is available in ensembl
#'
#' @param division "EnsemblVertebrates", alternatives: "EnsemblPlants", "EnsemblFungi"
#' @noRd
is.genome.available.ensembl <- function(db = "ensembl", organism,
                                        details = FALSE, divisions = ensembl_divisions()) {
  name <- accession <- accession <- assembly <- taxon_id <- NULL
  new.organism <- stringr::str_replace_all(organism, " ", "_")

  # For each ensembl division, check if it exists
  for (division in ensembl_divisions()) {
    ensembl.available.organisms <- get.ensembl.info(division = division)
    ensembl.available.organisms <- dplyr::filter(ensembl.available.organisms, !is.na(assembly))

    if (!is.taxid(organism)) {
      selected.organism <-
        dplyr::filter(
          ensembl.available.organisms,
          stringr::str_detect(name,
                              stringr::str_to_lower(new.organism)) |
            accession == organism, !is.na(assembly)
        )
    } else {
      selected.organism <-
        dplyr::filter(
          ensembl.available.organisms, taxon_id == as.integer(organism), !is.na(assembly))

    }
    if (nrow(selected.organism) > 0) break
  }



  if (!details) {
    if (nrow(selected.organism) == 0) {
      organism_no_hit_message_zero(organism, db)
      return(FALSE)
    }
    if (nrow(selected.organism) > 0) {
      message("A reference or representative genome assembly is available for '", organism, "'.")
      if (nrow(selected.organism) > 1) {
        organism_no_hit_message_more_than_one(organism, db)
      }
      return(TRUE)
    }
  }
  if (details)
    return(selected.organism)
}

organism_no_hit_message_zero <- function(organism, db) {
  message(
    "Unfortunatey, no entry for '",
    organism,
    "' was found in the '",
    db,
    "' database. ",
    "Please consider specifying ",
    paste0("'db = ", dplyr::setdiff(
      c("refseq", "genbank", "ensembl", "ensemblgenomes", "uniprot"), db
    ), collapse = "' or "),
    "' to check whether '",organism,"' is available in these databases."
  )
}

organism_no_hit_message_more_than_one <- function(organism, db) {
  message("More than one entry was found for '", organism, "'.",
          " Please consider to run the function 'is.genome.available()' and specify 'is.genome.available(organism = ",
          organism, ", db = ",db, ", details = TRUE)'.",
          " This will allow you to select the 'assembly_accession' identifier that can then be ",
          "specified in all get*() functions.")
}

context("Test: is.genome.available()")

test_that("The is.genome.available() interface to NCBI RefSeq
          works properly..", {
    skip_on_cran()
    skip_on_travis()
    # test whether interface to 'refseq' works properly
        expect_equal(nrow(is.genome.available(db = "refseq",
                            organism = "Saccharomyces cerevisiae",
                            details = TRUE)) > 0, TRUE)

    # test whether it works properly a second time based on the internal files
        expect_equal(suppressMessages(is.genome.available(db = "refseq",
                        organism = "Saccharomyces cerevisiae",
                        details = FALSE)), TRUE)

})

test_that("The is.genome.available() interface to NCBI RefSeq
          works properly using taxid..", {
              skip_on_cran()
              skip_on_travis()
              # test whether interface to 'refseq' works properly
              expect_equal(nrow(is.genome.available(db = "refseq",
                                  organism = "559292",
                                  details = TRUE)) > 0, TRUE)

              # test whether it works properly a second time based on the internal files
              expect_equal(suppressMessages(is.genome.available(db = "refseq",
                                  organism = "559292",
                                  details = FALSE)), TRUE)

          })


test_that("The is.genome.available() interface to NCBI RefSeq
          works properly using assembly accession id", {
              skip_on_cran()
              skip_on_travis()
              # test whether interface to 'refseq' works properly
              expect_equal(nrow(is.genome.available(db = "refseq",
                                                    organism = "GCF_000146045.2",
                                                    details = TRUE)) > 0, TRUE)

              # test whether it works properly a second time based on the internal files
              expect_equal(suppressMessages(is.genome.available(db = "refseq",
                                                                organism = "GCF_000146045.2",
                                                                details = FALSE)), TRUE)

          })

test_that("The is.genome.available() interface to NCBI Genbank
          works properly..", {
              skip_on_cran()
              skip_on_travis()
    # test whether interface to 'genbank' works properly
              expect_equal(nrow(is.genome.available(db = "genbank",
                        organism = "Saccharomyces cerevisiae",
                        details = TRUE)) > 0, TRUE)

    # test whether it works properly a second time based on the internal files
              expect_equal(suppressMessages(is.genome.available(db = "genbank",
                        organism = "Saccharomyces cerevisiae",
                        details = FALSE)), TRUE)

})

test_that("The is.genome.available() interface to NCBI Genbank
          works properly using taxid..", {
              skip_on_cran()
              skip_on_travis()
              # test whether interface to 'genbank' works properly
              expect_equal(nrow(is.genome.available(db = "genbank",
                                  organism = "559292",
                                  details = TRUE)) > 0, TRUE)

              # test whether it works properly a second time based on the internal files
              expect_equal(suppressMessages(is.genome.available(db = "genbank",
                                  organism = "559292",
                                  details = FALSE)), TRUE)

          })

test_that("The is.genome.available() interface to NCBI Genbank
          works properly using assembly accession id", {
              skip_on_cran()
              skip_on_travis()
              # test whether interface to 'genbank' works properly
              expect_equal(nrow(is.genome.available(db = "genbank",
                                                    organism = "GCA_000091065.2",
                                                    details = TRUE)) > 0, TRUE)

              # test whether it works properly a second time based on the internal files
              expect_equal(suppressMessages(is.genome.available(db = "genbank",
                                                                organism = "GCA_000091065.2",
                                                                details = FALSE)), TRUE)

          })

test_that("The is.genome.available() interface to Ensembl
          works properly..", {
              skip_on_cran()
              skip_on_travis()
    # test whether interface to 'ensembl' works properly
              expect_equal(nrow(is.genome.available(db = "ensembl",
                        organism = "Saccharomyces cerevisiae",
                        details = TRUE)) > 0, TRUE)

    # test whether interface to 'ensembl' works properly without details
              expect_equal(suppressMessages(is.genome.available(db = "ensembl",
                            organism = "Saccharomyces cerevisiae")), TRUE)
})

test_that("The is.genome.available() interface to Ensembl
          works properly using taxid..", {
              skip_on_cran()
              skip_on_travis()
              # test whether interface to 'ensembl' works properly
              expect_equal(nrow(is.genome.available(db = "ensembl",
                                                    organism = "4932",
                                                    details = TRUE)) > 0, TRUE)

              # test whether interface to 'ensembl' works properly without details
              expect_equal(suppressMessages(is.genome.available(db = "ensembl",
                                                                organism = "4932")), TRUE)
          })


test_that("The is.genome.available() interface to Ensembl
          works properly using accession id", {
              skip_on_cran()
              skip_on_travis()
              # test whether interface to 'ensembl' works properly
              expect_equal(nrow(is.genome.available(db = "ensembl",
                                                    organism = "GCA_000146045.2",
                                                    details = TRUE)) > 0, TRUE)

              # test whether interface to 'ensembl' works properly without details
              expect_equal(suppressMessages(is.genome.available(db = "ensembl",
                                                                organism = "GCA_000146045.2")), TRUE)
          })

test_that("The is.genome.available() error messages work properly..", {
    skip_on_cran()
    skip_on_travis()
    # test unknown organism error handling for 'refseq'
    expect_identical(suppressMessages(is.genome.available(organism = "Hrmo sapi",
                                         db = "refseq")),
                     FALSE)
    # test unknown organism error handling for 'genbank'
    expect_identical(suppressMessages(is.genome.available(organism = "Hrmo sapi",
                                         db = "genbank")),
                     FALSE)
    # test unknown organism error handling for 'ensembl'
    expect_identical(suppressMessages(is.genome.available(organism = "Hrmo sapi", db = "ensembl")),
                     FALSE)

    expect_identical(suppressMessages(is.genome.available(organism = "Hrmo sapi", db = "uniprot")),
                     FALSE)
})


test_that("The is.genome.available() interface to Uniprot
          works properly..", {
    skip_on_cran()
    skip_on_travis()
    # test whether interface to 'uniprot' works properly
    out1 <- is.genome.available(db = "uniprot",
                        organism = "Saccharomyces cerevisiae",
                        details = TRUE)

    # test whether interface to 'uniprot' works properly without details
    out2 <- is.genome.available(db = "uniprot",
                        organism = "Saccharomyces cerevisiae")

    expect_true(nrow(out1) > 0)
    expect_true(out2)
})

#' @title Retrieve All Available Attributes for a Specific Dataset
#' @description This function queries the BioMart Interface and returns a table
#' storing all available attributes for a specific dataset.
#' 
#' @param mart a character string specifying the database (mart) 
#' for which datasets shall be listed.
#' @param dataset a character string specifying the dataset for which 
#' attributes shall be listed.
#' @param mute_citation logical value indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # search for available datasets
#' getMarts()
#' 
#' # choose database (mart): ENSEMBL_MART_ENSEMBL
#' # and get a table of all available datasets from this BioMart database
#' head(getDatasets(mart = "ENSEMBL_MART_ENSEMBL"), 10)
#' 
#' # choose dataset: "hsapiens_gene_ensembl"
#' head(getAttributes(mart = "ENSEMBL_MART_ENSEMBL", 
#'                    dataset = "hsapiens_gene_ensembl") , 5)
#' }
#' @seealso \code{\link{getMarts}}, \code{\link{getDatasets}},
#' \code{\link{getFilters}}, \code{\link{organismBM}}, 
#' \code{\link{organismFilters}}, \code{\link{organismAttributes}}
#' @export

getAttributes <- function(mart, dataset, mute_citation = FALSE){
    
    if ((!is.character(mart)) || (!is.character(dataset)))
        stop("Please use a character string as mart or dataset.",
             call. = FALSE)
   
    if (!is.element(mart, getMarts()$mart))
        stop("Please select a valid mart with getMarts().", call. = FALSE)
    
  message("Starting retrieval of attribute information from mart ", mart, " and dataset ", dataset, " ...")
    # if (!is.element(dataset, getDatasets(mart = mart)$dataset))
    #     stop("Please select a valid dataset with getDatasets().", call. = FALSE)
     
    if (stringr::str_detect(mart, "ENSEMBL"))
        # connect to BioMart API
        url <-
            paste0(
              "http://ensembl.org/biomart/martservice?type=attributes&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    if (stringr::str_detect(mart, "plants"))
        # connect to BioMart API
        url <-
            paste0(
       "http://plants.ensembl.org/biomart/martservice?type=attributes&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    if (stringr::str_detect(mart, "fung"))
        # connect to BioMart API
        url <-
            paste0(
        "http://fungi.ensembl.org/biomart/martservice?type=attributes&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    
    if (stringr::str_detect(mart, "protist"))
        # connect to BioMart API
        url <-
            paste0(
      "http://protist.ensembl.org/biomart/martservice?type=attributes&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    if (stringr::str_detect(mart, "metazoa"))
        # connect to BioMart API
        url <-
            paste0(
      "http://metazoa.ensembl.org/biomart/martservice?type=attributes&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )

    testContent <-
        httr::content(httr::GET(url), as = "text", encoding = "UTF-8")
    if (testContent == "Attribute 'mains' does not exist\n") {
        warning("No attributes were available for mart = ",
                mart,
                " and dataset = ",
                dataset,
                ".",
                call. = FALSE)
        attrBioMart <- data.frame(name = "NA", description = "NA")
        return(attrBioMart)
    }
    
    attributesPage <- httr::handle(url)
    xmlContentAttributes <- httr::GET(handle = attributesPage)
    
    httr::stop_for_status(xmlContentAttributes)
    
    # extract attribute name and attribute description
    suppressWarnings(rawDF <-
                         do.call("rbind", apply(as.data.frame(strsplit(
                             httr::content(xmlContentAttributes, as = "text", 
                                           encoding = "UTF-8"),
                             "\n"
                         )), 1, function(x)
                             unlist(strsplit(x, "\t")))))
    
    colnames(rawDF) <- paste0("V", seq_len(ncol(rawDF)))
    
    attrBioMart <-
        as.data.frame(rawDF[, c("V1", "V2")],
                      stringsAsFactors = FALSE,
                      colClasses = rep("character", 2))
    colnames(attrBioMart) <- c("name", "description")
    
    please_cite_biomartr(mute_citation = mute_citation)
    
    return(attrBioMart)
}

context("Test: getAttributes()")


equal_lists <- function(df1, df2) {
        isTRUE(all.equal(df1, df2))
}


test_that("The getAttributes() interface works properly..",{

        skip_on_cran()
        skip_on_travis()

    expect_equal(getMarts()$mart[1], "ENSEMBL_MART_ENSEMBL")

    attrib <-
        getAttributes(mart = "ENSEMBL_MART_ENSEMBL",
                      dataset = "hsapiens_gene_ensembl",
                      mute_citation = TRUE)

    expect_equal(attrib[1 , 1], "ensembl_gene_id")
})

test_that("The getAttributes() throws error when wrong mart is selected",{

    skip_on_cran()
    skip_on_travis()

    expect_error(getAttributes(mart = "somethingelse",
                  dataset = "hsapiens_gene_ensembl"))

})

test_that("The getAttributes() throws error when wrong dataset is selected",{

    skip_on_cran()
    skip_on_travis()

    expect_error(getAttributes(mart = "hsapiens_gene_ensembl",
                  dataset = "somethingelse"))

})

#' @title Helper function to read md5checksum file from NCBI
#' @description NCBI stores md5checksum.txt files that can be retrieved
#' to check whether or not the downloaded file is corrupt.
#' @param md5path path to md5checksum.txt file.
#' @author Hajk-Georg Drost
#' @noRd
read_md5file <- function(md5path) {

    if (!file.exists(md5path))
        stop("The file path you specified does not seem to exist: '", md5path,"'.", call. = FALSE)

    file <-
        readr::read_delim(
            md5path,
            delim = "  ",
            col_names = c("md5", "file_name"),
            col_types = readr::cols(
                md5 = readr::col_character(),
                file_name = readr::col_character()
            )
        )

    return(file)
}

#' @title Retrieve UniProt Database Information File (STATS)
#' @description
#' The UniProt stores a \code{STATS} file to summarise all available information for their reference proteomes.
#' Users can now download this file and process it with \code{biomartr}.
#' @param update shall the internal \code{\link{cachedir}} file be deleted and the \code{STATS}
#' file freshly downloaded from the UniProt FTP servers? 
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # retrieve STATS file from UniProt
#' uniprot_info <- getUniProtSTATS(update = TRUE)
#' # look at results
#' uniprot_info
#' }
#' @export

getUniProtSTATS <- function(update = FALSE) {
  message("-> Start UniProt information retrieval from: https://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/reference_proteomes/STATS")
  tmp_file <- file.path(cachedir(),  "UniProtSTATS.tsv")
  if (file.exists(tmp_file) &&
      !update) {
    message("-> It seems a local version of the file is already available and 'update = FALSE', so the local file is being imported.")
    suppressWarnings(
      uniprot.info <-
        readr::read_tsv(
          tmp_file,
          skip = 16, # the first 16 lines of the STATS file are only annotation and not part of the dataset 
          col_names = FALSE,
          col_types = readr::cols(
            "X1" = readr::col_character(),
            "X2" = readr::col_integer(),
            "X3" = readr::col_character(),
            "X4" = readr::col_integer(),
            "X5" = readr::col_character(),
            "X6" = readr::col_character(),
            "X7" = readr::col_character(),
            "X8" = readr::col_character()
          )
        )
    )
    names(uniprot.info) <-
      c(
        "proteome_id",
        "taxid",
        "n_entries_fasta_canonical",
        "n_entries_fasta_isoforms",
        "n_entries_fasta_gene2acc_mapping",
        "assemby_release",
        "source_of_proteome_import",
        "species_name"
      )
  } else {
    rest_url <- uniprot_stats_url()
    rest_api_status <- curl::curl_fetch_memory(rest_url)
    if (rest_api_status$status_code != 200) {
      message(
        "The API '", uniprot_stats_url(), "' does not seem to
                respond or work properly. Is the homepage '", uniprot_stats_url(), "' currently available?",
        " Could it be that there is a firewall issue on your side? Please re-run the function and check if it works now."
      )
    }
    # download STATS file
    custom_download(url = uniprot_stats_url(), destfile = tmp_file)
    
    suppressWarnings(
      uniprot.info <-
        readr::read_tsv(
          tmp_file,
          skip = 16, # the first 15 lines of the STATS file are only annotation and not part of the dataset 
          col_names = FALSE,
          col_types = readr::cols(
            "X1" = readr::col_character(),
            "X2" = readr::col_integer(),
            "X3" = readr::col_character(),
            "X4" = readr::col_integer(),
            "X5" = readr::col_character(),
            "X6" = readr::col_character(),
            "X7" = readr::col_character(),
            "X8" = readr::col_character()
          )
        )
    )
    names(uniprot.info) <-
      c(
        "proteome_id",
        "taxid",
        "n_entries_fasta_canonical",
        "n_entries_fasta_isoforms",
        "n_entries_fasta_gene2acc_mapping",
        "assemby_release",
        "source_of_proteome_import",
        "species_name"
      )
  }
  message("-> Import successful!")
  return (uniprot.info)
}

#' @title Helper function to test if ENSEMBL server is reachable
#' @description This function pings the ENSEMBL FTP server to 
#' test whether or not a connection can be established.
#' @author Hajk-Georg Drost
#' @noRd
is.ensembl.alive <- function() {
    ping <- jsonlite::fromJSON(
        "http://rest.ensembl.org/info/ping?content-type=application/json")
    if (ping$ping == 0)
        message("The ENSEMBL site seems to be out of order. Sometimes the internet connection isn't stable and re-running the function might help. Otherwise, could there be an issue with the firewall? Please try to go to 
             'http://rest.ensembl.org/' and see if the site opens properly.")
}

#' @title Check whether an annotation file contains outlier lines
#' @description Some annotation files include lines with character lengths greater than 65000. 
#' This causes problems when trying to import such annotation files into R using \code{import}.
#' To overcome this issue, this function screens for such lines
#' in a given annotation file and removes these lines so that
#' \code{import} can handle the file.
#' @param annotation_file a file path to the annotation file.
#' @param remove_annotation_outliers shall outlier lines be removed from the input \code{annotation_file}? 
#' If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
#' for further exploration.
#' @author Hajk-Georg Drost
#' @examples \dontrun{
#' # download an example annotation file from NCBI RefSeq
#' Ath_path <- biomartr::getGFF(organism = "Arabidopsis thaliana")
#' # run annotation file check on the downloaded file
#' biomartr::check_annotation_biomartr(Ath_path)
#' # several outlier lines were detected, thus we re-run the
#' # function using 'remove_annotation_outliers = TRUE'
#' # to remove the outliers and overwrite the file
#' biomartr::check_annotation_biomartr(Ath_path, remove_annotation_outliers = TRUE)
#' }
#' @export

check_annotation_biomartr <- function(annotation_file, remove_annotation_outliers = FALSE) {
        
        if (!file.exists(annotation_file))
                stop("The file '", annotation_file, "' does not seem to exist. Please provide a valid path to the annotation file.", call. = FALSE)
        
        message("Importing '", annotation_file, "' ...")
        import_annotation <- readr::read_lines(annotation_file)
        outlier_lines <- which(nchar(import_annotation) > 65000)
        
        if (length(outlier_lines) == 0)
                return(annotation_file)
        
        if (length(outlier_lines) > 0 & !remove_annotation_outliers) {
                warning("Your annotation file '", annotation_file, "' contains ", length(outlier_lines), " lines that have more than 65000 characters.",
                        "This will cause the annotation import function to fail importing the annotation file into R.",
                        " If you wish to remove the outlier lines and overwrite the existing annotation file then re-run this function and specify 'remove_annotation_outliers = TRUE'. A file with the removed lines will then be stored in the temp directory for you to explore.")
                return(annotation_file)
        }
        
        if (length(outlier_lines) > 0 & remove_annotation_outliers) {
                message("Reading annotation file '", annotation_file, "' and removing all outlier lines with number of characters greater 65000 ...")
                message("Overwriting '", annotation_file, "' with removed outlier lines ...")
                
                file_extension_check <- unlist(stringr::str_split(annotation_file, "[.]"))
                if (file_extension_check[length(file_extension_check)] == "gz"){
                        message("Unzipping file ", annotation_file,"' ...")
                        R.utils::gunzip(annotation_file, destname = paste0(file_extension_check[1],".", file_extension_check[2]))
                }
                
                readr::write_lines(import_annotation[-outlier_lines], paste0(file_extension_check[1],".", file_extension_check[2]))
                
                message("The new annotation file was created and has been stored at '", paste0(file_extension_check[1],".", file_extension_check[2]),"'.")
                
                outlier_output_file <- file.path(tempdir(), paste0(basename(annotation_file),"_outlier_lines.txt"))
                
                readr::write_lines(import_annotation[outlier_lines], outlier_output_file)
                message("The outlier lines were stored in a separate file to explore at '", outlier_output_file, "'.")
                return(paste0(file_extension_check[1],".", file_extension_check[2]))
        }
        
}

#' @title Helper function to retrieve the assembly_summary.txt file from NCBI
#' @description Retrieval function of the assembly_summary.txt file from NCBI.
#' @inheritParams getKingdomAssemblySummary
#' @param kingdom kingdom for which assembly_summary.txt file shall be
#' retrieved. See also \code{\link{getKingdoms}}.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' test <- getSummaryFile("refseq","plant")
#' test
#' }
#' @seealso \code{\link{getKingdomAssemblySummary}},
#' \code{\link{getMetaGenomeSummary}}
#' @export

getSummaryFile <- function(db, kingdom,
                           file = assemblies_info_path(db, kingdom)){



  if (!is.element(db, c("refseq", "genbank")))
      stop("Please select one of the available data bases: 'refseq'
           or 'genbank'")

  if (!is.element(kingdom, getKingdoms(db = db)))
      stop(paste0(
          "Please select a valid kingdom: ",
          paste0(getKingdoms(db = db), collapse = ", ")
        ))
  url <- paste0("ftp://ftp.ncbi.nlm.nih.gov/genomes/",db, "/",
                kingdom, "/assembly_summary.txt")
  custom_download_check_local(url, file, rest_api_status = NULL, db = db)

  summary.file <- suppressWarnings(read_all_kingdoms_assemblies_info(file))

  return(summary.file)
}

assemblies_info_path <- function(db, kingdom = NULL, folder = cachedir()) {
  basename <- "AssemblyFilesAllKingdoms_"
  if (!is.null(kingdom)) {
    kingdom <- paste0(kingdom, "_")
    basename <- "assembly_summary_"
  }
  file.path(folder, paste0(basename, kingdom, db, ".txt"))
}

read_all_kingdoms_assemblies_info <- function(file) {
  tibble::as_tibble(
    readr::read_delim(
      file,
      comment = "#",
      delim = "\t",
      quote = "\"",
      escape_double = FALSE,
      col_names = c(
        "assembly_accession",
        "bioproject",
        "biosample",
        "wgs_master",
        "refseq_category",
        "taxid",
        "species_taxid",
        "organism_name",
        "infraspecific_name",
        "isolate",
        "version_status",
        "assembly_level",
        "release_type",
        "genome_rep",
        "seq_rel_date",
        "asm_name",
        "submitter",
        "gbrs_paired_asm",
        "paired_asm_comp",
        "ftp_path",
        "excluded_from_refseq",
        "relation_to_type_material",
        "asm_not_live_date",
        "assembly_type",
        "group",
        "genome_size",
        "genome_size_ungapped",
        "gc_percent",
        "replicon_count",
        "scaffold_count",
        "contig_count",
        "annotation_provider",
        "annotation_name",
        "annotation_date",
        "total_gene_count",
        "protein_coding_gene_count",
        "non_coding_gene_count",
        "pubmed_id"
      ),
      col_types = readr::cols(
        assembly_accession = readr::col_character(),
        bioproject = readr::col_character(),
        biosample = readr::col_character(),
        wgs_master = readr::col_character(),
        refseq_category = readr::col_character(),
        taxid = readr::col_integer(),
        species_taxid = readr::col_integer(),
        organism_name = readr::col_character(),
        infraspecific_name = readr::col_character(),
        isolate = readr::col_character(),
        version_status = readr::col_character(),
        assembly_level = readr::col_character(),
        release_type = readr::col_character(),
        genome_rep = readr::col_character(),
        seq_rel_date = readr::col_date(),
        asm_name = readr::col_character(),
        submitter = readr::col_character(),
        gbrs_paired_asm = readr::col_character(),
        paired_asm_comp = readr::col_character(),
        ftp_path = readr::col_character(),
        excluded_from_refseq = readr::col_character(),
        relation_to_type_material = readr::col_character(),
        asm_not_live_date = readr::col_character(),
        assembly_type = readr::col_character(),
        group = readr::col_character(),
        genome_size = readr::col_integer(),
        genome_size_ungapped = readr::col_integer(),
        gc_percent = readr::col_double(),
        replicon_count = readr::col_integer(),
        scaffold_count = readr::col_integer(),
        contig_count = readr::col_integer(),
        annotation_provider = readr::col_character(),
        annotation_name = readr::col_character(),
        annotation_date = readr::col_date(format = "%m/%d/%y"),
        total_gene_count = readr::col_integer(),
        protein_coding_gene_count = readr::col_integer(),
        non_coding_gene_count = readr::col_integer(),
        pubmed_id = readr::col_character()
      )
    )
  )
}

#' @title Import RNA as Biostrings or data.table object
#' @description This function reads an organism specific RNA stored in a
#' defined file format.
#' @param file a character string specifying the path to the file
#' storing the RNA.
#' @param format a character string specifying the file format used to store the
#'  genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.
#' @param obj.type a character string specifying the object stype in which the
#' genomic sequence shall be represented.
#' Either as \code{obj.type = "Biostrings"} (default) or
#' as \code{obj.type = "data.table"}.
#' @param ... additional arguments that are used by
#' \code{\link[seqinr]{read.fasta}}.
#' @author Hajk-Georg Drost
#' @details This function takes a string specifying the path to the RNA file
#' of interest as first argument. It is possible to read in different proteome
#' file standards such as \emph{fasta} or \emph{genebank}.
#' @return A data.table storing the gene id in the first column and the
#' corresponding sequence as string in the second column.
#' @family rna
#' @family readers
#' @import Biostrings
#' @export

read_rna <-
    function(file,
             format = "fasta",
             obj.type = "Biostrings",
             ...) {
        if (!is.element(format, c("fasta", "gbk")))
            stop("Please choose a file format that is supported
                 by this function.",
                 call. = FALSE)

        if (!is.element(obj.type, c("Biostrings", "data.table")))
            stop(
                "Please specify a valid object type:
                obj.type = 'Biostrings' (default) or obj.type = 'data.table'.",
                call. = FALSE
            )

        geneids <- NULL

        if (obj.type == "Biostrings") {
            tryCatch({
                RNA_file <-
                    Biostrings::readDNAStringSet(filepath = file,
                                                 format = format, ...)
            }, error = function(e) {
                stop(
                    paste0(
                        "File ",
                        file,
                        " could not be read properly. \n",
                        "Please make sure that ",
                        file,
                        " contains only RNA sequences and is in ",
                        format,
                        " format."
                    ),
                    call. = FALSE
                )
            })

            return(RNA_file)
        }

        if (obj.type == "data.table") {
            tryCatch({
                RNA_file <-
                    Biostrings::readDNAStringSet(filepath = file,
                                                 format = format, ...)

                RNA_names <-
                    as.vector(unlist(lapply(RNA_file@ranges@NAMES, function(x) {
                        return(strsplit(x, " ")[[1]][1])
                    })))

                RNA.dt <-
                    data.table::data.table(geneids = RNA_names ,
                                           seqs =
                                               tolower(as.character(RNA_file)))


                data.table::setkey(RNA.dt, geneids)


            }, error = function(e) {
                stop(
                    paste0(
                        "File ",
                        file,
                        " could not be read properly. \n",
                        "Please make sure that ",
                        file,
                        " contains only RNA sequences and is in ",
                        format,
                        " format."
                    ),
                    call. = FALSE
                )
            })
                return(RNA.dt)
        }
    }

context("Test: getRNA()")

test_that(
    "The getRNA() interface to NCBI RefSeq works properly (including when command is repeated)..",
    {
        skip_on_cran()
        skip_on_travis()
        # test proper download
        out1 <- read_rna(
            getRNA(
                db       = "refseq",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

        # test proper use of internal referece files when command is repeated
        out2 <- read_rna(
            getRNA(
                db       = "refseq",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )
      expect_equal(out1, out2)
    }
)


test_that(
    "The getRNA() interface to NCBI RefSeq works properly uding taxids (including when command is repeated)..",
    {
        skip_on_cran()
        skip_on_travis()
        # test proper download
        out1 <- read_rna(getRNA(
            db       = "refseq",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta")

        out2 <- read_rna(getRNA(
            db       = "refseq",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta")

        expect_equal(out1, out2)
    }
)

test_that("The getRNA() interface to NCBI RefSeq works properly using assembly ids ..",
          {
              skip_on_cran()
              skip_on_travis()
              # test proper download
              out1 <- read_rna(getRNA(
                  db       = "refseq",
                  organism = "GCF_000146045.2",
                  path     = tempdir(), mute_citation = TRUE
              ),
              format = "fasta")

              out2 <- read_rna(getRNA(
                  db       = "refseq",
                  organism = "GCF_000146045.2",
                  path     = tempdir(), mute_citation = TRUE
              ),
              format = "fasta")

              expect_equal(out1, out2)
          })


test_that(
    "The getRNA() interface to NCBI Genbank works properly (including when command is repeated)..",
    {
        skip_on_cran()
        skip_on_travis()
        # test proper download from genbank
        out1 <- read_rna(
            getRNA(
                db       = "genbank",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

        # test proper use of internal referece files when command is repeated
        out2 <- read_rna(
            getRNA(
                db       = "genbank",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

        expect_equal(out1, out2)
    }
)

test_that("The getRNA() interface to NCBI Genbank works properly using taxids (including when command is repeated)..",
          {
              skip_on_cran()
              skip_on_travis()
              # test proper download from genbank
              out1 <- read_rna(getRNA(
                  db       = "genbank",
                  organism = "559292",
                  path     = tempdir(), mute_citation = TRUE
              ),
              format = "fasta")

              out2 <- read_rna(getRNA(
                  db       = "genbank",
                  organism = "559292",
                  path     = tempdir(), mute_citation = TRUE
              ),
              format = "fasta")

              expect_equal(out1, out2)
          })



test_that("The getRNA() interface to NCBI Genbank works properly using accession ids (including when command is repeated)..",
          {
              skip_on_cran()
              skip_on_travis()
              # test proper download from genbank
              out1 <- read_rna(getRNA(
                  db       = "genbank",
                  organism = "GCA_000146045.2",
                  path     = tempdir(), mute_citation = TRUE
              ),
              format = "fasta")

              out2 <- read_rna(getRNA(
                  db       = "genbank",
                  organism = "GCA_000146045.2",
                  path     = tempdir(), mute_citation = TRUE
              ),
              format = "fasta")

              expect_equal(out1, out2)
          })


test_that("The getRNA() interface to Ensembl works properly (including when command is repeated)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from ensemblgenomes
    out1 <- read_rna(
        getRNA(
            db       = "ensembl",
            organism = "Saccharomyces cerevisiae",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    out2 <- read_rna(
        getRNA(
            db       = "ensembl",
            organism = "Saccharomyces cerevisiae",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    expect_equal(out1, out2)
})


test_that("The getRNA() interface to Ensembl works properly using taxid (including when command is repeated)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from ensemblgenomes
    # 4932
    out1 <- read_rna(
        getRNA(
            db       = "ensembl",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    out2 <- read_rna(
        getRNA(
            db       = "ensembl",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    expect_equal(out1, out2)
})


test_that("The getRNA() interface to Ensembl works properly using accession id (including when command is repeated)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from ensemblgenomes
    out1 <- read_rna(
        getRNA(
            db       = "ensembl",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    out2 <- read_rna(
        getRNA(
            db       = "ensembl",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    expect_equal(out1, out2)
})

#' @inherit getBio
#' @title Genome Annotation Retrieval (GFF3)
#' @description  Main retrieval function for GFF files of an
#' organism of interest. By specifying the scientific name of an organism of
#' interest the corresponding gff file storing the annotation  for the organism
#' of interest can be downloaded and stored locally. GFF files can be retrieved
#' from several databases.
#' @param path a character string specifying the location (a folder) in which
#' the corresponding annotation file shall be stored. Default is
#' \code{path = file.path("_ncbi_downloads","annotation")}.
#' @examples \dontrun{
#' # download the annotation of Arabidopsis thaliana from refseq
#' # and store the corresponding genome file in '_ncbi_downloads/annotation'
#' Athal_gff <- getGFF( db       = "refseq",
#'                organism = "Arabidopsis thaliana",
#'                path = file.path("_ncbi_downloads","annotation"),
#'                remove_annotation_outliers = TRUE)
#' Athal_gff_import <- read_gff(Athal_gff)
#'
#'
#' # download the genome of Arabidopsis thaliana from genbank
#' # and store the corresponding genome file in '_ncbi_downloads/annotation'
#' Athal_gff <- getGFF( db       = "genbank",
#'                organism = "Arabidopsis thaliana",
#'                path = file.path("_ncbi_downloads","annotation"),
#'                remove_annotation_outliers = TRUE)
#' Athal_gff_import <- read_gff(Athal_gff)
#'
#' # download the genome of Homo sapiens from ensembl
#' # and store the corresponding genome file in '_ncbi_downloads/annotation'
#' Hsap_gff <- getGFF( db       = "ensembl",
#'                organism = "Homo sapiens",
#'                path = file.path("_ncbi_downloads","annotation"),
#'                remove_annotation_outliers = TRUE)
#' Hsap_gff_import <- read_gff(Hsap_gff)
#'
#' }
#'
#' @family getBio
#' @family gff
#' @export

getGFF <- function(db = "refseq", organism, reference = FALSE,
                   skip_bacteria = TRUE, release = NULL, gunzip = FALSE,
                   remove_annotation_outliers = FALSE,
                   path = file.path("_ncbi_downloads", "annotation"),
                   mute_citation = FALSE, format = "gff3") {

   if (!is.element(db, c("refseq", "genbank", "ensembl")))
        stop(
            "Please select one of the available data bases: 'refseq',
            'genbank', or 'ensembl'."
        )
    message("-> Starting ", toupper(format)," retrieval of '", organism, "' from ", db, " ...")
    message("\n")

    if (!file.exists(path)) {
      dir.create(path, recursive = TRUE)
    }

    if (is.element(db, c("refseq", "genbank"))) {
      info <- get_file_refseq_genbank(db, organism, reference, skip_bacteria,
                                      release, gunzip, path, type = "gff")
      return(refseq_genbank_download_post_processing(info, organism, db, path,
                                                     gunzip,
                                                     remove_annotation_outliers,
                                                     format,
                                                     mute_citation = mute_citation))
    } else if (db == "ensembl") {
        genome.path <- getENSEMBL.Annotation(organism, type = "dna",
                                             release = release, path = path,
                                             format = format)
        return(ensembl_download_post_processing(genome.path, organism, format,
                                                remove_annotation_outliers,
                                                gunzip,
                                                mute_citation = mute_citation))
    }
}

#' @inherit getGFF
#' @title Genome Annotation Retrieval (GTF)
#' @description  Main retrieval function for GTF files of an
#' organism of interest. By specifying the scientific name of an organism of
#' interest the corresponding GTF file storing the annotation  for the organism
#' of interest can be downloaded and stored locally. GTF files can be retrieved
#' from several databases.
#' @examples \dontrun{
#' # download the annotation of Homo sapiens from ensembl
#' # and store the corresponding genome file in 'ensembl/annotation'
#' getGTF(db            = "ensembl",
#'        organism      = "Homo sapiens",
#'        path          = file.path("ensembl","annotation"))
#'
#' getGTF(db            = "ensembl",
#'        organism      = "Homo sapiens",
#'        path          = file.path("ensembl","annotation"),
#'        assembly_type = "primary_assembly")
#'
#' }
#'
#' @export
getGTF <-
  function(db = "ensembl",
           organism,
           remove_annotation_outliers = FALSE,
           path = file.path("ensembl", "annotation"),
           release = NULL,
           mute_citation = FALSE) {
    if (!is.element(db, c("ensembl")))
      stop( "Please select one of the available data bases: db = 'ensembl'.", call. = FALSE)
    getGFF(db = "ensembl",
           organism,
           remove_annotation_outliers = remove_annotation_outliers,
           path = path,
           release = release,
           mute_citation = mute_citation,
           format = "gtf")
}

context("Test: getGFF()")

test_that("The getGFF() interface works properly for NCBI RefSeq (repeating command)..",{

        skip_on_cran()
        skip_on_travis()
    # test proper download from refseq
    out1 <- getGFF( db       = "refseq",
            organism = "Saccharomyces cerevisiae",
            path = tempdir(), mute_citation = TRUE)

    out2 <- getGFF( db       = "refseq",
            organism = "Saccharomyces cerevisiae",
            path = tempdir(), mute_citation = TRUE)
    expect_equal(out1, out2)
})

test_that("The getGFF() interface works properly for NCBI RefSeq using taxid (repeating command)..",{

    skip_on_cran()
    skip_on_travis()
    # test proper download from refseq
    out1 <- getGFF( db       = "refseq",
            organism = "559292",
            path = tempdir(), mute_citation = TRUE)

    out2 <- getGFF( db       = "refseq",
            organism = "559292",
            path = tempdir(), mute_citation = TRUE)
    expect_equal(out1, out2)
})

test_that("The getGFF() interface works properly for NCBI Genbank (repeating command)..",{
        skip_on_travis()
        skip_on_cran()
    # test proper download from genbank
    out1 <- getGFF( db       = "genbank",
            organism = "Saccharomyces cerevisiae",
            path = tempdir(), mute_citation = TRUE)

    out2 <- getGFF( db       = "genbank",
            organism = "Saccharomyces cerevisiae",
            path = tempdir(), mute_citation = TRUE)
    expect_equal(out1, out2)
})


test_that("The getGFF() interface works properly for Ensembl (repeating command)",{
        skip_on_cran()
        skip_on_travis()

    # test proper download from Ensembl
    out1 <- getGFF( db       = "ensembl",
            organism = "Saccharomyces cerevisiae",
            path = tempdir(), mute_citation = TRUE)

    out2 <- getGFF( db       = "ensembl",
            organism = "Saccharomyces cerevisiae",
            path = tempdir(), mute_citation = TRUE)
    expect_equal(out1, out2)
})

test_that("The getGTF() interface works properly for Ensembl (repeating command)",{
  skip_on_cran()
  skip_on_travis()

  # test proper download from Ensembl
  out1 <- getGTF( db       = "ensembl",
          organism = "Saccharomyces cerevisiae",
          path = tempdir(), mute_citation = TRUE)

  out2 <- getGTF( db       = "ensembl",
          organism = "Saccharomyces cerevisiae",
          path = tempdir(), mute_citation = TRUE)
  expect_equal(out1, out2)
})

test_that("The getGFF() interface works properly for EnsemblGenomes with collections",{
  skip_on_cran()
  skip_on_travis()

  # Bacteria
  out1 <- getGFF( db       = "ensembl",
          organism = "Escherichia coli",
          path = tempdir(), mute_citation = TRUE)
  # Fungi
  out2 <- getGFF( db       = "ensembl",
          organism = "Acremonium chrysogenum",
          path = tempdir(), mute_citation = TRUE)
  # Protists
  getGFF( db       = "ensembl",
          organism = "Babesia bigemina",
          path = tempdir(), mute_citation = TRUE)
  expect_false(out1 == out2)
})

test_that("The getGTF() interface works properly for EnsemblGenomes with collections",{
  skip_on_cran()
  skip_on_travis()

  # Bacteria
  out1 <- getGTF( db       = "ensembl",
          organism = "Escherichia coli",
          path = tempdir(), mute_citation = TRUE)
  # Fungi
  out2 <- getGTF( db       = "ensembl",
          organism = "Acremonium chrysogenum",
          path = tempdir(), mute_citation = TRUE)
  # Protists
  getGTF( db       = "ensembl",
          organism = "Babesia bigemina",
          path = tempdir(), mute_citation = TRUE)
  expect_false(out1 == out2)
})

#' @title Helper function to retrieve species information from the ENSEMBL API
#' @description This function interfaces with the ENSEMBL API
#' (https://rest.ensembl.org/info/species?content-type=application/json)
#' and internally stores the output to use this information for subsequent
#' retrieval function calls.
#' @param update logical, default FALSE. If TRUE, force re-download of info.
#' @param division the ENSEMBL database (division) for which information shall
#' be retrieved (available options can be obtained with \code{\link{ensembl_divisions}}).
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # Look at available ENSEMBL division options
#' ensembl_divisions()
#' # Retrieve available information for EnsemblVertebrates
#' example <- get.ensembl.info(division = "EnsemblVertebrates")
#' example
#' # Update information file stored in the tempdir() folder.
#' example_update <- get.ensembl.info(division = "EnsemblVertebrates", update = TRUE)
#' example_update
#' }
#' @seealso \code{\link{ensembl_divisions}}, \code{\link{getKingdomAssemblySummary}}, \code{\link{getENSEMBLInfo}}
#' @export
get.ensembl.info <- function(update = FALSE, division) {
  tmp_file <- file.path(cachedir(), paste0(division, "_info.tsv"))
  if (file.exists(tmp_file) &&
        !update) {
        suppressWarnings(
            ensembl.info <-
                readr::read_tsv(
                    tmp_file,
                    col_names = TRUE,
                    col_types = readr::cols(
                        division = readr::col_character(),
                        taxon_id = readr::col_integer(),
                        name = readr::col_character(),
                        release = readr::col_integer(),
                        display_name = readr::col_character(),
                        accession = readr::col_character(),
                        common_name = readr::col_character(),
                        assembly = readr::col_character()
                    )
                )
        )

    } else {
        rest_url <- ensembl_rest_url_species_division(division)
        rest_api_status <- curl::curl_fetch_memory(rest_url)
        if (rest_api_status$status_code != 200) {
            message(
                "The API '", ensembl_rest_url(), "' does not seem to
                respond or work properly. Is the homepage '", ensembl_rest_url(), "' currently available?",
                " Could it be that there is a firewall issue on your side? Please re-run the function and check if it works now."
            )
        }

            ensembl.info <-
                tibble::as_tibble(
                    jsonlite::fromJSON(
                        rest_url
                    )$species
                )

        aliases <- groups <- NULL
        ensembl.info <-
            dplyr::select(ensembl.info, -aliases, -groups)

        readr::write_tsv(ensembl.info, tmp_file)
    }

    return(ensembl.info)
}


uniprot_stats_url <- function(){
  return("https://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/reference_proteomes/STATS")
}


ensembl_rest_url <- function() {
  "https://rest.ensembl.org"
}

#' @title List all available ENSEMBL divisions
#' @description
#' Retrieve a list of available databases on ENSEMBL for which \code{get.ensembl.info} can be retrieved.
#' @author Hajk-Georg Drost
#' @examples
#' ensembl_divisions()
#' @export
ensembl_divisions <- function() {
  c("EnsemblVertebrates", "EnsemblPlants", "EnsemblFungi", "EnsemblMetazoa",
    "EnsemblBacteria", "EnsemblProtists")
}

ensembl_divisions_short <- function(ensembl_as_empty = TRUE, bacteria = TRUE) {
  div  <-
    c(EnsemblVertebrates = "", EnsemblPlants = "plants", EnsemblFungi = "fungi",
    EnsemblBacteria = "bacteria", EnsemblMetazoa = "metazoa", EnsemblProtists = "protists")
  if (!ensembl_as_empty) div[1] <- "ensembl"
  if (!bacteria) div <- div[!(div == "bacteria")]
  return(div)
}

ensembl_rest_url_species <- function() {
  file.path(ensembl_rest_url(), "info/species")
}


#' Get supported species from given division
#'
#' @param division "EnsemblVertebrates", alternatives:
#'  "EnsemblPlants", "EnsemblFungi", "EnsemblMetazoa", "EnsemblBacteria"
#' @noRd
ensembl_rest_url_species_division <- function(division = "EnsemblVertebrates") {
  stopifnot(division %in% ensembl_divisions())
  suffix_url <- paste0("?division=", division,
                       "&content-type=application/json")
  file.path(ensembl_rest_url_species(), suffix_url)
}

ensembl_current_release <- function(division = "EnsemblVertebrates") {
  if (division == "EnsemblVertebrates") {
    jsonlite::fromJSON(
      file.path(ensembl_rest_url(), "info/data/?content-type=application/json")
    )$releases
  } else {
    jsonlite::fromJSON(
      file.path(ensembl_rest_url(),
                "info/eg_version?content-type=application/json"))$version
  }
}

ensembl_all_releases <- function(division = "EnsemblVertebrates") {
  seq_len(as.integer(ensembl_current_release(division)))
}

ensembl_rest_url_assembly <- function(organism) {
  file.path(ensembl_rest_url(),
            paste0("info/assembly/", organism,
                   "?content-type=application/json"
  ))
}

ensembl_ftp_server_url <- function(division = "EnsemblVertebrates") {
  if (division == "EnsemblVertebrates") {
    "https://ftp.ensembl.org"
  } else {
    "http://ftp.ensemblgenomes.org"
  }
}

ensembl_ftp_server_url_release <- function(division, release = NULL) {
  if (division == "EnsemblVertebrates") {
    if (is.null(release)) {
      "pub/current_"
    } else paste0("pub/release-", release ,"/")

  } else {ensembl_divisions
    short_name <- ensembl_divisions_short()[division]
    if (is.null(release)) {
      paste0("pub/current/", short_name, "/")
    } else paste0("pub/release-", release ,"/", short_name, "/")
  }
}

ensembl_ftp_server_url_format <- function(division, release = NULL, format) {
  release_stem <- ensembl_ftp_server_url_release(division, release)
  paste0(release_stem, format, "/")
}

ensembl_ftp_server_url_format_full <- function(division, release = NULL, format) {
  file.path(ensembl_ftp_server_url(division),
            ensembl_ftp_server_url_format(division, release, format))
}

ensembl_ftp_server_url_release_style_fasta <- function(division, release = NULL) {
  ensembl_ftp_server_url_format(division, release, "fasta")
}

ensembl_ftp_server_url_release_style_gtf <- function(division, release = NULL, format = "gtf") {
  ensembl_ftp_server_url_format(division, release, format)
}


ensembl_ftp_server_url_fasta <- function(division, release = NULL) {
  file.path(ensembl_ftp_server_url(division),
            ensembl_ftp_server_url_release_style_fasta(division, release))
}

ensembl_ftp_server_url_gtf <- function(division = "EnsemblVertebrates",
                                       release = NULL, format = "gtf") {
  file.path(ensembl_ftp_server_url(division),
            ensembl_ftp_server_url_release_style_gtf(division, release, format))
}

#' @title Download a NCBI Database to Your Local Hard Drive
#' @description This function allows users to download a database selected by
#' \code{\link{listDatabases}} to their local hard drive.
#' @param db a character string specifying the database that shall be downloaded
#'  (selected from \code{\link{listDatabases}}).
#' @param path a character string specifying the location (a folder) in
#' which the corresponding database shall be stored.
#' Default is \code{path = "database"}.
#' In case this folder does not exist yet, it will be created.
#' @details
#' This function downloads large databases to your hard drive.
#' For this purpose a folder
#' named \code{database} (default) is created and the correspondning
#' database then stored in this folder.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#'   # search for available NCBI nr databases
#'   listNCBIDatabases(db = "nr")
#'   # select NCBI nr version 27 =  "nr.27.tar.gz"
#'   # and download it to your hard drive
#'   # -> please note that large databases take some time for download!
#'   download.database(db = "nr.27.tar.gz")
#' }
#' @seealso \code{\link{download.database.all}}, \code{\link{listDatabases}}
#' @return File path to the downloaded database file.
#' @export

download.database <- function(db, path = "database") {
    # test if internet connection is available
    connected.to.internet()
  withr::local_options(timeout = max(30000000000, getOption("timeout")))
  
    if (!is.element(db, listNCBIDatabases(db = db)))
        stop(
            paste0("The specified database '",
            db,
            "' could not be found on NCBI.",
            "Please use the listNCBIDatabases() command to retrieve available ", 
            "databases or check if the name was written correctly.", 
            collapse = ""),
            call. = FALSE
        )
    
    message("Starting download process of file: ", db, " ...")
    
    if (!file.exists(path))
        dir.create(path, recursive = TRUE)
    
    if (file.exists(file.path(path, db))) {
        message("The file '", db, " exists already and will not be re-downloaded.")
        return(file.path(path, db))
    }
    
    tryCatch({
        custom_download(
            paste0("ftp://ftp.ncbi.nlm.nih.gov/blast/db/", db),
            file.path(path, db),
            mode = "wb"
        )
        
        custom_download(
            paste0(
                "ftp://ftp.ncbi.nlm.nih.gov/blast/db/",
                paste0(db, ".md5")
            ),
            file.path(path, paste0(db, ".md5")),
            mode = "wb"
        )
    }, error = function(e)
        message(
            paste0("The FTP site 'ftp://ftp.ncbi.nlm.nih.gov/blast/db/",
            db,
            "' cannot be reached. Are you connected to the internet or could there be a firewall issue or did ",
            "something go wrong with the connection to the NCBI server?", 
            collapse = "")
        ))
    
    # test check sum
    md5_file <-
        readr::read_lines(file.path(path, paste0(db, ".md5")))
    md5_sum <- unlist(stringr::str_split(md5_file, " "))[1]
    
    message("Checking md5 hash of file: ", db , " ...")
    if (!(tools::md5sum(file.path(path, db)) == md5_sum)) {
        warning(
            paste0("The md5 hash between the downloaded file and the file ", 
                   "stored at NCBI do not match. Therefore, the currupted file '",file.path(path, db),"' has been removed.",
                   " This can happen due to internet connection breaks which corrupt the downloaded file. Please download the file '",
                   db,
                   "' again either by re-running the function."))
        unlink(file.path(path, paste0(db, ".md5")))
        unlink(file.path(path, db, ".md5"))
        
        return(FALSE)
    }
        
    unlink(file.path(path, paste0(db, ".md5")))
    message("The md5 hash of file '", db, "' matches!")
    message("File '",
            file.path(path, db),
            " has successfully been retrieved.")
    return(file.path(path, db))
}

context("Test: download.database()")

test_that("The download.database() throws error when wrong input
          database is specified..",{
              skip_on_cran()
              skip_on_travis()
              
     expect_error(download.database(db = "nr"))     
              
})

#' @title List number of available genomes in each kingdom of life
#' @description Users can retrieve the available number of sequenced 
#' genomes per kingdom.
#' @param db a character string specifying the database for which genome 
#' availability shall be checked, 
#' e.g. \code{db = "refseq"}, \code{db = "genbank"}, \code{db = "ensembl"}.
#' @author Hajk-Georg Drost
#' @examples 
#' \dontrun{
#' # list number of available genomes in refseq for each kingdom of life
#' listKingdoms(db = "refseq")
#' # example for genbank
#' listKingdoms(db = "genbank")
#' # example for ensembl
#' listKingdoms(db = "ensembl")
#' }
#' @seealso \code{\link{listGenomes}}, \code{\link{is.genome.available}}, 
#' \code{\link{listGroups}}
#' @export

listKingdoms <- function(db = "refseq") {
    organism_name <- kingdoms <- name <- division <- NULL
    
    listgenomes.data <-
        listGenomes(db = db,
                    type = "kingdom",
                    details = TRUE)
    
    if (is.element(db, c("refseq", "genbank"))) {
        uniq.species <-
            dplyr::summarise(
                dplyr::group_by(listgenomes.data, organism_name, kingdoms),
                unique_elements = dplyr::n_distinct(organism_name)
            )
        
        return(table(uniq.species$kingdoms))
    }
    
    if (is.element(db, c("ensembl"))) {
        uniq.species <-
            dplyr::summarise(
                dplyr::group_by(listgenomes.data, name, division),
                unique_elements = dplyr::n_distinct(name)
            )
        
        return(table(uniq.species$division))
    }
}

#' @title Retrieve available kingdoms of life
#' @description A short list of available kingdoms of life
#' @param db a character string specifying the database from which the genome
#' shall be retrieved: \code{db = "refseq"}, \code{db = "genbank"},
#' \code{db = "ensembl"}, \code{db = "ensemblgenomes"}.
#' Default is \code{db = "refseq"}.
#' @author Hajk-Georg Drost
#' @examples
#' # retrieve kingdoms available from refseq
#' getKingdoms(db = "refseq")
#'
#' # retrieve kingdoms available from genbank
#' getKingdoms(db = "genbank")
#' @seealso \code{\link{meta.retrieval}}, \code{\link{getGenome}},
#' \code{\link{getProteome}}, \code{\link{getCDS}}, \code{\link{getGroups}}
#' @export

getKingdoms <- function(db = "refseq") {
  withr::local_options(timeout = max(30000000, getOption("timeout")))

    if (!is.element(db, c("refseq", "genbank", "ensembl", "ensemblgenomes")))
        stop("Please select one of the available data bases: 'refseq',
             'genbank', 'ensembl', or 'ensemblgenomes'.", call. = FALSE)

    if (db == "refseq") {
        return(getKingdomRefseq())
    }

    if (db == "genbank") {
        return(getKingdomGenbank())
    }

    if (db %in% c("ensembl", "ensemblgenomes")) {
        return(getKingdomEnsembl())
    }
}

getKingdomGenbank <- function() {
  c(
    "archaea",
    "bacteria",
    "fungi",
    "invertebrate",
    "plant",
    "protozoa",
    "vertebrate_mammalian",
    "vertebrate_other"
  )
}

getKingdomRefseq <- function() {
  c(getKingdomGenbank(), "viral")
}

getKingdomEnsembl <- function() {
  ensembl_divisions_short()
}

context("Test: getKingdoms()")

test_that("The getKingdoms() interface works properly..",{
    skip_on_cran()
    skip_on_travis()
    
    expect_equal(
        getKingdoms(),
        c(
            "archaea",
            "bacteria",
            "fungi",
            "invertebrate",
            "plant",
            "protozoa",
            "vertebrate_mammalian",
            "vertebrate_other",
            "viral"
        )
    )
    
})

#' @title List number of available genomes in each taxonomic group
#' @description Users can retrieve the available number of sequenced 
#' genomes per group. Only available for \code{db = "refseq"} and 
#' \code{db = "genbank"}.
#' @param db a character string specifying the database for which genome 
#' availability shall be checked. Available options are:
#' \itemize{
#' \item \code{db = "refseq"} 
#' \item \code{db = "genbank"}
#' }
#' @param kingdom a kingdom specification retrieved by 
#' \code{\link{getKingdoms}}.
#' @param details shall all species corresponding to the specified 
#' \code{kingdom} be returned? Default is \code{details = FALSE}.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # example for refseq
#' listGroups(db = "refseq")
#' # example for genbank
#' listGroups(db = "genbank")
#' ### in case groups should be specified by kingdom
#' # first, retrieve available kingdom names
#' listKingdoms()
#' # now we choose kingdom "bacteria"
#' listGroups(db = "refseq", kingdom = "bacteria")
#' # or
#' listGroups(db = "genbank", kingdom = "bacteria")
#' }
#' @seealso \code{\link{listGenomes}}, \code{\link{is.genome.available}}, 
#' \code{\link{listKingdoms}}
#' @export

listGroups <-
    function(db = "refseq",
             kingdom = "all",
             details = FALSE) {
        if (!is.element(db, c("refseq", "genbank")))
            stop("Unfortunately, only db = 'refseq' and db = 'genbank' 
                 provide group information.")
        
        if (!is.element(kingdom, c(getKingdoms(), "all")))
            stop(
                "Please choose a kingdom that is supported by NCBI RefSeq or 
                NCBI Genbank. See getKingdoms() for details.",
                call. = FALSE
            )
        
        organism_name <- group <- kingdoms <- subgroup <- NULL
        
        listgenomes.data <-
            listGenomes(db = db,
                        type = "group",
                        details = TRUE)
        
        if (kingdom != "all") {
            # case Animals
            if (is.element(kingdom,
                           c(
                               "invertebrate",
                               "vertebrate_mammalian",
                               "vertebrate_other"
                           ))) {
                if (kingdom == "vertebrate_mammalian") {
                    # filter for kingdom before groups are determined
                    group.y <- NULL
                    listgenomes.data <-
                        dplyr::filter(
                            listgenomes.data,
                            kingdoms == "Eukaryota",
                            group.y == "Animals",
                            subgroup == "Mammals"
                        )
                }
                
                if (kingdom == "invertebrate") {
                    group.y <- NULL
                    # filter for kingdom before groups are determined
                    listgenomes.data <-
                        dplyr::filter(
                            listgenomes.data,
                            kingdoms == "Eukaryota",
                            group.y == "Animals",
                            subgroup %in% c(
                                "Insects",
                                "Other Animals",
                                "Roundworms",
                                "Flatworms"
                            )
                        )
                }
                
                if (kingdom == "vertebrate_other") {
                  group.y <- NULL  
                  # filter for kingdom before groups are determined
                    listgenomes.data <-
                        dplyr::filter(
                            listgenomes.data,
                            kingdoms == "Eukaryota",
                            group.y == "Animals",
                            subgroup %in% c("Amphibians", "Birds", "Fishes", 
                                            "Reptiles")
                        )
                }
            }
            # case Plants
            if (is.element(kingdom, c("plant"))) {
              group.y <- NULL  
              # filter for kingdom before groups are determined
                listgenomes.data <-
                    dplyr::filter(listgenomes.data,
                                  kingdoms == "Eukaryota",
                                  group.y == "Plants")
            }
            # case Fungi
            if (is.element(kingdom, c("fungi"))) {
              group.y <- NULL  
              # filter for kingdom before groups are determined
                listgenomes.data <-
                    dplyr::filter(listgenomes.data,
                                  kingdoms == "Eukaryota",
                                  group.y == "Fungi")
            }
            # case Protozoa
            if (is.element(kingdom, c("protozoa"))) {
              group.y <- NULL  
              # filter for kingdom before groups are determined
                listgenomes.data <-
                    dplyr::filter(listgenomes.data,
                                  kingdoms == "Eukaryota",
                                  group.y == "Protists")
            }
            # case Viruses
            if (is.element(kingdom, c("viral"))) {
                # filter for kingdom before groups are determined
                listgenomes.data <-
                    dplyr::filter(listgenomes.data,
                                  kingdoms %in% c("Viruses", "Viroids"))
            }
            # case Bacteria
            if (is.element(kingdom, c("bacteria"))) {
                # filter for kingdom before groups are determined
                listgenomes.data <-
                    dplyr::filter(listgenomes.data, kingdoms == "Bacteria")
            }
            # case Archaea
            if (is.element(kingdom, c("archaea"))) {
                # filter for kingdom before groups are determined
                listgenomes.data <-
                    dplyr::filter(listgenomes.data, kingdoms == "Archaea")
            }
            
            # change only first character to upper upper case
            #kingdom <- toupper_first_char(kingdom)
            
            if (nrow(listgenomes.data) == 0)
                stop("Unfortunately, no group is available for ",
                     kingdom,
                     ".",
                     call. = FALSE)
        }
        
        uniq.species <-
            dplyr::summarise(
                dplyr::group_by(listgenomes.data, organism_name, subgroup),
                unique_elements = dplyr::n_distinct(organism_name)
            )
        
        if (!details) {
            return(table(uniq.species$subgroup))
        } else {
            return(uniq.species)
        }
    }

#' @title Gene Ontology Query
#' @description This function takes a gene id as character vector from a given 
#' query organism and returns the corresponding GO terms and additional GO 
#' information.
#' @param organism a character string specifying the scientific name 
#' of a query organism.
#' @param genes a character vector storing the gene ids of a organisms 
#' of interest to be queried against Ensembl Biomart.
#' @param filters a character vector specifying the filter (query key) for 
#' the Ensembl Biomart query, e.g. \code{filter} = \code{"ensembl_gene_id"}.
#' @param ... additional parameters that can be passed to the 
#' \code{\link{biomart}} function.
#' @author Hajk-Georg Drost
#' @details 
#' This function takes the scientific name of a query organism, a set of genes 
#' for which GO terms and additional information shall be retrieved, and a 
#' filter argument that specifies the attribute for the query genes.  
#' @examples
#' \dontrun{ 
#' GO_tbl <- getGO(organism = "Arabidopsis thaliana", 
#'                 genes    = c("AT1G06090", "AT1G06100"),
#'                 filters  = "ensembl_gene_id")
#' 
#' # look at the result
#' head(GO_tbl)
#' }
#' @seealso \code{\link{biomart}}, \code{\link{organismFilters}}, 
#' \code{\link{organismBM}}, \code{\link[biomaRt]{getBM}}, 
#' \code{\link{getMarts}}, \code{\link{getDatasets}}, 
#' \code{\link{getFilters}}
#' @export

getGO <-
    function(organism,
             genes,
             filters,
             ...) {
        
    name  <- NULL

    orgAttr <-
        organismAttributes(organism = organism, topic = "go")
    
    GOattributes <- c("go_linkage_type",
                      "goslim_goa_accession",
                      "goslim_goa_description")
    
    GOattr_df <-
        dplyr::filter(orgAttr, name %in% GOattributes)
    
    if (dim(GOattr_df)[1] == 0)
        stop("Unfortunately for '",
             organism,
             "' no GO attributes could be found.")
    
    m <- names(table(GOattr_df[ , "mart"]))
    d <- names(table(GOattr_df[ , "dataset"]))
    
    if (m %in% c("ensembl")) {
        GOattributes <- "go_linkage_type"
    } else {
        GOattributes <- c("goslim_goa_description", "goslim_goa_accession")
    }
    
    if ((length(m) > 1) | (length(d) > 1))
        stop(
            "GO related attributes have been found in more than one mart or 
            dataset: mart = ",
            m,
            " ; dataset = ",
            d
        )
    
    return(
        biomart(
            genes      = genes,
            mart       = m,
            dataset    = d,
            attributes = GOattributes,
            filters    = filters,
            ...
        )
    )
}

#' @title Genomic Data Retrieval
#'
#' @description This package interacts with a suite of web Application
#' Programming Interfaces and FTP sites to perform automated genomic data
#' retieval and annotation information retrieval.
#'
#' @section About:
#' To automate the retrieval process on a meta-genomic scale, this package
#'  provides useful interface functions for genomic sequence retrieval and
#'  functional annotation retrieval.
#'  The major aim of \code{biomartr} is to facilitate computational
#'  reproducibility and large-scale handling of genomic data for
#'  (meta-)genomic analyses.
#'
#' In detail, \code{biomartr} aims to provide users with an easy to use
#' framework to obtain genome, proteome, CDS, GFF (annotation), genome
#' assembly quality, and metagenome project data. Furthermore, an interface to
#' the Ensembl Biomart database allows users to retrieve functional annotation
#' for genomic loci.
#' Users can download entire databases
#' such as
#' \itemize{
#' \item \code{NCBI RefSeq}
#' \item \code{NCBI nr}
#' \item \code{NCBI nt}
#' \item \code{NCBI Genbank}
#' \item \code{NCBI nt}
#' \item \code{Ensembl}
#' \item \code{Ensembl Genomes}
#' \item \code{UniProt}
#' }
#' @name biomartr-package
#' @aliases biomartr
#' @docType package
#' @author Hajk-Georg Drost \email{hajk-georg.drost@tuebingen.mpg.de}
#' @keywords package
## usethis namespace: start
#' @importFrom data.table data.table
#' @importFrom data.table :=
#' @importFrom data.table .SD
#' @importFrom data.table .BY
#' @importFrom data.table .N
#' @importFrom data.table .I
#' @importFrom data.table .GRP
#' @importFrom data.table .NGRP
#' @importFrom data.table .EACHI
#' @importFrom data.table fread fwrite rbindlist data.table chmatch
## usethis namespace: end
NULL

#' @title Retrieve summary statistics for a coding sequence (CDS) file
#' @description A summary statistics of specific CDS features is returned.
#' @param file file path to a CDS file in \code{fasta} format.
#' @param organism character string specifying the organism at hand.
#' @author Hajk-Georg Drost
#' @details 
#' The summary statistics include:
#' \itemize{
#' \item \code{total_seqs}: 
#' \item \code{nnn_abs}: The total number of NNN's 
#' (over all chromosomes/scaffolds/contigs) in all coding sequences combined
#' \item \code{nnn_perc}: The percentage (relative frequency) of NNN's 
#' (over all chromosomes/scaffolds/contigs) compared to the total number of 
#' nucleotides of all coding sequences
#' }
#' @seealso \code{\link{getCollection}}, \code{\link{getCDS}}, \code{\link{read_cds}}, \code{\link{summary_genome}}
#' @export 
summary_cds <- function(file,
                        organism) {
        if (!file.exists(file))
                stop("Please provide a valid file path to your CDS file.", call. = FALSE)
        
        cds_seq <- Biostrings::readDNAStringSet(filepath = file, format = "fasta")
       if (length(cds_seq) == 0)
               stop("Please provide a file that contains CDS sequences.", call. = FALSE)
        
        total_seqs <- length(cds_seq)
        total_corrupt_cds <- sum(cds_seq@ranges@width %% 3 > 0)
        corrupt_cds_perc <- total_corrupt_cds / total_seqs
        
        all_cds_seqs <- as.character(cds_seq)
        
        start_AND_end <- sum(sapply(all_cds_seqs, function(x) {
                triplet_start <- stringr::str_sub(x, 1, 3)
                triplet_end <-
                        stringr::str_sub(x, nchar(x) - 2, nchar(x))
                
                return((triplet_start == "ATG") &&
                               (triplet_end == "TGA"))
        }))
        
        ONLY_start <- sum(sapply(all_cds_seqs, function(x) {
                triplet_start <- stringr::str_sub(x, 1, 3)
                return(triplet_start == "ATG")
        }))
        
        start_with_N <- sum(sapply(all_cds_seqs, function(x) {
                triplet_start <- stringr::str_sub(x, 1, 1)
                return(triplet_start == "N")
        }))
        
        ONLY_end <- sum(sapply(all_cds_seqs, function(x) {
                triplet_end <-
                        stringr::str_sub(x, nchar(x) - 2, nchar(x))
                
                return(triplet_end == "TGA")
        }))
        
        end_with_N <- sum(sapply(all_cds_seqs, function(x) {
                triplet_end <-
                        stringr::str_sub(x, nchar(x), nchar(x))
                
                return(triplet_end == "N")
        }))
        
        # count absolute number of NNN's in genome
        NNN_abs <- sum(as.numeric(Biostrings::vcountPattern("N", cds_seq)))
        
        # relative frequency of NNN's in genome
        NNN_freq <-
                sum(as.numeric(Biostrings::vcountPattern("N", cds_seq))) / sum(as.numeric(cds_seq@ranges@width))
        
        res <- tibble::tibble(organism = organism,
                              n_seqs = total_seqs,
                              n_corrupt_cds = total_corrupt_cds,
                              rel_corrupt_cds = corrupt_cds_perc,
                              n_start_atg_and_end_tga = start_AND_end,
                              rel_start_atg_and_end_tga = start_AND_end / total_seqs,
                              n_start_atg = ONLY_start,
                              rel_start_atg = ONLY_start / total_seqs,
                              n_start_n = start_with_N,
                              n_end_tga = ONLY_end,
                              rel_end_tga = ONLY_end / total_seqs,
                              n_end_n = end_with_N,
                              n_nnn = NNN_abs,
                              rel_nnn = NNN_freq
                              
        )
        
        return(res)
}

#' @title Retrieve a List of Available NCBI Databases for Download  
#' @description This function allows you to retrieve a list of database names 
#' and versions that can be downloaded from correspondning servers.
#' @param db a character string specifying the name of the database that shall 
#' be searched for.
#' @param update a logical value specifying whether or not the local 
#' listDatabases.txt file shall be updated by remote access to NCBI.
#' @description
#' Database retrieval is crucial for most biological studies and analyses.
#' There is a vast diversity of databases that can be accessed remotely or that 
#' can be downloaded to your local machine. This function provides an interface 
#' to databases that can be downloaded from NCBI servers and lists all available
#' databases and their database version to be able to select an appropriate 
#' database for download with \code{\link{download.database}}.
#' @author Hajk-Georg Drost
#' @examples \dontrun{
#' # retrieve all versions of the NCBI 'nr' database that can be downloaded
#' listNCBIDatabases(db = "nr")
#' 
#' # analogous:
#' # listNCBIDatabases(db = "cdd")
#' # listNCBIDatabases(db = "nt")
#' # listNCBIDatabases(db = "gss")
#' # listNCBIDatabases(db = "refseq_protein")
#' }
#' @seealso \code{\link{download.database}}, \code{\link{download.database.all}}
#' @export

listDatabases <- function(db = "nr", update = FALSE) {
    if (is.element("listDatabases", as.character(match.call()[[1]])))
        warning(
      "Please use listNCBIDatabases() instead of listDatabases(), 
      in the next version of biomartr this function will be deprecated.",
            call. = FALSE
        )
    
    
    if (!file.exists(file.path(tempdir(), "_ncbi_downloads"))) {
        dir.create(file.path(tempdir(), "_ncbi_downloads"))
    }
    
    db.file.name <-
        file.path(tempdir(), "_ncbi_downloads", "listDatabases.txt")
    
    if (update) {
        if (file.exists(db.file.name))
            unlink(db.file.name)
    }
    
    if (file.exists(db.file.name)) {
        listDBs <-
            utils::read.csv(db.file.name,
                            sep = ";",
                            header = FALSE)
    } else {
        # retrieve all available databases from NCBI
        listDBs <-
            strsplit(
                RCurl::getURL(
                    url         = "ftp://ftp.ncbi.nlm.nih.gov/blast/db/",
                    ftp.use.epsv = FALSE,
                    dirlistonly  = TRUE
                ),
                "\n"
            )
        
        utils::write.table(
            x         = listDBs[[1]],
            file      = db.file.name,
            quote     = FALSE,
            col.names = FALSE,
            row.names = FALSE,
            sep       = ";"
        )
    }
    
    if (db == "all") {
        listDBs2 <- listDBs[[1]]
        all_databases <-
            listDBs2[-which(unlist(lapply(listDBs2, function(x)
                stringr::str_detect(x, ".md5"))))]
        # available DBs
        dbs <-
            as.vector(names(table(unlist(
                lapply(all_databases, function(x)
                    unlist(stringr::str_split(x, "[.]"))[1])
            ))))
        return(dbs[-which(dbs == "README")])
        
    } else {
        listDBs2 <- listDBs[[1]]
        # select all database versions of 'db'
        DBName <-
            listDBs2[unlist(lapply(listDBs2, function(x)
                stringr::str_detect(x, paste0("^", db))))]
        
        if (length(DBName) == 0)
            stop("No entries for db = '", db, "' could not be found.", 
                 call. = FALSE)

        # delete md5 entries
        return(as.vector(DBName[-which(unlist(lapply(DBName, function(x)
            stringr::str_detect(x, ".md5"))))]))
    }
}

clean.str.brackets <- function(string) {

    str.new <- stringr::str_replace(string,"\\(","\\\\\\\\(")
    str.new <- stringr::str_replace(str.new,"\\)","\\\\\\\\)")

    return(str.new)
}

is.taxid <- function(x) {
  return(stringr::str_count(x, "[:digit:]") == nchar(x))
}


docFile <-
  function(file.name = doc$file_name,
           organism = doc$organism,
           url = doc$url,
           database = doc$database,
           path = doc$path,
           refseq_category = doc$refseq_category,
           assembly_accession = doc$assembly_accession,
           bioproject = doc$bioproject,
           biosample = doc$bioproject,
           taxid = doc$taxid,
           infraspecific_name = doc$infraspecific_name,
           version_status = doc$version_status,
           release_type = doc$release_type,
           genome_rep = doc$genome_rep,
           seq_rel_date = doc$seq_rel_date,
           submitter = doc$submitter,
           doc = NULL) {
    local.org <- stringr::str_replace_all(organism, "-", "_")
    local.org <- stringr::str_replace_all(organism, "\\/", "_")

    sink(file.path(path, paste0(
      "doc_", local.org, "_db_", database, ".txt"
    )))

    cat(paste0("File Name: ", file.name))
    cat("\n")
    cat(paste0("Organism Name: ", organism))
    cat("\n")
    if (database %in% c("refseq", "genbank"))
      cat(paste0("Database: NCBI ", database))
    if (database %in% c("ensembl", "uniprot"))
      cat(paste0("Database: ", database))
    cat("\n")
    cat(paste0("URL: ", url))
    cat("\n")
    cat(paste0("Download_Date: ", date()))
    cat("\n")
    cat(paste0("refseq_category: ", refseq_category))
    cat("\n")
    cat(paste0("assembly_accession: ", assembly_accession))
    cat("\n")
    cat(paste0("bioproject: ", bioproject))
    cat("\n")
    cat(paste0("biosample: ", biosample))
    cat("\n")
    cat(paste0("taxid: ", taxid))
    cat("\n")
    cat(paste0("infraspecific_name: ", infraspecific_name))
    cat("\n")
    cat(paste0("version_status: ", version_status))
    cat("\n")
    cat(paste0("release_type: ", release_type))
    cat("\n")
    cat(paste0("genome_rep: ", genome_rep))
    cat("\n")
    cat(paste0("seq_rel_date: ", seq_rel_date))
    cat("\n")
    cat(paste0("submitter: ", submitter))

    sink()

  }



setTMPFile <- function(file.name){

        tempfile(pattern = file.name, tmpdir = tempdir(), fileext = "")
}


getTMPFile <- function(file.name){

        file.path(tempdir(),file.name)

}

#' Gunzip and checks post download
#'
#' @noRd
gunzip_and_check <- function(file, gunzip = FALSE,
                             remove_annotation_outliers = FALSE, format,
                             mute_citation = FALSE) {
  final_path <- file
  if (gunzip) {
    final_path <- unlist(stringr::str_replace(final_path, "[.]gz", ""))
  }

  message("-> The ", toupper(format), " has been downloaded to '",
          dirname(final_path), "' and has been named '",
          basename(final_path), "'."
  )

  if (gunzip) {
    message("-> Unzipping downloaded file ...")
    R.utils::gunzip(file, destname = final_path)
  }
  if (format == "gff3") {
    output_path <- check_annotation_biomartr(final_path, remove_annotation_outliers)
  }


  please_cite_biomartr(mute_citation = mute_citation)
  return(final_path)
}

#' Validates md5 sum test given url and local file
#' @param md5_local the path to download md5 file
#' @param md5_url the md5 url to download
#' @param local_file the existing local file to check
#' @param file_url full path to online file,
#' the file named paste0("./", basename(file_url)) must exist in md5 name list.
#' @noRd
md5_sum_test <- function(md5_local, md5_url, local_file, file_url) {
  if (!file.exists(local_file))
    stop("The local file: ", local_file, " does not exist to check md5 sum of!")
  custom_download(
    md5_url,
    md5_local,
    mode = "wb"
  )
  # test check sum
  md5_file_path <- md5_local
  md5_file <- read_md5file(md5_file_path)

  file_name <- NULL

  md5_sum <- dplyr::filter(md5_file,
                           file_name == paste0("./", basename(file_url)))$md5

  message("-> Checking md5 hash of file: ",
          local_file , " (md5: ", md5_sum, ")", " ...")

  if (!(tools::md5sum(local_file) == md5_sum))
    stop(
      paste0(
        "Please download the file '",
        md5_file_path,
        "' again. The md5 hash between the downloaded file and the file ",
        "stored at NCBI do not match.",
        collapse = ""
      )
    )
  unlink(md5_file_path)
  message("-> The md5 hash of file '", md5_file_path, "' matches!")
  return(invisible(NULL))
}

test <- function(x){ print(paste0("Test ",x," passed.","\n"))}

#' Get directory to store back end files like kingdom summaries etc
#' @param non_temp_cache "~/.biomartr_cache_dir.rds",
#' @return reads the rds file, and returns the path for local cache, if not
#' existing, use tempdir().
#' @export
#' @examples
#' cachedir()
#' @family cachedir
cachedir <- function(non_temp_cache = "~/.biomartr_cache_dir.rds") {
  if (file.exists(non_temp_cache)) {
    readRDS(non_temp_cache)
  } else tempdir()
}

#' Set directory to store back end files like kingdom summaries etc
#' @param path the path to cache dir, example "~/Bio_data/biomartr_cache/"
#' @return invisible(NULL), only save the file to path location
#' @export
#' @examples
#' # By default it is tempdir()
#' cachedir()
#' # cachedir_set("~/Bio_data/biomartr_cache/")
#' cachedir()
#' @family cachedir
cachedir_set <- function(path) {
  dir.create(path, showWarnings = FALSE, recursive = TRUE)
  saveRDS(path, file = "~/.biomartr_cache_dir.rds")
}

#' @inherit getBio
#' @title Genome Retrieval
#' @description  Main genome retrieval function for an organism of interest.
#' By specifying the scientific name of an organism of interest the
#' corresponding fasta-file storing the genome of the organism of interest
#' can be downloaded and stored locally. Genome files can be retrieved from
#' several databases. In addition, the genome summary statistics for the
#' retrieved species is stored locally to provide users with
#' insights regarding the genome assembly quality (see \code{\link{summary_genome}} for details).
#' This is useful when comparing genomes with large difference in genome assembly qualities.
#' @family getBio
#' @family genome
#' @export
getGenome <- function(db = "refseq",
             organism,
             reference = FALSE,
             skip_bacteria = TRUE,
             release = NULL,
             gunzip = FALSE,
             path = file.path("_ncbi_downloads", "genomes"),
             assembly_type = "toplevel",
             mute_citation = FALSE,
             analyse_genome = FALSE
             ) {

       if (!is.element(db, c("refseq", "genbank", "ensembl")))
            stop(
                "Please select one of the available data bases: 'refseq',
                'genbank', or 'ensembl'.",
                call. = FALSE
            )
        if (!(assembly_type %in% c("toplevel", "primary_assembly")))
            stop("Please select one the available assembly types: \ntoplevel, primary_assembly")
        if ((db != "ensembl") && (assembly_type != "toplevel"))
            stop( "The assembly_type argument is not default value.",
            "Don't change this argument when not using db = 'ensembl'.", call. = FALSE)

        if (!is.logical(reference))
            stop("Please specify 'reference' as either TRUE or FALSE.", call. = FALSE)


      message("-> Starting genome retrieval of '", organism, "' from ", db, " ...")
      message("\n")

      # create result folder
      if (!file.exists(path)) {
        dir.create(path, recursive = TRUE)
      }

    if (is.element(db, c("refseq", "genbank"))) {
      info <- get_file_refseq_genbank(db, organism, reference, skip_bacteria,
                                      release, gunzip, path, assembly_type,
                                      type = "genome")
      refseq_genbank_download_post_processing(info, organism, db, path,
                                              gunzip,
                                              remove_annotation_outliers = FALSE,
                                              format = "genome",
                                              analyse_genome = analyse_genome,
                                              mute_citation = mute_citation)
    } else if (db %in% c("ensembl", "ensemblgenomes")) {
        genome.path <- getENSEMBL.Seq(organism, type = "dna",
                                      id.type = assembly_type,
                                      release = release,
                                      path = path)

        ensembl_download_post_processing(genome.path, organism,
                                         format = "genome",
                                         gunzip = gunzip,
                                         mute_citation = mute_citation)
    }
}

context("Test: getGenome()")

test_that("The getGenome() interface works properly for NCBI RefSeq (including when command is repeated)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from genbank
    out1 <- read_genome(
        getGenome(
            db       = "refseq",
            organism = "Saccharomyces cerevisiae",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    # test proper use of internal referece files when command is repeated
    out2 <- read_genome(
        getGenome(
            db       = "refseq",
            organism = "Saccharomyces cerevisiae",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    expect_s4_class(out1, class = "DNAStringSet")
    expect_false(any(out1 != out2))
})


test_that("The getGenome() interface works properly for NCBI RefSeq using taxid..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from genbank
    out1 <- read_genome(
        getGenome(
            db       = "refseq",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_s4_class(out1, class = "DNAStringSet")
})

test_that("The getGenome() interface works properly for NCBI RefSeq using assembly id..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from genbank
    out1 <- read_genome(
        getGenome(
            db       = "refseq",
            organism = "GCF_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_s4_class(out1, class = "DNAStringSet")
})



test_that("The getGenome() interface works properly for NCBI Genbank (including when command is repeated)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from genbank
    out1 <- read_genome(
        getGenome(
            db       = "genbank",
            organism = "Saccharomyces cerevisiae",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    # test proper use of internal referece files when command is repeated
    out2 <- read_genome(
        getGenome(
            db       = "genbank",
            organism = "Saccharomyces cerevisiae",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_s4_class(out1, class = "DNAStringSet")
    expect_false(any(out1 != out2))
})

test_that("The getGenome() interface works properly for NCBI Genbank using taxid (including when command is repeated)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from genbank
    out1 <- read_genome(
        getGenome(
            db       = "genbank",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    # test proper use of internal referece files when command is repeated
    out2 <- read_genome(
        getGenome(
            db       = "genbank",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_s4_class(out1, class = "DNAStringSet")
    expect_false(any(out1 != out2))
})

test_that("The getGenome() interface works properly for NCBI Genbank using accession ids (including when command is repeated)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from genbank
    out1 <- read_genome(
        getGenome(
            db       = "genbank",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    # test proper use of internal referece files when command is repeated
    out2 <- read_genome(
        getGenome(
            db       = "genbank",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_s4_class(out1, class = "DNAStringSet")
    expect_false(any(out1 != out2))
})

test_that("The getGenome() interface works properly for ENSEMBL (including repeating function call)..",{

        skip_on_cran()
        skip_on_travis()
    # test proper download from ENSEMBL
        out1 <- read_genome(
            getGenome(
                db       = "ensembl",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

        out2 <- read_genome(
            getGenome(
                db       = "ensembl",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )
        expect_s4_class(out1, class = "DNAStringSet")
        expect_false(any(out1 != out2))
})

test_that("The getGenome() interface works properly for ENSEMBL using taxid (including repeating function call)..",{

    skip_on_cran()
    skip_on_travis()
    # test proper download from ENSEMBL
    out1 <- read_genome(
        getGenome(
            db       = "ensembl",
            organism = "4932",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    out2 <- read_genome(
        getGenome(
            db       = "ensembl",
            organism = "4932",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_s4_class(out1, class = "DNAStringSet")
    expect_false(any(out1 != out2))
})


test_that("The getGenome() interface works properly for ENSEMBL using accession id (including repeating function call)..",{

    skip_on_cran()
    skip_on_travis()
    # test proper download from ENSEMBL
    out1 <- read_genome(
        getGenome(
            db       = "ensembl",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    out2 <- read_genome(
        getGenome(
            db       = "ensembl",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_s4_class(out1, class = "DNAStringSet")
    expect_false(any(out1 != out2))
})

test_that("The getGenome() interface works properly for ENSEMBL with collections",{

  skip_on_cran()
  skip_on_travis()
  # test proper download from ENSEMBL
  # Bacteria
  out1 <- getGenome(db       = "ensembl",
            organism = "Escherichia coli",
            path = tempdir(), mute_citation = TRUE)
  # Fungi
  out2 <- getGenome(db       = "ensembl",
            organism = "Acremonium chrysogenum",
            path = tempdir(), mute_citation = TRUE)
  # Protists
  getGenome(db       = "ensembl",
            organism = "Babesia bigemina",
            path = tempdir(), mute_citation = TRUE)
  expect_false(out1 == out2)
})

test_that("The getGenome() error messages work properly for ENSEMBL..", {
    skip_on_cran()
    skip_on_travis()

    out1 <- getGenome(
        db       = "ensembl",
        organism = "Saccharomycesi",
        path     = tempdir(), mute_citation = TRUE
    )
    expect_type(out1, "logical")
    expect_false(out1 == TRUE)
})

test_that("The getGenome() error messages work properly for NCBI RefSeq", {
    skip_on_cran()
    skip_on_travis()

    expect_equal(getGenome(
        db       = "refseq",
        organism = "Saccharomycesi",
        path     = tempdir(), mute_citation = TRUE
    ), "Not available")
})

#' @title Proteome Retrieval
#' @description Main proteome retrieval function for an organism of interest.
#' By specifying the scientific name of an organism of interest the
#' corresponding fasta-file storing the proteome of the organism of interest
#' can be downloaded and stored locally. Proteome files can be retrieved from
#' several databases.
#' @inheritParams getGenome
#' @param path a character string specifying the location (a folder) in which
#' the corresponding proteome shall be stored. Default is
#' \code{path} = \code{file.path("_ncbi_downloads","proteomes")}.
#' @param update logical, default TRUE. (Uniprot only for now!)
#' If species info file exists already,
#' do not re download, makes it faster but the file can be old, i.e. no longer
#' as complete as it could be.
#' @author Hajk-Georg Drost
#' @details Internally this function loads the the overview.txt file from NCBI:
#'
#'  refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/
#'
#'  genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/
#'
#' and creates a directory '_ncbi_downloads/proteomes' to store
#' the proteome of interest as fasta file for future processing.
#' @return File path to downloaded proteome.
#' @examples \dontrun{
#' # download the proteome of Arabidopsis thaliana from NCBI RefSeq
#' # and store the corresponding proteome file in '_ncbi_downloads/refseq/proteomes'
#' file_path <- getProteome( db       = "refseq",
#'              organism = "Arabidopsis thaliana",
#'              path     = file.path("_ncbi_downloads","refseq","proteomes") )
#' # import proteome into R session
#' Ath_proteome <- read_proteome(file_path, format = "fasta")
#'
#' # download the proteome of Arabidopsis thaliana from NCBI Genbank
#' # and store the corresponding proteome file in '_ncbi_downloads/genbank/proteomes'
#' file_path <- getProteome( db       = "genbank",
#'              organism = "Arabidopsis thaliana",
#'              path     = file.path("_ncbi_downloads","genbank","proteomes") )
#' # import proteome into R session
#' Ath_proteome <- read_proteome(file_path, format = "fasta")
#'
#  # download the proteome of Arabidopsis thaliana from UniProt
#' # and store the corresponding proteome file in '_downloads/uniprot/proteomes'
#' file_path <- getProteome( db       = "uniprot",
#'              organism = "Arabidopsis thaliana",
#'              path     = file.path("_downloads","uniprot","proteomes") )
#'# import proteome into R session
#'Ath_proteome <- read_proteome(file_path, format = "fasta")
#'
#' # download the proteome of Arabidopsis thaliana from ENSEMBL
#' # and store the corresponding proteome file in '_downloads/ensembl/proteomes'
#' file_path <- getProteome( db       = "ensembl",
#'              organism = "Arabidopsis thaliana",
#'              path     = file.path("_downloads","ensembl","proteomes") )
#' # import proteome into R session
#' Ath_proteome <- read_proteome(file_path, format = "fasta")
#' }
#' @family getBio
#' @family proteome
#' @export

getProteome <-
    function(db = "refseq",
             organism,
             reference = TRUE,
             skip_bacteria = TRUE,
             release = NULL,
             gunzip = FALSE,
             update = TRUE,
             path = file.path("_ncbi_downloads", "proteomes"),
             mute_citation = FALSE) {
  if (!is.element(db, c("refseq", "genbank",
                        "ensembl", "ensemblgenomes", "uniprot")))
      stop(
          "Please select one of the available data bases:
          'refseq', 'genbank', 'ensembl', 'ensemblgenomes' or 'uniprot'.",
          call. = FALSE
      )

  # create result folder
  if (!file.exists(path)) {
    dir.create(path, recursive = TRUE)
  }

  if (db == "ensemblgenomes") {
          organism_name <- is.genome.available(db = db, organism = organism, details = TRUE)$display_name[1]

          if (!is.na(organism_name))
                  message("Starting proteome retrieval of '", organism_name, "' from ", db, " ...")
          if (is.na(organism_name))
                  message("Starting proteome retrieval of '", organism, "' from ", db, " ...")

          message("\n")
  } else {
      message("-> Starting proteome retrieval of '", organism, "' from ", db, " ...")
      message("\n")
  }

    if (is.element(db, c("refseq", "genbank"))) {
      info <- get_file_refseq_genbank(db, organism, reference, skip_bacteria,
                                      release, gunzip, path, type = "protein")
      refseq_genbank_download_post_processing(info, organism, db, path,
                                              gunzip,
                                              remove_annotation_outliers = FALSE,
                                              format = "protein",
                                              mute_citation = mute_citation)
    } else if (db %in% c("ensembl", "ensemblgenomes")) {
        proteome.path <- getENSEMBL.Seq(organism, type = "pep",
                                        id.type = "all",
                                        release = release,
                                        path = path)
        return(ensembl_download_post_processing(proteome.path, organism,
                                                format = "pep.fa",
                                                remove_annotation_outliers = FALSE,
                                                gunzip,
                                                mute_citation = mute_citation))


    } else if (db == "uniprot") {
        getUniProtSeq(organism = organism, path = path, update = update,
                      gunzip = gunzip, mute_citation = mute_citation)
    }
}

context("Test: getProteome()")

test_that("The getProteome() interface to NCBI RefSeq works properly (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download
    out1 <-
        read_proteome(
            getProteome(
                db       = "refseq",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

    # test proper use of internal referece files when command is repeated
    out2 <-
        read_proteome(
            getProteome(
                db       = "refseq",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )
    expect_false(length(out1) != length(out2) | length(out1) == 0)
})


test_that("The getProteome() interface to NCBI RefSeq works properly using taxid (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download
    out1 <-   read_proteome(
            getProteome(
                db       = "refseq",
                organism = "559292",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

    # test proper use of internal referece files when command is repeated
    out2 <-  read_proteome(
            getProteome(
                db       = "refseq",
                organism = "559292",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )
        expect_false(length(out1) != length(out2) | length(out1) == 0)
})


test_that("The getProteome() interface to NCBI RefSeq works properly using accession id (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download
    out1 <- read_proteome(
        getProteome(
            db       = "refseq",
            organism = "GCF_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    # test proper use of internal referece files when command is repeated
    out2 <- read_proteome(
        getProteome(
            db       = "refseq",
            organism = "GCF_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_false(length(out1) != length(out2) | length(out1) == 0)
})

test_that("The getProteome() interface to NCBI Genbank works properly (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download from genbank
    out1 <-   read_proteome(
            getProteome(
                db       = "genbank",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

    # test proper use of internal referece files when command is repeated
    out2 <-   read_proteome(
            getProteome(
                db       = "genbank",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )
        expect_false(length(out1) != length(out2) | length(out1) == 0)
})


test_that("The getProteome() interface to NCBI Genbank works properly using taxid (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download
    out1 <- read_proteome(
        getProteome(
            db       = "genbank",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    # test proper use of internal referece files when command is repeated
    out2 <- read_proteome(
        getProteome(
            db       = "genbank",
            organism = "559292",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_false(length(out1) != length(out2) | length(out1) == 0)
})


test_that("The getProteome() interface to NCBI Genbank works properly using accession (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    # test proper download
    out1 <- read_proteome(
        getProteome(
            db       = "genbank",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    # test proper use of internal referece files when command is repeated
    out2 <- read_proteome(
        getProteome(
            db       = "genbank",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_false(length(out1) != length(out2) | length(out1) == 0)
})

test_that("The getProteome() interface to Ensembl works properly (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    out1 <- read_proteome(
            getProteome(
                db       = "ensembl",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )

    out2 <-  read_proteome(
            getProteome(
                db       = "ensembl",
                organism = "Saccharomyces cerevisiae",
                path     = tempdir(), mute_citation = TRUE
            ),
            format = "fasta"
        )
        expect_false(length(out1) != length(out2) | length(out1) == 0)
})

test_that("The getProteome() interface to Ensembl works properly using taxid (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    out1 <- read_proteome(
        getProteome(
            db       = "ensembl",
            organism = "4932",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    out2 <- read_proteome(
        getProteome(
            db       = "ensembl",
            organism = "4932",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_false(length(out1) != length(out2) | length(out1) == 0)
})


test_that("The getProteome() interface to Ensembl works properly using accession ids (including repeating command)..", {
    skip_on_cran()
    skip_on_travis()
    out1 <- read_proteome(
        getProteome(
            db       = "ensembl",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )

    out2 <- read_proteome(
        getProteome(
            db       = "ensembl",
            organism = "GCA_000146045.2",
            path     = tempdir(), mute_citation = TRUE
        ),
        format = "fasta"
    )
    expect_false(length(out1) != length(out2) | length(out1) == 0)
})

test_that("The getProteome() interface to Ensembl works properly (For organisms with collections)..", {
  skip_on_cran()
  skip_on_travis()

  out <- getProteome(
    db       = "ensembl",
    organism = "Escherichia coli",
    path     = tempdir(), mute_citation = TRUE
  )

  expect_false(is.logical(out))
})

test_that("The getProteome() interface to Uniprot works properly ..", {
  skip_on_cran()
  skip_on_travis()

  out1 <- getProteome(
    db       = "uniprot",
    organism = "Saccharomyces cerevisiae",
    path     = tempdir(), mute_citation = TRUE
  )

  out2 <- getProteome(
    db       = "uniprot",
    organism = "Homo sapiens",
    path     = tempdir(), mute_citation = TRUE
  )

  expect_false(is.logical(c(out1, out2)))
  expect_false(anyNA(c(out1, out2)))
  expect_false(identical(out1, out2))
})

#' @title Retrieve NCBI GENOME_REPORTS file
#' @description Retrieves NCBI GENOME_REPORTS file from
#' ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/overview.txt.
#' @param local_file character, file path, default: file.path(cachedir(), "_ncbi_downloads", "overview.txt")
#' @return a tibble object with the report
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' report <- getGENOMEREPORT()
#' report
#' }
#' @export

getGENOMEREPORT <- function(local_file = file.path(cachedir(), "_ncbi_downloads", "overview.txt")) {
  withr::local_options(timeout = max(30000000, getOption("timeout")))

    if (!dir.exists(dirname(local_file))) {
        dir.create(dirname(local_file), recursive = TRUE, showWarnings = FALSE)
    }

    if (!file.exists(local_file)) {
        tryCatch({
            downloader::download(
               "ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/overview.txt",
                local_file,
                mode = "wb"
            )
        }, error = function(e)
            message(
                "Unfortunately, the FTP site
                'ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/overview.txt'
                does not seem to be reachable. Is your current internet connection stable? Can you reach the
                FTP site 'ftp://ftp.ncbi.nlm.nih.gov'?"
            ))

        # NCBI limits requests to three per second
    }

    suppressWarnings(
        ncbi_overview <-
            readr::read_tsv(
                local_file,
                comment = "#",
                col_names = c(
                    "organism_name",
                    "kingdoms",
                    "group",
                    "subgroup",
                    "file_size_MB",
                    "chrs",
                    "organelles",
                    "plasmids",
                    "bio_projects"
                ),
                col_types = readr::cols(
                    organism_name = readr::col_character(),
                    kingdoms = readr::col_character(),
                    group = readr::col_character(),
                    subgroup = readr::col_character(),
                    file_size_MB = readr::col_double(),
                    chrs = readr::col_integer(),
                    organelles = readr::col_integer(),
                    plasmids = readr::col_integer(),
                    bio_projects = readr::col_integer()
                )
            )
    )

    return(ncbi_overview)
}

#' @title Repeat Masker Retrieval
#' @description  Main Repeat Masker output retrieval function for an 
#' organism of interest.
#' By specifying the scientific name of an organism of interest the 
#' corresponding Repeat Masker file storing the genome of the organism of 
#' interest can be downloaded and stored locally. 
#' Repeat Masker files can be retrieved from several databases.
#' @param db a character string specifying the database from which the genome 
#' shall be retrieved:
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' }
#' @param organism a character string specifying the scientific name of the 
#' organism of interest, e.g. \code{organism = "Homo sapiens"}.
#' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.
#' @param skip_bacteria Due to its enormous dataset size (> 700MB as of July 2023), 
#' the bacterial summary file will not be loaded by default anymore. If users
#' wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the 
#' bacterial summary file will be downloaded.
#' @param path a character string specifying the location (a folder) in which 
#' the corresponding file shall be stored. Default is 
#' \code{path} = \code{file.path("_ncbi_downloads","repeatmasker")}.
#' @param mute_citation logical value indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @details Internally this function loads the the overview.txt file from NCBI:
#' 
#'  refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/
#' 
#'  genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/
#'  
#' and creates a directory '_ncbi_downloads/repeatmasker' to store
#' the files of interest as fasta file for future processing.
#' In case the corresponding fasta file already exists within the
#' '_ncbi_downloads/repeatmasker' folder and is accessible within the workspace,
#' no download process will be performed.
#' @return File path to downloaded Repeat Masker output file.
#' @examples \dontrun{
#' 
#' # download the Repeat Masker output file of Homo sapiens from refseq
#' # and store the corresponding genome file in '_ncbi_downloads/genomes'
#' file_path <- getRepeatMasker( db       = "refseq", 
#'              organism = "Homo sapiens", 
#'              path = file.path("_ncbi_downloads","repeatmasker"))
#' 
#' Hsap_repeatmasker <- read_rm(file_path)
#' 
#' }
#' 
#' @seealso \code{\link{getGenome}}, \code{\link{getProteome}}, \code{\link{getCDS}}, 
#' \code{\link{getGFF}}, \code{\link{getRNA}}, \code{\link{getCollection}}, \code{\link{meta.retrieval}}, 
#' \code{\link{read_rm}}
#' @export

getRepeatMasker <-
    function(db = "refseq",
             organism,
             reference = FALSE,
             skip_bacteria = TRUE,
             path = file.path("_ncbi_downloads", "repeatmasker"),
             mute_citation = FALSE) {
        
    if (!is.element(db, c("refseq", "genbank")))
            stop(
                "Please select one of the available data bases for which ",
                "Repeat Masker output files are available: 'refseq' or ",
                "'genbank'.",
                call. = FALSE
            )
        
        message("-> Starting Repeat Masker retrieval of '", organism, "' from ", db, " ...")
        message("\n")
        
        if (is.element(db, c("refseq", "genbank"))) {
            # get Kingdom Assembly Summary file
            AssemblyFilesAllKingdoms <-
                getKingdomAssemblySummary(db = db, skip_bacteria = skip_bacteria)
            
            # test whether or not genome is available
            suppressMessages(is.genome.available(organism = organism, db = db, skip_bacteria = skip_bacteria))
            
            if (!file.exists(path)) {
                dir.create(path, recursive = TRUE)
            }
            
            organism_name <-
                refseq_category <- version_status <- ftp_path <- NULL
            
            organism <-
                stringr::str_replace_all(organism, "\\(", "")
            organism <-
                stringr::str_replace_all(organism, "\\)", "")
            
            assembly_accession <- taxid <- NULL
            
            if (reference) {
                if (!is.taxid(organism)) {
                    FoundOrganism <-
                        dplyr::filter(
                            AssemblyFilesAllKingdoms,
                            stringr::str_detect(organism_name, organism) | 
                                stringr::str_detect(assembly_accession, organism),
                            ((refseq_category == "representative genome") |
                                 (refseq_category == "reference genome")
                            ),
                            (version_status == "latest"), !is.na(ftp_path)
                        ) 
                } else {
                    FoundOrganism <-
                        dplyr::filter(
                            AssemblyFilesAllKingdoms,
                            taxid == as.integer(organism),
                            ((refseq_category == "representative genome") |
                                 (refseq_category == "reference genome")
                            ),
                            (version_status == "latest"), !is.na(ftp_path))
                }
            } else {
                if (!is.taxid(organism)) {
                    FoundOrganism <-
                        dplyr::filter(
                            AssemblyFilesAllKingdoms,
                            stringr::str_detect(organism_name, organism) |
                                stringr::str_detect(assembly_accession, organism),
                            (version_status == "latest"), !is.na(ftp_path)
                        ) 
                } else {
                    FoundOrganism <-
                        dplyr::filter(
                            AssemblyFilesAllKingdoms,
                            taxid == as.integer(organism),
                            (version_status == "latest"), !is.na(ftp_path)
                        ) 
                }
            }
            
            if (nrow(FoundOrganism) == 0) {
                    message(
                            paste0(
                                    "----------> No reference genome or representative genome was found for '",
                                    organism, "'. Thus, download for this species has been omitted.",
                                    " Have you tried to specify 'reference = FALSE' ?"
                            )
                    )
                    return("Not available")
            } else {
                if (nrow(FoundOrganism) > 1) {
                    warnings(
                        "More than one entry has been found for '",
                        organism,
                        "'. Only the first entry '",
                        FoundOrganism[1, 1],
                        "' has been used for subsequent genome retrieval."
                    )
                    FoundOrganism <- FoundOrganism[1,]
                }
                
                organism <-
                    stringr::str_replace_all(organism, " ", "_")
                
                download_url <-
                    paste0(FoundOrganism$ftp_path,
                           "/",
                           paste0(
                               basename(FoundOrganism$ftp_path),
                               "_rm.out.gz"
                           ))
                
                # if (!exists.ftp.file(url = paste0(FoundOrganism$ftp_path, "/"),
                #                      file.path = download_url)) {
                #     message(
                #         "Unfortunately no genome file could be found for organism '",
                #         organism,
                #         "'. Thus, the download of this organism has been omitted."
                #     )
                #     return(FALSE)
                # }
                
                # download_url <- paste0(query$ftp_path,query$`
                # assembly_accession`,"_",query$asm_name,"_genomic.fna.gz")
                
                local.org <-
                    stringr::str_replace_all(organism, "-", "_")
                local.org <-
                    stringr::str_replace_all(organism, "\\/", "_")
                
                if (nrow(FoundOrganism) == 1) {
                    if (file.exists(file.path(
                        path,
                        paste0(local.org, "_rm_", db, ".out.gz")
                    ))) {
                        message(
                            "-> File ",
                            file.path(
                                path,
                                paste0(local.org, "_rm_", db, ".out.gz")
                            ),
                            " exists already. Thus, download has been skipped."
                        )
                    } else {
                        tryCatch({
                            utils::capture.output(
                                custom_download(
                                    download_url,
                                    destfile = file.path(
                                        path,
                                        paste0(local.org,
                                               "_rm_", db, ".out.gz")
                                    ),
                                    mode = "wb"
                                )
                            )
                        }, error = function(e)
                        {
                          message(
                            "The download session seems to have timed out at the FTP site '",
                            download_url, "'. This could be due to an overload of queries to the databases.",
                            " Please restart this function to continue the data retrieval process or wait ",
                            "for a while before restarting this function in case your IP address was logged due to an query overload on the server side."
                          )
                          return("Not available")
                        })
                    }
                            
                            message("-> RepeatMasker download of ", organism, " is completed!")
                                
                            # download md5checksum file for organism of interest
                            custom_download(
                              paste0(FoundOrganism$ftp_path,"/md5checksums.txt"),
                              file.path(path, 
                                        paste0(local.org, "_md5checksums.txt")),
                              mode = "wb"
                            )
                            
                            # test check sum
                            md5_file_path <- file.path(path, 
                                                       paste0(local.org, 
                                                              "_md5checksums.txt"))
                            md5_file <-
                              read_md5file(md5_file_path)
                            
                            file_name <- NULL
                            
                            md5_sum <- dplyr::filter(md5_file,
                                                     file_name == paste0("./", paste0(
                                                       basename(FoundOrganism$ftp_path),
                                                       "_rm.out.gz"
                                                     )))$md5
                            
                            # check if md5 info is available
                            if (!is.character(md5_sum)){
                            message("-> Checking md5 hash of file: ", 
                                    file.path(
                                      path,
                                      paste0(local.org, "_rm_", db, ".out.gz")
                                    ) , " (md5: ", md5_sum, ")", " ...")
                            
                            if (!(tools::md5sum(file.path(
                              path,
                              paste0(local.org, "_rm_", db, ".out.gz")
                            )) == md5_sum))
                              stop(
                                paste0(
                                  "Please download the file '",
                                  md5_file_path,
                                  "' again. The md5 hash between the downloaded file and the file ",
                                  "stored at NCBI do not match.",
                                  collapse = ""
                                )
                              )
                            unlink(md5_file_path)
                            message("The md5 hash of file '", md5_file_path, "' matches!")
                            }
                    
                    docFile(
                        file.name = paste0(local.org, "_rm_", db, 
                                           ".out.gz"),
                        organism  = organism,
                        url       = download_url,
                        database  = db,
                        path      = path,
                        refseq_category = FoundOrganism$refseq_category,
                        assembly_accession = FoundOrganism$assembly_accession,
                        bioproject = FoundOrganism$bioproject,
                        biosample = FoundOrganism$biosample,
                        taxid = FoundOrganism$taxid,
                        infraspecific_name = FoundOrganism$infraspecific_name,
                        version_status = FoundOrganism$version_status,
                        release_type = FoundOrganism$release_type,
                        genome_rep = FoundOrganism$genome_rep,
                        seq_rel_date = FoundOrganism$seq_rel_date,
                        submitter = FoundOrganism$submitter
                    )
                    
                    doc <- tibble::tibble(
                        file_name = paste0(ifelse(is.taxid(organism), paste0("taxid_", local.org), local.org), "_genomic_", db,
                                           ".fna.gz"),
                        organism  = organism,
                        url       = download_url,
                        database  = db,
                        path      = path,
                        refseq_category = FoundOrganism$refseq_category,
                        assembly_accession = FoundOrganism$assembly_accession,
                        bioproject = FoundOrganism$bioproject,
                        biosample = FoundOrganism$biosample,
                        taxid = FoundOrganism$taxid,
                        infraspecific_name = FoundOrganism$infraspecific_name,
                        version_status = FoundOrganism$version_status,
                        release_type = FoundOrganism$release_type,
                        genome_rep = FoundOrganism$genome_rep,
                        seq_rel_date = FoundOrganism$seq_rel_date,
                        submitter = FoundOrganism$submitter
                        
                    )
                    
                    readr::write_tsv(doc, file = file.path(path,paste0("doc_",local.org,"_db_",db,".tsv")))
                    
                    message(
                        paste0(
                            "-> The Repeat Masker output file of '",
                            organism,
                            "' has been downloaded to '",
                            path,
                            "' and has been named '",
                            paste0(local.org, "_rm_", db, ".out.gz"),
                            "'."
                        )
                    )
                    please_cite_biomartr(mute_citation = mute_citation)
                    
                    return(file.path(
                        path,
                        paste0(local.org, "_rm_", db, ".out.gz")
                    ))
                } else {
                    message(
                        "Something went wrong when trying to download file: ",
                        download_url,
                        " ... Sometimes the internet connection isn't stable and re-running the function might help. Otherwise, could there be an issue with the firewall?"
                    )
                }
            }
        }
}

#' @title Retrieve Ensembl Biomart attributes for a query organism
#' @description In addition to the \code{\link{organismBM}} function, 
#' this function returns all available attributes that can be accessed through 
#' different marts and datasets for a given query organism.
#' @param organism a character string specifying the scientific name 
#' of a query organism.
#' @param update a logical value specifying whether or not the local 
#' listMart.txt, listDatasets.txt, and listAttributes_organism.txt files shall 
#' be updated by remote access to BioMart.
#' @param topic a character string specifying a topic (category) of attributes, 
#' e.g. \code{topic} = \code{"id"}.
#' @author Hajk-Georg Drost
#' @return a data.frame storing corresponding attribute names, description, 
#' datasets, and marts.
#' @details
#' For a given query organism, this function retrieves all available attributes 
#' that can be accessed through different marts and datasets.
#' 
#' Sometimes the same attribute names correspond to different datasets and 
#' marts causing problems when using \code{\link{getMarts}}. The approach 
#' introduced by this function provides (again) a organism centric way of 
#' accessing organism specific attributes.
#' 
#' The \code{topic} argument allows the user to search for specific attribute 
#' topics/categories for faster filtering.
#' @note
#' When you run this function for the first time, the data retrieval procedure 
#' will take some time, due to the remote access to BioMart. The corresponding 
#' result is then saved in a *.txt file within the \code{\link{tempdir}}
#' directory named "_biomart/listMarts.txt","_biomart/listDatasets.txt", and 
#' "_biomart/listAttributes_organism.txt", allowing subsequent queries to 
#' perform much faster.
#' @examples
#' \dontrun{ 
#' # search for attribute topic id
#' head(organismAttributes("Homo sapiens", topic = "id"), 20)
#' }
#' @references
#' \url{http://biomart.org/}
#' 
#' Mapping identifiers for the integration of genomic datasets with the
#' R/Bioconductor package biomaRt. Steffen Durinck, Paul T. Spellman, Ewan
#' Birney and Wolfgang Huber, Nature Protocols 4, 1184-1191 (2009).
#' 
#' BioMart and Bioconductor: a powerful link between biological databases and
#' microarray data analysis. Steffen Durinck, Yves Moreau, Arek Kasprzyk, Sean
#' Davis, Bart De Moor, Alvis Brazma and Wolfgang Huber, Bioinformatics 21,
#' 3439-3440 (2005).
#' @seealso \code{\link{organismFilters}}, \code{\link{organismBM}}, 
#' \code{\link{biomart}}, \code{\link[biomaRt]{listAttributes}}
#' @export

organismAttributes <-
    function(organism,
             update = FALSE,
             topic = NULL) {
        
    
    mart <- dataset <- NULL
    
    # retrieve all available marts for the organism of interest
    orgBM <- organismBM(organism = organism, update = update)
    message("\n")
    message("Starting retrieval of all available BioMart attributes for ", organism, " ...")
    
    orgMarts <- names(table(orgBM$mart))
    martList <-
        lapply(orgMarts, function(mart)
            dplyr::filter(orgBM, mart == mart))
    
    attrTXT <-
        paste0("listAttributes_", stringr::str_replace(organism, " ", "_"))
    
    if (!file.exists(file.path(tempdir(), "_biomart"))) {
        dir.create(file.path(tempdir(), "_biomart"))
    }
    
    if (!file.exists(file.path(tempdir(), "_biomart",
                               paste0(attrTXT, ".txt")))) {
        attrList <- lapply(martList, function(mart) {
            mart <- as.data.frame(mart)
            
            mart_tbl <-
                do.call(rbind, lapply(seq_len(nrow(mart)),
                                      function(dataset) {
                                              org_name_tmp <- unlist(stringr::str_split(mart$dataset[dataset], "_"))[1]
                                              if (!is.element(mart$dataset[dataset], c(paste0(org_name_tmp, "_structvar_som"), paste0(org_name_tmp, "_structvar")))) {
                                                  message("Processing mart ", mart$mart[dataset], " and dataset ", mart$dataset[dataset], " ...")
                                                  tryCatch({attr_tbl <- 
                                                      getAttributes(dataset = mart$dataset[dataset],
                                                                    mart    = mart$mart[dataset])
                                                  
                                                  datasetVec <-
                                                      rep(mart$dataset[dataset], nrow(attr_tbl))
                                                  
                                                  attr_tbl <-
                                                      dplyr::mutate(attr_tbl, dataset = datasetVec)
                                                  return(attr_tbl)
                                                  }, error = function(e) {message("No entries found ...")})
                                              }
                                          
                                      }))
            
            martVec <-
                rep(mart$mart[1], nrow(mart_tbl))
            mart_tbl <-
                dplyr::mutate(mart_tbl, mart = martVec)
            return(mart_tbl)
        })
        
        readr::write_tsv(
            do.call(rbind, attrList),
            file.path(tempdir(), "_biomart", paste0(attrTXT, ".txt")),
            col_names = TRUE
        )
    }
    
    attributeTable <-
        readr::read_tsv(
            file.path(tempdir(), "_biomart", paste0(attrTXT, ".txt")),
            col_names = TRUE,
            col_types = readr::cols(
                "name" = readr::col_character(),
                "description" = readr::col_character(),
                "dataset" = readr::col_character(),
                "mart" = readr::col_character()
            )
        )
    
    #         summ_attrTable <- dplyr::summarise(
    # dplyr::group_by(attributeTable, name),
    # description = names(table(description)),
    # mart        = names(table(mart)),
    # dataset     = names(table(dataset)))
    
    if (!is.null(topic)) {
        findTopic <-
            which(unlist(lapply(attributeTable$name, function(x)
                stringr::str_detect(x, topic))))
        
        if (dim(attributeTable[findTopic , ])[1] == 0)
            stop("Unfortunately the topic '", topic , "' could not be found.", call. = FALSE)
        
        return(attributeTable[findTopic , ])
        
    } else {
        return(attributeTable)
    }
}

#' @inherit getBioSet
#' @title CDS retrieval of multiple species
#' @description Main CDS retrieval function for a set of organism of interest.
#' By specifying the scientific names of the organisms of interest the corresponding fasta-files storing the CDS of the organisms of interest
#' will be downloaded and stored locally. CDS files can be retrieved from several databases.
#' @family getBioSet
#' @family cds
#' @export
getCDSSet <-
  function(db = "refseq",
           organisms,
           reference = FALSE,
           release = NULL,
           gunzip = TRUE,
           update = FALSE,
           path = "set_CDS") {
    getBioSet(db, organisms, set_type = "cds", reference,
              release = release, gunzip = gunzip, update = update,
              path = path)
}

#' @inherit getBioSet
#' @title Proteome retrieval of multiple species
#' @description Main proteome retrieval function for a set of organism of interest.
#' By specifying the scientific names of the organisms of interest the corresponding fasta-files storing the proteome of the organisms of interest
#' will be downloaded and stored locally. proteome files can be retrieved from several databases.
#' @param path a character string specifying the location (a folder) in which
#' the corresponding proteomes shall be stored. Default is \code{path} = \code{"set_proteomes"}.
#' @author Hajk-Georg Drost
#' @examples \dontrun{
#' # download the proteomes of three different species at the same time
#' #### Database: NCBI RefSeq
#' file_paths <- getProteomeSet(db = "refseq", organisms = c("Arabidopsis thaliana",
#'                                       "Arabidopsis lyrata",
#'                                        "Capsella rubella"))
#' # look at file paths
#' file_paths
#'
#' #### Database: NCBI Genbank
#' file_paths <- getProteomeSet(db = "genbank", organisms = c("Arabidopsis thaliana",
#'                                       "Arabidopsis lyrata",
#'                                        "Capsella rubella"))
#' # look at file paths
#' file_paths
#'
#' # download the proteomes of three different species at the same time
#' #### Database: ENSEMBL
#' file_paths <- getProteomeSet(db = "ensembl", organisms = c("Homo sapiens",
#'                                       "Mus musculus",
#'                                        "Caenorhabditis elegans"))
#' # look at file paths
#' file_paths
#'
#' # download the proteomes of three different species at the same time
#' #### Database: UniProt
#' file_paths <- getProteomeSet(db = "uniprot", organisms = c("Homo sapiens",
#'                                       "Mus musculus",
#'                                        "Caenorhabditis elegans"))
#' # look at file paths
#' file_paths
#' }
#'
#' @family getBioSet
#' @family proteome
#' @export
getProteomeSet <-
  function(db = "refseq",
           organisms,
           reference = FALSE,
           release = NULL,
           skip_bacteria = TRUE,
           gunzip = TRUE,
           update = FALSE,
           path = "set_proteomes",
           mute_citation = FALSE) {
    getBioSet(db,
              organisms,
              set_type = "proteome",
              reference = reference,
              release = release,
              skip_bacteria = skip_bacteria,
              gunzip = gunzip,
              update = update,
              path = path,
              mute_citation = mute_citation)
  }

#' @inherit getBioSet
#' @title GFF retrieval of multiple species
#' @description Main GFF retrieval function for a set of organism of interest.
#' By specifying the scientific names of the organisms of interest the corresponding fasta-files storing the GFF of the organisms of interest
#' will be downloaded and stored locally. GFF files can be retrieved from several databases.
#' @family getBioSet
#' @family gff
#' @export
getGFFSet <-
  function(db = "refseq",
           organisms,
           reference = FALSE,
           release = NULL,
           skip_bacteria = TRUE,
           gunzip = TRUE,
           remove_annotation_outliers = FALSE,
           update = FALSE,
           format = "gff3",
           path = "set_GFF",
           mute_citation = FALSE) {
    getBioSet(db,
              organisms,
              set_type = "gff",
              reference = reference,
              release = release,
              skip_bacteria = skip_bacteria,
              gunzip = gunzip,
              remove_annotation_outliers = remove_annotation_outliers,
              update = update,
              format = format,
              path = path,
              mute_citation = mute_citation)
  }

#' @inherit getBioSet
#' @title Genome Retrieval of multiple species
#' @description Main genome retrieval function for a set of organism of interest.
#' By specifying the scientific names of the organisms of interest the corresponding fasta-files storing the genome of the organisms of interest
#' will be downloaded and stored locally. Genome files can be retrieved from several databases.
#' @param path a character string specifying the location (a folder) in which
#' the corresponding genomes shall be stored. Default is
#' \code{path} = \code{"set_genomes"}.
#' @family getBioSet
#' @family genome
#' @export
getGenomeSet <-
  function(db = "refseq",
           organisms,
           reference = FALSE,
           release = NULL,
           skip_bacteria = TRUE,
           gunzip = TRUE,
           update = FALSE,
           path = "set_genomes",
           assembly_type = "toplevel",
           mute_citation = FALSE) {
    getBioSet(db,
              organisms,
              set_type = "genome",
              reference = reference,
              release = release,
              skip_bacteria = skip_bacteria,
              gunzip = gunzip,
              update = update,
              path = path,
              assembly_type = assembly_type,
              mute_citation = mute_citation)

  }

#' @inherit getBioSet
#' @title RNA Retrieval of multiple species
#' @description Main RNA retrieval function for a set of organism of interest.
#' By specifying the scientific names of the organisms of interest the corresponding fasta-files storing the RNA of the organisms of interest
#' will be downloaded and stored locally. RNA files can be retrieved from several databases.
#' @param path a character string specifying the location (a folder) in which
#' the corresponding RNAs shall be stored. Default is \code{path} = \code{"set_RNAs"}.
#' @family getBioSet
#' @family rna
#' @export
getRNASet <-
  function(db = "refseq",
           organisms,
           reference = FALSE,
           release = NULL,
           skip_bacteria = TRUE,
           gunzip = TRUE,
           update = FALSE,
           path = "set_RNAs",
           mute_citation = FALSE) {
    getBioSet(db,
              organisms,
              set_type = "genome",
              reference = reference,
              release = release,
              skip_bacteria = skip_bacteria,
              gunzip = gunzip,
              update = update,
              path = path,
              mute_citation = mute_citation)
  }

#' @inherit getBioSet
#' @title Retrieve a Collection: Genome, Proteome, CDS, RNA, GFF, Repeat Masker, AssemblyStats of multiple species
#' @description Main collection retrieval function for an organism of interest.
#' By specifying the scientific name of an organism of interest a collection consisting of
#' the genome file, proteome file, CDS file, RNA file, GFF file, Repeat Masker file, AssemblyStats
#' file of the organism of interest
#' can be downloaded and stored locally. Collections can be retrieved from
#' several databases.
#' @param path a character string specifying the location (a folder) in which
#' the corresponding collection shall be stored. Default is
#' \code{path} = \code{file.path("_db_downloads","collections")}.
#' @family getBioSet
#' @family collection
#' @export
getCollectionSet <-
  function(db = "refseq",
           organisms,
           reference = FALSE,
           release = NULL,
           skip_bacteria = TRUE,
           gunzip = TRUE,
           update = FALSE,
           remove_annotation_outliers = TRUE,
           path = "set_collections",
           mute_citation = FALSE) {
    getBioSet(db,
              organisms,
              set_type = "collection",
              reference = reference,
              release = release,
              skip_bacteria = skip_bacteria,
              gunzip = gunzip,
              update = update,
              remove_annotation_outliers = remove_annotation_outliers,
              path = path,
              mute_citation = mute_citation)
}

context("Test: getBioSet()")

test_that("The getGFFSet() interface works properly for ensembl (repeating command)..",{
  path <- file.path(tempdir(), "GFF_set")
  out1 <- getGFFSet("ensembl",
                    organisms = c("Saccharomyces cerevisiae", "Escherichia coli"),
                    path = path, mute_citation = T)
  out2 <- getGFFSet("ensembl",
                    organisms = c("Saccharomyces cerevisiae", "Escherichia coli"),
                    path = path, mute_citation = T)
  expect_equal(length(out1), 2)
  expect_equal(as.character(out1), as.character(out2))
})

refseq_genbank_download_post_processing <- function(info, organism, db, path,
                                                    gunzip,
                                                    remove_annotation_outliers,
                                                    format,
                                                    analyse_genome = FALSE,
                                                    mute_citation = FALSE) {
  if ((is.logical(info[1]) && !info) || (info[1] == "Not available"))
    return("Not available")

  local.org <- info$local.org
  local_file <- info$local_file
  download_url <- info$download_url
  FoundOrganism <- info$FoundOrganism
  # Save Annotation information
  doc <- tibble::tibble(
    file_name = local_file,
    organism  = organism,
    url       = download_url,
    database  = db,
    path      = path,
    refseq_category = FoundOrganism$refseq_category,
    assembly_accession = FoundOrganism$assembly_accession,
    bioproject = FoundOrganism$bioproject,
    biosample = FoundOrganism$biosample,
    taxid = FoundOrganism$taxid,
    infraspecific_name = FoundOrganism$infraspecific_name,
    version_status = FoundOrganism$version_status,
    release_type = FoundOrganism$release_type,
    genome_rep = FoundOrganism$genome_rep,
    seq_rel_date = FoundOrganism$seq_rel_date,
    submitter = FoundOrganism$submitter
  )
  docFile(doc = doc)

  if (analyse_genome) {
    genome_summary_stats <- summary_genome(file = local_file, organism = organism)
    genome_analysis_path <- file.path(path, paste0("doc_",local.org,"_db_",db,"_summary_statistics.tsv"))
    readr::write_tsv(genome_summary_stats, file = genome_analysis_path)
  }


  doc_file_path <- file.path(path,
                             paste0("doc_", local.org, "_db_", db, ".tsv"))
  readr::write_tsv(doc, file = doc_file_path)
  gunzip_and_check(local_file, gunzip, remove_annotation_outliers, format,
                   mute_citation = mute_citation)
}

tidy_name <- function(x, split_on = "_") {
    new_spec_names <- unlist(lapply(x, function(y) {
        split_str <- unlist(stringr::str_split(y, split_on))
            return(paste0(
                stringr::str_to_upper(stringr::str_sub(split_str[1], 1, 1)),
                stringr::str_sub(split_str[1], 2, nchar(split_str[1])),
                stringr::str_to_upper(stringr::str_sub(split_str[2],1,1)),
                stringr::str_sub(split_str[2], 2, nchar(split_str[2])),
                collapse = ""
            ))
    }))

    return(new_spec_names)
}

tidy_name2 <- function(x) {
  tidy_name(x, split_on = " ")
}

#' @title Transform 1st character of a string to upper case
#' @description Helper function to only transform 1st character of a string
#' to upper case.
#' @param string character string that shall be transformed.
#' @author Hajk-Georg Drost
#' @noRd
toupper_first_char <- function(string) {
  return(stringr::str_c(stringr::str_to_upper(stringr::str_sub(string,1,1)),
                        stringr::str_sub(string,2,nchar(string)),
                        collapse = ""))
}

ensembl_fix_wrong_naming <- function(organism) {
  organism <-
    stringr::str_replace_all(organism, " sp ", " sp. ")
  organism <-
    stringr::str_replace_all(organism, " pv ", " pv. ")
  organism <-
    stringr::str_replace_all(organism, " str ", " str. ")
  organism <-
    stringr::str_replace_all(organism, " subsp ", " subsp. ")
  organism <-
    stringr::str_replace_all(organism, "\\(", "")
  organism <-
    stringr::str_replace_all(organism, "\\)", "")
}

ensembl_proper_organism_name <- function(ensembl_summary) {
  new.organism <- ensembl_summary$name[1]
  new.organism <-
    paste0(
      stringr::str_to_upper(stringr::str_sub(new.organism, 1, 1)),
      stringr::str_sub(new.organism, 2, nchar(new.organism))
    )
  new.organism <- ensembl_fix_wrong_naming(new.organism)
}

lower_cap_underscore_organism_name <- function(organism) {
  stringr::str_to_lower(stringr::str_replace_all(organism, " ", "_"))
}

#' Generic Bio data set extractor
#'
#' Usually you want to use one of the specific set extractors
#' @inheritParams getBio
#' @param organisms a character vector storing the names of the organisms than shall be retrieved.
#' There are three available options to characterize an organism:
#' @param path character, default location is paste0("set_", toupper(set_type))
#' @param set_type the biological sequence type that shall be retrieved. 
#' Available options are
#' \itemize{
#' \item \code{set_type = "genome"}
#' \item \code{set_type = "proteome"}
#' \item \code{set_type = "cds"}
#' \item \code{set_type = "gff"}
#' \item \code{set_type = "rna"}
#' \item \code{set_type = "collection"} (all the others)
#' }
#' @author Hajk-Georg Drost
#' @details Internally this function loads the the overview.txt file from NCBI:
#'
#'  refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/
#'
#'  genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/
#'
#' and creates a directory 'set_CDSs' to store
#' the CDSs of interest as fasta files for future processing.
#' In case the corresponding fasta file already exists within the
#' 'set_CDSs' folder and is accessible within the workspace,
#' no download process will be performed.
#' @return File path to downloaded genomes
#' (names are identifiers: 'new' (file was downloaded now),
#' 'old' files did already exist)
#' @examples \dontrun{
#' getBioSet("refseq", organisms = c("Arabidopsis thaliana",
#'                                   "Arabidopsis lyrata",
#'                                   "Capsella rubella"), 
#'                                   set_type = "cds")
#' }
#' @family getBioSet
#' @seealso \code{\link{getBio}}
#' @export
getBioSet <- function(db = "refseq",
                      organisms,
                      set_type,
                      reference = FALSE,
                      release = NULL,
                      gunzip = TRUE,
                      update = FALSE,
                      skip_bacteria = TRUE,
                      path = paste0("set_", toupper(set_type)),
                      remove_annotation_outliers = FALSE,
                      assembly_type = "toplevel",
                      format = "gff3",
                      mute_citation = FALSE) {
  stopifnot(length(organisms) > 0)
  set_upper <- toupper(set_type)
  message(
    "Starting ", set_upper, " retrieval of the following ", set_upper, "s: ",
    paste0(organisms, collapse = ", "),
    " ..."
  )

  if (!dir.exists(path)) {
    message("Generating folder ", path, " ...")
    dir.create(path, recursive = TRUE, showWarnings = FALSE)
  }

  if (!dir.exists(file.path(path, "documentation")))
    dir.create(file.path(path, "documentation"), showWarnings = FALSE)

  message("\n")
  organisms <- check_if_tidy_exists(organisms, path, update, set_type)
  to_run <- names(organisms) == "new"
  if (any(to_run)) {
    message("\n")
    paths <- c()
    for (organism in organisms[to_run]) {
      out <-  getBio(db, organism, type = set_type, reference = reference,
                     release = release, gunzip = gunzip, update = update,
                     skip_bacteria = skip_bacteria, path = path,
                     remove_annotation_outliers = remove_annotation_outliers,
                     assembly_type = assembly_type, format = format,
                     mute_citation = TRUE)
      paths <- c(paths, out)
      message("\n")
    }
    make_summary_doc_set(path)
    mark_as_complete_set(organisms, path, paths, set_type)
  }

  dt <- data.table::fread(done_organisms_bioset_file(path, set_type))
  paths <- dt[data.table::chmatch(organism, tidy_name2(organisms)),]$file
  names(paths) <- names(organisms)

  please_cite_biomartr(mute_citation = mute_citation)
  return(paths)
}

#' A wrapper to all bio getters, selected with 'type' argument
#'
#' @param db a character string specifying the database from which the genome
#' shall be retrieved:
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' \item \code{db = "ensembl"}
#' }
#' @param organism Organism selector id,
#' there are three options to characterize an organism:
#' \itemize{
#' \item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
#' \item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
#' \item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
#' }
#' @param type biological sequence type. (alternatives are: genome, gff, cds, rna, proteome, collection (all the others))
#' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.
#' @param release a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
#' that the most recent database version is used. \code{release = 75} would for human would give the stable
#' GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
#' if the standard format before that.
#' @param gunzip a logical, indicating whether or not files should be unzipped.
#' @param update logical, default FALSE. Updated backend cached files needed.
#' Usually keep this false, to make ut run much faster. Only set to TRUE, if you
#' believe you cache is outdated (Species only exist in newest release etc)
#' @param skip_bacteria Due to its enormous dataset size (> 700MB as of July 2023),
#' the bacterial summary file will not be loaded by default anymore. If users
#' wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
#' bacterial summary file will be downloaded.
#' @param path character, default location is paste0("set_", toupper(type))
#' @param remove_annotation_outliers shall outlier lines be removed from the input \code{annotation_file}?
#' If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
#' for further exploration.
#' @param analyse_genome logical, default FALSE. If TRUE, get general genome statistics like
#' gc content etc. For more details, see ?summary_genome
#' @param assembly_type a character, default "toplevel". id type of assembly, either "toplevel" or "primary_assembly" usually.
#' @param format "gff3", alternative "gtf" for ensembl.
#' @param mute_citation logical, default FALSE, indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @details Internally this function loads the the overview.txt file from NCBI:
#'
#'  refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/
#'
#'  genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/
#'
#' and creates a directory relative to file type, if you get fasta genomes it will be
#' _ncbi_downloads/genomes'.
#' In case the corresponding fasta file already exists within the
#' '_ncbi_downloads/genomes' folder and is accessible within the workspace,
#' no download process will be performed.
#' For other file types the same rule applies.
#' @family getBio
#' @return File path to downloaded genome.
getBio <- function(db = "refseq",
                   organism,
                   type,
                   reference = FALSE,
                   release = NULL,
                   gunzip = FALSE,
                   update = FALSE,
                   skip_bacteria = TRUE,
                   path = paste0("set_", toupper(type)),
                   remove_annotation_outliers = FALSE,
                   analyse_genome = FALSE,
                   assembly_type = "toplevel",
                   format = "gff3",
                   mute_citation = FALSE) {
  if (type == "genome") {
    getGenome(db, organism, reference = reference,
              skip_bacteria = skip_bacteria, release = release,
              gunzip = gunzip, path = path, assembly_type = assembly_type,
              mute_citation = mute_citation, analyse_genome = analyse_genome)
  } else if (type == "gff") {
    getGFF(db, organism, reference = reference,
           skip_bacteria = skip_bacteria, release = release,
           gunzip = gunzip,
           remove_annotation_outliers = remove_annotation_outliers,
           path = path,
           mute_citation = mute_citation, format = format)
  } else if (type == "cds") {
    getCDS(db, organism, reference = reference,
           skip_bacteria = skip_bacteria,
           release = release,
           gunzip = gunzip,
           path = path,
           mute_citation = mute_citation)
  } else if (type == "rna") {
    getRNA(db, organism, reference = reference,
           skip_bacteria = skip_bacteria,
           release = release,
           gunzip = gunzip,
           path = path,
           mute_citation = mute_citation)
  } else if (type %in% c("protein", "proteome")) {
    getProteome(db, organism, reference = reference,
                skip_bacteria = skip_bacteria,
                release = release,
                gunzip = gunzip,
                path = path,
                mute_citation = mute_citation)
  } else if (type == "collection") {
    getCollection(db, organism, reference = reference,
                  skip_bacteria = skip_bacteria,
                  release = release,
                  gunzip = gunzip,
                  remove_annotation_outliers = FALSE,
                  path = path,
                  mute_citation = mute_citation)
  } else stop("invalid bio getter type selected")
}

make_summary_doc_set <- function(path, out = file.path(path, "documentation", paste0(basename(path), "_summary.csv"))) {
  meta_files <- list.files(path)
  meta_files <- meta_files[stringr::str_detect(meta_files, "doc_")]
  if (length(meta_files) == 0) return(invisible(NULL))

  file.rename(from = file.path(path, meta_files), to = file.path(path, "documentation", meta_files))

  doc_tsv_files <- file.path(path,"documentation", meta_files[stringr::str_detect(meta_files, "[.]tsv")])

  summary_log <- data.table::rbindlist(lapply(doc_tsv_files, function(data) {
    suppressMessages(data.table::fread(data))
  }))

  data.table::fwrite(summary_log, out)
  message("A summary file (which can be used as supplementary information file in publications)",
  " containig retrieval information for all species has been stored at '", out, "'.")
  return(invisible(NULL))
}

check_if_tidy_exists <- function(organisms, path, update, set_type) {
  # Load done csv if existing
  done_org_list <- done_organisms_bioset_file(path, set_type)
  names(organisms) <- rep("new", length(organisms))
  if (file.exists(done_org_list)) {
    done_org_list <- data.table::fread(done_org_list)

    organisms_short <- tidy_name2(organisms)
    # If update, remove all
    done_orgs <- done_org_list$organism %in% organisms_short
    if (any(done_orgs)) {
      if (update) {
        file.remove(done_org_list[done_orgs,]$file)
      } else {
        match <- organisms_short %in% done_org_list$organism

        names(organisms)[match] <- "exist"
      }
    }
    if (all(done_orgs) & !update) {
      message("- Skipping computation of set (all species already done),",
              " set update to TRUE if you want to run again")
    }
  }

  return(organisms)
}

mark_as_complete_set <- function(organisms, path, new_files, set_type) {
  # Load done csv if existing
  stopifnot(names(organisms) %in% c("new", "exist"))

  # If update, remove all
  to_be_appended <- names(organisms) == "new"
  if (any(to_be_appended)) {
    done_org_list_file <- done_organisms_bioset_file(path, set_type)
    if (file.exists(done_org_list_file)) {
      done_org_list <- data.table::fread(done_org_list)
    } else done_org_list <- data.table::data.table()
    organisms_short <- tidy_name2(organisms)
    append <- data.table::data.table(organism = organisms_short[to_be_appended],
                                     file = new_files[to_be_appended])
    new_list <- data.table::rbindlist(list(done_org_list, append))
    data.table::fwrite(new_list, done_org_list_file)
    message("- List of completed organisms updated in: ", done_org_list_file)
  }
  return(invisible(NULL))
}

done_organisms_bioset_file <- function(path, set_type) {
  file.path(path, paste0(set_type, "_done_organisms.csv"))
}

# All these will be removed


#' @title Annotation file retrieval from ENSEMBLGENOMES
#' @description Helper function for retrieving GFF files
#' of a particular organism of interest from the ENSEMBLGENOMES ftp server.
#' This function will be deprecated!
#' @param organism scientific name of the query organism
#' @param type type of biological sequence
#' @param id.type ENSEMBLGENOMES id type
#' @param release the ENSEMBLGENOMES release. Default is \code{release = NULL} meaning that the current (most recent) version is used.
#' @param path where shall file be saved?
#' @author Hajk-Georg Drost
#' @noRd

getENSEMBLGENOMES.Annotation <- function(organism, type = "dna", release = NULL, path) {
  getENSEMBL.Annotation(organism, type = type, path = path, release = release)
}

#' This will be deprecated, use getENSEMBL.gtf.
#' @inheritParams getENSEMBL.gtf
#' @author Hajk-Georg Drost
#' @noRd

getENSEMBLGENOMES.gtf <- function(organism, type = "dna", path, release = NULL) {
  getENSEMBL.gtf(organism, type = type, path = path, release = release)
}

#' @title Helper function for retrieving biological sequence files from
#' ENSEMBLGENOMES.
#' @description This function downloads gff files of query organisms from
#' ENSEMBLGENOMES. This will be deprecated, use getENSEMBL.Seq.
#' @inheritParams getENSEMBL.Seq
#' @author Hajk-Georg Drost
#' @noRd

getENSEMBLGENOMES.Seq <- function(organism, release = NULL, type = "dna",
                                  id.type = "toplevel", path) {
  getENSEMBL.Seq(organism, type = type, id.type = id.type, release = release, path)
}

#' @title Retrieve ENSEMBLGENOMES info file
#' @description Retrieve species and genome information from
#' http://rest.ensemblgenomes.org/info/species?content-type=application/json/.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' info.file <- getENSEMBLGENOMESInfo()
#' info.file
#' }
#' @export
getENSEMBLGENOMESInfo <- function() {
  ENSEMBLGENOMESInfoTable <- get.ensembl.info(update = TRUE)
  return(ENSEMBLGENOMESInfoTable)
}

#' @title Download all elements of an NCBI databse
#' @description The \code{\link{download.database}} functions allows users to
#' retrieve individual packages of a NCBI database. This function is designed to
#' retrieve the entire database selected by the users (hence all packages 
#' corresponding to this database).
#' @param db a character string specifying the database that shall be downloaded
#' (selected from \code{\link{listDatabases}}).
#' @param path a character string specifying the location (a folder) in which
#' the corresponding
#' database shall be stored. In case this folder does not exist yet,
#' it will be created.
#' @author Hajk-Georg Drost
#' @examples 
#' \dontrun{
#' # search for available NCBI databases
#'   listNCBIDatabases(db = "all")
#' # choose database NCBI nr and download compelete database
#'   download.database.all(db = "nr", path = "nr")
#' }
#' @seealso \code{\link{download.database}}, \code{\link{listNCBIDatabases}}
#' @return A character vector storing the file paths of the downloaded databases.
#' @export

download.database.all <- function(db, path = NULL) {
    
    db_chunks <- listNCBIDatabases(db = db)
    db_chunks <- db_chunks[-which(stringr::str_detect(db_chunks, "[.]json"))]
    
    message("Starting download of the files: ",
            paste0(db_chunks, collapse = ", "),
            " ...")
    message("This download process may take a while due to the large size of the individual data chunks ...")
    
    if (is.null(path)) {
        path <- db
    }
    
    dld_paths <- list(length(db_chunks))
    
    for (i in seq_len(length(db_chunks))) {
        dld_paths[i] <- list(download.database(db = db_chunks[i], path = path))
    }
    
    corrupt_md5 <- any(unlist(lapply(dld_paths, is.logical)))
    which_corrupter_md5 <- which(unlist(lapply(dld_paths, is.logical)))
    
    if (corrupt_md5)
        warning("The file(s) ", paste0(db_chunks[which_corrupter_md5], collapse = ", "), " had corrupted md5 check sum(s). You can simply re-run this function to re-download corrupted files.")
    
    message("Download process is finished and files are stored in '",
            path, "'.")
    
    if (corrupt_md5)
        return(unlist(dld_paths[-which_corrupter_md5]))
    
    if (!corrupt_md5)
        return(unlist(dld_paths))
}

#' @title List available metagenomes on NCBI Genbank
#' @description List available metagenomes on NCBI genbank. NCBI genbank 
#' allows users to download entire metagenomes of several metagenome projects. 
#' This function lists all available metagenomes that can then be downloaded via
#' \code{\link{getMetaGenomes}}.
#' @param details a boolean value specifying whether only the scientific names 
#' of stored metagenomes shall be returned (\code{details = FALSE}) or all 
#' information such as "organism_name","bioproject", 
#' etc (\code{details = TRUE}).
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # retrieve available metagenome projects at NCBI Genbank
#' listMetaGenomes()
#' # retrieve detailed information on available metagenome projects 
#' # at NCBI Genbank
#' listMetaGenomes(details = TRUE)
#' }
#' @seealso \code{\link{getMetaGenomes}}, \code{\link{getMetaGenomeSummary}}
#' @export
 
listMetaGenomes <- function(details = FALSE) {

    metagenome.summary <- getMetaGenomeSummary()    
    
    if (!details)
        return(unique(metagenome.summary$organism_name))
    
    if (details)
        return(metagenome.summary)
}

#' @title Perform Meta-Genome Retrieval
#' @description Download genomes, proteomes, cds, gff, rna, or assembly stats
#' files of all species within a kingdom of life.
#' @param db a character string specifying the database from which the genome
#' shall be retrieved:
#'
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' \item \code{db = "emsembl"}
#' }
#' @param kingdom a character string specifying the kingdom of the organisms
#' of interest, e.g.
#'
#' \itemize{
#' \item For \code{NCBI RefSeq}:
#' \itemize{
#' \item \code{kingdom = "archaea"}
#' \item \code{kingdom = "bacteria"}
#' \item \code{kingdom = "fungi"}
#' \item \code{kingdom = "invertebrate"}
#' \item \code{kingdom = "plant"}
#' \item \code{kingdom = "protozoa"}
#' \item \code{kingdom = "viral"}
#' \item \code{kingdom = "vertebrate_mammalian"}
#' \item \code{kingdom = "vertebrate_other"}
#' }
#' \item For \code{NCBI Genbank}:
#' \itemize{
#' \item \code{kingdom = "archaea"}
#' \item \code{kingdom = "bacteria"}
#' \item \code{kingdom = "fungi"}
#' \item \code{kingdom = "invertebrate"}
#' \item \code{kingdom = "plant"}
#' \item \code{kingdom = "protozoa"}
#' \item \code{kingdom = "vertebrate_mammalian"}
#' \item \code{kingdom = "vertebrate_other"}
#' }
#' \item For \code{ENSEMBL}:
#' \itemize{
#' \item \code{kingdom = "EnsemblVertebrates"}
#' \item \code{kingdom = "EnsemblPlants"}
#' \item \code{kingdom = "EnsemblFungi"}
#' \item \code{kingdom = "EnsemblMetazoa"}
#' \item \code{kingdom = "EnsemblBacteria"}
#' \item \code{kingdom = "EnsemblProtists"}
#' }
#' }
#'
#' Available kingdoms can be retrieved with \code{\link{getKingdoms}}.
#' @param group only species belonging to this subgroup will be downloaded.
#' Groups can be retrieved with \code{\link{getGroups}}.
#' @param type type of sequences that shall be retrieved. Options are:
#'
#' \itemize{
#'  \item \code{type = "genome"} :
#'  (for genome assembly retrieval; see also \code{\link{getGenome}}),
#'  \item \code{type = "proteome"} :
#'  (for proteome retrieval; see also \code{\link{getProteome}}),
#'  \item \code{type = "cds"} :
#'  (for coding sequence retrieval; see also \code{\link{getCDS}}),
#'  \item \code{type = "gff"} :
#' (for annotation file retrieval in gff format; see also \code{\link{getGFF}}),
#' \item \code{type = "gtf"} :
#' (for annotation file retrieval in gtf format (only for ensembl and
#'  ensemblgenomes); see also \code{\link{getGTF}})
#'  \item \code{type = "rna"} :
#'  (for RNA file retrieval in fasta format; see also \code{\link{getRNA}}),
#'  \item \code{type = "rm"} :
#'  (for Repeat Masker output file retrieval; see also
#'  \code{\link{getRepeatMasker}}),
#'  \item \code{type = "assemblystats"} :
#'  (for genome assembly quality stats file retrieval;
#'  see also \code{\link{getAssemblyStats}}).
#'  }
#' @param restart_at_last a logical value indicating whether or not \code{meta.retrieval} should pick up at the last species when re-running the function.
#' \itemize{
#' \item If \code{restart_at_last = TRUE} (Default) then \code{meta.retrieval} will skip all organisms that are already present in the folder
#' and will start downloading all remaining species. However, this way \code{meta.wretrieval} will not be able to check whether
#' already downloaded organism files are corrupted or not by checking the md5 checksum.
#' \item If \code{restart_at_last = FALSE} then \code{meta.retrieval} will start from the beginning and crawl through already downloaded
#' organism files and check whether already downloaded organism files are corrupted or not by checking the md5 checksum.
#' After checking existing files the function will start downloading all remaining organisms.
#' }
#' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database
#' as either a reference genome or a representative genome. Options are:
#' \itemize{
#' \item \code{reference = FALSE} (Default): all organisms (reference, representative, and non-representative genomes) are downloaded.
#' \item \code{reference = TRUE}: organisms that are downloaded must be either a reference or representative genome. Thus, most genomes which are usually non-reference genomes
#' will not be downloaded.
#' }
#' @param combine just in case \code{type = "assemblystats"} is specified, shall
#' assemby stats of individual species be imported and combined to a
#' \code{\link{data.frame}}?
#' @param path path to the folder in which downloaded genomes shall be stored.
#' By default the kingdom name is used to name the output folder.
#' @author Hajk-Georg Drost
#' @details This function aims to perform bulk retrieval of the genomes,
#' proteomes, cds, etc. of species that belong to the same kingdom of life or
#' to the same subgroup.
#' @examples
#' \dontrun{
#' # get all available kingdoms for refseq
#' getKingdoms(db = "refseq")
#' # download all vertebrate genomes from refseq
#' meta.retrieval(kingdom = "vertebrate_mammalian",
#'                db = "refseq",
#'                type = "genome")
#'
#' # get all available kingdoms for genbank
#' getKingdoms(db = "genbank")
#' # download all vertebrate genomes from genbank
#' meta.retrieval(kingdom = "vertebrate_mammalian",
#'                db = "genbank",
#'                type = "genome")
#'
#'
#' # In case users do not wish to retrieve genomes from an entire kingdom,
#' # but rather from a subgoup (e.g. from species belonging to the
#' # Gammaproteobacteria class, a subgroup of the bacteria kingdom),
#' # they can use the following workflow"
#' # First, users can again consult the getKingdoms() function to retrieve
#' # kingdom information.
#' getKingdoms(db = "refseq")
#'
#' # In this example, we will choose the bacteria kingdom.
#' # Now, the getGroups() function allows users to obtain available
#' # subgroups of the bacteria kingdom.
#' getGroups(db = "refseq", kingdom = "bacteria")
#'
#' # Now we choose the group Gammaproteobacteria and specify
#' # the group argument in the meta.retrieval() function
#' meta.retrieval(kingdom = "bacteria",
#'    roup = "Gammaproteobacteria",
#'    db = "refseq",
#'    type = "genome")
#' }
#' @family meta_retrival
#' @return a character vector storing the file paths of the retrieved files.
#' @export
meta.retrieval <- function(db         = "refseq",
                           kingdom,
                           group = NULL,
                           type       = "genome",
                           restart_at_last = TRUE,
                           reference  = FALSE,
                           combine    = FALSE,
                           path = NULL) {

    division <- subgroup <- NULL

    subfolders <- getKingdoms(db = db)

    if (!is.element(kingdom, subfolders))
        stop(paste0(
            "Please select a valid kingdom: ",
            paste0(subfolders, collapse = ", ")
        ), call. = FALSE)

    if (!is.null(group))
        if (!is.element(group, getGroups(kingdom = kingdom, db = db)))
            stop(
                "Please specify a group that is supported by getGroups().
                Your specification '",
                group,
                "' does not exist in getGroups(kingdom = '",
                kingdom,
                "', db = '",
                db,
                "'). Maybe you used a different db argument in getGroups()?",
                call. = FALSE
            )

    if (!is.element(type,
                    c("genome", "proteome", "CDS","cds", "gff",
                      "rna", "assemblystats", "gtf", "rm")))
        stop(
        "Please choose either type: type = 'genome', type = 'proteome',
        type = 'CDS', type = 'gff', type = 'gtf',
        type = 'rna', type = 'rm', or type = 'assemblystats'.",
            call. = FALSE
        )

    if (!is.element(db, c("refseq", "genbank", "ensembl")))
        stop(
            "Please select einter db = 'refseq', db = 'genbank', or
            db = 'ensembl'.",
            call. = FALSE
        )

    if ((stringr::str_to_upper(type) == "CDS") && (db == "genbank"))
        stop("Genbank does not store CDS data. Please choose 'db = 'refseq''.",
             call. = FALSE)

    if ((type == "gtf") && (is.element(db, c("genbank", "refseq"))))
            stop("GTF files are only available for type = 'ensembl' and type = 'ensemblgebomes'.")

    if (type == "assemblystats" &&
        !is.element(db, c("refseq", "genbank")))
        stop(
            "Unfortunately, assembly stats files are only available for
            db = 'refseq' and db = 'genbank'.",
            call. = FALSE
        )

    if (combine && type != "assemblystats")
        stop(
            "Only option type = 'assemblystats' can use combine = TRUE. Please
            specify: type = 'assemblystats' and combine = TRUE.",
            call. = FALSE
        )

    if ((type == "rm") && (!is.element(db, c("refseq", "genbank"))))
        stop("Repeat Masker output files can only be retrieved from 'refseq'",
             " or 'genbank'.", call. = FALSE)


    if (is.element(db, c("refseq", "genbank"))) {
        if (is.null(group)) {
            assembly.summary.file <-
                getSummaryFile(db = db, kingdom = kingdom)
            #assembly.summary.file <-
            #    dplyr::mutate(assembly.summary.file,
            #    organism_name = clean.str.brackets(organism_name))
            FinalOrganisms <-
                unique(assembly.summary.file$organism_name)
            #organism_name <- NULL

        }

        if (!is.null(group)) {
            groups.selection <-
                listGroups(kingdom = kingdom,
                           db = db,
                           details = TRUE)
            groups.selection <-
                dplyr::filter(groups.selection, subgroup %in% group)
            FinalOrganisms <- unique(groups.selection$organism_name)
        }
    }

    if (db %in% c("ensembl", "ensemblgenomes")) {
        ensembl_names <- getKingdoms(db = db)
        summary.file <- get.ensembl.info(division = names(ensembl_names[which(ensembl_names == kingdom)]))
        FinalOrganisms <- unique(summary.file$name)
        FinalOrganisms <-
            stringr::str_replace_all(FinalOrganisms, "_", " ")
        stringr::str_sub(FinalOrganisms, 1, 1) <-
            stringr::str_to_upper(stringr::str_sub(FinalOrganisms, 1, 1))
    }


    if (is.null(group))
        message(paste0(
            "Starting meta retrieval of all ",
            type,
            " files for kingdom: ",
            kingdom,
            " from database: ", db,
            "."
        ))
    if (!is.null(group))
        message(
            paste0(
                "Starting meta retrieval of all ",
                type,
                " files within kingdom '",
                kingdom,
                "' and subgroup '",
                group,"' ",
                " from database: ",
                db,
                "."
            )
        )
    message("\n")

    if (!is.null(path)) {
        if (!file.exists(path)) {
            message("Generating folder ", path, " ...")
            dir.create(path, recursive = TRUE)
        }

        if (!file.exists(file.path(path, "documentation")))
            dir.create(file.path(path, "documentation"))
    } else {
        if (!file.exists(kingdom)) {
            message("Generating folder ", kingdom, " ...")
            dir.create(kingdom, recursive = TRUE)
        }
        if (!file.exists(file.path(kingdom, "documentation")))
            dir.create(file.path(kingdom, "documentation"))
    }

    paths <- vector("character", length(FinalOrganisms))


    if (is.element(db, c("refseq", "genbank"))) {
        if (type == "genome") internal_type <- "genomic"
        if (type == "proteome") internal_type <- "protein"
        if (stringr::str_to_upper(type) == "CDS") internal_type <- "cds"
        if (type == "gff") internal_type <- "genomic"
        if (type == "gtf") internal_type <- db
        if (type == "rna") internal_type <- "rna"
        if (type == "rm") internal_type <- "rm"
        if (type == "assemblystats") internal_type <- "assembly"


        if (!is.null(path)) {
            .existingOrgs <- existingOrganisms(path = path, .type = internal_type)
        } else {
            .existingOrgs <- existingOrganisms(path = kingdom, .type = internal_type)
        }

    }


    if (db == "ensembl") {

        if (type == "genome") internal_type <- "dna"
        if (type == "proteome") internal_type <- "pep"
        if (stringr::str_to_upper(type) == "CDS") internal_type <- "cds"
        if (type == "gff") internal_type <- "ensembl"
        if (type == "gtf") internal_type <- "ensembl"
        if (type == "rna") internal_type <- "ncrna"

        if (!is.null(path)) {
            .existingOrgs <- existingOrganisms_ensembl(path = path, .type = internal_type)
        } else {
            .existingOrgs <- existingOrganisms_ensembl(path = kingdom, .type = internal_type)
        }
    }

    if (db == "ensemblgenomes") {
        if (type == "genome") internal_type <- "dna"
        if (type == "proteome") internal_type <- "pep"
        if (stringr::str_to_upper(type) == "CDS") internal_type <- "cds"
        if (type == "gff") internal_type <- "ensemblgenomes"
        if (type == "gtf") internal_type <- "ensemblgenomes"
        if (type == "rna") internal_type <- "ncrna"

        if (!is.null(path)) {
            .existingOrgs <- existingOrganisms_ensembl(path = path, .type = internal_type)
        } else {
            .existingOrgs <- existingOrganisms_ensembl(path = kingdom, .type = internal_type)
        }

    }

    if (length(.existingOrgs) > 0) {
        if (restart_at_last) {
            FinalOrganisms <- dplyr::setdiff(FinalOrganisms, .existingOrgs)
            if (length(FinalOrganisms) > 0) {
                message("Skipping already downloaded species: ", paste0(.existingOrgs, collapse = ", "))
                message("\n")
            }
        }
    }

    if (length(FinalOrganisms) > 0) {

        if (type == "genome") {
            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getGenome(db        = db,
                                          organism  = FinalOrganisms[i],
                                          reference = reference,
                                          path      = kingdom,
                                          mute_citation = TRUE)
                    message("\n")
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getGenome(db       = db,
                                          organism = FinalOrganisms[i],
                                          reference = reference,
                                          path     = path,
                                          mute_citation = TRUE)
                    message("\n")
                }
            }
        }

        if (type == "proteome") {
            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getProteome(db       = db,
                                            organism = FinalOrganisms[i],
                                            reference = reference,
                                            path     = kingdom,
                                            mute_citation = TRUE)
                    message("\n")
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getProteome(db       = db,
                                            organism = FinalOrganisms[i],
                                            reference = reference,
                                            path     = path,
                                            mute_citation = TRUE)
                    message("\n")
                }
            }
        }

        if (stringr::str_to_upper(type) == "CDS") {
            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getCDS(db       = db,
                                       organism = FinalOrganisms[i],
                                       reference = reference,
                                       path     = kingdom,
                                       mute_citation = TRUE)
                    message("\n")
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getCDS(db       = db,
                                       organism = FinalOrganisms[i],
                                       reference = reference,
                                       path     = path,
                                       mute_citation = TRUE)
                    message("\n")
                }
            }
        }

        if (type == "gff") {
            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getGFF(db       = db,
                                       organism = FinalOrganisms[i],
                                       reference = reference,
                                       path     = kingdom,
                                       mute_citation = TRUE)
                    message("\n")
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getGFF(db       = db,
                                       organism = FinalOrganisms[i],
                                       reference = reference,
                                       path     = path,
                                       mute_citation = TRUE)
                    message("\n")
                }
            }
        }

        if (type == "gtf") {
            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getGTF(db       = db,
                                       organism = FinalOrganisms[i],
                                       path     = kingdom,
                                       mute_citation = TRUE)
                    message("\n")
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getGTF(db       = db,
                                       organism = FinalOrganisms[i],
                                       path     = path,
                                       mute_citation = TRUE)
                    message("\n")
                }
            }
        }

        if (type == "rm") {
            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getRepeatMasker(db       = db,
                                                organism = FinalOrganisms[i],
                                                reference = reference,
                                                path     = kingdom,
                                                mute_citation = TRUE)
                    message("\n")
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getRepeatMasker(db       = db,
                                                organism = FinalOrganisms[i],
                                                reference = reference,
                                                path     = path,
                                                mute_citation = TRUE)
                    message("\n")
                }
            }
        }

        if (type == "rna") {
            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    paths[i] <- getRNA(db       = db,
                                       organism = FinalOrganisms[i],
                                       reference = reference,
                                       path     = kingdom,
                                       mute_citation = TRUE)
                    message("\n")
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    getRNA(db       = db,
                           organism = FinalOrganisms[i],
                           reference = reference,
                           path     = path,
                           mute_citation = TRUE)
                    message("\n")
                }
            }
        }

        if (type == "assemblystats") {
            stats.files <- vector("list", length(FinalOrganisms))

            if (is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    if (combine) {
                        stats.files[i] <- list(
                            getAssemblyStats(
                                db       = db,
                                organism = FinalOrganisms[i],
                                reference = reference,
                                path     = kingdom,
                                type     = "import",
                                mute_citation = TRUE
                            )
                        )
                        names(stats.files[i]) <- FinalOrganisms[i]
                        message("\n")
                    } else {
                        paths[i] <- getAssemblyStats(
                            db       = db,
                            organism = FinalOrganisms[i],
                            reference = reference,
                            path     = kingdom,
                            mute_citation = TRUE
                        )
                        message("\n")
                    }
                }
            }

            if (!is.null(path)) {
                for (i in seq_len(length(FinalOrganisms))) {
                    if (combine) {
                        stats.files[i] <- list(
                            getAssemblyStats(
                                db       = db,
                                organism = FinalOrganisms[i],
                                reference = reference,
                                path     = path,
                                type     = "import",
                                mute_citation = TRUE
                            )
                        )
                        names(stats.files[i]) <- FinalOrganisms[i]
                    } else {
                        paths[i] <- getAssemblyStats(
                            db       = db,
                            reference = reference,
                            organism = FinalOrganisms[i],
                            path     = path,
                            mute_citation = TRUE
                        )
                    }
                }
            }
        }


        if (!is.null(path)) {
            meta_files <- list.files(path)
            meta_files <- meta_files[stringr::str_detect(meta_files, "doc_")]
            file.rename(from = file.path(path, meta_files), to = file.path(path, "documentation", meta_files))

            doc_tsv_files <- file.path(path,"documentation", meta_files[stringr::str_detect(meta_files, "[.]tsv")])

            summary_log <- dplyr::bind_rows(lapply(doc_tsv_files, function(data) {
              if (fs::file_size(data) > 0)
                suppressMessages(readr::read_tsv(data))
            }))

            readr::write_excel_csv(summary_log, file.path(path, "documentation", paste0(kingdom, "_summary.csv")))
            message("A summary file (which can be used as supplementary information file in publications) containig retrieval information for all ",kingdom," species has been stored at '",file.path(path, "documentation", paste0(kingdom, "_summary.csv")),"'.")

        } else {
            meta_files <- list.files(kingdom)
            meta_files <- meta_files[stringr::str_detect(meta_files, "doc_")]
            file.rename(from = file.path(kingdom, meta_files), to = file.path(kingdom, "documentation", meta_files))

            doc_tsv_files <- file.path(kingdom,"documentation", meta_files[stringr::str_detect(meta_files, "[.]tsv")])

            summary_log <- dplyr::bind_rows(lapply(doc_tsv_files, function(data) {
              if (fs::file_size(data) > 0)
                suppressMessages(readr::read_tsv(data))
            }))

            readr::write_excel_csv(summary_log, file.path(kingdom, "documentation", paste0(kingdom, "_summary.csv")))
            message("A summary file (which can be used as supplementary information file in publications) containig retrieval information for all ",kingdom," species has been stored at '",file.path(kingdom, "documentation", paste0(kingdom, "_summary.csv")),"'.")

        }

        if (combine) {
            stats.files <- dplyr::bind_rows(stats::na.omit(stats.files))
            message("Finished meta retieval process.")
            return(stats.files)
        }

        message("Finished meta retieval process.")
        return(paths[!is.element(paths, c("FALSE", "Not available"))])
    } else {

        if (!is.null(path)) {
            meta_files <- list.files(path)
            meta_files <- meta_files[stringr::str_detect(meta_files, "doc_")]
            file.rename(from = file.path(path, meta_files), to = file.path(path, "documentation", meta_files))

            doc_tsv_files <- file.path(path,"documentation", meta_files[stringr::str_detect(meta_files, "[.]tsv")])

            summary_log <- dplyr::bind_rows(lapply(doc_tsv_files, function(data) {
              if (fs::file_size(data) > 0)
                suppressMessages(readr::read_tsv(data))
            }))

            readr::write_excel_csv(summary_log, file.path(path, "documentation", paste0(kingdom, "_summary.csv")))
            message("A summary file (which can be used as supplementary information file in publications) containig retrieval information for all ",kingdom," species has been stored at '",file.path(path, "documentation", paste0(kingdom, "_summary.csv")),"'.")

        } else {
            meta_files <- list.files(kingdom)
            meta_files <- meta_files[stringr::str_detect(meta_files, "doc_")]
            file.rename(from = file.path(kingdom, meta_files), to = file.path(kingdom, "documentation", meta_files))

            doc_tsv_files <- file.path(kingdom,"documentation", meta_files[stringr::str_detect(meta_files, "[.]tsv")])

            summary_log <- dplyr::bind_rows(lapply(doc_tsv_files, function(data) {
              if (fs::file_size(data) > 0)
                suppressMessages(readr::read_tsv(data))
            }))

            readr::write_excel_csv(summary_log, file.path(kingdom, "documentation", paste0(kingdom, "_summary.csv")))
            message("A summary file (which can be used as supplementary information file in publications) containig retrieval information for all ",kingdom," species has been stored at '",file.path(kingdom, "documentation", paste0(kingdom, "_summary.csv")),"'.")

        }

        message("The ", type,"s of all species have already been downloaded! You are up to date!")
        please_cite_biomartr()
    }
}

#' @title Helper function to query sub marts from Ensembl Biomart
#' @description Internal interface function to the Ensembl Biomart API
#' @param submart type of submart
#' @author Hajk-Georg Drost
#' @noRd
getSubMarts <- function(submart = "ensembl") {

    mart_names <- as.character(ensembl_divisions_short(FALSE))
    # TODO: Is there no way to get bacteria in?
    mart_names <- mart_names[!(mart_names %in% "bacteria")]

    if (!is.element(submart, mart_names))
        stop(
            "Please select a submart that is supported by ensembl:\n",
            paste(mart_names, collapse = ", "),
            call. = FALSE
        )
    # Create URLs
    marts <- paste0("http://", mart_names, ".ensembl",".org")
    marts[1] <- "http://www.ensembl.org"
    port_id <- ":80/biomart/martservice?"
    request_url <- "type=registry&requestid=biomart"
    marts <- paste0(marts, port_id, request_url)
    names(marts) <- mart_names
    # Example: "http://fungi.ensembl.org:80/biomart/martservice?type=registry&requestid=biomart"

    biomartPage <- httr::handle(marts[submart])

    xmlContentMarts <- httr::GET(handle = biomartPage)

    # test whether or not a connection could be established
    httr::stop_for_status(xmlContentMarts)

    # parse Mart information
    doc <- suppressMessages(XML::xmlTreeParse(
        xmlContentMarts,
        useInternalNodes = TRUE,
        encoding = "UTF-8"
    ))
    rootNode <- XML::xmlRoot(doc)

    # extract available databases
    databases <- as.data.frame(XML::xmlSApply(rootNode, function(x)
        XML::xmlGetAttr(x, "name")))

    # extract available database versions
    displayNames <- as.data.frame(XML::xmlSApply(rootNode, function(x)
        XML::xmlGetAttr(x, "displayName")))

    # ectract information whether or not the corresponding database is visible
    visible <- as.data.frame(XML::xmlSApply(rootNode, function(x)
        XML::xmlGetAttr(x, "visible")))

    dbBioMart <- tibble::tibble(
        mart = as.character(databases[ , 1]),
        version = as.character(displayNames[ , 1]),
        visible = as.character(visible[ , 1])
    )

    mart <- version <- NULL

    dbBioMart <- dplyr::select(dplyr::filter(dbBioMart, visible != "0"),
                               mart, version)

    return(dbBioMart)
}

#' @title Retrieve summary statistics for a genome assembly file
#' @description A summary statistics of specific genome features is generated.
#' These statistics are useful to assess the genome quality of retrieved genome assemblies
#' when performing comparative genomics tasks. This way, users can assess whether or not
#' patterns found based on genome comparisons aren't just a technical artifact of
#' differences in genome assembly quality.
#' @param file file path to a genome assembly file in \code{fasta} format.
#' @param organism character string specifying the organism at hand.
#' @author Hajk-Georg Drost
#' @details The summary statistics include:
#' \itemize{
#' \item \code{genome_size_mbp}: Genome size in mega base pairs
#' \item \code{n50_mbp}: The N50 contig size of the genome assembly in mega base pairs
#' \item \code{n_seqs}: The number of chromosomes/scaffolds/contigs of the genome assembly file
#' \item \code{n_nnn}: The absolute number of NNNs (over all chromosomes or scaffolds or contigs) in the genome assembly file
#' \item \code{rel_nnn}: The percentage (relative frequency) of NNNs (over all chromosomes or scaffolds or contigs) compared to the total number of
#' nucleotides in the genome assembly file
#' \item \code{genome_entropy}: The \code{Shannon Entropy} of the genome assembly file (median entropy over all individual chromosome entropies)
#' \item \code{n_gc}: The total number of GCs
#' (over all chromosomes or scaffolds or contigs) in the genome assembly file
#' \item \code{rel_gc}: The (relative frequency) of GCs
#' (over all chromosomes or scaffolds or contigs) compared to the total number of
#' nucleotides in the genome assembly file
#' }
#' @seealso \code{\link{summary_cds}}, \code{\link{getCollection}}, \code{\link{getGenome}}, \code{\link{read_genome}}
#' @examples \dontrun{
#' # retrieve genome from NCBI RefSeq
#' Sc <- biomartr::getGenome(db = "refseq", organism = "Saccharomyces cerevisiae")
#' # compute genome assembly summary statistics
#' Sc_genome_summary <- summary_genome(file = Sc, organism = "Saccharomyces cerevisiae")
#' # look at results
#' Sc_genome_summary
#' }
#' @export
summary_genome <- function(file,
                           organism) {
        if (!file.exists(file))
                stop("Please provide a valid file path to your genome assembly file.", call. = FALSE)
        message("Starting genome summary analysis:")
        genome_seq <- Biostrings::readDNAStringSet(filepath = file, format = "fasta")
        # determine letter frequency as probablity distr (relative frequency of nucleotides)
        nucl_feq <- Biostrings::letterFrequency(genome_seq, letters = c("A","C","G","T"), as.prob = TRUE)
        # compute the median rel feq over all chromosomes
        nucl_re_freq <- apply(nucl_feq, 1, philentropy::H)
        # compute the entropy of the nucleotide distributions
        genome_entropy <- stats::median(nucl_re_freq)

        # compute genome size in Mega base pairs
        genome_size_nucl_mbp <-
                sum(as.numeric(genome_seq@ranges@width)) / 1000000

        # total number of all nucleotides in the genome
        length_all_nucl <- sum(as.numeric(genome_seq@ranges@width))
        # count absolute number of NNN's in genome
        NNN_abs <- sum(as.numeric(Biostrings::vcountPattern("N", genome_seq)))

        # relative frequency of NNN's in genome
        NNN_freq <- NNN_abs / length_all_nucl

        # count absolute number of GC's in genome
        GC_abs <- sum(as.numeric(Biostrings::vcountPattern("GC", genome_seq)))

        # relative frequency of GC's in genome
        GC_freq <- GC_abs / length_all_nucl

        # compute N50 of genome assembly
        genome_N50 <- N50(Biostrings::width(genome_seq)) / 1000000

        res <- tibble::tibble(organism = organism,
                              genome_size_mbp = genome_size_nucl_mbp,
                              n50_mbp = genome_N50,
                              n_seqs = length(genome_seq),
                              n_nnn = NNN_abs,
                              rel_nnn = NNN_freq,
                              genome_entropy = genome_entropy,
                              n_gc = GC_abs,
                              rel_gc = GC_freq
                              )

        return(res)
}

#' @title Helper function to perform customized downloads
#' @description To achieve the most stable download experience,
#' ftp file downloads are customized for each operating system.
#' @param url the url to download
#' @param ... additional arguments that shall be passed to
#' \code{\link[downloader]{download}}
#' @author Hajk-Georg Drost
#' @noRd
custom_download <- function(url, ...) {

  withr::local_options(timeout = max(30000000, getOption("timeout")))

    if (RCurl::url.exists(url)) {
        operating_sys <- Sys.info()[1]

        if (operating_sys == "Darwin") {
            downloader::download(
                url = url, ...,
                cacheOK = FALSE,
                quiet = TRUE
            )

        }

        if (operating_sys == "Linux") {
            downloader::download(
                url = url, ...,
                cacheOK = FALSE,
                quiet = TRUE
            )
        }

        if (operating_sys == "Windows") {
            downloader::download(url = url, ...,
                                 cacheOK = FALSE,
                                 quiet = TRUE)
        }
    } else {
        message(
            "The FTP link: '",url,"' seems not to be available at the moment. This might either be due to an instable internet connection, a firewall issue, a wrong organism name, or due to the fact that the specified organism is not available in the database you selected. If it is an internet connection issue, could you please try to re-run the function to see whether it works now?"
        )
        return(FALSE)
    }
}

#' An wrapper to custom_download which checks if local files exists
#' @noRd
custom_download_check_local <- function(url, local_file, rest_api_status, db = "ensembl", ...) {
  withr::local_options(timeout = max(30000000, getOption("timeout")))

  if (file.exists(local_file)) {
    message("File ", local_file,
            " exists already. Thus, download has been skipped.")
  } else {
    if (!is.null(rest_api_status) && rest_api_status$release_coord_system_version == "not_found") {
      message("Found organism but given release number did not specify existing file
                     in ensembl, maybe it is too old? Check that it exists on ensembl
                     first at all.")
      return(FALSE)
    }

    tryCatch({
      custom_download(url, destfile = local_file, mode = "wb")
    }, error = function(e)
      if (db == "ensembl") {
        message(
          "Something went wrong while trying to reach the file '",url,
          "'. This could be due to an instable internet connection or incorrect file path on the ENSEMBL ftp server. Please check if you are able to reach '",url, "' in your web browser.",
          " In some cases ENSEMBL released a new database version and path names or the API weren't updated yet. Please give it a few days time or contact the ENSEMBL helpdesk."
        )
      } else {
        message(
          "The download session seems to have timed out at the FTP site '", url,
          "'. This could be due to an overload of queries to the databases.",
          " Please restart this function to continue the data retrieval process or wait ",
          "for a while before restarting this function in case your IP address was logged due to an query overload on the server side."
        )
      }
      )
  }
  return(invisible(NULL))
}

test_url_status <- function(url, organism) {

  test_status <- curl::curl_fetch_memory(url)

  if (test_status$status_code == 200) {
    json.qry.info <-
      jsonlite::fromJSON(url)
    return(json.qry.info)
  } else {
    message(
      "Something went wrong when trying to access the url: ",
      url,
      ". It seems like the organism '",
      organism,
      "' does not exist in this database. Could it be that the organism name is misspelled? Thus, download has been omitted."
    )
    return(FALSE)
  }
}

#' @title Helper function to test connection and availability of
#' queried FTP files. This version work when ssl certificates are too strict.
#' @description To make sure that the automatically generated query path to
#' the ftp stored file on NCBI or ENSEMBL actually exists, this helper function
#' makes a test query.
#' @author Hajk-Georg Drost
#' @importFrom  RCurl url.exists
#' @importFrom XML getHTMLLinks
#' @noRd
#' @import curl
exists.ftp.file.new <- function(url, file.path) {

  url_dir_safe <- gsub("//$", "/", paste0(dirname(url), "/"))
  if (!RCurl::url.exists(url_dir_safe))
    return(FALSE)

  con <- RCurl::getURL(url_dir_safe, ftp.use.epsv = FALSE, dirlistonly = TRUE)

  dir_files <- XML::getHTMLLinks(con)

  return(is.element(as.character(basename(file.path)),
                    dir_files))
}

N50 <- function(len) {
    # sort scaffold or chromosome lenghts in descending order
    len.sorted <- rev(sort(len))
    # compute N50 over all scaffold or chromosome lenghts in Mbp
    N50 <- len.sorted[cumsum(as.numeric(len.sorted)) >= sum(len.sorted)*0.5][1] / 1000000
    return(N50)
}

select_assembly_refseq_genbank <- function(organism, AssemblyFilesAllKingdoms,
                                           reference, db) {
  organism_name <- taxid <-
    refseq_category <- version_status <- assembly_accession <- ftp_path <- NULL

  if (reference) {
    if (!is.taxid(organism)) {
      FoundOrganism <-
        dplyr::filter(
          AssemblyFilesAllKingdoms,
          stringr::str_detect(organism_name, organism) |
            assembly_accession ==  organism,
          ((refseq_category == "representative genome") |
             (refseq_category == "reference genome")
          ),
          (version_status == "latest"), !is.na(ftp_path)
        )
    } else {
      FoundOrganism <-
        dplyr::filter(
          AssemblyFilesAllKingdoms,
          taxid == as.integer(organism),
          ((refseq_category == "representative genome") |
             (refseq_category == "reference genome")
          ),
          (version_status == "latest"), !is.na(ftp_path))
    }
  } else {
    if (!is.taxid(organism)) {
      FoundOrganism <-
        dplyr::filter(
          AssemblyFilesAllKingdoms,
          stringr::str_detect(organism_name, organism) |
            assembly_accession == organism,
          (version_status == "latest"), !is.na(ftp_path)
        )
    } else {
      FoundOrganism <-
        dplyr::filter(
          AssemblyFilesAllKingdoms,
          taxid == as.integer(organism),
          (version_status == "latest"), !is.na(ftp_path)
        )
    }
  }

  if (nrow(FoundOrganism) == 0) {
    message(
      "----------> No reference genome or representative genome was found for '",
      organism, "'. Thus, download for this species has been omitted.",
      " Have you tried to specify getGenome(db = '",db,"', organism = '",organism,"' , reference = FALSE) ?",
      " Alternatively, you can retrieve genome assemblies using the NCBI accession ID or NCBI Taxonomy ID.",
      " See '?'is.genome.available' for examples."
    )
    return("Not available")
  }
  if (nrow(FoundOrganism) > 1) {

    is_reference_genome <- FoundOrganism$refseq_category == "reference genome"
    if (any(is_reference_genome)) {
      FoundOrganism <- FoundOrganism[is_reference_genome == TRUE, ]
    }
    is_complete_genome <- FoundOrganism$assembly_level == "Complete Genome"
    if (any(is_complete_genome)) {
      FoundOrganism <- FoundOrganism[is_complete_genome == TRUE, ]
    }

    FoundOrganism <- FoundOrganism[1, ]

    message(
      "More than one entry has been found for '",
      organism, "'. Only the first entry '", FoundOrganism$organism_name[1], "' has been used for subsequent genome retrieval.",
      " If you wish to download a different version, please use the NCBI accession ID when specifying the 'organism' argument.",
      " See ?is.genome.available for examples."
    )
  }
  return(FoundOrganism)
}

ftp_url_refseq_genbank <- function(assembly, type) {
  stem_url <- paste0(assembly$ftp_path, "/", basename(assembly$ftp_path))

  if (type == "genome") {
    url <- paste0(stem_url, "_genomic.fna.gz")
  } else if (type == "gff") {
    url <- paste0(stem_url, "_genomic.gff.gz")
  } else if (type == "cds") {
    url <- paste0(stem_url, "_cds_from_genomic.fna.gz")
  } else if (type == "rna") {
    url <- paste0(stem_url, "_rna_from_genomic.fna.gz")
  } else if (type == "protein") {
    url <- paste0(stem_url, "_protein.faa.gz")
  } else stop("invalid refseq/genbank file type selected")

  return(url)
}

local_path_refseq_genbank <- function(path, local.org, db, type) {
  local_file <- file.path(path, local.org)

  if (type == "genome") {
    local_file <- paste0(local_file, "_genomic_", db, ".fna.gz")
  } else if (type == "gff") {
    local_file <- paste0(local_file, "_genomic_", db, ".gff.gz")
  } else if (type == "cds") {
    local_file <- paste0(local_file, "_cds_from_genomic_", db, ".fna.gz")
  } else if (type == "rna") {
    local_file <- paste0(local_file, "_rna_from_genomic_", db, ".fna.gz")
  } else if (type == "protein") {
    local_file <- paste0(local_file, "_protein_", db, ".faa.gz")
  } else stop("invalid refseq/genbank file type selected")

  return(local_file)
}


get_file_refseq_genbank <- function(db = "refseq",
                                    organism,
                                    reference = FALSE,
                                    skip_bacteria = TRUE,
                                    release = NULL,
                                    gunzip = FALSE,
                                    path = file.path("_ncbi_downloads", "genomes"),
                                    assembly_type = "toplevel",
                                    type = "genome") {
  # get Kingdom Assembly Summary file
  AssemblyFilesAllKingdoms <-
    getKingdomAssemblySummary(db = db, skip_bacteria = skip_bacteria)

  # test whether or not genome is available
  if (!suppressMessages(is.genome.available(organism = organism, db = db, skip_bacteria = skip_bacteria))) {
    message(
      "Unfortunately no ",toupper(type), " file could be found for organism '",
      organism, "'. Thus, the download of this organism has been omitted. Have you tried to specify 'reference = FALSE' ?"
    )
    return("Not available")
  }

  organism <- stringr::str_replace_all(organism, "\\(", "")
  organism <- stringr::str_replace_all(organism, "\\)", "")

  FoundOrganism <- select_assembly_refseq_genbank(organism, AssemblyFilesAllKingdoms,
                                                  reference, db)
  download_url <- ftp_url_refseq_genbank(FoundOrganism, type)

  organism <- stringr::str_replace_all(organism, " ", "_")
  local.org <- stringr::str_replace_all(organism, "-", "_")
  local.org <- stringr::str_replace_all(local.org, "\\/", "_")
  local_file <- local_path_refseq_genbank(path, local.org, db, type)

  custom_download_check_local(download_url, local_file, NULL, "refgen")
  message("-> ", toupper(type)," download of ", organism, " is completed!")

  md5_sum_test(md5_local = file.path(path, paste0(local.org, "_md5checksums.txt")),
               md5_url = paste0(FoundOrganism$ftp_path,"/md5checksums.txt"),
               local_file = local_file, file_url = download_url)

  return(list(local_file = local_file, download_url = download_url,
              local.org = local.org, FoundOrganism = FoundOrganism))
}

#' @inherit getBio
#' @title Coding Sequence Retrieval
#' @description Main retrieval function for coding sequences (CDS)
#' of an organism of interest.
#' By specifying the scientific name of an organism of interest the
#' corresponding fasta-file storing the CDS information for the organism
#' of interest can be downloaded and stored locally. CDS files can be retrieved
#' from several databases.
#' @param path a character string specifying the location (a folder)
#' in which the corresponding CDS file shall be stored.
#' Default is \code{path} = \code{file.path("_ncbi_downloads","CDS")}.
#' @examples
#' \dontrun{
#' # download the genome of Arabidopsis thaliana from refseq
#' # and store the corresponding genome CDS file in '_ncbi_downloads/CDS'
#' file_path <- getCDS( db       = "refseq",
#'              organism = "Arabidopsis thaliana",
#'              path     = file.path("_ncbi_downloads","CDS"))
#'
#' Ath_CDS <- read_cds(file_path, format = "fasta")
#'
#' }
#' @family getBio
#' @family cds
#' @export
getCDS <-
    function(db = "refseq",
             organism,
             reference = FALSE,
             skip_bacteria = TRUE,
             release = NULL,
             gunzip = FALSE,
             path = file.path("_ncbi_downloads", "CDS"),
             mute_citation = FALSE) {

    if (!is.element(db, c("refseq", "genbank",
                          "ensembl")))
        stop(
            "Please select one of the available data bases:
            'refseq', 'genbank', or 'ensembl'.",
            call. = FALSE
        )

    message("-> Starting CDS retrieval of '", organism, "' from ", db, " ...")
    message("\n")

    # create result folder
    if (!file.exists(path)) {
      dir.create(path, recursive = TRUE)
    }

    if (is.element(db, c("refseq", "genbank"))) {
      info <- get_file_refseq_genbank(db, organism, reference, skip_bacteria,
                                      release, gunzip, path, type = "cds")
      refseq_genbank_download_post_processing(info, organism, db, path,
                                              gunzip,
                                              remove_annotation_outliers = FALSE,
                                              format = "cds",
                                              mute_citation = mute_citation)
    } else if (db %in% c("ensembl", "ensemblgenomes")) {
        # download CDS sequence from ENSEMBL
        cds.path <- getENSEMBL.Seq(organism, type = "cds", id.type = "all",
                        release = release, path = path)

        ensembl_download_post_processing(cds.path, organism,
                                         format = "cds",
                                         gunzip = gunzip,
                                         mute_citation = mute_citation)
    }
}

#' @title Retrieve metagenomes from NCBI Genbank
#' @description Retrieve available metagenomes from NCBI Genbank. 
#' NCBI Genbank allows users to download entire metagenomes of several 
#' metagenome projects. This function downloads available metagenomes that can 
#' then be downloaded via \code{\link{getMetaGenomes}}.
#' @param name metagenome name retrieved by \code{\link{listMetaGenomes}}.
#' @param path a character string specifying the location (a folder) in 
#' which the corresponding metagenome shall be stored. 
#' Default is \code{path} = \code{file.path("_ncbi_downloads","metagenome")}.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # Frist, retrieve a list of available metagenomes
#' listMetaGenomes()
#' 
#' # Now, retrieve the 'human gut metagenome'
#' getMetaGenomes(name = "human gut metagenome")
#' }
#' @seealso \code{\link{getMetaGenomeAnnotations}}, 
#' \code{\link{listMetaGenomes}}  
#' @export

getMetaGenomes <-
    function(name,
             path = file.path("_ncbi_downloads", "metagenome")) {
        if (!is.element(name, listMetaGenomes(details = FALSE)))
            stop(
                paste0("Unfortunately, the metagenome '",
                name,
                "' is not available. Please consult the listMetaGenomes() ",
                "function for available metagenomes."), call. = FALSE
            )
        
        if (!file.exists(path)) {
            dir.create(path, recursive = TRUE)
        }
        
        organism_name <- NULL
        
        # retrieve metagenomes assembly_summary.txt file
        mgs <- getMetaGenomeSummary()
        
        metagenomes.members <-
            dplyr::filter(mgs, organism_name == name)
        file.names <- metagenomes.members$ftp_path
        
        for (i in seq_len(length(file.names))) {
            download_url <-
                paste0(
                    file.names[i],
                    "/",
                    paste0(
                        metagenomes.members$assembly_accession[i],
                        "_",
                        metagenomes.members$asm_name[i],
                        "_genomic.fna.gz"
                    )
                )
            
            tryCatch({
                utils::capture.output(downloader::download(
                    download_url,
                    destfile = file.path(path, paste0(
                        basename(file.names[i]), "_genomic.fna.gz"
                    )),
                    mode = "wb"
                ))
            }, error = function(e)
                message(
                    "Unfortunately, the FTP site 'ftp://ftp.ncbi.nlm.nih.gov/' cannot be reached.",
                    " Are you able to access the FTP site '",
                    download_url,"' in your browser? Please try to re-run this function ",
                    "and see if it might work then ... Sometimes an unstable internet connection or firewall issues are causing this function to fail."
                ))
            
            docFile(
                file.name = paste0(basename(file.names[i]), "_genomic.fna.gz"),
                organism  = basename(file.names[i]),
                url       = download_url,
                database  = "Genbank metagenomes",
                path      = path,
                refseq_category = metagenomes.members$refseq_category[i],
                assembly_accession = metagenomes.members$assembly_accession[i],
                bioproject = metagenomes.members$bioproject[i],
                biosample = metagenomes.members$biosample[i],
                taxid = metagenomes.members$taxid[i],
                infraspecific_name = metagenomes.members$infraspecific_name[i],
                version_status = metagenomes.members$version_status[i],
                release_type = metagenomes.members$release_type[i],
                genome_rep = metagenomes.members$genome_rep[i],
                seq_rel_date = metagenomes.members$seq_rel_date[i],
                submitter = metagenomes.members$submitter[i]
            )
        }
        
        message(
            "The metagenome of '",
            name,
            "' has been downloaded to '",
            path,
            "'."
        )
        
        file.paths <- file.path(path, list.files(path = path))
        # return only file paths without "*.txt"
        return(file.paths[!unlist(lapply(file.paths, function(x)
            stringr::str_detect(x, "[.]txt")))])
    }

#' @title Retrieve and summarise the assembly_summary.txt files from
#' NCBI for all kingdoms
#' @description Retrieval function of the assembly_summary.txt file
#' from NCBI for all kingdoms.
#' The assembly_summary.txt files store available species on NCBI.
#' @param db database name. E.g. \code{refseq} or \code{genbank}.
#' @param skip_bacteria Due to its enormous dataset size (> 700MB as of July 2023),
#' the bacterial summary file will not be loaded by default anymore. If users
#' wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
#' bacterial summary file will be downloaded.
#' @param file path, local path to total summary file, default is in tmp folder.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # This example will run the default version of this function
#' # whereby information for Bacteria are not downloaded
#' test <- getKingdomAssemblySummary(db = "genbank", skip_bacteria = TRUE)
#' test
#' # Users can then retrieve information for Bacteria by skip_bacteria = FALSE
#' test2 <- getKingdomAssemblySummary(db = "genbank", skip_bacteria = FALSE)
#' test2
#' }
#' @seealso \code{\link{getSummaryFile}}, \code{\link{getMetaGenomeSummary}}, \code{\link{get.ensembl.info}}
#' @export

getKingdomAssemblySummary <- function(db, skip_bacteria = TRUE,
                                      file = assemblies_info_path(db)
                                      ) {
    if (!is.element(db, c("refseq", "genbank")))
        stop("Please select one of the available data bases:
             'refseq' or 'genbank'")

    if (is.element(db, c("refseq", "genbank"))) {
        # if AssemblyFilesAllKingdoms.txt file was already generated/downloaded
        # then use the local version stored in temp()
        if (file.exists(file) & (skip_bacteria | file.exists(assemblies_info_path(db, "bacteria")))) {
          AssemblyFilesAllKingdoms <- suppressWarnings(read_all_kingdoms_assemblies_info(file))
        } else {
            # otherwise download all assembly_summary.txt files for all kingdoms
            # and store the AssemblyFilesAllKingdoms.txt file locally
            # retrieve the assembly_summary.txt files for all kingdoms
            cat("It seems that this is the first time you run this command for",db,".")
            cat("\n")
            message("Thus, 'assembly_summary.txt' files for all kingdoms will be retrieved from ",db,". ")
            message("Don't worry this has to be done only once if you don't restart your R session.")
            message("\n")
            kgdoms <- getKingdoms(db = db)
            storeAssemblyFiles <- vector("list", length(kgdoms))

            if (skip_bacteria){
                cat( "Due to its extended dataset size (GenBank: >700 MB, RefSeq: >150 MB) Kingdom 'bacteria' will not be downloaded by default anymore. To also include 'bacteria' please specify the argument 'skip_bacteria = FALSE'" )
                cat("\n")
                cat("\n")
            }

            for (i in seq_along(kgdoms)) {
                if (kgdoms[i] == "bacteria" && (db == "genbank" || db == "refseq") && skip_bacteria){
                    cat("--------> Skipping bacteria download .....")
                    cat("\n")
                    cat("\n")
                } else {
                    cat("-> Starting download for:", kgdoms[i])
                    cat("\n")
                    storeAssemblyFiles[i] <-
                        list(getSummaryFile(db = db, kingdom = kgdoms[i]))
                }
            }

            AssemblyFilesAllKingdoms <-
                dplyr::bind_rows(storeAssemblyFiles)

            readr::write_tsv(AssemblyFilesAllKingdoms, file)

            message("\n")
            message("Completed!")
            message("Now continue with species download ...")
        }
    }

    orgs <-
        stringr::str_replace_all(AssemblyFilesAllKingdoms$organism_name,
                                 "\\(", "")
    orgs <- stringr::str_replace_all(orgs, "\\)", "")

    AssemblyFilesAllKingdoms <-
        dplyr::mutate(AssemblyFilesAllKingdoms, organism_name = orgs)

    return(AssemblyFilesAllKingdoms)
}

context("Test: getKingdomAssemblySummary()")

test_that("The getKingdomAssemblySummary() interface works properly
          for NCBI RefSeq..",{
    skip_on_cran()
    skip_on_travis()
    # KingdomAssembly Summary Info file retrieval from RefSeq
    all <- getKingdomAssemblySummary("refseq")
    expect_false(nrow(all) == 0)
})


test_that("The getKingdomAssemblySummary() interface works properly
          for NCBI Genbank..",{
    skip_on_cran()
    skip_on_travis()
    # KingdomAssembly Summary Info file retrieval from Genbank
    all <- getKingdomAssemblySummary("genbank")
    expect_false(nrow(all) == 0)
})

#' @title Retrieve available groups for a kingdom of life (only available for NCBI RefSeq and NCBI Genbank)
#' @description A short list of available groups for a kingdom of life.
#' @param db a character string specifying the database from which the genome 
#' shall be retrieved: 
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' }
#' 
#' Default is \code{db = "refseq"}.
#' @param kingdom a character string specifying for which kingdom of life 
#' groups shall be retrieved. See \code{\link{getKingdoms}} for details.
#' @author Hajk-Georg Drost
#' @examples
#' # get possible kigdom names
#' getKingdoms(db = "refseq")
#' \dontrun{
#' # retrieve subgroups for vertebrate_mammalian available from refseq
#' getGroups(db = "refseq", kingdom = "vertebrate_mammalian")
#' 
#' # get possible kigdom names
#' getKingdoms(db = "genbank")
#' # retrieve subgroups for vertebrate_mammalian available from genbank
#' getGroups(db = "genbank", kingdom = "vertebrate_mammalian")
#' }
#' @seealso \code{\link{meta.retrieval}}, \code{\link{getGenome}}, 
#' \code{\link{getProteome}}, \code{\link{getCDS}}, \code{\link{getKingdoms}}
#' @export

getGroups <- function(db = "refseq", kingdom) {
    
    if (!is.element(db, c("refseq", "genbank")))
        stop("Group information is unfortunately only available for db = 'refseq' and db = 'genbank'.", call. = FALSE)
    
    groups <- listGroups(db = db, kingdom = kingdom, details = FALSE)
    return(names(groups))
}

context("Test: getGroups()")

test_that("The getGroups() interface works properly... test: archaea",{

    skip_on_cran()
    skip_on_travis()

    expect_equal(
        getGroups("archaea", db = "refseq"),
        c("Archaeoglobi", "Candidatus Bathyarchaeota",
          "Candidatus Culexarchaeota", "Candidatus Korarchaeota",
          "Candidatus Lokiarchaeota", "Candidatus Micrarchaeota",
          "Candidatus Nanohaloarchaeota", "Methanomada group",
          "Methanonatronarchaeia", "Nanoarchaeota",
          "Nitrososphaerota", "Stenosarchaea group",
          "Thermococci", "Thermoplasmata", "Thermoproteota")
    )
})

test_that("The getGroups() interface works properly... test: bacteria",{

    skip_on_cran()
    skip_on_travis()
    # Too many groups now
    # just test that it outputs correct format and does not fail
    expect_type(
        getGroups("bacteria", db = "refseq"),
        "character"
    )
})

test_that("The getGroups() interface works properly... test: fungi",{

    skip_on_cran()
    skip_on_travis()

    expect_equal(
        getGroups("fungi", db = "refseq"),
        c(
            "Ascomycetes", "Basidiomycetes", "Other Fungi"
        )
    )
})


test_that("The getGroups() interface works properly... test: invertebrate",{

    skip_on_cran()
    skip_on_travis()

    expect_equal(
        getGroups("invertebrate", db = "refseq"),
        c(
            "Flatworms", "Insects", "Other Animals", "Roundworms"
        )
    )
})


test_that("The getGroups() interface works properly... test: plant",{

    skip_on_cran()
    skip_on_travis()

    expect_equal(
        getGroups("plant", db = "refseq"),
        c(
            "Green Algae", "Land Plants"
        )
    )
})


test_that("The getGroups() interface works properly... test: protozoa",{

    skip_on_cran()
    skip_on_travis()

    expect_equal(
        getGroups("protozoa", db = "refseq"),
        c(
            "Apicomplexans", "Kinetoplasts", "Other Protists"
        )
    )
})

test_that("The getGroups() interface works properly... test:
          vertebrate_mammalian",{

              skip_on_cran()
              skip_on_travis()

    expect_equal(
        getGroups("vertebrate_mammalian", db = "refseq"),
        c(
            "Mammals"
        )
    )
})

test_that("The getGroups() interface works properly...
          test: vertebrate_other",{

              skip_on_cran()
              skip_on_travis()

    expect_equal(
        getGroups("vertebrate_other", db = "refseq"),
        c(
            "Amphibians", "Birds", "Fishes", "Reptiles"
        )
    )
})

#' @title Download sequence or annotation from ENSEMBL
#' @description Backend function for retrieving files sequence and annotation
#' files from the ENSEMBL ftp server
#' @inheritParams getBio
#' @param type character, biological sequence type (e.g. "dna", "cds")
#' @param id.type a character, default "toplevel". id type of assembly, either "toplevel" or "primary_assembly" usually.
#' @param path location where file shall be stored.
#' @author Hajk-Georg Drost
#' @return either a character path to downloaded file, or a logical FALSE, specifying failure.
getENSEMBL <- function(organism, type = "dna", id.type = "toplevel", release = NULL, path, format) {

  annotation_formats <- c("gff3", "gtf")
  all_format <- c(annotation_formats, "fasta")
  if (!is.element(format, all_format))
    stop("Please a 'format' argument supported by this function: ",
         paste(all_format, collapse = ", "),".")

  region_types <- c("dna", "cds", "pep", "ncrna")
  if (!is.element(type, region_types))
    stop("Please a 'type' argument supported by this function: ",
         paste(region_types, collapse = ", "),".")

  ensembl_summary <- ensembl_assembly_hits(organism)
  if (isFALSE(ensembl_summary)) return(FALSE)
  release <- validate_release(release, ensembl_summary)

  # Check if assembly can be reached
  new.organism <- ensembl_proper_organism_name(ensembl_summary)
  rest_url <- ensembl_rest_url_assembly(new.organism)
  rest_api_status <- test_url_status(url = rest_url, organism = organism)
  if (isFALSE(rest_api_status)) return(FALSE)


  # construct retrieval query
  # Before 75, ensembl used .release extension on assembly

  all_possible_assemblies <- rest_api_status$coord_system_versions

  # construct retrieval query
  kingdom <- ensembl_summary$division[1]
  core_path <- ensembl_ftp_server_url_format_full(kingdom, release, format)
  # Go through all possible assemblies, from newest to oldest, only 1 will match!
  rest_api_status$release_coord_system_version <- "not_found"
  for (assembly_option in all_possible_assemblies) {
    ensembl.qry <- ftp_url_ensembl(core_path, new.organism, assembly_option,
                                   ensembl_summary, format, type, id.type,
                                   release)
    if (!isFALSE(ensembl.qry)) {
      assembly_is_correct <- exists.ftp.file.new(ensembl.qry, ensembl.qry)
      if (assembly_is_correct) {
        rest_api_status$release_coord_system_version <- assembly_option
        break
      }
    }
  }

  local_file <- local_path_ensembl(path, new.organism, rest_api_status,
                                   format, type, release, id.type)

  custom_download_check_local(ensembl.qry, local_file, rest_api_status)
  return(c(local_file, ensembl.qry))
}

local_path_ensembl <- function(path, new.organism, rest_api_status,
                               format, type, release, id.type) {
  assembly <- rest_api_status$release_coord_system_version
  if (format == "fasta") {
    ensembl_seq_local_path(path, new.organism, assembly,
                           type, id.type)
  } else {
    file.path(
      path,
      paste0(ensembl_assembly_stem(new.organism, assembly),
             release, ".", "_ensembl",
             ".", format, ".gz"))
  }
}

#' Get the ftp relative file name of the assembly file wanted
#' @noRd
ftp_url_ensembl <- function(core_path, new.organism, assembly,
                            ensembl_summary, format, type, id.type = "none",
                            release = NULL) {

  dir <- ftp_url_ensembl_dir(core_path, ensembl_summary, new.organism, type)
  url_stem <- file.path(dir, ensembl_assembly_stem(new.organism, assembly))

  if (!is.null(type)) type <- paste0(type, ".")
  if (!is.null(id.type) && id.type == "none") id.type <- NULL
  if (!is.null(id.type)) id.type <- paste0(id.type, ".")
  release. <- release
  is_fasta <- format == "fasta"
  old_fasta_naming <- is_fasta && is.numeric(release) && release <= 75

  if (!is_fasta | old_fasta_naming) {
    release. <- paste0(release, ".")
  } else release. <- NULL

  url <- paste0(url_stem, release.)
  if (is_fasta) {
    format <- "fa"
    url <- paste0(url_stem, type, id.type)
  }
  url <- paste0(url, format, ".gz")
  return(url)
}

#' Get the ftp directory of the assembly
#' @noRd
ftp_url_ensembl_dir <- function(core_path, ensembl_summary, new.organism, type) {
  collection_folder <- get_collection_id(ensembl_summary)
  if (isFALSE(collection_folder)) return(FALSE)
  dir <- paste0(core_path,
                collection_folder,
                stringr::str_to_lower(new.organism),
                "/")
  if (basename(core_path) == "fasta") dir <- paste0(dir, type, "/")
  return(dir)
}

ensembl_assembly_stem <- function(new.organism, assembly_option, suffix = ".") {
  paste0(
    stringr::str_to_title(new.organism),
    ".",
    assembly_option,
    suffix)
}

ensembl_seq_file_base <- function(new.organism, assembly_option, type,
                                  id.type) {
  paste0(
    ensembl_assembly_stem(new.organism, assembly_option),
    type,
    ifelse(id.type == "none", "", paste0(".", id.type)),
    ".fa.gz"
  )
}
ensembl_seq_local_path <- function(path, new.organism, assembly,
                                   type, id.type) {
  file.path(
    path,
    ensembl_seq_file_base(new.organism, assembly, type,
                          id.type)
  )
}

ensembl_gtf_local_path <- function(path, new.organism, rest_api_status,
                                   type, id.type) {
  assembly_option <- rest_api_status$release_coord_system_version
  file.path(
    path,
    ensembl_seq_file_base(new.organism, assembly_option, type,
                          id.type)
  )
}

#' @title Import Genome Assembly as Biostrings or data.table object
#' @description This function reads an organism specific genome stored in a
#' defined file format.
#' @param file a character string specifying the path to the file
#' storing the genome.
#' @param format a character string specifying the file format used to store
#' the genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.
#' @param obj.type a character string specifying the object stype in which
#' the genomic sequence shall be represented. Either as
#' \code{obj.type = "Biostrings"} (default) or
#' as \code{obj.type = "data.table"}.
#' @param ... additional arguments that are used by the
#' \code{\link[seqinr]{read.fasta}} function.
#' @author Hajk-Georg Drost
#' @details This function takes a string specifying the path to the genome file
#' of interest as first argument (e.g. the path returned by
#' \code{\link{getGenome}}).
#' @return Either a \code{Biostrings} or \code{data.table} object.
#' @family genome
#' @family readers
#' @import Biostrings
#' @export

read_genome <-
    function(file,
             format = "fasta",
             obj.type = "Biostrings",
             ...) {
        if (!is.element(format, c("fasta", "gbk")))
            stop("Please choose a file format that is
                 supported by this function.",
                 call. = FALSE)

        if (!file.exists(file))
            stop("The file path you specified does not seem to exist: '", file,"'.", call. = FALSE)

        if (!is.element(obj.type, c("Biostrings", "data.table")))
            stop(
                "Please specify a valid object type: obj.type = 'Biostrings'
                (default) or obj.type = 'data.table'.",
                call. = FALSE
            )
        geneids <- NULL

        if (obj.type == "Biostrings") {
            tryCatch({
                genome <-
                    Biostrings::readDNAStringSet(filepath = file,
                                                 format = format, ...)
            }, error = function(e) {
                stop(
                    paste0(
                        "File ",
                        file,
                        " could not be read properly. \n",
                        "Please make sure that ",
                        file,
                        " contains only DNA sequences and is in ",
                        format,
                        " format."
                    ),
                    call. = FALSE
                )
            })

            return(genome)
        }

        if (obj.type == "data.table") {
            tryCatch({
                genome <-
                    Biostrings::readDNAStringSet(filepath = file,
                                                 format = format, ...)
                genome_names <-
                    as.vector(unlist(lapply(genome@ranges@NAMES, function(x) {
                        return(strsplit(x, " ")[[1]][1])
                    })))
                genome.dt <-
                    data.table::data.table(geneids = genome_names,
                                           seqs = tolower(as.character(genome)))

                data.table::setkey(genome.dt, geneids)

            }, error = function(e) {
                stop(
                    paste0(
                        "File ",
                        file,
                        " could not be read properly. \n",
                        "Please make sure that ",
                        file,
                        " contains only DNA sequences and is in ",
                        format,
                        " format."
                    ),
                    call. = FALSE
                )
            })

            return(genome.dt)
        }
    }

#' @title Interface function to UniProt
#' @noRd
getUniProtSeq <-
  function(organism,
           path = NULL,
           gunzip = FALSE,
           update = FALSE,
           mute_citation = FALSE) {
    organism_new <- stringr::str_replace_all(organism, " ", "%20")
    organism_name_path <-
      stringr::str_replace_all(organism, " ", "_")

    message("-> Retrieve UniProt information for organism: ", organism)
    # retrieve all uniprot info for this organism
    uniprot_species_info <- getUniProtInfo(organism, update = update)

    if (nrow(uniprot_species_info) > 0) {
      # TODO: Generalize this part too
      download_file <-
        paste0(uniprot_species_info$upid,
               "_",
               uniprot_species_info$taxonomy,
               ".fasta.gz")

      # arachea
      kingdom <- uniprot_species_info$superregnum
      proteom_url <- "https://ftp.uniprot.org/pub/databases/uniprot/current_release/knowledgebase/reference_proteomes"
      query <- file.path(proteom_url,
                         stringr::str_to_title(kingdom),
                         uniprot_species_info$upid,
                         download_file)


      local_proteome <- ifelse(is.null(path),
                               file.path(tempdir(), download_file),
                               file.path(path, download_file))
      custom_download(url = query, destfile = local_proteome)

      organism_fasta_file <- tryCatch({
        Biostrings::readBStringSet(local_proteome)
      }, error = function(e)
        message(
          "Something went wrong when trying to access the Uniprot API .Thus the query ",
          query,
          ". could not be used to retrieve a fasta file for '",
          organism,
          "'. Sometimes the internet connection isn't stable and re-running the function might help. Otherwise, could there be an issue with the firewall?"
        ))

      path_or_wd <- ifelse(is.null(path), getwd(), path)
      local_file <- file.path(path_or_wd,
                              paste0(organism_name_path,
                                     "_protein_uniprot.faa.gz"))

      message("-> Write downloaded *.fasta file to local disk ...")

      Biostrings::writeXStringSet(
        organism_fasta_file,
        filepath = local_file,
        compress = TRUE
      )

      docFile(
        file.name = basename(local_file),
        organism  = uniprot_species_info$name,
        url       = query,
        database  = "uniprot",
        path      = path_or_wd,
        refseq_category = "",
        assembly_accession = "",
        bioproject = "",
        biosample = "",
        taxid = uniprot_species_info$taxonomy,
        infraspecific_name = "",
        version_status = "",
        release_type = "",
        genome_rep = "",
        seq_rel_date = "",
        submitter = ""
      )


      final_path <-
        gunzip_and_check(local_file,
                         gunzip,
                         format = "proteome",
                         mute_citation = mute_citation)
      
      return(final_path)
    } else {
      warning(
        "Unfortunately, no entry for '",
        organism,
        "' has been found. Are you certain that you typed the scientific name correctly, e.g. Homo sapiens (capital letter in the first name)?",
        call. = FALSE
      )
      return(NA)
    }
}

existingOrganisms <- function(path, .type) {
    .files <- list.files(path)
    .files <- .files[!stringr::str_detect(.files, "doc_")]
    .files <- .files[!stringr::str_detect(.files, "md5cheksum")]
    .files <- .files[!stringr::str_detect(.files, "documentation")]
    
    .orgs <- unlist(purrr::map(.files, function(x, type = .type) {
        .data <- unlist(stringr::str_split(x, "_"))
        if (length(.data) > 0) {
            
            which_found <- which(.data == type)
            if (length(which_found) > 0) {
                .data <- .data[-c(which_found:length(.data))]
                return(paste0(.data, collapse = " "))
            } else {
             warning("It seems like there are some files in download folder that are neither pre-downloaded species files nor doc_, documentation, or md5checksum files.", call. = FALSE)   
            }
        }
    }))
    
    return(.orgs)
}

#' @title Main BioMart Query Function
#' @description This function takes a set of gene ids and the biomart
#' specifications and performs a biomart query for the given set of gene ids.
#' @param genes a character vector storing the gene ids of a organisms
#'  of interest to be queried against BioMart.
#' @param mart a character string specifying the mart to be used. Users
#' can obtain available marts using \code{\link{getMarts}}.
#' @param dataset a character string specifying the dataset within the mart to
#'  be used, e.g. \code{dataset} = \code{"hsapiens_gene_ensembl"}.
#' @param attributes a character vector specifying the attributes that shall be
#'  used, e.g. \code{attributes} = 
#'  \code{c("start_position","end_position","description")}.
#' @param filters a character vector specifying the filter (query key) for the 
#' BioMart query, e.g. \code{filter} = \code{"ensembl_gene_id"}.
#' @param ... additional parameters for the
#'  \code{\link[biomaRt]{getBM}} function.
#' @param mute_citation logical value indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @details This function is the main query function of the biomartr package.
#'
#' It enables to fastly access annotations of a given gene set based 
#' on the \pkg{biomaRt} package
#' implemented by Steffen Durinck et al.
#'
#' @examples
#' \dontrun{
#' # 1) select a mart
#' getMarts()
#'
#' # we will select mart 'plants_mart' and search for available datasets
#' getDatasets(mart = "plants_mart")
#'
#' # we choose dataset 'athaliana_eg_gene' and run biomart()
#' # using mart: 'plants_mart', dataset: "athaliana_eg_gene"
#' # attributes: c("start_position","end_position","description")
#' # for an example gene set of Arabidopsis thaliana:
#' # c("AT1G06090", "AT1G06100", "AT1G06110", "AT1G06120",
#' #    "AT1G06130", "AT1G06200")
#'
#' biomart(genes      = c("AT1G06090", "AT1G06100",
#'                        "AT1G06110", "AT1G06120",
#'                        "AT1G06130", "AT1G06200"),
#'         mart       = "plants_mart",
#'         dataset    = "athaliana_eg_gene",
#'         attributes = c("start_position","end_position","description"),
#'         filters    = "ensembl_gene_id")
#'}
#'
#' @return A data.table storing the initial query gene vector in 
#' the first column, the output
#' gene vector in the second column, and all attributes in
#' the following columns.
#' @seealso \code{\link{organismFilters}}, \code{\link{organismBM}},
#' \code{\link[biomaRt]{listAttributes}}, \code{\link[biomaRt]{getBM}}
#' @import biomaRt
#' @export
biomart <- function(genes,
                    mart,
                    dataset,
                    attributes,
                    filters,
                    mute_citation = FALSE,
                    ...) {
        
        message("Starting BioMart query ...")
        
    if (stringr::str_detect(mart, "ENSEMBL"))
        # connect to BioMart API
        m <- biomaRt::useMart(mart, host = "https://www.ensembl.org")
    
    if (stringr::str_detect(mart, "plants"))
        # connect to BioMart API
        m <- biomaRt::useMart(mart, host = "https://plants.ensembl.org")
    
    if (stringr::str_detect(mart, "fung"))
        # connect to BioMart API
        m <- biomaRt::useMart(mart, host = "https://fungi.ensembl.org")
    
    if (stringr::str_detect(mart, "protist"))
        # connect to BioMart API
        m <- biomaRt::useMart(mart, host = "https://protist.ensembl.org")
    
    if (stringr::str_detect(mart, "metazoa"))
        # connect to BioMart API
        m <- biomaRt::useMart(mart, host = "https://metazoa.ensembl.org")
    
    d <- biomaRt::useDataset(dataset = dataset, mart = m)
    
    # establishing a biomaRt connection and retrieving the 
    # information for the given gene list
    query <-
        biomaRt::getBM(
            attributes = as.character(c(filters, attributes)),
            filters    = as.character(filters),
            values     = as.character(genes),
            mart       = d,
            ...
        )
    
    colnames(query) <- c(filters, attributes)
    
    genes <- tibble::tibble(as.character(genes))
    colnames(genes) <- as.character(filters)
    
    tbl_biomart <-
        merge(query, genes, by = filters, incomparables = NA)
    
    please_cite_biomartr(mute_citation = mute_citation)
    return(tbl_biomart)
    
}

context("Test: biomart()")

test_that("The biomart() interface works properly..",{

    skip_on_cran()
    skip_on_travis()

    expect_gt(nrow(getMarts()), 6)
    expect_output(
            biomart(
                genes      = "GUCA2A",
                mart       = "ENSEMBL_MART_ENSEMBL",
                dataset    = "hsapiens_gene_ensembl",
                attributes = c("start_position", "end_position", "description"),
                filters    = "hgnc_symbol"
            )[1, 1:3]
        )
})

#' @title Retrieve available database releases or versions
#' of ENSEMBL
#' @description Retrieve available database releases or versions of
#' ENSEMBL.
#' @param db a character string specifying the database from which
#' available resease versions shall be retrieved:
#' \itemize{
#' \item \code{db = "ensembl"}
#' }
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # retrieve available resease versions of ENSEMBL
#' getReleases("ensembl")
#' }
#' @export
getReleases <- function(db = "ensembl") {

    if (!is.element(db, c("ensembl")))
        stop(
            "Please select one of the available databases: 'ensembl'.",
            call. = FALSE
        )

    if (db %in% c("ensembl", "ensemblgenomes")) {

        tryCatch({
        current_release <- ensembl_current_release()
        }, error = function(e)
            message(
                "The ensembl rest API '", ensembl_rest_url(), "' does not seem to be reachable.
                Could you please check whether you are connected to the internet?
                Is it possible to access the homepage '", ensembl_rest_url(),
                "' via your browser?"
            ))

        message("The current ENSEMBL release is release-", current_release, ".")
        return(paste0("release-", seq_len(current_release)))
    }
}

#' @title Helper function to test if a stable internet connection 
#' can be established.
#' @description All retrieval functions need a stable
#' internet connection to work properly. This internal function pings 
#' the google homepage and throws an error if it cannot be reached.
#' @author Hajk-Georg Drost
#' @noRd
connected.to.internet <- function() {
    if (curl::curl_fetch_memory("www.google.com")$status_code == 200) {
        return(TRUE)
    } else {
        message(
            "It seems that you are not connected to the internet. A query to www.google.com was not successful. Could you please check?"
        )
    }
}

#' @inherit getBio
#' @title RNA Sequence Retrieval
#' @description Main retrieval function for RNA sequences of an organism
#' of interest. By specifying the scientific name of an organism of interest the
#' corresponding fasta-file storing the RNA information for the organism
#' of interest can be downloaded and stored locally.
#' RNA files can be retrieved from several databases.
#'
#' @param path a character string specifying the location (a folder) in which
#' the corresponding
#' CDS file shall be stored. Default is
#' \code{path} = \code{file.path("_ncbi_downloads","RNA")}.
#' @examples
#' \dontrun{
#' # download the RNA of Arabidopsis thaliana from refseq
#' # and store the corresponding RNA file in '_ncbi_downloads/RNA'
#' file_path <- getRNA( db       = "refseq",
#'              organism = "Arabidopsis thaliana",
#'              path     = file.path("_ncbi_downloads","RNA"))
#'
#' Ath_RNA <- read_rna(file_path, format = "fasta")
#' }
#' @family getBio
#' @family rna
#' @export
getRNA <-
    function(db = "refseq",
             organism,
             reference = FALSE,
             skip_bacteria = TRUE,
             release = NULL,
             assembly_type = "toplevel",
             path = file.path("_ncbi_downloads", "RNA"),
             gunzip = FALSE,
             mute_citation = FALSE) {


      if (!is.element(db, c("refseq", "genbank", "ensembl")))
          stop(
              "Please select one of the available data bases:
              'refseq', 'genbank', or 'ensembl'.",
              call. = FALSE
          )

      if (db == "ensemblgenomes") {
              organism_name <- is.genome.available(db = db, organism = organism, details = TRUE)$display_name[1]

              if (!is.na(organism_name))
                      message("-> Starting RNA retrieval of '", organism_name, "' from ", db, " ...")
              if (is.na(organism_name))
                      message("-> Starting RNA retrieval of '", organism, "' from ", db, " ...")
      } else {
          message("-> Starting RNA retrieval of '", organism, "' from ", db, " ...")
          message("\n")
      }

      if (!file.exists(path)) {
        dir.create(path, recursive = TRUE)
      }

      if (is.element(db, c("refseq", "genbank"))) {
        info <- get_file_refseq_genbank(db, organism, reference, skip_bacteria,
                                        release, gunzip, path, assembly_type,
                                        type = "rna")
        refseq_genbank_download_post_processing(info, organism, db, path,
                                                gunzip,
                                                remove_annotation_outliers = FALSE,
                                                format = "rna",
                                                mute_citation = mute_citation)
      } else if (db %in% c("ensembl", "ensemblgenomes")) {
          rna.path <- getENSEMBL.Seq(
                          organism,
                          type = "ncrna",
                          id.type = "none",
                          release = release,
                          path = path
                  )

          ensembl_download_post_processing(rna.path, organism,
                                           format = "ncrna",
                                           gunzip = gunzip,
                                           mute_citation = mute_citation)
      }
  }

#' @inherit getBio
#' @title Retrieve a Collection: Genome, Proteome, CDS, RNA, GFF, Repeat Masker, AssemblyStats
#' @description  Main collection retrieval function for an organism of interest.
#' By specifying the scientific name of an organism of interest a collection consisting of
#' the genome file, proteome file, CDS file, RNA file, GFF file, Repeat Masker file, AssemblyStats
#' file of the organism of interest
#' can be downloaded and stored locally. Collections can be retrieved from
#' several databases.
#' @param path a character string specifying the location (a folder) in which
#' the corresponding collection shall be stored. Default is
#' \code{path} = \code{file.path("_db_downloads","collections")}.
#' @examples \dontrun{
#' # download the collection of Homo sapiens from refseq
#' # and store the corresponding genome file in '_ncbi_downloads/collection'
#'  Hsap_collection <- getCollection( db       = "refseq",
#'              organism = "Homo sapiens",
#'              path = file.path("_db_downloads","collections"))
#' # download the collection of Homo sapiens from genbank
#' # and store the corresponding genome file in '_ncbi_downloads/collection'
#'  Hsap_collection <- getCollection( db       = "genbank",
#'              organism = "Homo sapiens",
#'              path = file.path("_db_downloads","collections"))
#' # download the collection of Homo sapiens from ensembl
#' # and store the corresponding genome file in '_ncbi_downloads/collection'
#'  Hsap_collection <- getCollection( db       = "ensembl",
#'              organism = "Homo sapiens",
#'              path = file.path("_db_downloads","collections"))
#' }
#'
#' @family getBio
#' @family collection
#' @export
getCollection <-
        function(db = "refseq",
                 organism,
                 reference = TRUE,
                 skip_bacteria = TRUE,
                 release = NULL,
                 gunzip = FALSE,
                 remove_annotation_outliers = FALSE,
                 path = file.path("_db_downloads","collections"),
                 mute_citation = FALSE
        ) {

    new_name <- stringr::str_replace_all(organism," ","_")
    message("-> Starting collection retrieval (genome, proteome, cds, gff/gtf, rna, repeat masker, assembly stats) for ", new_name, " ...")

    org_exists <- is.genome.available(db = db, organism, details = TRUE)

    if (isFALSE(org_exists) || length(org_exists) == 0)
        stop("-> No entry was found for organism ",organism,". Could the name be misspelled?",  call. = FALSE)

    path <- file.path(path, db, new_name)
    if (!dir.exists(path)) dir.create(path, recursive = TRUE)



    # retrieve genome assembly
    species_genome <-
            getGenome(
                    db = db,
                    organism = organism,
                    reference = reference,
                    skip_bacteria = skip_bacteria,
                    release = release,
                    gunzip = gunzip,
                    path = path,
                    mute_citation = TRUE
            )
    message("\n")
    # retrieve proteome
    species_proteome <-
            getProteome(
                    db = db,
                    organism = organism,
                    reference = reference,
                    skip_bacteria = skip_bacteria,
                    release = release,
                    gunzip = gunzip,
                    path = path,
                    mute_citation = TRUE
            )

    message("\n")
    # retrieve coding sequences
    species_cds <-
            getCDS(
                    db = db,
                    organism = organism,
                    reference = reference,
                    skip_bacteria = skip_bacteria,
                    release = release,
                    gunzip = gunzip,
                    path = path,
                    mute_citation = TRUE
            )
    message("\n")
    # retrieve corresponding gff file
    species_gff <-
            getGFF(
                    db = db,
                    organism = organism,
                    reference = reference,
                    skip_bacteria = skip_bacteria,
                    release = release,
                    gunzip = gunzip,
                    remove_annotation_outliers = remove_annotation_outliers,
                    path = path,
                    mute_citation = TRUE
            )
    message("\n")
    if (is.element(db, c("ensembl"))) {
            species_gtf <-
                    getGTF(
                            db = db,
                            organism = organism,
                            remove_annotation_outliers = remove_annotation_outliers,
                            path = path
                    )
            message("\n")
    }

    # retrieve RNA
    species_rna <-
            getRNA(
                    db = db,
                    organism = organism,
                    reference = reference,
                    skip_bacteria = skip_bacteria,
                    path = path,
                    mute_citation = TRUE
            )
    message("\n")

    if (is.element(db, c("refseq", "genbank"))) {
            # retrieve RepeatMasker output
            species_rm <-
                    getRepeatMasker(
                            db = db,
                            organism = organism,
                            reference = reference,
                            skip_bacteria = skip_bacteria,
                            path = path,
                            mute_citation = TRUE
                    )
            message("\n")

            # retrieve assembly stats
            species_stats <-
                    getAssemblyStats(
                            db = db,
                            organism = organism,
                            reference = reference,
                            skip_bacteria = skip_bacteria,
                            path = path,
                            mute_citation = TRUE
                    )
    }
    # TODO: WHAT IS THIS ? getwd() ? This looks very dangerous
    if (!file.exists(file.path(getwd(), path, "doc"))) {
            dir.create(file.path(getwd(), path, "doc"))
    }

    doc_files <- list.files(file.path(getwd(), path))
    file.remove(file.path(getwd(), path, doc_files[stringr::str_detect(doc_files, "md5checksums.txt")]))
    doc_files <- doc_files[stringr::str_detect(doc_files, "doc_")]
    file.rename(file.path(getwd(), path, doc_files),
                file.path(getwd(), path, "doc", doc_files))

    doc_tsv <- doc_files[stringr::str_detect(doc_files, paste0(db, "[.]tsv"))]

    if (is.element(db, c("refseq", "genbank"))) {
            tsv_file <-
                    readr::read_tsv(
                            file.path(getwd(), path, "doc", doc_tsv),
                            col_types = readr::cols(
                                    file_name = readr::col_character(),
                                    organism = readr::col_character(),
                                    url = readr::col_character(),
                                    database = readr::col_character(),
                                    path = readr::col_character(),
                                    refseq_category = readr::col_character(),
                                    assembly_accession = readr::col_character(),
                                    bioproject = readr::col_character(),
                                    biosample = readr::col_character(),
                                    taxid = readr::col_integer(),
                                    infraspecific_name = readr::col_character(),
                                    version_status = readr::col_character(),
                                    release_type = readr::col_character(),
                                    genome_rep = readr::col_character(),
                                    seq_rel_date = readr::col_date(format = ""),
                                    submitter = readr::col_character()
                            ),
                            col_names = TRUE
                    )
    }

    if (is.element(db, c("ensembl"))) {
            tsv_file <-
                    readr::read_tsv(
                            file.path(getwd(), path, "doc", doc_tsv),
                            col_types = readr::cols(
                                    file_name = readr::col_character(),
                                    organism = readr::col_character(),
                                    database = readr::col_character(),
                                    download_data = readr::col_character(),
                                    assembly_name = readr::col_character(),
                                    assembly_date = readr::col_character(),
                                    genebuild_last_geneset_update = readr::col_character(),
                                    assembly_accession = readr::col_character(),
                                    genebuild_initial_release_date = readr::col_character()
                            ),
                            col_names = TRUE
                    )
    }
    message("-> Collection retrieval finished successfully!")
    message("\n")
    please_cite_biomartr(mute_citation = mute_citation)
    return(file.path(getwd(), path))
}

#' @title Retrieve Ensembl Biomart marts and datasets for a query organism
#' @description This function returns either all available biomart connections 
#' for all available organisms for which biomart access is possible, or 
#' (when specified) returns all organism specific biomart connections.
#' @param organism a character string specifying the scientific name of a 
#' query organism. Default is \code{organism} = \code{NULL}. In this case all 
#' available biomart connections are returned.
#' @param update a logical value specifying whether or not the local 
#' listMart.txt and listDatasets.txt files shall be updated by remote access
#'  to BioMart. 
#' @param mute_citation logical value indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @details
#' This function collects all available biomart connections and returns a table 
#' storing the organism for which biomart connections are available as well as 
#' the corresponding mart and database.
#' @note
#' When you run this function for the first time, the data retrieval procedure 
#' will take some time, due to the remote access to BioMart. The corresponding 
#' result is then saved in a *.txt file named "_biomart/listDatasets.txt" in the
#' \code{\link{tempdir}} directory, allowing subsequent queries to perform
#'  much faster.
#' @examples
#' \dontrun{ 
#' # returning all available biomart connections
#' head(organismBM(), 20)
#' # retrieving all available datasets and biomart connections for
#' # a specific query organism (scientific name)
#' organismBM(organism = "Homo sapiens")
#' # you can also update the downloaded version using 
#' # the "update = TRUE" argument
#' head(organismBM(update = TRUE), 20)
#' }
#' @references
#' 
#' \url{http://biomart.org/}
#' 
#' Mapping identifiers for the integration of genomic datasets with the
#' R/Bioconductor package biomaRt. Steffen Durinck, Paul T. Spellman, Ewan
#' Birney and Wolfgang Huber, Nature Protocols 4, 1184-1191 (2009).
#' 
#' BioMart and Bioconductor: a powerful link between biological databases and
#' microarray data analysis. Steffen Durinck, Yves Moreau, Arek Kasprzyk, Sean
#' Davis, Bart De Moor, Alvis Brazma and Wolfgang Huber, Bioinformatics 21,
#' 3439-3440 (2005).
#' @seealso \code{\link{getMarts}}, \code{\link{getDatasets}}, 
#' \code{\link{biomart}}, \code{\link{organismFilters}}, 
#' \code{\link{organismAttributes}}
#' @export

organismBM <- function(organism = NULL, update = FALSE, mute_citation = TRUE) {
    organism_name <- description <- mart <- dataset <- NULL
    
    message("Starting retrieval of all available BioMart datasets for ", organism, " ...")
    if (update) {
        if (file.exists(getTMPFile(file.path("_biomart", "listMarts.txt"))))
            unlink(getTMPFile(file.path("_biomart", "listMarts.txt")))
        
    }
    
    if (!file.exists(getTMPFile(file.path("_biomart", "listMarts.txt")))) {
        if (!file.exists(file.path(tempdir(), "_biomart")))
            dir.create(file.path(tempdir(), "_biomart"))
        
        all_marts <- droplevels.data.frame(getMarts())
        readr::write_tsv(
            all_marts,
            file.path(tempdir(), "_biomart", "listMarts.txt"),
            col_names = TRUE
        )
        
    }
    
    if (file.exists(getTMPFile(file.path("_biomart", "listMarts.txt"))))
        all_marts <-
            as.data.frame(readr::read_tsv(
                file.path(tempdir(), "_biomart", "listMarts.txt"),
                col_types = readr::cols(
                    "mart" = readr::col_character(),
                    "version" = readr::col_character()
                )
            ))
    
    if (update) {
        if (file.exists(file.path(tempdir(), "_biomart", "listDatasets.txt")))
            unlink(file.path(tempdir(), "_biomart", "listDatasets.txt"))
        
    }
    
    if (!file.exists(file.path(tempdir(), "_biomart", "listDatasets.txt"))) {
        remove.corrupt.marts <- which(all_marts[ , "mart"] %in% c("pride", "metazoa_variations"))
        
        if (length(remove.corrupt.marts) > 0) {
            all_marts <- all_marts[-remove.corrupt.marts , ]
        }
        
        message("Datasets for the following marts will be retrieved:")
        print(all_marts)
        
        
        all_datasets <-
            do.call(rbind, lapply(unlist(all_marts[ , "mart"]),
                                  function(mart) {
                                    message("Processing mart ", mart, " ...")
                                      df <- getDatasets(mart = mart, mute_citation = TRUE)
                                      df <-
                                          dplyr::mutate(tibble::as_tibble(df), 
                                                mart = rep(mart, nrow(df)))
                                      return(df)
                                  }))
        
        readr::write_tsv(
            all_datasets,
            file.path(tempdir(), "_biomart", "listDatasets.txt"),
            col_names = TRUE
        )
    }
    
    if (file.exists(file.path(tempdir(), "_biomart", "listDatasets.txt")))
        all_datasets <-
            readr::read_tsv(
                file.path(tempdir(), "_biomart", "listDatasets.txt"),
                col_types = readr::cols(
                    "dataset" = readr::col_character(),
                    "description" = readr::col_character(),
                    "version" = readr::col_character(),
                    "mart" = readr::col_character()
                )
            )
    
    all_datasets <- dplyr::mutate(all_datasets,
                                  organism_name = 
                               unlist(lapply(unlist(all_datasets[ , "dataset"]),
                               function(x)
                               unlist(strsplit(x, "_")[[1]][1]))))
    
    all_datasets <-
        dplyr::select(all_datasets,
                      organism_name,
                      description,
                      mart,
                      dataset,
                      version)
    
    if (!is.null(organism)) {
        organism <- unlist(stringr::str_split(organism, " "))
        organism <-
      stringr::str_c(stringr::str_to_lower(stringr::str_sub(organism[1], 1, 1)),
                           organism[2],
                           collapse = "")
        
        res <-
            dplyr::filter(all_datasets,
                          stringr::str_detect(organism_name, organism))
        
        if (dim(res)[1] == 0) {
            stop("Unfortunately, no entry for '",
                 organism,
                 "' has been found.")
        } else {
          please_cite_biomartr(mute_citation = mute_citation)
          
            return(dplyr::filter(
                all_datasets,
                stringr::str_detect(organism_name, organism)
            ))
        }
    }
    
    message("Retrieval finished successfully.")
    if (is.null(organism))
        return(all_datasets)
}

context("Test: organismBM()")

test_that("The organismBM() interface works properly..",{
        
        skip_on_cran()
        skip_on_travis()
        
        expect_output(organismBM("Homo sapiens"))
})

please_cite_biomartr <- function(mute_citation = FALSE) {
  if (!mute_citation) {
    cat("\n")
    cat("\n")
    cat("Please cite: Drost HG, Paszkowski J. Biomartr: genomic data retrieval with R. Bioinformatics (2017) 33(8): 1216-1217. doi:10.1093/bioinformatics/btw821.")
    cat("\n")
  }
}

#' @title Retrieve All Available Filters for a Specific Dataset
#' @description This funcion queries the BioMart API and returns a table
#' storing all available filters for a specific dataset.
#' @param mart a character string specifying the database (mart) for which 
#' datasets shall be listed.
#' @param dataset a character string specifying the dataset for which filters 
#' shall be listed.
#' @param mute_citation logical value indicating whether citation message should be muted.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # search for available datasets
#' # getMarts()
#' # choose database (mart): "ENSEMBL_MART_ENSEMBL"
#' # head(getDatasets(mart = "ENSEMBL_MART_ENSEMBL"), 10)
#' # choose dataset: "hsapiens_gene_ensembl"
#' head(getFilters(mart = "ENSEMBL_MART_ENSEMBL", 
#'                 dataset = "hsapiens_gene_ensembl") , 5)
#' }
#' @seealso \code{\link{getMarts}}, \code{\link{getDatasets}}, 
#' \code{\link{getAttributes}}, \code{\link{organismBM}}, 
#' \code{\link{organismFilters}}, \code{\link{organismAttributes}}
#' @export

getFilters <- function(mart, dataset, mute_citation = FALSE){
    
    if ((!is.character(mart)) || (!is.character(dataset)))
        stop("Please use a character string as mart or dataset.", call. = FALSE)
    
    message("Starting retrieval of filters information from mart ", mart, " and dataset ", dataset, " ...")
    
    
    if (stringr::str_detect(mart, "ENSEMBL"))
        # connect to BioMart API
        url <-
            paste0(
                "http://ensembl.org/biomart/martservice?type=filters&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    if (stringr::str_detect(mart, "plants"))
        # connect to BioMart API
        url <-
            paste0(
          "http://plants.ensembl.org/biomart/martservice?type=filters&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    if (stringr::str_detect(mart, "fung"))
        # connect to BioMart API
        url <-
            paste0(
           "http://fungi.ensembl.org/biomart/martservice?type=filters&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    
    if (stringr::str_detect(mart, "protist"))
        # connect to BioMart API
        url <-
            paste0(
         "http://protist.ensembl.org/biomart/martservice?type=filters&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    if (stringr::str_detect(mart, "metazoa"))
        # connect to BioMart API
        url <-
            paste0(
         "http://metazoa.ensembl.org/biomart/martservice?type=filters&dataset=",
                dataset,
                "&requestid=biomart&mart=",
                mart
            )
    
    
    testContent <- httr::content(httr::GET(url), as = "text")
    
    if ((testContent == "Attribute 'mains' does not exist\n") ||
        is.na(testContent)) {
        warning("No attributes were available for mart = ",
                mart,
                " and dataset = ",
                dataset,
                ".",
                call. = FALSE)
        filterBioMart <- data.frame(name = "NA", description = "NA")
        return(filterBioMart)
    }
    
    filterPage <- httr::handle(url)
    xmlContentFilters <- httr::GET(handle = filterPage)
    
    httr::stop_for_status(xmlContentFilters)
    
    
    tryCatch({
        # extract attribute name and attribute description
        suppressWarnings(rawDF <-
                             do.call("rbind", apply(as.data.frame(strsplit(
                                 httr::content(xmlContentFilters, as = "text", 
                                               encoding = "UTF-8"),
                                 "\n"
                             )), 1, function(x)
                                 unlist(strsplit(x, "\t")))))
        
        colnames(rawDF) <- paste0("V", seq_len(ncol(rawDF)))
        
        filterBioMart <-
            as.data.frame(rawDF[, c("V1", "V2")],
                          stringsAsFactors = FALSE,
                          colClasses = rep("character", 2))
        colnames(filterBioMart) <- c("name", "description")
        
        please_cite_biomartr(mute_citation = mute_citation)
        
        return(filterBioMart)
        
    }, error = function(e)
        stop(
            "Your input mart '",
            mart,
            "' or dataset '",
            dataset,
            "' could not be found. Please use getMarts() to choose from 
            available marts."
        ))
}

context("Test: getFilters()")

equal_lists <- function(df1, df2) {
        isTRUE(all.equal(df1, df2))
}


test_that("The getFilters() interface works properly..",{

    skip_on_cran()
    skip_on_travis()

    marts <- getMarts()$mart
    elsembl_mart <- which(stringr::str_match(marts, "ENSEMBL_MART_ENSEMBL") != "NA")

    attrib <-
        getFilters(mart = as.character(marts[elsembl_mart]),
                   dataset = "hsapiens_gene_ensembl",
                   mute_citation = TRUE)

    expect_equal(attrib[1 , 1], "chromosome_name")

})

#' @title List All Available Genomes either by kingdom, group, or subgroup
#' @description This function retrieves the names of all genomes available on
#' the NCBI ftp:// server and stores the results in a file named 'overview.txt'
#' inside the directory _ncbi_downloads' that is built inside the workspace.
#' @param db a character string specifying the database for which genome
#' availability shall be checked. Available options are:
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' \item \code{db = "ensembl"}
#' }
#' @param type a character string specifying a potential filter of available
#' genomes. Available options are:
#' \itemize{
#' \item \code{type = "all"}
#' \item \code{type = "kingdom"}
#' \item \code{type = "group"}
#' \item \code{type = "subgroup"}
#' }
#' @param subset a character string or character vector specifying a subset of
#' \code{type}. E.g. if users are interested in retrieving all
#' \code{Eukaryota} species, they can specify: \code{type = "kingdom"} and
#' \code{subset = "Eukaryota"}.
#' @param details a boolean value specifying whether only the scientific names
#' of stored genomes shall be returned (details = FALSE) or all information such
#' as
#' \itemize{
#' \item \code{organism_name}
#' \item \code{kingdoms}
#' \item \code{group}
#' \item \code{subgroup}
#' \item \code{file_size_MB}, etc.
#' }
#' @param skip_bacteria Due to its enormous dataset size (> 700MB as of July 2023),
#' the bacterial summary file will not be loaded by default anymore. If users
#' wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
#' bacterial summary file will be downloaded.
#' @author Hajk-Georg Drost
#' @details Internally this function loads the the overview.txt file from NCBI
#' and creates a directory '_ncbi_downloads' in the \code{temdir()}
#' folder to store the overview.txt file for future processing. In case the
#' overview.txt file already exists within the '_ncbi_downloads' folder and is
#' accessible within the workspace, no download process will be performed again.
#' @note
#' Please note that the ftp:// connection relies on the NCBI or ENSEMBL server
#' and cannot be accurately accessed via a proxy.
#' @examples
#' \dontrun{
#' # print details for refseq
#' listGenomes(db = "refseq")
#' # print details for all plants in refseq
#' listGenomes(db = "refseq", type = "kingdom")
#' # print details for all plant groups in refseq
#' listGenomes(db = "refseq", type = "group")
#' # print details for all plant subgroups in refseq
#' listGenomes(db = "refseq", type = "subgroup")
#' # Ensembl
#' listGenomes(db = "ensembl", type = "kingdom", subset = "EnsemblVertebrates")
#' }
#' @export

listGenomes <-
    function(db = "refseq",
             type = "all",
             subset = NULL,
             details = FALSE,
             skip_bacteria = FALSE) {
    if (!is.element(db, c("refseq", "genbank",
                          "ensembl")))
        stop(
            "Please specify a database that is supported by this function.
            E.g. 'refseq', 'genbank', or 'ensembl'.",
            call. = FALSE
        )

    if (!is.element(type, c("all", "kingdom", "group", "subgroup")))
        stop(
            "Please specify a type that is supported by this function.
            E.g. 'all', kingdom', 'group', or 'subgroup'.",
            call. = FALSE
        )

    subgroup <- division <- NULL

    if (is.element(db, c("refseq", "genbank"))) {
        # retrieve genome report overview file
        ncbi_overview <- getGENOMEREPORT()

        # get Kingdom Assembly Summary file
        AssemblyFilesAllKingdoms <-
            getKingdomAssemblySummary(db = db, skip_bacteria = skip_bacteria)

        # join tables to retrieve kingdom, group, subgroup information for
        # refseq/genbank organisms
        joined.df <-
            dplyr::inner_join(AssemblyFilesAllKingdoms, ncbi_overview,
                              by = "organism_name")

        kingdoms <- group <-  NULL

        if (type == "all") {
            if (details) {
                if (!is.null(subset)) {
                    warning(
                        "For option type = 'all' no subset can be specified.",
                        " Please select another type and then specify subset = '",
                        subset,
                        "'.",
                        call. = FALSE
                    )
                }

                return(tibble::as_tibble(joined.df))
            }

            if (!details) {
                if (!is.null(subset)) {
                    warning(
                        "For option type = 'all' no subset can be specified.",
                        " Please select another type and then specify subset = '",
                        subset,
                        "'.",
                        call. = FALSE
                    )
                }

                return(unique(joined.df$organism_name))
            }
        }

        if (type == "kingdom") {
            if (details) {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(joined.df$kingdoms)
                    ))))
                        stop(
                            "Unfortunately, not all members of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'Eukaryota' instead of 'eukaryota'.",
                            call. = FALSE
                        )

                    return(dplyr::filter(joined.df,
                                         is.element(kingdoms, subset)))
                } else {
                    return(tibble::as_tibble(joined.df))
                }
            }

            if (!details) {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(joined.df$kingdoms)
                    ))))
                        stop(
                            "Unfortunately, not all members of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'Eukaryota' instead of 'eukaryota'.",
                            call. = FALSE
                        )
                    return(unique(
                        dplyr::filter(joined.df,
                        is.element(kingdoms, subset))$organism_name
                    ))
                } else {
                    return(unique(joined.df$organism_name))
                }
            }
        }

        if (type == "group") {
            if (details) {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(joined.df$group)
                    ))))
                        stop(
                            "Unfortunately, not all memebrs of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'Eukaryota' instead of 'eukaryota'.",
                            call. = FALSE
                        )
                    return(dplyr::filter(joined.df,
                                         is.element(group, subset)))
                } else {
                    return(tibble::as_tibble(joined.df))
                }
            }

            if (!details) {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(joined.df$group)
                    ))))
                        stop(
                            "Unfortunately, not all memebrs of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'Eukaryota' instead of 'eukaryota'.",
                            call. = FALSE
                        )
                    return(unique(
                        dplyr::filter(joined.df,
                        is.element(group, subset))$organism_name
                    ))
                } else {
                    return(unique(joined.df$organism_name))
                }
            }
        }

        if (type == "subgroup") {
            if (details) {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(joined.df$subgroup)
                    ))))
                        stop(
                            "Unfortunately, not all memebrs of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'Eukaryota' instead of 'eukaryota'.",
                            call. = FALSE
                        )
                    return(dplyr::filter(joined.df,
                                         is.element(subgroup, subset)))
                } else {
                    return(tibble::as_tibble(joined.df))
                }
            }

            if (!details) {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(joined.df$subgroup)
                    ))))
                        stop(
                            "Unfortunately, not all memebrs of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'Eukaryota' instead of 'eukaryota'.",
                            call. = FALSE
                        )
                    return(unique(
                        dplyr::filter(joined.df,
                        is.element(subgroup, subset))$organism_name
                    ))
                } else {
                    return(unique(joined.df$organism_name))
                }
            }
        }
    }

    if (db %in% c("ensembl", "ensemblgenomes")) {
        if (!is.element(type, c("all", "kingdom")))
            stop(
                "Unfortunately, ENSEMBL only provides kingdom information and no group or subgroup information.",
                call. = FALSE
            )

        ensemblinfo <- getENSEMBLInfo()

        if (details) {
            if (type == "all")
                return(ensemblinfo)
            if (type == "kingdom") {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(ensemblinfo$division)
                    ))))
                        stop(
                            "Unfortunately, not all memebrs of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'EnsemblMetazoa' instead of 'ensemblmetazoa'.",
                            call. = FALSE
                        )
                    return(dplyr::filter(
                        ensemblinfo,
                        is.element(division, subset)
                    ))

                } else {
                    return(tibble::as_tibble(ensemblinfo))
                }
            }
        }

        if (!details) {
            if (type == "all")
                return(unique(ensemblinfo$name))
            if (type == "kingdom") {
                if (!is.null(subset)) {
                    if (!all(is.element(subset, names(
                        table(ensemblinfo$division)
                    ))))
                        stop(
                            "Unfortunately, not all memebrs of your specified subset '",
                            paste0(subset, ", '"),
                            " could be found. Search terms are case sensitive, so you could try to type:",
                            " 'EnsemblMetazoa' instead of 'ensemblmetazoa'.",
                            call. = FALSE
                        )
                    return(unique(
                        dplyr::filter(
                            ensemblinfo,
                            is.element(division, subset)
                        )$name
                    ))

                } else {
                    return(unique(ensemblinfo$name))
                }
            }
        }
    }
}

#' @title Retrieve information about available Ensembl Biomart databases
#' @description This funcion queries the Ensembl Biomart API and returns a table
#' storing information about all available Ensembl Biomart databases.
#' @author Hajk-Georg Drost
#' @examples
#' \dontrun{
#' # get a table of all available databases from Ensembl Biomart
#' getMarts()
#'  }
#' @seealso \code{\link{getDatasets}}, \code{\link{getAttributes}},
#' \code{\link{getFilters}}, \code{\link{organismBM}},
#' \code{\link{organismFilters}}, \code{\link{organismAttributes}}
#' @export
getMarts <- function(){

    submarts <- ensembl_divisions_short(FALSE, bacteria = FALSE)
    submarts.df <- vector("list", length(submarts))

    # Only fail if primary mart is down, else notify which are down
    for (i in seq_along(submarts)) {
      tryCatch({
        submarts.df[i] <- list(getSubMarts(submarts[i]))

        }, error = function(e) {
        message(
            "It seems like the BioMart server could not be reached for mart: ", submarts[i],
            ". This is either due to a server maintainence downtime or may be due to an instable internet connection on your side."
        )
        if (i == 1) stop()
        })

    }

    return(dplyr::bind_rows(submarts.df))
}

context("Test: getMarts()")

test_that("The getMarts() interface works properly..",{

        skip_on_cran()
        skip_on_travis()

        marts <- getMarts()
        expect_false(nrow(marts) == 0)
})

#' @title Import Repeat Masker output file
#' @description This function reads an organism specific 
#' Repeat Masker output file.
#' @param file a character string specifying the path to the file 
#' storing the Repeat Masker output (e.g. retrieved with \code{\link{getRepeatMasker}}).
#' @author Hajk-Georg Drost
#' @details This function takes a string specifying the path to the 
#' Repeat Masker output file of interest as first argument.
#' @seealso \code{\link{getRepeatMasker}}, \code{\link{read_genome}}, 
#' \code{\link{read_proteome}}, \code{\link{read_gff}}, \code{\link{read_rna}}
#' @export

read_rm <- function(file) {
    
    rm_file <- readr::read_lines(file = file, skip = 2)
    rm_file <- lapply(rm_file, function(x) {
        str.res <- unlist(stringr::str_split(x, "\\s+"))[-1]
        str.res <- str.res[1:14]
        return(str.res)
    })
    
    
    rm_file <- as.data.frame(do.call(rbind, rm_file))
    colnames(rm_file) <- c(
        "sw_score",
        "perc_div",
        "perc_del",
        "perc_insert",
        "qry_id",
        "qry_start",
        "qry_end",
        "qry_left",
        "matching_repeat",
        "repeat_id",
        "matching_class",
        "no_bp_in_complement",
        "in_repeat_start",
        "in_repeat_end"
    )
    qry_end <- qry_start <- NULL
    
    rm_file <- tibble::as_tibble(rm_file)
    
    nrow_before_filtering <- nrow(rm_file)
        
    suppressWarnings(rm_file <- dplyr::mutate(rm_file,
                             qry_start = as.integer(qry_start),
                             qry_end = as.integer(qry_end)))
    
    
    rm_file <-
        dplyr::filter(
            rm_file,
            !is.na(qry_start),
            !is.na(qry_end)
        )
    
    rm_file <-
        dplyr::mutate(
            rm_file,
            qry_width = as.integer(qry_end - qry_start + 1L))

    nrow_after_filtering <- nrow(rm_file)
    
    
    if ((nrow_before_filtering - nrow_after_filtering) > 0)
        message((nrow_before_filtering - nrow_after_filtering) + 1 , " out of ",nrow_before_filtering," rows ~ ", round(((nrow_before_filtering - nrow_after_filtering) + 1) / nrow_before_filtering, 3) , "% were removed from the imported RepeatMasker file, ",
                "because they contained 'NA' values in either 'qry_start' or 'qry_end'.")
    
    return(rm_file)
}

#' @title Perform Meta-Genome Retrieval of all organisms in all kingdoms of life
#' @description Download genomes, proteomes, cds, gff, rna, or assembly stats
#' files of individual species of all kingdoms of life.
#' @param db a character string specifying the database from which the genome
#' shall be retrieved:
#' \itemize{
#' \item \code{db = "refseq"}
#' \item \code{db = "genbank"}
#' \item \code{db = "emsembl"}
#' \item \code{db = "ensemblgenomes"}
#' }
#' @param type type of sequences that shall be retrieved. Options are:
#' \itemize{
#'  \item \code{type = "genome"} :
#'  for genome assembly retrieval; see also \code{\link{getGenome}}),
#'  \item \code{type = "proteome"} :
#'  (for proteome retrieval; see also \code{\link{getProteome}}),
#'  \item \code{type = "cds"} :
#'  (for coding sequence retrieval; see also \code{\link{getCDS}}),
#'  \item \code{type = "gff"} :
#' (for annotation file retrieval in gff format; see also \code{\link{getGFF}}),
#' \item \code{type = "gtf"} :
#' (for annotation file retrieval in gtf format
#' (only for ensembl and ensemblgenomes); see also \code{\link{getGTF}}),
#'  \item \code{type = "rna"} :
#'  (for RNA file retrieval in fasta format; see also \code{\link{getRNA}}),
#'  \item \code{type = "rm"} :
#'  (for Repeat Masker output file retrieval; see also
#'  \code{\link{getRepeatMasker}}),
#'  \item \code{type = "assemblystats"} (for genome assembly quality stats
#'  file retrieval; see also \code{\link{getAssemblyStats}}).
#'  }
#' @param reference a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database
#' as either a reference genome or a representative genome. Options are:
#' \itemize{
#' \item \code{reference = FALSE} (Default): all organisms (reference, representative, and non-representative genomes) are downloaded.
#' \item \code{reference = TRUE}: organisms that are downloaded must be either a reference or representative genome. Thus, most genomes which are usually non-reference genomes
#' will not be downloaded.
#' }
#' @author Hajk-Georg Drost
#' @details This function aims to perform bulk retrieval of all genomes
#' of species for all kingdoms of life.
#' @examples
#' \dontrun{
#' # download all genomes from refseq
#' meta.retrieval.all(db = "refseq", type = "genome")
#' # download all vertebrate genomes from genbank
#' meta.retrieval.all(db = "genbank", type = "genome")
#' # download all vertebrate genomes from ensemblgenomes
#' meta.retrieval.all(db = "genbank", type = "ensemblgenomes")
#' }
#' @return a character vector storing the file paths of the retrieved files.
#' @family meta_retrival
#' @export
meta.retrieval.all <- function(db = "refseq", type = "genome", reference = FALSE) {
    message("Starting ", type, " meta retrieval process of all species individually from database: ", db," ...")
    # retrieve all genomes from all kingdoms of life
    paths <- unlist(lapply(getKingdoms(db = db),
                           function(x) meta.retrieval(x, type = type,
                                                            db = db,
                                                            group = NULL,
                                                      reference = reference)))
    message("Meta retrieval process... finished!")
    return(paths)
}

#' @title Helper function to test connection and availability of
#' queried FTP files.
#' @description To make sure that the automatically generated query path to
#' the ftp stored file on NCBI or ENSEMBL actually exists, this helper function
#' makes a test query.
#' @author Hajk-Georg Drost
#' @noRd
#' @import curl
exists.ftp.file <- function(url, file.path) {

    tryCatch({test_connection <- curl::curl_fetch_memory(url = url)}, error = return(FALSE))

    if (test_connection$status_code == 226) {
        con <-
            curl::curl(url = url,
                       open = "rb",
                       curl::new_handle(dirlistonly = TRUE))

        ftp.content <-
            suppressMessages(readr::read_delim(con,
                                               delim = "\n",
                                               col_names = FALSE))
        close(con)
        return(is.element(as.character(basename(file.path)),
                          as.character(ftp.content$X1)))
    } else {
        return(FALSE)
    }
}

clean_species_names_helper <- function(folder_files, gunzip = TRUE, as.list = TRUE) {

    file_ext <- "[.]*a$"

    if (any(stringr::str_detect(folder_files, "[.]faa.gz$"))) {
        seq_type <- "ncbi_protein"
        file_ext <- "[.]faa$"
    }

    if (any(stringr::str_detect(folder_files, "[.]fna.gz$"))) {
        seq_type <- "ncbi_nucleotide"
        file_ext <- "[.]fna$"
    }
    if (any(stringr::str_detect(folder_files, "[.]gff.gz$"))) {
        seq_type <- "ncbi_gff"
        file_ext <- "[.]gff$"
    }
    if (any(stringr::str_detect(folder_files, "[.]out.gz$"))) {
        seq_type <- "ncbi_rm"
        file_ext <- "[.]out$"
    }
    if (any(stringr::str_detect(folder_files, "[.]gff3.gz$"))) {
        seq_type <- "ensembl_gff3"
        file_ext <- "[.]gff3$"
    }
    if (any(stringr::str_detect(folder_files, "[.]gtf.gz$"))) {
        seq_type <- "ensembl_gtf"
        file_ext <- "[.]gtf$"
    }
    if (any(stringr::str_detect(folder_files, "[.]fa.gz$"))) {
        seq_type <- "ensembl_fasta"
        file_ext <- "[.]fa$"
    }


    # remove doc, md5checksum files, and already unzipped files
    find_doc <- which(stringr::str_detect(folder_files, "doc_"))
    find_md5 <- which(stringr::str_detect(folder_files, "md5checksum"))
    find_documentaion <- which(stringr::str_detect(folder_files, "documentation"))
    find_unzipped_files <- which(stringr::str_detect(folder_files, file_ext))
    find_rs_graphics <- which(stringr::str_detect(folder_files, "rs-graphics-"))

    if (length(c(find_doc, find_md5, find_documentaion, find_unzipped_files)) > 0) {
        folder_files_reduced <- folder_files[-c(find_doc, find_md5, find_documentaion, find_unzipped_files, find_rs_graphics)]
    }

    if (length(folder_files_reduced) == 0) {
        return(folder_files[-c(find_doc, find_md5, find_documentaion)])
    } else {
        input_files <- folder_files_reduced
    }

    input_files_without_appendix <- unlist(lapply(input_files, function(x) return(unlist(stringr::str_split(x, "[.]"))[1])))

    file_ext <- stringr::str_replace(file_ext, "\\$", "")
    file_ext <- stringr::str_replace(file_ext, "\\[.]", "")

    if (gunzip)
        output_files <- paste0(tidy_name(input_files_without_appendix), ".", file_ext)

    if (!gunzip)
        output_files <- paste0(tidy_name(input_files_without_appendix),".",file_ext,".gz")

    if (as.list) return(list(output_files = output_files, input_files = input_files,
                             input_files_without_appendix = input_files_without_appendix))
    return(output_files)
}

#' @title Retrieve ENSEMBL info file
#' @description Retrieve species and genome information from
#' http://rest.ensembl.org/info/species?content-type=application/json/.
#' @param update logical, default TRUE. Update cached list, if FALSE use existing
#' (if it exists)
#' @author Hajk-Georg Drost
#' @return a tibble table storing info for all available ENSEMBL divisions.
#' @examples
#' \dontrun{
#' # look at available divisions
#' ensembl_divisions()
#' # retrieve information for all ENSEMBL divisions at once
#' test <- getENSEMBLInfo()
#' test
#' # retrieve information for a particular ENSEMBL division (e.g. EnsemblVertebrates)
#' test_vertebrates <- get.ensembl.info(update = TRUE, division = "EnsemblVertebrates")
#' test_vertebrates
#' }
#' @seealso \code{\link{ensembl_divisions}}, \code{\link{get.ensembl.info}}, \code{\link{getKingdomAssemblySummary}}
#' @export
getENSEMBLInfo <- function(update = TRUE) {
  all_divisions <- ensembl_divisions()
  ENSEMBLInfoTable <- vector("list", length(all_divisions))

  for (i in seq_len(length(all_divisions))) {
    cat("Starting information retrieval for:", all_divisions[i])
    cat("\n")
    ENSEMBLInfoTable[[i]] <-
      get.ensembl.info(update, division = all_divisions[i])
  }

  return(dplyr::bind_rows(ENSEMBLInfoTable))
}

ensembl_assembly_hits <- function(organism) {
  ensembl_summary <-
    suppressMessages(is.genome.available.ensembl(
      organism = organism,
      db = "ensembl",
      details = TRUE
    ))

  if (nrow(ensembl_summary) == 0) {
    message(
      "Unfortunately, organism '",
      organism,
      "' does not exist in this database. ",
      "Could it be that the organism name is misspelled? Thus, download has been omitted."
    )
    return(FALSE)
  }
  ensembl_summary <-
    ensembl_summaries_filter(ensembl_summary, organism)
  return(ensembl_summary)
}

ensembl_summaries_filter <- function(ensembl_summary, organism) {
  if (nrow(ensembl_summary) > 1) {
    taxon_id <- assembly <- name <- accession <- NULL
    ensembl_summary_copy <- ensembl_summary
    if (is.taxid(organism)) {
      ensembl_summary <-
        dplyr::filter(ensembl_summary,
                      taxon_id == as.integer(organism),
                      !is.na(assembly))
    } else {
      ensembl_summary <-
        dplyr::filter(
          ensembl_summary,
          (name == lower_cap_underscore_organism_name(organism)) |
            (accession == organism),
          !is.na(assembly)
        )
      if (nrow(ensembl_summary) == 0) {
        ensembl_summary <-
          dplyr::filter(ensembl_summary_copy, !is.na(assembly))
      }
    }
    if (nrow(ensembl_summary) == 0) {
      print(ensembl_summary_copy)
      stop("All assemblies removed by filter (more info in lines above)")
    }
  }
  return(ensembl_summary)
}

is.taxid <- function(x) {
  return(stringr::str_count(x, "[:digit:]") == nchar(x))
}

validate_release <- function(release, ensembl_summary) {
  if (!is.null(release)) {
    release <- as.numeric(release)
    if (!is.element(release, ensembl_all_releases()))
      stop("Please provide a release number that is supported by ENSEMBL.", call. = FALSE)
  } else release <- ensembl_current_release(ensembl_summary$division[1])

  if (is.numeric(release)) {
    if (release <= 46) {
      message("ensembl release <= 46 is not supported")
      return(FALSE)
    }
  }
  return(release)
}

#' Check if genome is available in ensembl
#' @param organism which organism, scientific name
#' @param details logical, default FALSE, return logical only, else table of info
#' @param division "EnsemblVertebrates", alternatives: "EnsemblPlants",
#' "EnsemblFungi", "EnsemblMetazoa" and "EnsemblProtists"
#' @return logical, if details is TRUE, then returns table of details.
#' @noRd
is.genome.available.ensembl <- function(db = "ensembl",
                                        organism,
                                        details = FALSE,
                                        divisions = ensembl_divisions()) {
  name <- accession <- accession <- assembly <- taxon_id <- NULL
  new.organism <- stringr::str_replace_all(organism, " ", "_")

  # For each ensembl division, check if it exists
  for (division in ensembl_divisions()) {
    ensembl.available.organisms <- get.ensembl.info(division = division)
    ensembl.available.organisms <-
      dplyr::filter(ensembl.available.organisms, !is.na(assembly))

    if (!is.taxid(organism)) {
      selected.organism <-
        dplyr::filter(
          ensembl.available.organisms,
          stringr::str_detect(name,
                              stringr::str_to_lower(new.organism)) |
            accession == organism,!is.na(assembly)
        )
    } else {
      selected.organism <-
        dplyr::filter(
          ensembl.available.organisms,
          taxon_id == as.integer(organism),!is.na(assembly)
        )

    }
    if (nrow(selected.organism) > 0)
      break
  }



  if (!details) {
    if (nrow(selected.organism) == 0) {
      organism_no_hit_message_zero(organism, db)
      return(FALSE)
    }
    if (nrow(selected.organism) > 0) {
      message("A reference or representative genome assembly is available for '",
              organism,
              "'.")
      if (nrow(selected.organism) > 1) {
        organism_no_hit_message_more_than_one(organism, db)
      }
      return(TRUE)
    }
  }
  if (details)
    return(selected.organism)
}

organism_no_hit_message_zero <- function(organism, db) {
  message(
    "Unfortunatey, no entry for '",
    organism,
    "' was found in the '",
    db,
    "' database. ",
    "Please consider specifying ",
    paste0("'db = ", dplyr::setdiff(
      c("refseq", "genbank", "ensembl", "ensemblgenomes", "uniprot"),
      db
    ), collapse = "' or "),
    "' to check whether '",
    organism,
    "' is available in these databases."
  )
}

organism_no_hit_message_more_than_one <- function(organism, db) {
  message(
    "More than one entry was found for '",
    organism,
    "'.",
    " Please consider to run the function 'is.genome.available()' and specify 'is.genome.available(organism = ",
    organism,
    ", db = ",
    db,
    ", details = TRUE)'.",
    " This will allow you to select the 'assembly_accession' identifier that can then be ",
    "specified in all get*() functions."
  )
}

collection_table <- function(division = "EnsemblBacteria") {
  base_name_file <- paste0(division, ".txt")
  local_file <- file.path(tempdir(), base_name_file)

  if (!file.exists(local_file)) {
    url <- paste0(ensembl_ftp_server_url(division),"/",
                  ensembl_ftp_server_url_release(division),
                  "species_", base_name_file)
    tryCatch({
      custom_download(
        url,
        destfile = local_file,
        mode = "wb"
      )
    }, error = function(e) {
      message(
        "Something went wrong when accessing the the file",
        " Are you connected to the internet? ",
        "Is the homepage '", url,"' ",
        "currently available? Could it be that the scientific name is mis-spelled or includes special characters such as '.' or '('?"
      )
    })
  }
  suppressWarnings(
    collection <-
      readr::read_delim(
        local_file,
        delim = "\t",
        quote = "\"",
        escape_backslash = FALSE,
        col_names = c(
          "name",
          "species",
          "division",
          "taxonomy_id",
          "assembly",
          "assembly_accession",
          "genebuild",
          "variation",
          "microarray",
          "pan_compara",
          "peptide_compara",
          "genome_alignments",
          "other_alignments",
          "core_db",
          "species_id"
        ),
        col_types = readr::cols(
          name = readr::col_character(),
          species = readr::col_character(),
          division = readr::col_character(),
          taxonomy_id = readr::col_integer(),
          assembly = readr::col_character(),
          assembly_accession = readr::col_character(),
          genebuild = readr::col_character(),
          variation = readr::col_character(),
          microarray = readr::col_character(),
          pan_compara = readr::col_character(),
          peptide_compara = readr::col_character(),
          genome_alignments = readr::col_character(),
          other_alignments = readr::col_character(),
          core_db = readr::col_character(),
          species_id = readr::col_integer()
        ),
        comment = "#"
      )
  )
}

get_collection_id <- function(ensembl_summary) {
  division <- ensembl_summary$division[1]
  if (!(division %in% c("EnsemblBacteria", "EnsemblFungi",
                                           "EnsemblProtists")))
    return("") # Only these have collection folder structure

  get.org.info <- ensembl_summary[1,]
  collection_info <- collection_table(division)
  assembly <- NULL
  collection_info <-
    dplyr::filter(collection_info,
                  assembly == gsub("_$", "", get.org.info$assembly))

  if (nrow(collection_info) == 0) {
    message(
      "Unfortunately organism '",
      ensembl_summary$display_name,
      "' could not be found. Have you tried another database yet? ",
      "E.g. db = 'ensembl'? Thus, download for this species is omitted."
    )
    return(FALSE)
  }

  if (is.na(collection_info$core_db[1]) || collection_info$core_db[1] == "N") {
    # TODO make sure this is safe
    # In theory this should mean that the file exist outside collection folders
    return("")
  }
  collection <- paste0(paste0(unlist(
    stringr::str_split(collection_info$core_db[1], "_")
  )[1:3], collapse = "_"), "/")

  return(collection)
}


assembly_summary_and_rest_status <- function(organism) {
  ensembl_summary <- ensembl_assembly_hits(organism)
  if (isFALSE(ensembl_summary)) return(FALSE)

  # Check if assembly can be reached (TODO: remove, as this is already done)
  new.organism <- ensembl_proper_organism_name(ensembl_summary)
  rest_url <- ensembl_rest_url_assembly(new.organism)
  rest_api_status <- test_url_status(url = rest_url, organism = organism)
  if (isFALSE(rest_api_status)) return(FALSE)
  json.qry.info <- rest_api_status
  return(list(new.organism = new.organism,
              ensembl_summary = ensembl_summary,
              json.qry.info = json.qry.info))
}

write_assembly_docs_ensembl <- function(genome.path, new.organism, db, json.qry.info,
                                        path = dirname(genome.path[1]), append = NULL) {
  # generate Genome documentation
  sink(file.path(path, paste0("doc_", new.organism, "_db_", db, append,".txt")))

  cat(paste0("File Name: ", genome.path[1]))
  cat("\n")
  cat(paste0("Download Path: ", genome.path[2]))
  cat("\n")
  cat(paste0("Organism Name: ", new.organism))
  cat("\n")
  cat(paste0("Database: ", db))
  cat("\n")
  cat(paste0("Download_Date: ", date()))
  cat("\n")
  cat(paste0("assembly_name: ", json.qry.info$assembly_name))
  cat("\n")
  cat(paste0("assembly_date: ", json.qry.info$assembly_date))
  cat("\n")
  cat(
    paste0(
      "genebuild_last_geneset_update: ",
      json.qry.info$genebuild_last_geneset_update
    )
  )
  cat("\n")
  cat(paste0(
    "assembly_accession: ",
    json.qry.info$assembly_accession
  ))
  cat("\n")
  cat(
    paste0(
      "genebuild_initial_release_date: ",
      json.qry.info$genebuild_initial_release_date
    )
  )

  sink()

  doc <- tibble::tibble(
    file_name = genome.path[1],
    download_path = genome.path[2],
    organism = new.organism,
    database = db,
    download_data = date(),
    assembly_name = ifelse(!is.null(json.qry.info$assembly_name), json.qry.info$assembly_name, "none"),
    assembly_date = ifelse(!is.null(json.qry.info$assembly_date), json.qry.info$assembly_date, "none"),
    genebuild_last_geneset_update = ifelse(!is.null(json.qry.info$genebuild_last_geneset_update), json.qry.info$genebuild_last_geneset_update, "none"),
    assembly_accession = ifelse(!is.null(json.qry.info$assembly_accession), json.qry.info$assembly_accession, "none"),
    genebuild_initial_release_date = ifelse(!is.null(json.qry.info$genebuild_initial_release_date), json.qry.info$genebuild_initial_release_date, "none")
  )

  readr::write_tsv(doc, file = file.path(
    path,
    paste0("doc_", new.organism, "_db_", db, append, ".tsv"))
  )
  return(invisible(NULL))
}

ensembl_download_post_processing <- function(genome.path, organism, format,
                                             remove_annotation_outliers = FALSE,
                                             gunzip = FALSE, db = "ensembl",
                                             mute_citation = FALSE) {
  if (is.logical(genome.path[1]) && !genome.path) {
    return(FALSE)
  } else {
    # Format specific behaviors
    if (format == "gtf") {append <- "_gtf_"} else append <- NULL

    info <- assembly_summary_and_rest_status(organism)

    write_assembly_docs_ensembl(genome.path, new.organism = info$new.organism,
                                db = db, json.qry.info = info$json.qry.info, append = append)
    local_file <- genome.path[1]

    gunzip_and_check(local_file, gunzip, remove_annotation_outliers, format, mute_citation)
  }
}

#' @title Import Genome Assembly Stats File
#' @description This function reads an organism specific Genome Assembly
#' Stats file that was retrieved with \code{\link{getAssemblyStats}}.
#' @param file a character string specifying the path to the file storing
#' the Genome Assembly Stats file.
#' @param type a tibble object, either \code{type = "raw"} to import the entire genome assembly
#' stats file or \code{type = "stats"} to import overall statistics including
#' all chromosomes, mitochondria and plastids.
#' @author Hajk-Georg Drost
#' @details This function takes a string specifying the path to the Genome
#' Assembly Stats file of interest (e.g. the path returned by
#' \code{\link{getAssemblyStats}}) and imports it.
#' @seealso \code{\link{getAssemblyStats}}, \code{\link{read_genome}},
#' \code{\link{read_proteome}}, \code{\link{read_cds}}, \code{\link{read_gff}}
#' @export
read_assemblystats <- function(file, type = "raw") {


    if (!is.element(type, c("raw", "stats")))
        stop(
            "Please choose a type that is supported byt this function,
            e.g. type = 'raw' or type = 'stats'.",
            call. = FALSE
        )

    unit_name <- molecule_name <- statistic <- NULL

    suppressMessages(assemblystats_file <- readr::read_delim(
        file,
        delim = "\t",
        col_names = c(
            "unit_name",
            "molecule_name",
            "molecule_type",
            "sequence_type",
            "statistic",
            "value"
        ),
        col_types = readr::cols(
            "unit_name" = readr::col_character(),
            "molecule_name" = readr::col_character(),
            "molecule_type" = readr::col_character(),
            "sequence_type" = readr::col_character(),
            "statistic" = readr::col_character(),
            "value" = readr::col_number()
        ),
        comment = "#"
    ))

    if (type == "stats") {
    # select all features (including chromosomes, mitochondria, and plastids)
        assemblystats_file.all.features <-
            dplyr::filter(assemblystats_file,
                          unit_name == "all",
                          molecule_name == "all")

        total.length <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "total-length")
        spanned.gaps <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "spanned-gaps")
        unspanned.gaps <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "unspanned-gaps")
        region.count <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "region-count")
        scaffold.count <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "scaffold-count")
        scaffold.N50 <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "scaffold-N50")
        scaffold.L50 <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "scaffold-L50")
        scaffold.N75 <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "scaffold-N75")
        scaffold.N90 <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "scaffold-N90")
        contig.count <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "contig-count")
        contig.N50 <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "contig-N50")
        total.gap.length <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "total-gap-length")
        molecule.count <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "molecule-count")
        top.level.count <-
            dplyr::filter(assemblystats_file.all.features,
                          statistic == "top-level-count")

        assemblystats_file.all.features.short <-
            tibble::tibble(
                total_length = ifelse(nrow(total.length) > 0,
                                      total.length$value, NA),
                spanned_gaps = ifelse(nrow(spanned.gaps) > 0,
                                      spanned.gaps$value, NA),
                unspanned_gaps = ifelse(nrow(unspanned.gaps) > 0,
                                        unspanned.gaps$value, NA),
                region_count = ifelse(nrow(region.count) > 0,
                                      region.count$value, NA),
                scaffold_count = ifelse(nrow(scaffold.count) > 0,
                                        scaffold.count$value, NA),
                scaffold_N50 = ifelse(nrow(scaffold.N50) > 0,
                                      scaffold.N50$value, NA),
                scaffold_L50 = ifelse(nrow(scaffold.L50) > 0,
                                      scaffold.L50$value, NA),
                scaffold_N75 = ifelse(nrow(scaffold.N75) > 0,
                                      scaffold.N75$value, NA),
                scaffold_N90 = ifelse(nrow(scaffold.N90) > 0,
                                      scaffold.N90$value, NA),
                contig_count = ifelse(nrow(contig.count) > 0,
                                      contig.count$value, NA),
                contig_N50 = ifelse(nrow(contig.N50) > 0,
                                    contig.N50$value, NA),
                total_gap_length = ifelse(nrow(total.gap.length) > 0,
                                          total.gap.length$value, NA),
                molecule_count = ifelse(nrow(molecule.count) > 0,
                                        molecule.count$value, NA),
                top_level_count = ifelse(nrow(top.level.count) > 0,
                                         top.level.count$value, NA)
            )

        return(assemblystats_file.all.features.short)
    }

    if (type == "raw") {
        return(assemblystats_file)
    }
}

#' @title Retrieve All Organism Names Stored on refseq
#' @description This function extracts all organism names (scientific names)
#' for which genomes, proteomes, and CDS files are stored on 
#' the NCBI refseq server.
#' @author Hajk-Georg Drost
#' @export

refseqOrganisms <- function() {
    refseq_kingdoms <-   c(
        "archaea",
        "bacteria",
        "fungi",
        "invertebrate",
        "plant",
        "protozoa",
        "vertebrate_mammalian",
        "vertebrate_other"
    )

    all_refseqOrgs <-
        as.vector(unlist(lapply(refseq_kingdoms, function(kingdom)
            strsplit(
                RCurl::getURL(
                    paste0(
                        "ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/",
                        kingdom,
                        "/"
                    ),
                    ftp.use.epsv = FALSE,
                    dirlistonly = TRUE
                ),
                "\n"
            ))))
    
    
    all_refseqOrgs <-
        stringr::str_replace(all_refseqOrgs, "_", " ")
    
    return(all_refseqOrgs)
}

#' @title Import Proteome as Biostrings or data.table object
#' @description This function reads an organism specific proteome stored in a
#' defined file format.
#' @param file a character string specifying the path to the file storing
#' the proteome.
#' @param format a character string specifying the file format used to store the
#' genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.
#' @param obj.type a character string specifying the object stype in which the
#' genomic sequence shall be represented.
#' Either as \code{obj.type = "Biostrings"} (default) or as
#' \code{obj.type = "data.table"}.
#' @param ... additional arguments that are used by
#' \code{\link[seqinr]{read.fasta}}.
#' @author Hajk-Georg Drost
#' @details This function takes a string specifying the path to the
#' proteome file of interest as first argument.
#'
#' It is possible to read in different proteome file standards such as
#' \emph{fasta} or \emph{genebank}.
#' @return Either a \code{Biostrings} or \code{data.table} object.
#' @family readers
#' @family proteome
#' @import Biostrings
#' @export
read_proteome <-
    function(file,
             format = "fasta",
             obj.type = "Biostrings",
             ...) {
        if (!is.element(format, c("fasta", "gbk")))
            stop("Please choose a file format that is supported by
                 this function.",
                 call. = FALSE)

        if (!is.element(obj.type, c("Biostrings", "data.table")))
            stop(
                "Please specify a valid object type:
                obj.type = 'Biostrings' (default) or obj.type = 'data.table'.",
                call. = FALSE
            )

        if (!file.exists(file))
            stop("The file path you specified does not seem to exist: '", file,"'.", call. = FALSE)


        geneids <- NULL

        if (obj.type == "Biostrings") {
            tryCatch({
                proteome <-
                    Biostrings::readAAStringSet(filepath = file,
                                                format = format, ...)
            }, error = function(e) {
                stop(
                    paste0(
                        "File ",
                        file,
                        " could not be read properly. \n",
                        "Please make sure that ",
                        file,
                        " contains only amino acid sequences and is in ",
                        format,
                        " format."
                    ),
                    call. = FALSE
                )
            })

            return(proteome)
        }

        if (obj.type == "data.table") {
            tryCatch({
                proteome <-
                    Biostrings::readAAStringSet(filepath = file,
                                                format = format, ...)
                proteome_names <-
                    as.vector(unlist(lapply(proteome@ranges@NAMES, function(x) {
                        return(strsplit(x, " ")[[1]][1])
                    })))
                proteome.dt <-
                    data.table::data.table(geneids = proteome_names,
                                           seqs =
                                               tolower(as.character(proteome)))

                data.table::setkey(proteome.dt, geneids)

            }, error = function(e) {
                stop(
                    paste0(
                        "File ",
                        file,
                        " could not be read properly. \n",
                        "Please make sure that ",
                        file,
                        " contains only amino acid sequences and is in ",
                        format,
                        " format."
                    ),
                    call. = FALSE
                )
            })

            return(proteome.dt)
        }
    }

context("Test: getMetaGenomeAnnotations()")

test_that("The getMetaGenomeAnnotations() throws error when wrong 'name'
          argument is selected..",{
            skip_on_cran()
            skip_on_travis()
            
    expect_error(getMetaGenomeAnnotations(name = "somethingelse"))
})

---
title: "Meta-Genome Retrieval"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Meta-Genome Retrieval}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, message = FALSE}
options(width = 750)
knitr::opts_chunk$set(
  comment = "#>",
  error = FALSE,
  tidy = FALSE)
```

> **_NOTE:_** To make sure that you have a sufficiently stable (internet) connection between R and the respective databases, please set the default `timeout` setting __on your local machine__ from 60sec to at least 30000sec before running any retrieval functions via:

```r
options(timeout = 30000)
```


## Topics
- [1. Perform Meta-Genome Retrieval for Specific Kingdoms of Life](#perform-meta-genome-retrieval)
    - [1.1 Retrieve Genomic Sequences](#retrieve-genomic-sequences)
        - [1.1.1 Retrieval from `NCBI RefSeq`](#retrieval-from-ncbi-refseq)
        - [1.1.2 Retrieval from `NCBI Genbank`](#retrieval-from-ncbi-genbank)
        - [1.1.3 Retrieval from `ENSEMBL`](#retrieval-from-ensembl)
    - [1.2 Retrieve Protein Sequences](#retrieve-protein-sequences)
        - [1.2.1 Retrieval from `NCBI RefSeq`](#retrieval-from-ncbi-refseq-1)
        - [1.2.2 Retrieval from `NCBI Genbank`](#retrieval-from-ncbi-genbank-1)
        - [1.2.3 Retrieval from `ENSEMBL`](#retrieval-from-ensembl-1)
    - [1.3 Retrieve CDS Sequences](#retrieve-cds-sequences)
        - [1.3.1 Retrieval from `NCBI RefSeq`](#retrieval-from-ncbi-refseq-2)
        - [1.3.2 Retrieval from `NCBI Genbank`](#retrieval-from-ncbi-genbank-2)
        - [1.3.3 Retrieval from `ENSEMBL`](#retrieval-from-ensembl-2)
    - [1.4 Retrieve GFF files](#retrieve-gff-files)
    - [1.5 Retrieve GTF files](#retrieve-gtf-files)
    - [1.6 Retrieve RNA sequences](#retrieve-rna-sequences)
        - [1.6.1 Retrieval from `NCBI RefSeq`](#retrieval-from-ncbi-refseq-3)
        - [1.6.2 Retrieval from `NCBI Genbank`](#retrieval-from-ncbi-genbank-3)
        - [1.6.3 Retrieval from `ENSEMBL`](#retrieval-from-ensembl-3)
    - [1.7 Retrieve Repeat Masker Repeat Annotation File](#retrieve-repeat-masker-sequences)
        - [1.7.1 Retrieval from `NCBI RefSeq`](#retrieval-from-ncbi-refseq-4)
        - [1.7.2 Retrieval from `NCBI Genbank`](#retrieval-from-ncbi-genbank-4)
- [2. Retrieve groups or subgroups of species ](#retrieve-groups-or-subgroups-of-species)
    - [2.1 Example retrieval of all `Gammaproteobacteria` genomes from `NCBI RefSeq`](#example-retrieval-of-all-gammaproteobacteria-genomes-from-ncbi-refseq)
    - [2.2 Example retrieval of all `Adenoviridae` genomes from `NCBI RefSeq`](#example-retrieval-of-all-adenoviridae-genomes-from-ncbi-refseq)
- [3. Meta retrieval of genome assembly quality information](#meta-retrieval-of-genome-assembly-quality-information)
- [4. Retrieve data from metagenome projects such as `human gut metagenome` project from `NCBI Genbank`](#metagenome-project-retrieval-from-ncbi-genbank)
- [5. Retrieve Individual Genomes for all Species in the Tree of Life](#retrieve-individual-genomes-for-all-species-in-the-tree-of-life)
    - [5.1 Example: Genome Retrieval](#genome-retrieval)
    - [5.2 Example: Proteome Retrieval](#proteome-retrieval)

## Perform Meta-Genome Retrieval

The number of genome assemblies generated and stored in sequence databases is growing exponentially every year. With the availability of this growing amount of genomic data, meta-genomics studies become more and more popular. By using this bulk of genomes for comparing them to thousands of other genomes
new structural patterns and evolutionary insights can be obtained.
However, the first step in any meta-genomics study is the retrieval of the genomes, proteomes, coding sequences or annotation files that shall be compared
and investigated. For this purpose, the `meta.retrieval()` and `meta.retrieval.all()` functions allows users to perform straightforward meta-genome retrieval of hundreds of genomes, proteomes, CDS, etc in R. Finally, in addition to the retrieved sequence information the `meta.retrieval()` and `meta.retrieval.all()` functions will
generate a `summary file` which contains information about the genome version, genome status, submitter, etc for each organism to promote
computational and scientific reproducibility of the meta-genomics study at hand. This `summary file` can for example be attached as `Supplementary Data` to the respective study. 


#### Getting Started

The `meta.retrieval()` and `meta.retrieval.all()` functions aim to simplify the genome retrieval and computational reproducibility process for meta-genomics studies. Both functions allow users to either download genomes, proteomes, CDS, etc for species within a specific kingdom or subgroup of life (`meta.retrieval()`) or of all species of all kingdoms (`meta.retrieval.all()`). Before `biomartr` users had to write `shell` scripts to download respective genomic data. However, since many meta-genomics packages exist for the R programming language, I implemented this functionality for easy integration into existing R workflows and for easier reproducibility.

For example, the pipeline logic of the [magrittr](https://github.com/tidyverse/magrittr) package can be used with
`meta.retrieval()` and `meta.retrieval.all()` as follows.

```{r,eval=FALSE}
# download all vertebrate genomes, then apply ...
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "genome") %>% ...
```

Here `...` denotes any subsequent meta-genomics analysis. Hence, `meta.retrieval()` enables the pipeline methodology for meta-genomics. 

### Retrieve Genomic Sequences

To retrieve a list of all available kingdoms stored in the `NCBI RefSeq`,  `NCBI Genbank`, and `ENSEMBL` databases users can consult the `getKingdoms()` function which stores a list of all available kingdoms of life for the corresponding database. 

### Example `NCBI RefSeq`:

```{r, eval=FALSE}
getKingdoms(db = "refseq")
```

```
[1] "archaea"              "bacteria"             "fungi"                "invertebrate"        
[5] "plant"                "protozoa"             "vertebrate_mammalian" "vertebrate_other"    
[9] "viral"
```

### Example `NCBI Genbank`:

```{r, eval=FALSE}
getKingdoms(db = "genbank")
```

```
[1] "archaea"              "bacteria"             "fungi"               
[4] "invertebrate"         "plant"                "protozoa"            
[7] "vertebrate_mammalian" "vertebrate_other"
```

In these examples the difference betwenn `db = "refseq"` and `db = "genbank"` is that `db = "genbank"` does not store `viral` information.


### Example `ENSEMBL`

```{r, eval=FALSE}
getKingdoms(db = "ensembl")
```

```
[1] "EnsemblVertebrates"                                             
```

The `ENSEMBL` database does not differentiate between different kingdoms, but specialized on storing high-quality reference genomes of diverse biological disciplines.

#### Retrieval from `NCBI RefSeq`

Download all mammalian vertebrate genomes from `NCBI RefSeq`.

```{r,eval=FALSE}
# download all vertebrate genomes
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "genome", reference = FALSE)
```

The argument `kingdom` specifies the kingdom selected with `getKingdoms()` from which
genomes of organisms shall be retrieved. The `db` argument specifies the database
from which respective genomes shall be downloaded. The argument `type` specifies
that `genome assembly` files shall be retrieved. The argument `reference` indicates whether or not a genome shall be downloaded if it isn't marked in the database as either a `reference genome` or a `representative genome`. Options are:

- `reference = FALSE` (__Default__): all organisms (reference, representative, and non-representative genomes) are downloaded.
- `reference = TRUE`: organisms that are downloaded must be either a `reference` or `representative genome`. Thus, most genomes which are usually non-reference genomes will not be downloaded and the user will retrieve much less organisms than are stored in databases.

When running this command all geneomes are stored in a folder which is either named according to the kingdom
(in this case `vertebrate_mammalian`). Alternatively, users can specify
the `out.folder` argument to define a custom output folder path. 

Internally, in this example `meta.retrieval()` will generate a folder named `vertebrate_mammalian`
in which respective genomes will be stored. In addition, the `vertebrate_mammalian`
folder contains a folder named `documentation` which stores individual documentation
files for each individual organism and a `summary file` which stores documentation
for all retrieved organisms. This `summary file` can then be used as `Supplementary Data`
in studies to promote computational reproducibility.

An example documentation file of an individual organism looks like this:

```
File Name: Mus_musculus_genomic_genbank.gff.gz
Organism Name: Mus_musculus
Database: NCBI genbank
URL: ftp://ftp.ncbi.nlm.nih.gov/genomes/all/GCA/000/001/635/GCA_000001635.7_GRCm38.p5/GCA_000001635.7_GRCm38.p5_genomic.gff.gz
Download_Date: Mon Nov 14 12:43:45 2016
refseq_category: reference genome
assembly_accession: GCA_000001635.7
bioproject: PRJNA20689
biosample: NA
taxid: 10090
infraspecific_name: NA
version_status: latest
release_type: Patch
genome_rep: Full
seq_rel_date: 2016-06-29
submitter: Genome Reference Consortium
```


An example `summary file` of all organism looks like this (here we use 105 Plant species as an example):

```
# A tibble: 105 x 16
   file_name     organism   url             database path  refseq_category
   <chr>         <chr>      <chr>           <chr>    <chr> <chr>          
 1 Aegilops_tau Aegilops_ ftp://ftp.ncbi refseq   Prot representative
 2 Amborella_tr Amborella ftp://ftp.ncbi refseq   Prot representative
 3 Ananas_comos Ananas_co ftp://ftp.ncbi refseq   Prot representative
 4 Arabidopsis_ Arabidops ftp://ftp.ncbi refseq   Prot representative
 5 Arabidopsis_ Arabidops ftp://ftp.ncbi refseq   Prot reference geno
 6 Arachis_dura Arachis_d ftp://ftp.ncbi refseq   Prot representative
 7 Arachis_ipae Arachis_i ftp://ftp.ncbi refseq   Prot representative
 8 Asparagus_of Asparagus ftp://ftp.ncbi refseq   Prot representative
 9 Auxenochlore Auxenochl ftp://ftp.ncbi refseq   Prot representative
10 Bathycoccus_ Bathycocc ftp://ftp.ncbi refseq   Prot representative
# ... with 95 more rows, and 10 more variables: assembly_accession <chr>,
#   bioproject <chr>, biosample <chr>, taxid <int>,
#   infraspecific_name <chr>, version_status <chr>, release_type <chr>,
#   genome_rep <chr>, seq_rel_date <date>, submitter <chr>
```

#### Restarting a corrupted download

Unfortunately, when downloading large amounts of genomes the NCBI RefSeq database
limits the number of queries from an individual IP address. This causes
that the download process might stop or time out at a particular step.
To overcome this limitation users can simply __re-run__ the `meta.retrieval()` command they previously executed 
and specify the argument `restart_at_last` which has the following two options:

- If `restart_at_last = TRUE` (__Default__) then `meta.retrieval()` will skip all organisms that are already present in the folder and will start downloading all remaining species (thus will pick up from where the initial download process stopped). However, this way `meta.retrieval()` will not be able to check whether already downloaded organism files are corrupted or not by checking the `md5 checksum` of the respective file. Thus, I recommend to download the last organism before `meta.retrieval()` stopped manually using `getGenome()` to make sure that the respective file is not corrupted.
- If `restart_at_last = FALSE` then `meta.retrieval()` will start from the beginning and crawl through already downloaded organism files and check whether already downloaded organism files are corrupted or not by checking the `md5 checksum` (this procedure takes longer than `restart_at_last = TRUE`). After checking existing files the function will start downloading all remaining organisms.

#### Un-zipping downloaded files

After downloading genomes users can format the output of `meta.retrieval()` by first un-zipping downloaded files and renaming them for more convenient downstream data analysis (e.g. from `Saccharomyces_cerevisiae_cds_from_genomic_refseq.fna.gz` to `Scerevisiae.fa`).

The easiest way to use `clean.retrieval()` in combination with `meta.retrieval()` is to use the pipe operator from the `magrittr` package:

```r
library(magrittr)
meta.retrieval(kingdom = "vertebrate_mammalian", 
               db = "refseq", 
               type = "genome") %>% 
    clean.retrieval()
```

In the first step, genome assembly files are downloaded with `meta.retrieval` and
subsequently (`%>%`) un-zipped and re-named using `clean.retrieval()`.



Example `Bacteria`
```{r,eval=FALSE}
# download all bacteria genomes
meta.retrieval(kingdom = "bacteria", db = "refseq", type = "genome", reference = FALSE)
```

Example `Viruses`
```{r,eval=FALSE}
# download all virus genomes
meta.retrieval(kingdom = "viral", db = "refseq", type = "genome", reference = FALSE)
```

Example `Archaea`
```{r,eval=FALSE}
# download all archaea genomes
meta.retrieval(kingdom = "archaea", db = "refseq", type = "genome", reference = FALSE)
```

Example `Fungi`
```{r,eval=FALSE}
# download all fungi genomes
meta.retrieval(kingdom = "fungi", db = "refseq", type = "genome", reference = FALSE)
```

Example `Plants`
```{r,eval=FALSE}
# download all plant genomes
meta.retrieval(kingdom = "plant", db = "refseq", type = "genome", reference = FALSE)
```

Example `Invertebrates`
```{r,eval=FALSE}
# download all invertebrate genomes
meta.retrieval(kingdom = "invertebrate", db = "refseq", type = "genome", reference = FALSE)
```

Example `Protozoa`
```{r,eval=FALSE}
# download all invertebrate genomes
meta.retrieval(kingdom = "protozoa", db = "refseq", type = "genome", reference = FALSE)
```

#### Retrieval from `NCBI Genbank`

Alternatively, download all mammalian vertebrate genomes from `NCBI Genbank`, e.g.

```{r,eval=FALSE}
# download all vertebrate genomes
meta.retrieval(kingdom = "vertebrate_mammalian", db = "genbank", type = "genome", reference = FALSE)
```

Example `Bacteria`
```{r,eval=FALSE}
# download all bacteria genomes
meta.retrieval(kingdom = "bacteria", db = "genbank", type = "genome", reference = FALSE)
```

Example `Archaea`
```{r,eval=FALSE}
# download all archaea genomes
meta.retrieval(kingdom = "archaea", db = "genbank", type = "genome", reference = FALSE)
```

Example `Fungi`
```{r,eval=FALSE}
# download all fungi genomes
meta.retrieval(kingdom = "fungi", db = "genbank", type = "genome", reference = FALSE)
```

Example `Plants`
```{r,eval=FALSE}
# download all plant genomes
meta.retrieval(kingdom = "plant", db = "genbank", type = "genome", reference = FALSE)
```

Example `Invertebrates`
```{r,eval=FALSE}
# download all invertebrate genomes
meta.retrieval(kingdom = "invertebrate", db = "genbank", type = "genome", reference = FALSE)
```

Example `Protozoa`
```{r,eval=FALSE}
# download all invertebrate genomes
meta.retrieval(kingdom = "protozoa", db = "genbank", type = "genome", reference = FALSE)
```

#### Retrieval from `ENSEMBL`

```{r,eval=FALSE}
# download all genomes from ENSEMBL
meta.retrieval(kingdom = "Ensembl", db = "ensembl", type = "genome", reference = FALSE)
```

### Retrieve groups or subgroups of species 

In case users do not wish to retrieve genomes from an entire kingdom, but rather from
a group or subgoup (e.g. from species belonging to the `Gammaproteobacteria` class, a subgroup of the `bacteria` kingdom), they can use the following workflow.

#### Example retrieval of all `Gammaproteobacteria` genomes from `NCBI RefSeq`:

First, users can again consult the `getKingdoms()` function to retrieve kingdom information.

```{r, eval=FALSE}
getKingdoms(db = "refseq")
```

```
[1] "archaea"              "bacteria"             "fungi"                "invertebrate"        
[5] "plant"                "protozoa"             "vertebrate_mammalian" "vertebrate_other"    
[9] "viral"
```

In this example, we will choose the `bacteria` kingdom. Now, the `getGroups()` function allows users to obtain
available subgroups of the `bacteria` kingdom.

```{r, eval=FALSE}
getGroups(db = "refseq", kingdom = "bacteria")
```

```
 [1] "Acidithiobacillia"                     "Acidobacteriia"                       
 [3] "Actinobacteria"                        "Alphaproteobacteria"                  
 [5] "Aquificae"                             "Armatimonadetes"                      
 [7] "Bacteroidetes/Chlorobi group"          "Balneolia"                            
 [9] "Betaproteobacteria"                    "Blastocatellia"                       
[11] "Candidatus Kryptonia"                  "Chlamydiae"                           
[13] "Chloroflexi"                           "Cyanobacteria/Melainabacteria group"  
[15] "Deinococcus-Thermus"                   "delta/epsilon subdivisions"           
[17] "Endomicrobia"                          "Fibrobacteres"                        
[19] "Firmicutes"                            "Fusobacteriia"                        
[21] "Gammaproteobacteria"                   "Gemmatimonadetes"                     
[23] "Kiritimatiellaeota"                    "Nitrospira"                           
[25] "Planctomycetes"                        "Spirochaetia"                         
[27] "Synergistia"                           "Tenericutes"                          
[29] "Thermodesulfobacteria"                 "Thermotogae"                          
[31] "unclassified Acidobacteria"            "unclassified Bacteria (miscellaneous)"
[33] "unclassified Proteobacteria"           "Verrucomicrobia"                      
[35] "Zetaproteobacteria" 
```

Please note, that the `kingdom` argument specified in `getGroups()` needs to match with an available kingdom retrieved with `getKingdoms()`.
It is also important that in both cases: `getKingdoms()` and `getGroups()` the same database should be specified.


Now we choose the group `Gammaproteobacteria` and specify the `group` argument in the `meta.retrieval()` function.

```{r, eval=FALSE}
meta.retrieval(kingdom = "bacteria", group = "Gammaproteobacteria", db = "refseq", type = "genome", reference = FALSE)
```

Using this command, all bacterial (`kingdom = "bacteria"`) genomes (`type = "genome"`) that belong to the group `Gammaproteobacteria` (`group = "Gammaproteobacteria"`) will be retrieved from NCBI RefSeq (`db = "refseq"`).

Alternatively, `Gammaproteobacteria` genomes can be retrieved from NCBI Genbank by exchanging `db = "refseq"` to `db = "genbank"`.
If users wish to download proteome, CDS, or GFF files instead of genomes, they can specify the argument: `type = "proteome"`, `type = "cds"`, or `type = "gff"`.

#### Example retrieval of all `Adenoviridae` genomes from `NCBI RefSeq`:

Retrieve groups for viruses.
```{r, eval=FALSE}
getGroups(db = "refseq", kingdom = "viral")
```

```
 [1] "Adenoviridae"                                        "Alloherpesviridae"                                  
  [3] "Alphaflexiviridae"                                   "Alphatetraviridae"                                  
  [5] "Alvernaviridae"                                      "Amalgaviridae"                                      
  [7] "Ampullaviridae"                                      "Anelloviridae"                                      
  [9] "Apple fruit crinkle viroid"                          "Apple hammerhead viroid-like circular RNA"          
 [11] "Apscaviroid"                                         "Arenaviridae"                                       
 [13] "Arteriviridae"                                       "Ascoviridae"                                        
 [15] "Asfarviridae"                                        "Astroviridae"                                       
 [17] "Avsunviroid"                                         "Baculoviridae"                                      
 [19] "Barnaviridae"                                        "Benyviridae"                                        
 [21] "Betaflexiviridae"                                    "Bicaudaviridae"                                     
 [23] "Birnaviridae"                                        "Bornaviridae"                                       
 [25] "Bromoviridae"                                        "Bunyaviridae"                                       
 [27] "Caliciviridae"                                       "Carmotetraviridae"                                  
 [29] "Caulimoviridae"                                      "Cherry leaf scorch small circular viroid-like RNA 1"
 [31] "Cherry small circular viroid-like RNA"               "Chrysoviridae"                                      
 [33] "Circoviridae"                                        "Closteroviridae"                                    
 [35] "Cocadviroid"                                         "Coleviroid"                                         
 [37] "Coronaviridae"                                       "Corticoviridae"                                     
 [39] "Cystoviridae"                                        "Dicistroviridae"                                    
 [41] "Elaviroid"                                           "Endornaviridae"                                     
 [43] "Filoviridae"                                         "Flaviviridae"                                       
 [45] "Fusarividae"                                         "Fuselloviridae"                                     
 [47] "Gammaflexiviridae"                                   "Geminiviridae"                                      
 [49] "Genomoviridae"                                       "Globuloviridae"                                     
 [51] "Grapevine latent viroid"                             "Guttaviridae"                                       
 [53] "Hepadnaviridae"                                      "Hepeviridae"                                        
 [55] "Herpesviridae"                                       "Hostuviroid"                                        
 [57] "Hypoviridae"                                         "Hytrosaviridae"                                     
 [59] "Iflaviridae"                                         "Inoviridae"                                         
 [61] "Iridoviridae"                                        "Lavidaviridae"                                      
 [63] "Leviviridae"                                         "Lipothrixviridae"                                   
 [65] "Luteoviridae"                                        "Malacoherpesviridae"                                
 [67] "Marnaviridae"                                        "Marseilleviridae"                                   
 [69] "Megabirnaviridae"                                    "Mesoniviridae"                                      
 [71] "Microviridae"                                        "Mimiviridae"                                        
 [73] "Mulberry small circular viroid-like RNA 1"           "Mymonaviridae"                                      
 [75] "Myoviridae"                                          "Nanoviridae"                                        
 [77] "Narnaviridae"                                        "Nimaviridae"                                        
 [79] "Nodaviridae"                                         "Nudiviridae"                                        
 [81] "Nyamiviridae"                                        "Ophioviridae"                                       
 [83] "Orthomyxoviridae"                                    "Other"                                              
 [85] "Papillomaviridae"                                    "Paramyxoviridae"                                    
 [87] "Partitiviridae"                                      "Parvoviridae"                                       
 [89] "Pelamoviroid"                                        "Permutotetraviridae"                                
 [91] "Persimmon viroid"                                    "Phycodnaviridae"                                    
 [93] "Picobirnaviridae"                                    "Picornaviridae"                                     
 [95] "Plasmaviridae"                                       "Pneumoviridae"                                      
 [97] "Podoviridae"                                         "Polydnaviridae"                                     
 [99] "Polyomaviridae"                                      "Pospiviroid"                                        
[101] "Potyviridae"                                         "Poxviridae"                                         
[103] "Quadriviridae"                                       "Reoviridae"                                         
[105] "Retroviridae"                                        "Rhabdoviridae"                                      
[107] "Roniviridae"                                         "Rubber viroid India/2009"                           
[109] "Rudiviridae"                                         "Secoviridae"                                        
[111] "Siphoviridae"                                        "Sphaerolipoviridae"                                 
[113] "Sunviridae"                                          "Tectiviridae"                                       
[115] "Togaviridae"                                         "Tombusviridae"                                      
[117] "Totiviridae"                                         "Turriviridae"                                       
[119] "Tymoviridae"                                         "unclassified"                                       
[121] "unclassified Pospiviroidae"                          "Virgaviridae"
```

Now we can choose `Adenoviridae` as group argument for the `meta.retrieval()` function.

```{r, eval=FALSE}
meta.retrieval(kingdom = "viral", group = "Adenoviridae", db = "refseq", type = "genome", reference = FALSE)
```

Again, by exchanging `type = "genome"` by either `type = "proteome"`, `type = "cds"`, `type = "rna"`, `type = "assemblystats"`, or `type = "gff"`, users can retrieve proteome, CDS, RNA, genome assembly statistics or GFF files instead of genomes.


### Meta retrieval of genome assembly quality information

Although much effort is invested to increase the genome assembly quality
when new genomes are published or new versions are released, the influence
of genome assembly quality on downstream analyses cannot be neglected.
A rule of thumb is, that the larger the genome the more prone it is to
genome assembly errors and therefore, a reduction of assembly quality.

In [Veeckman et al., 2016](https://doi.org/10.1105/tpc.16.00349) the authors conclude:

> As yet, no uniform metrics or standards are in place to estimate the completeness of a genome assembly or
> the annotated gene space, despite their importance for downstream analyses

In most metagenomics studies, however, the influence or bias of genome assembly quality on the 
outcome of the analysis (e.g. comparative genomics, annotation, etc.) is neglected. To better 
grasp the genome assembly quality, the NCBI databases store genome assembly statistics of some
species for which genome assemblies are available. An example assembly statistics report can
be found at: ftp://ftp.ncbi.nlm.nih.gov/genomes/all/GCF/000/001/405/GCF_000001405.36_GRCh38.p10/GCF_000001405.36_GRCh38.p10_assembly_stats.txt. 

The `biomartr` package allows users to retrieve these genome assembly stats file in an automated way by specifying the argument `type = "assemblystats"` and `combine = TRUE`.
Please make sure that `combine = TRUE` when selecting `type = "assemblystats"`.

```{r, eval=FALSE}
# show all elements of the data.frame
options(tibble.print_max = Inf)
# retrieve genome assembly stats for all mammal genome assemblies
# and store these stats in a data.frame
mammals.gc <- meta.retrieval(kingdom = "vertebrate_mammalian", 
                             db      = "refseq", 
                             type    = "assemblystats", 
                             combine = TRUE)

mammals.gc
```

```
                    species total_length spanned_gaps unspanned_gaps region_count scaffold_count
                      <chr>        <int>        <int>          <int>        <int>          <int>
1  Ornithorhynchus anatinus   1995607322       243698            137            0         200283
2      Sarcophilus harrisii           NA       201317              0            0          35974
3      Dasypus novemcinctus           NA       268413              0            0          46559
4       Erinaceus europaeus           NA       219764              0            0           5803
5         Echinops telfairi           NA       269444              0            0           8402
6           Pteropus alecto   1985975446       104566              0            0          65598
7     Rousettus aegyptiacus   1910250568          559              0            0             NA
8        Callithrix jacchus           NA       184972           2242            0          16399
9  Cebus capucinus imitator           NA       133441              0            0           7156
10          Cercocebus atys           NA        65319              0            0          11433
# ... with 89 more rows, and 9 more variables: scaffold_N50 <int>, scaffold_L50 <int>,
#   scaffold_N75 <int>, scaffold_N90 <int>, contig_count <int>, contig_N50 <int>, total_gap_length <int>,
#   molecule_count <int>, top_level_count <int>
```

Analogously, this information can be retrieved for each kingdom other than `kingdom = "vertebrate_mammalian"`. Please consult `getKingdoms()` for available kingdoms.

### Metagenome project retrieval from NCBI Genbank

NCBI Genbank stores [metagenome projects](ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/metagenomes/) in addition to species specific genome, proteome or CDS sequences. To retrieve these metagenomes users can perform the following combination of commands:

First, users can list the project names of available metagenomes by typing

```{r,eval=FALSE}
# list available metagenomes at NCBI Genbank
listMetaGenomes()
```

```
[1] "metagenome"                     "human gut metagenome"           "epibiont metagenome"           
 [4] "marine metagenome"              "soil metagenome"                "mine drainage metagenome"      
 [7] "mouse gut metagenome"           "marine sediment metagenome"     "termite gut metagenome"        
[10] "hot springs metagenome"         "human lung metagenome"          "fossil metagenome"             
[13] "freshwater metagenome"          "saltern metagenome"             "stromatolite metagenome"       
[16] "coral metagenome"               "mosquito metagenome"            "fish metagenome"               
[19] "bovine gut metagenome"          "chicken gut metagenome"         "wastewater metagenome"         
[22] "microbial mat metagenome"       "freshwater sediment metagenome" "human metagenome"              
[25] "hydrothermal vent metagenome"   "compost metagenome"             "wallaby gut metagenome"        
[28] "groundwater metagenome"         "gut metagenome"                 "sediment metagenome"           
[31] "ant fungus garden metagenome"   "food metagenome"                "hypersaline lake metagenome"   
[34] "hydrocarbon metagenome"         "activated sludge metagenome"    "viral metagenome"              
[37] "bioreactor metagenome"          "wasp metagenome"                "permafrost metagenome"         
[40] "sponge metagenome"              "aquatic metagenome"             "insect gut metagenome"         
[43] "activated carbon metagenome"    "anaerobic digester metagenome"  "rock metagenome"               
[46] "terrestrial metagenome"         "rock porewater metagenome"      "seawater metagenome"           
[49] "scorpion gut metagenome"        "soda lake metagenome"           "glacier metagenome"
```

Internally the `listMetaGenomes()` function downloads the assembly_summary.txt file from ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/metagenomes/ to retrieve
available metagenome information. This procedure might take a few seconds during the first run of `listMetaGenomes()`. Subsequently, the assembly_summary.txt file will be stored in the `tempdir()` directory to
achieve a much faster access of this information during following uses of `listMetaGenomes()`.

In case users wish to retrieve detailed information about available metagenome projects they can specify the `details = TRUE` argument.

```{r,eval=FALSE}
# show all elements of the data.frame
options(tibble.print_max = Inf)
# detailed information on available metagenomes at NCBI Genbank
listMetaGenomes(details = TRUE)
```

```
# A tibble: 857 x 21
   assembly_accession bioproject    biosample     wgs_master refseq_category  taxid species_taxid
                <chr>      <chr>        <chr>          <chr>           <chr>  <int>         <int>
1     GCA_000206185.1 PRJNA32359 SAMN02954317 AAGA00000000.1              na 256318        256318
2     GCA_000206205.1 PRJNA32355 SAMN02954315 AAFZ00000000.1              na 256318        256318
3     GCA_000206225.1 PRJNA32357 SAMN02954316 AAFY00000000.1              na 256318        256318
4     GCA_000208265.2 PRJNA17779 SAMN02954240 AASZ00000000.1              na 256318        256318
5     GCA_000208285.1 PRJNA17657 SAMN02954268 AATO00000000.1              na 256318        256318
6     GCA_000208305.1 PRJNA17659 SAMN02954269 AATN00000000.1              na 256318        256318
7     GCA_000208325.1 PRJNA16729 SAMN02954263 AAQL00000000.1              na 256318        256318
8     GCA_000208345.1 PRJNA16729 SAMN02954262 AAQK00000000.1              na 256318        256318
9     GCA_000208365.1 PRJNA13699 SAMN02954283 AAFX00000000.1              na 256318        256318
10    GCA_900010595.1 PRJEB11544 SAMEA3639840 CZPY00000000.1              na 256318        256318
# ... with 847 more rows, and 14 more variables: organism_name <chr>, infraspecific_name <chr>,
#   isolate <chr>, version_status <chr>, assembly_level <chr>, release_type <chr>, genome_rep <chr>,
#   seq_rel_date <date>, asm_name <chr>, submitter <chr>, gbrs_paired_asm <chr>, paired_asm_comp <chr>,
#   ftp_path <chr>, excluded_from_refseq <chr>
```

Finally, users can retrieve available metagenomes using `getMetaGenomes()`. The `name`
argument receives the metagenome project name retrieved with `listMetaGenomes()`.
The `path` argument specifies the folder path in which corresponding genomes shall be stored.

```{r,eval=FALSE}
# retrieve all genomes belonging to the human gut metagenome project
getMetaGenomes(name = "human gut metagenome", path = file.path("_ncbi_downloads","human_gut"))
```

```
1] "The metagenome of 'human gut metagenome' has been downloaded to '_ncbi_downloads/human_gut'."
  [1] "_ncbi_downloads/human_gut/GCA_000205525.2_ASM20552v2_genomic.fna.gz"
  [2] "_ncbi_downloads/human_gut/GCA_000205765.1_ASM20576v1_genomic.fna.gz"
  [3] "_ncbi_downloads/human_gut/GCA_000205785.1_ASM20578v1_genomic.fna.gz"
  [4] "_ncbi_downloads/human_gut/GCA_000207925.1_ASM20792v1_genomic.fna.gz"
  [5] "_ncbi_downloads/human_gut/GCA_000207945.1_ASM20794v1_genomic.fna.gz"
  [6] "_ncbi_downloads/human_gut/GCA_000207965.1_ASM20796v1_genomic.fna.gz"
  [7] "_ncbi_downloads/human_gut/GCA_000207985.1_ASM20798v1_genomic.fna.gz"
  [8] "_ncbi_downloads/human_gut/GCA_000208005.1_ASM20800v1_genomic.fna.gz"
  [9] "_ncbi_downloads/human_gut/GCA_000208025.1_ASM20802v1_genomic.fna.gz"
 [10] "_ncbi_downloads/human_gut/GCA_000208045.1_ASM20804v1_genomic.fna.gz"
 [11] "_ncbi_downloads/human_gut/GCA_000208065.1_ASM20806v1_genomic.fna.gz"
 [12] "_ncbi_downloads/human_gut/GCA_000208085.1_ASM20808v1_genomic.fna.gz"
 [13] "_ncbi_downloads/human_gut/GCA_000208105.1_ASM20810v1_genomic.fna.gz"
 [14] "_ncbi_downloads/human_gut/GCA_000208125.1_ASM20812v1_genomic.fna.gz"
 [15] "_ncbi_downloads/human_gut/GCA_000208145.1_ASM20814v1_genomic.fna.gz"
 [16] "_ncbi_downloads/human_gut/GCA_000208165.1_ASM20816v1_genomic.fna.gz"
 ...
```

Internally, `getMetaGenomes()` creates a folder specified in the `path` argument.
Genomes associated with the metagenomes project specified in the `name` argument
will then be downloaded and stored in this folder. As return value `getMetaGenomes()`
returns the file paths to the genome files which can then be used as input to the `read*()` functions.

Alternatively or subsequent to the metagenome retrieval, users can retrieve annotation files of genomes belonging to a metagenome project
selected with `listMetaGenomes()` by using the `getMetaGenomeAnnotations()` function.


```{r,eval=FALSE}
# retrieve all genomes belonging to the human gut metagenome project
getMetaGenomeAnnotations(name = "human gut metagenome", path = file.path("_ncbi_downloads","human_gut","annotations"))
```

```
[1] "The annotations of metagenome 'human gut metagenome' have been downloaded and stored at '_ncbi_downloads/human_gut/annotations'."
  [1] "_ncbi_downloads/human_gut/annotations/GCA_000205525.2_ASM20552v2_genomic.gff.gz"
  [2] "_ncbi_downloads/human_gut/annotations/GCA_000205765.1_ASM20576v1_genomic.gff.gz"
  [3] "_ncbi_downloads/human_gut/annotations/GCA_000205785.1_ASM20578v1_genomic.gff.gz"
  [4] "_ncbi_downloads/human_gut/annotations/GCA_000207925.1_ASM20792v1_genomic.gff.gz"
  [5] "_ncbi_downloads/human_gut/annotations/GCA_000207945.1_ASM20794v1_genomic.gff.gz"
  [6] "_ncbi_downloads/human_gut/annotations/GCA_000207965.1_ASM20796v1_genomic.gff.gz"
  [7] "_ncbi_downloads/human_gut/annotations/GCA_000207985.1_ASM20798v1_genomic.gff.gz"
  [8] "_ncbi_downloads/human_gut/annotations/GCA_000208005.1_ASM20800v1_genomic.gff.gz"
  [9] "_ncbi_downloads/human_gut/annotations/GCA_000208025.1_ASM20802v1_genomic.gff.gz"
 [10] "_ncbi_downloads/human_gut/annotations/GCA_000208045.1_ASM20804v1_genomic.gff.gz"
 [11] "_ncbi_downloads/human_gut/annotations/GCA_000208065.1_ASM20806v1_genomic.gff.gz"
 [12] "_ncbi_downloads/human_gut/annotations/GCA_000208085.1_ASM20808v1_genomic.gff.gz"
 [13] "_ncbi_downloads/human_gut/annotations/GCA_000208105.1_ASM20810v1_genomic.gff.gz"
 [13] "_ncbi_downloads/human_gut/annotations/GCA_000208105.1_ASM20810v1_genomic.gff.gz"
 [14] "_ncbi_downloads/human_gut/annotations/GCA_000208125.1_ASM20812v1_genomic.gff.gz"
 [15] "_ncbi_downloads/human_gut/annotations/GCA_000208145.1_ASM20814v1_genomic.gff.gz"
 [16] "_ncbi_downloads/human_gut/annotations/GCA_000208165.1_ASM20816v1_genomic.gff.gz"
 ...
```

The file paths of the downloaded `*.gff` are retured by `getMetaGenomeAnnotations()` and can be used
as input for the `read.gff()` function in the [seqreadr](https://github.com/HajkD/seqreadr) package.


#### Retrieve Protein Sequences

Download all mammalian vertebrate proteomes.

#### Retrieval from `NCBI RefSeq`:
```{r,eval=FALSE}
# download all vertebrate genomes
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "proteome", reference = FALSE)
```


#### Retrieval from `NCBI Genbank`:
```{r,eval=FALSE}
# download all vertebrate genomes
meta.retrieval(kingdom = "vertebrate_mammalian", db = "genbank", type = "proteome", reference = FALSE)
```


#### Retrieval from `ENSEMBL`:

```{r,eval=FALSE}
# download all Ensembl proteome sequneces
meta.retrieval(kingdom = "Ensembl", db = "ensembl", type = "proteome", reference = FALSE)
```

#### Retrieve CDS Sequences

Download all mammalian vertebrate CDS from RefSeq (Genbank does not store CDS data).

#### Retrieval from `NCBI RefSeq`:
```{r,eval=FALSE}
# download all vertebrate genomes
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "cds", reference = FALSE)
```

#### Retrieval from `NCBI Genbank`:
```{r,eval=FALSE}
# download all vertebrate genomes
meta.retrieval(kingdom = "vertebrate_mammalian", db = "genbank", type = "cds", reference = FALSE)
```

#### Retrieval from `ENSEMBL`:

```{r,eval=FALSE}
# download all Ensembl CDS sequneces
meta.retrieval(kingdom = "Ensembl", db = "ensembl", type = "cds", reference = FALSE)
```

### Retrieve GFF files

Download all mammalian vertebrate gff files.

Example `NCBI RefSeq`:
```{r,eval=FALSE}
# download all vertebrate gff files
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "gff", reference = FALSE)
```


Example `NCBI Genbank`:
```{r,eval=FALSE}
# download all vertebrate gff files
meta.retrieval(kingdom = "vertebrate_mammalian", db = "genbank", type = "gff", reference = FALSE)
```

### Retrieve GTF files

Download all mammalian vertebrate gtf files.

Example `ENSEMBL`:
```{r,eval=FALSE}
# download all vertebrate gff files
meta.retrieval(kingdom = "Ensembl", db = "ensembl", type = "gtf", reference = FALSE)
```


### Retrieve RNA sequences

Download all mammalian vertebrate RNA sequences from `NCBI RefSeq` and `NCBI Genbank`.

#### Retrieval from `NCBI RefSeq`:
```{r,eval=FALSE}
# download all vertebrate RNA sequneces
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "rna", reference = FALSE)
```

#### Retrieval from `NCBI Genbank`:
```{r,eval=FALSE}
# download all vertebrate RNA sequneces
meta.retrieval(kingdom = "vertebrate_mammalian", db = "genbank", type = "rna", reference = FALSE)
```

#### Retrieval from `ENSEMBL`:

```{r,eval=FALSE}
# download all Ensembl RNA sequneces
meta.retrieval(kingdom = "Ensembl", db = "ensembl", type = "rna", reference = FALSE)
```

### Retrieve Repeat Masker Sequences

Download all mammalian vertebrate Repeat Masker Annotation files from `NCBI RefSeq` and `NCBI Genbank`.

#### Retrieval from `NCBI RefSeq`:
```{r,eval=FALSE}
# download all vertebrate RNA sequneces
meta.retrieval(kingdom = "vertebrate_mammalian", db = "refseq", type = "rm", reference = FALSE)
```

#### Retrieval from `NCBI Genbank`:
```{r,eval=FALSE}
# download all vertebrate RNA sequneces
meta.retrieval(kingdom = "vertebrate_mammalian", db = "genbank", type = "rm", reference = FALSE)
```

Users can obtain alternative kingdoms using `getKingdoms()`.

## Retrieve Individual Genomes for all Species in the Tree of Life

If users wish to download the all genomes, proteome, CDS, or gff files for all species
available in RefSeq or Genbank, they can use the `meta.retrieval.all()` function for this purpose.

### Genome Retrieval

Example `RefSeq`:
```{r,eval=FALSE}
# download all geneomes stored in RefSeq
meta.retrieval.all(db = "refseq", type = "genome", reference = FALSE)
```

Example `Genbank`:
```{r,eval=FALSE}
# download all geneomes stored in Genbank
meta.retrieval.all(db = "genbank", type = "genome", reference = FALSE)
```

### Proteome Retrieval

Example `RefSeq`:
```{r,eval=FALSE}
# download all proteome stored in RefSeq
meta.retrieval.all(db = "refseq", type = "proteome", reference = FALSE)
```

Example `Genbank`:
```{r,eval=FALSE}
# download all proteome stored in Genbank
meta.retrieval.all(db = "genbank", type = "proteome", reference = FALSE)
```

Again, by exchanging `type = "proteome"` by either 

- `type = "genome"`
- `type = "cds"`
- `type = "rna"`
- `type = "assemblystats"`
- `type = "gff"`

users can retrieve genome, CDS, RNA, genome assembly statistics or GFF files instead of proteomes.

---
title: "Ensembl BioMart Examples"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Ensembl BioMart Examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\usepackage[utf8]{inputenc}
---

```{r, echo = FALSE, message = FALSE}
options(width = 750)
knitr::opts_chunk$set(
  comment = "#>",
  error = FALSE,
  tidy = FALSE)
```

## Use Case #1: Functional Annotation of Genes Sharing a Common Evolutionary History

Evolutionary Transcriptomics aims to predict stages or periods of evolutionary conservation in
biological processes on the transcriptome level. However, finding genes sharing a [common evolutionary history](https://drostlab.github.io/myTAI/articles/Enrichment.html) could reveal how the the biological process might have evolved in the first place.

In this `Use Case` we will combine functional and biological annotation obtained with `biomartr` with enriched genes obtained with [PlotEnrichment()](https://drostlab.github.io/myTAI/articles/Enrichment.html). 

### Step 1

For the following example we will use the dataset an enrichment analyses found in [PlotEnrichment()](https://drostlab.github.io/myTAI/articles/Enrichment.html).

Install and load the [myTAI](https://github.com/drostlab/myTAI) package:

```{r, eval=FALSE}
# install myTAI
install.packages("myTAI")

# load myTAI
library(myTAI)
```

Download the `Phylostratigraphic Map` of _D. rerio_:

```r
# download the Phylostratigraphic Map of Danio rerio
# from Sestak and Domazet-Loso, 2015
```

The dataset comes from `Supplementary file 3` of this publication: https://academic.oup.com/mbe/article/32/2/299/1058654#77837069

After downloading `Supplementary file 3`, you will find the file `TableS3-2.xlsx` which can be used in the following `biomartr` functions.



Read the `*.xlsx` file storing the `Phylostratigraphic Map` of _D. rerio_ and format it for the use with `myTAI`:

```r
# install the readxl package
install.packages("readxl")

# load package readxl
library(readxl)

# read the excel file
DrerioPhyloMap.MBEa <- read_excel("TableS3-2.xlsx", sheet = 1, skip = 4)

# format Phylostratigraphic Map for use with myTAI
Drerio.PhyloMap <- DrerioPhyloMap.MBEa[ , 1:2]

# have a look at the final format
head(Drerio.PhyloMap)
```

```
  Phylostrata            ZFIN_ID
1           1 ZDB-GENE-000208-13
2           1 ZDB-GENE-000208-17
3           1 ZDB-GENE-000208-18
4           1 ZDB-GENE-000208-23
5           1  ZDB-GENE-000209-3
6           1  ZDB-GENE-000209-4
```

Now, `Drerio.PhyloMap` stores the `Phylostratigraphic Map` of _D. rerio_ which is used
as background set to perform enrichment analyses with `PlotEnrichment()` from `myTAI`.

### Enrichment Analyses

Now, the `PlotEnrichment()` function visualizes the over- and underrepresented `Phylostrata` of brain specific genes when compared with the total number of genes stored in the `Phylostratigraphic Map` of _D. rerio_.


```{r,eval=FALSE}
library(readxl)

# read expression data (organ specific genes) from Sestak and Domazet-Loso, 2015
Drerio.OrganSpecificExpression <- read_excel("TableS3-2.xlsx", sheet = 2, skip = 3)

# select only brain specific genes
Drerio.Brain.Genes <- unlist(unique(na.omit(Drerio.OrganSpecificExpression[ , "brain"])))

# visualize enriched Phylostrata of genes annotated as brain specific
PlotEnrichment(Drerio.PhyloMap,
               test.set     = Drerio.Brain.Genes,
               measure      = "foldchange",
               use.only.map = TRUE,
               legendName   = "PS")
```

Users will observe that for example brain genes deriving from PS5 are significantly enriched. 


Now we can select all brain genes originating in PS5 using the `SelectGeneSet()` function from `myTAI`. Please notice that `SelectGeneSet()` can only be used with phylostratigraphic maps only (`use.map.only = TRUE` argument) since myTAI version > 0.3.0.

```{r,eval=FALSE}
BrainGenes <- SelectGeneSet(ExpressionSet = Drerio.PhyloMap,
                            gene.set      = Drerio.Brain.Genes,
                            use.only.map  = TRUE)

# select only brain genes originating in PS5
BrainGenes.PS5 <- BrainGenes[which(BrainGenes[ , "Phylostrata"] == 5), ]

# look at the results
head(BrainGenes.PS5)
```

```
      Phylostrata           ZFIN_ID
14851           5 ZDB-GENE-000210-6
14852           5 ZDB-GENE-000210-7
14853           5 ZDB-GENE-000328-4
14856           5 ZDB-GENE-000411-1
14857           5 ZDB-GENE-000427-4
14860           5 ZDB-GENE-000526-1
```

Now users can perform the `biomart()` function to obtain the functional annotation of brain genes originating in PS5.

For this purpose, first we need to find the filter name of the corresponding gene ids such as `ZDB-GENE-000210-6`.

```{r, eval=FALSE}
# find filter for zfin.org ids
organismFilters("Danio rerio", topic = "zfin_id")
```

```
                            name                           description             dataset
52                  with_zfin_id                       with ZFIN ID(s) drerio_gene_ensembl
53  with_zfin_id_transcript_name          with ZFIN transcript name(s) drerio_gene_ensembl
103                      zfin_id ZFIN ID(s) [e.g. ZDB-GENE-060825-136] drerio_gene_ensembl
274                 with_zfin_id                       with ZFIN ID(s)    drerio_gene_vega
286                      zfin_id ZFIN ID(s) [e.g. ZDB-GENE-121214-212]    drerio_gene_vega
366                 with_zfin_id                       with ZFIN ID(s) drerio_gene_ensembl
367 with_zfin_id_transcript_name          with ZFIN transcript name(s) drerio_gene_ensembl
417                      zfin_id ZFIN ID(s) [e.g. ZDB-GENE-060825-136] drerio_gene_ensembl
588                 with_zfin_id                       with ZFIN ID(s)    drerio_gene_vega
600                      zfin_id ZFIN ID(s) [e.g. ZDB-GENE-121214-212]    drerio_gene_vega
680                 with_zfin_id                       with ZFIN ID(s) drerio_gene_ensembl
681 with_zfin_id_transcript_name          with ZFIN transcript name(s) drerio_gene_ensembl
731                      zfin_id ZFIN ID(s) [e.g. ZDB-GENE-060825-136] drerio_gene_ensembl
902                 with_zfin_id                       with ZFIN ID(s)    drerio_gene_vega
914                      zfin_id ZFIN ID(s) [e.g. ZDB-GENE-121214-212]    drerio_gene_vega
                    mart
52  ENSEMBL_MART_ENSEMBL
53  ENSEMBL_MART_ENSEMBL
103 ENSEMBL_MART_ENSEMBL
274 ENSEMBL_MART_ENSEMBL
286 ENSEMBL_MART_ENSEMBL
366 ENSEMBL_MART_ENSEMBL
367 ENSEMBL_MART_ENSEMBL
417 ENSEMBL_MART_ENSEMBL
588 ENSEMBL_MART_ENSEMBL
600 ENSEMBL_MART_ENSEMBL
680 ENSEMBL_MART_ENSEMBL
681 ENSEMBL_MART_ENSEMBL
731 ENSEMBL_MART_ENSEMBL
902 ENSEMBL_MART_ENSEMBL
914 ENSEMBL_MART_ENSEMBL
```

Now users can retrieve the corresponding GO attribute of _D. rerio_ with `organismAttributes`.

```{r,eval=FALSE}
# find go attribute term for D. rerio
organismAttributes("Danio rerio", topic = "go")
```

```
                                              name                             description
33                                           go_id                       GO Term Accession
36                                 go_linkage_type                   GO Term Evidence Code
38                            goslim_goa_accession                 GOSlim GOA Accession(s)
39                          goslim_goa_description                  GOSlim GOA Description
516                  ggorilla_homolog_ensembl_gene                 Gorilla Ensembl Gene ID
517  ggorilla_homolog_canomical_transcript_protein      Canonical Protein or Transcript ID
518               ggorilla_homolog_ensembl_peptide              Gorilla Ensembl Protein ID
519                    ggorilla_homolog_chromosome                 Gorilla Chromosome Name
520                   ggorilla_homolog_chrom_start           Gorilla Chromosome Start (bp)
521                     ggorilla_homolog_chrom_end             Gorilla Chromosome End (bp)
522                ggorilla_homolog_orthology_type                           Homology Type
523                       ggorilla_homolog_subtype                                Ancestor
524          ggorilla_homolog_orthology_confidence    Orthology confidence [0 low, 1 high]
525                       ggorilla_homolog_perc_id   % Identity with respect to query gene
526                    ggorilla_homolog_perc_id_r1 % Identity with respect to Gorilla gene
527                            ggorilla_homolog_dn                                      dN
528                            ggorilla_homolog_ds                                      dS
1240                                         go_id                                   GO ID
1241                                      quick_go                             Quick GO ID
1370                                         go_id                       GO Term Accession
1373                               go_linkage_type                   GO Term Evidence Code
1375                          goslim_goa_accession                 GOSlim GOA Accession(s)
1376                        goslim_goa_description                  GOSlim GOA Description
1853                 ggorilla_homolog_ensembl_gene                 Gorilla Ensembl Gene ID
1854 ggorilla_homolog_canomical_transcript_protein      Canonical Protein or Transcript ID
1855              ggorilla_homolog_ensembl_peptide              Gorilla Ensembl Protein ID
1856                   ggorilla_homolog_chromosome                 Gorilla Chromosome Name
1857                  ggorilla_homolog_chrom_start           Gorilla Chromosome Start (bp)
1858                    ggorilla_homolog_chrom_end             Gorilla Chromosome End (bp)
1859               ggorilla_homolog_orthology_type                           Homology Type
1860                      ggorilla_homolog_subtype                                Ancestor
1861         ggorilla_homolog_orthology_confidence    Orthology confidence [0 low, 1 high]
1862                      ggorilla_homolog_perc_id   % Identity with respect to query gene
1863                   ggorilla_homolog_perc_id_r1 % Identity with respect to Gorilla gene
1864                           ggorilla_homolog_dn                                      dN
1865                           ggorilla_homolog_ds                                      dS
2577                                         go_id                                   GO ID
2578                                      quick_go                             Quick GO ID
2707                                         go_id                       GO Term Accession
2710                               go_linkage_type                   GO Term Evidence Code
2712                          goslim_goa_accession                 GOSlim GOA Accession(s)
2713                        goslim_goa_description                  GOSlim GOA Description
3190                 ggorilla_homolog_ensembl_gene                 Gorilla Ensembl Gene ID
3191 ggorilla_homolog_canomical_transcript_protein      Canonical Protein or Transcript ID
3192              ggorilla_homolog_ensembl_peptide              Gorilla Ensembl Protein ID
3193                   ggorilla_homolog_chromosome                 Gorilla Chromosome Name
3194                  ggorilla_homolog_chrom_start           Gorilla Chromosome Start (bp)
3195                    ggorilla_homolog_chrom_end             Gorilla Chromosome End (bp)
3196               ggorilla_homolog_orthology_type                           Homology Type
3197                      ggorilla_homolog_subtype                                Ancestor
3198         ggorilla_homolog_orthology_confidence    Orthology confidence [0 low, 1 high]
3199                      ggorilla_homolog_perc_id   % Identity with respect to query gene
3200                   ggorilla_homolog_perc_id_r1 % Identity with respect to Gorilla gene
3201                           ggorilla_homolog_dn                                      dN
3202                           ggorilla_homolog_ds                                      dS
3914                                         go_id                                   GO ID
3915                                      quick_go                             Quick GO ID
                 dataset                 mart
33   drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
36   drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
38   drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
39   drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
516  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
517  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
518  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
519  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
520  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
521  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
522  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
523  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
524  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
525  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
526  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
527  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
528  drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1240    drerio_gene_vega ENSEMBL_MART_ENSEMBL
1241    drerio_gene_vega ENSEMBL_MART_ENSEMBL
1370 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1373 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1375 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1376 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1853 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1854 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1855 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1856 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1857 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1858 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1859 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1860 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1861 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1862 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1863 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1864 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
1865 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
2577    drerio_gene_vega ENSEMBL_MART_ENSEMBL
2578    drerio_gene_vega ENSEMBL_MART_ENSEMBL
2707 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
2710 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
2712 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
2713 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3190 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3191 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3192 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3193 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3194 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3195 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3196 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3197 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3198 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3199 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3200 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3201 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3202 drerio_gene_ensembl ENSEMBL_MART_ENSEMBL
3914    drerio_gene_vega ENSEMBL_MART_ENSEMBL
3915    drerio_gene_vega ENSEMBL_MART_ENSEMBL
```

Now users can specify the filter `zfin_id` and attribute `go_id` to retrieve the GO terms of corresponding gene ids (__Please note that this will take some time__).

```{r, eval=FALSE}
# retrieve GO terms of D. rerio brain genes originating in PS5
GO_tbl.BrainGenes <- biomart(genes      = unlist(BrainGenes.PS5[ , "ZFIN_ID"]),
                             mart       = "ENSEMBL_MART_ENSEMBL",
                             dataset    = "drerio_gene_ensembl",
                             attributes = "go_id",
                             filters    = "zfin_id")

head(GO_tbl.BrainGenes)
```

```
            zfin_id      go_id
1 ZDB-GENE-000210-6 GO:0060037
2 ZDB-GENE-000210-6 GO:0046983
3 ZDB-GENE-000210-7 GO:0046983
4 ZDB-GENE-000328-4 GO:0007275
5 ZDB-GENE-000328-4 GO:0007166
6 ZDB-GENE-000328-4 GO:0035567
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/organismAttributes.R
\name{organismAttributes}
\alias{organismAttributes}
\title{Retrieve Ensembl Biomart attributes for a query organism}
\usage{
organismAttributes(organism, update = FALSE, topic = NULL)
}
\arguments{
\item{organism}{a character string specifying the scientific name 
of a query organism.}

\item{update}{a logical value specifying whether or not the local 
listMart.txt, listDatasets.txt, and listAttributes_organism.txt files shall 
be updated by remote access to BioMart.}

\item{topic}{a character string specifying a topic (category) of attributes, 
e.g. \code{topic} = \code{"id"}.}
}
\value{
a data.frame storing corresponding attribute names, description, 
datasets, and marts.
}
\description{
In addition to the \code{\link{organismBM}} function, 
this function returns all available attributes that can be accessed through 
different marts and datasets for a given query organism.
}
\details{
For a given query organism, this function retrieves all available attributes 
that can be accessed through different marts and datasets.

Sometimes the same attribute names correspond to different datasets and 
marts causing problems when using \code{\link{getMarts}}. The approach 
introduced by this function provides (again) a organism centric way of 
accessing organism specific attributes.

The \code{topic} argument allows the user to search for specific attribute 
topics/categories for faster filtering.
}
\note{
When you run this function for the first time, the data retrieval procedure 
will take some time, due to the remote access to BioMart. The corresponding 
result is then saved in a *.txt file within the \code{\link{tempdir}}
directory named "_biomart/listMarts.txt","_biomart/listDatasets.txt", and 
"_biomart/listAttributes_organism.txt", allowing subsequent queries to 
perform much faster.
}
\examples{
\dontrun{ 
# search for attribute topic id
head(organismAttributes("Homo sapiens", topic = "id"), 20)
}
}
\references{
\url{http://biomart.org/}

Mapping identifiers for the integration of genomic datasets with the
R/Bioconductor package biomaRt. Steffen Durinck, Paul T. Spellman, Ewan
Birney and Wolfgang Huber, Nature Protocols 4, 1184-1191 (2009).

BioMart and Bioconductor: a powerful link between biological databases and
microarray data analysis. Steffen Durinck, Yves Moreau, Arek Kasprzyk, Sean
Davis, Bart De Moor, Alvis Brazma and Wolfgang Huber, Bioinformatics 21,
3439-3440 (2005).
}
\seealso{
\code{\link{organismFilters}}, \code{\link{organismBM}}, 
\code{\link{biomart}}, \code{\link[biomaRt]{listAttributes}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getFilters.R
\name{getFilters}
\alias{getFilters}
\title{Retrieve All Available Filters for a Specific Dataset}
\usage{
getFilters(mart, dataset, mute_citation = FALSE)
}
\arguments{
\item{mart}{a character string specifying the database (mart) for which 
datasets shall be listed.}

\item{dataset}{a character string specifying the dataset for which filters 
shall be listed.}

\item{mute_citation}{logical value indicating whether citation message should be muted.}
}
\description{
This funcion queries the BioMart API and returns a table
storing all available filters for a specific dataset.
}
\examples{
\dontrun{
# search for available datasets
# getMarts()
# choose database (mart): "ENSEMBL_MART_ENSEMBL"
# head(getDatasets(mart = "ENSEMBL_MART_ENSEMBL"), 10)
# choose dataset: "hsapiens_gene_ensembl"
head(getFilters(mart = "ENSEMBL_MART_ENSEMBL", 
                dataset = "hsapiens_gene_ensembl") , 5)
}
}
\seealso{
\code{\link{getMarts}}, \code{\link{getDatasets}}, 
\code{\link{getAttributes}}, \code{\link{organismBM}}, 
\code{\link{organismFilters}}, \code{\link{organismAttributes}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listKingdoms.R
\name{listKingdoms}
\alias{listKingdoms}
\title{List number of available genomes in each kingdom of life}
\usage{
listKingdoms(db = "refseq")
}
\arguments{
\item{db}{a character string specifying the database for which genome 
availability shall be checked, 
e.g. \code{db = "refseq"}, \code{db = "genbank"}, \code{db = "ensembl"}.}
}
\description{
Users can retrieve the available number of sequenced 
genomes per kingdom.
}
\examples{
\dontrun{
# list number of available genomes in refseq for each kingdom of life
listKingdoms(db = "refseq")
# example for genbank
listKingdoms(db = "genbank")
# example for ensembl
listKingdoms(db = "ensembl")
}
}
\seealso{
\code{\link{listGenomes}}, \code{\link{is.genome.available}}, 
\code{\link{listGroups}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_database.R
\name{download.database}
\alias{download.database}
\title{Download a NCBI Database to Your Local Hard Drive}
\usage{
download.database(db, path = "database")
}
\arguments{
\item{db}{a character string specifying the database that shall be downloaded
(selected from \code{\link{listDatabases}}).}

\item{path}{a character string specifying the location (a folder) in
which the corresponding database shall be stored.
Default is \code{path = "database"}.
In case this folder does not exist yet, it will be created.}
}
\value{
File path to the downloaded database file.
}
\description{
This function allows users to download a database selected by
\code{\link{listDatabases}} to their local hard drive.
}
\details{
This function downloads large databases to your hard drive.
For this purpose a folder
named \code{database} (default) is created and the correspondning
database then stored in this folder.
}
\examples{
\dontrun{
  # search for available NCBI nr databases
  listNCBIDatabases(db = "nr")
  # select NCBI nr version 27 =  "nr.27.tar.gz"
  # and download it to your hard drive
  # -> please note that large databases take some time for download!
  download.database(db = "nr.27.tar.gz")
}
}
\seealso{
\code{\link{download.database.all}}, \code{\link{listDatabases}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGFF.R
\name{getGTF}
\alias{getGTF}
\title{Genome Annotation Retrieval (GTF)}
\usage{
getGTF(
  db = "ensembl",
  organism,
  remove_annotation_outliers = FALSE,
  path = file.path("ensembl", "annotation"),
  release = NULL,
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organism}{Organism selector id,
there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{remove_annotation_outliers}{shall outlier lines be removed from the input \code{annotation_file}?
If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
for further exploration.}

\item{path}{a character string specifying the location (a folder) in which
the corresponding annotation file shall be stored. Default is
\code{path = file.path("_ncbi_downloads","annotation")}.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}
}
\value{
File path to downloaded genome.
}
\description{
Main retrieval function for GTF files of an
organism of interest. By specifying the scientific name of an organism of
interest the corresponding GTF file storing the annotation  for the organism
of interest can be downloaded and stored locally. GTF files can be retrieved
from several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory relative to file type, if you get fasta genomes it will be
_ncbi_downloads/genomes'.
In case the corresponding fasta file already exists within the
'_ncbi_downloads/genomes' folder and is accessible within the workspace,
no download process will be performed.
For other file types the same rule applies.
}
\examples{
\dontrun{
# download the annotation of Homo sapiens from ensembl
# and store the corresponding genome file in 'ensembl/annotation'
getGTF(db            = "ensembl",
       organism      = "Homo sapiens",
       path          = file.path("ensembl","annotation"))

getGTF(db            = "ensembl",
       organism      = "Homo sapiens",
       path          = file.path("ensembl","annotation"),
       assembly_type = "primary_assembly")

}

}
\seealso{
Other getBio: 
\code{\link{getBio}()},
\code{\link{getCDS}()},
\code{\link{getCollection}()},
\code{\link{getGenome}()},
\code{\link{getProteome}()},
\code{\link{getRNA}()}

Other gff: 
\code{\link{getGFFSet}()},
\code{\link{read_gff}()}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getENSEMBL.R
\name{getENSEMBL.Seq}
\alias{getENSEMBL.Seq}
\title{Helper function for retrieving biological sequence files from ENSEMBL}
\usage{
getENSEMBL.Seq(
  organism,
  type = "dna",
  id.type = "toplevel",
  release = NULL,
  path
)
}
\arguments{
\item{organism}{Organism selector id,
there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{type}{character, biological sequence type (e.g. "dna", "cds")}

\item{id.type}{a character, default "toplevel". id type of assembly, either "toplevel" or "primary_assembly" usually.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{path}{location where file shall be stored.}
}
\value{
either a character path to downloaded file, or a logical FALSE, specifying failure.
}
\description{
This function downloads gff files of query
organisms from ENSEMBL.
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cachedir_set}
\alias{cachedir_set}
\title{Set directory to store back end files like kingdom summaries etc}
\usage{
cachedir_set(path)
}
\arguments{
\item{path}{the path to cache dir, example "~/Bio_data/biomartr_cache/"}
}
\value{
invisible(NULL), only save the file to path location
}
\description{
Set directory to store back end files like kingdom summaries etc
}
\examples{
# By default it is tempdir()
cachedir()
# cachedir_set("~/Bio_data/biomartr_cache/")
cachedir()
}
\seealso{
Other cachedir: 
\code{\link{cachedir}()}
}
\concept{cachedir}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.retrieval.R
\name{meta.retrieval}
\alias{meta.retrieval}
\title{Perform Meta-Genome Retrieval}
\usage{
meta.retrieval(
  db = "refseq",
  kingdom,
  group = NULL,
  type = "genome",
  restart_at_last = TRUE,
  reference = FALSE,
  combine = FALSE,
  path = NULL
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:

\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "emsembl"}
}}

\item{kingdom}{a character string specifying the kingdom of the organisms
of interest, e.g.

\itemize{
\item For \code{NCBI RefSeq}:
\itemize{
\item \code{kingdom = "archaea"}
\item \code{kingdom = "bacteria"}
\item \code{kingdom = "fungi"}
\item \code{kingdom = "invertebrate"}
\item \code{kingdom = "plant"}
\item \code{kingdom = "protozoa"}
\item \code{kingdom = "viral"}
\item \code{kingdom = "vertebrate_mammalian"}
\item \code{kingdom = "vertebrate_other"}
}
\item For \code{NCBI Genbank}:
\itemize{
\item \code{kingdom = "archaea"}
\item \code{kingdom = "bacteria"}
\item \code{kingdom = "fungi"}
\item \code{kingdom = "invertebrate"}
\item \code{kingdom = "plant"}
\item \code{kingdom = "protozoa"}
\item \code{kingdom = "vertebrate_mammalian"}
\item \code{kingdom = "vertebrate_other"}
}
\item For \code{ENSEMBL}:
\itemize{
\item \code{kingdom = "EnsemblVertebrates"}
\item \code{kingdom = "EnsemblPlants"}
\item \code{kingdom = "EnsemblFungi"}
\item \code{kingdom = "EnsemblMetazoa"}
\item \code{kingdom = "EnsemblBacteria"}
\item \code{kingdom = "EnsemblProtists"}
}
}

Available kingdoms can be retrieved with \code{\link{getKingdoms}}.}

\item{group}{only species belonging to this subgroup will be downloaded.
Groups can be retrieved with \code{\link{getGroups}}.}

\item{type}{type of sequences that shall be retrieved. Options are:

\itemize{
 \item \code{type = "genome"} :
 (for genome assembly retrieval; see also \code{\link{getGenome}}),
 \item \code{type = "proteome"} :
 (for proteome retrieval; see also \code{\link{getProteome}}),
 \item \code{type = "cds"} :
 (for coding sequence retrieval; see also \code{\link{getCDS}}),
 \item \code{type = "gff"} :
(for annotation file retrieval in gff format; see also \code{\link{getGFF}}),
\item \code{type = "gtf"} :
(for annotation file retrieval in gtf format (only for ensembl and
 ensemblgenomes); see also \code{\link{getGTF}})
 \item \code{type = "rna"} :
 (for RNA file retrieval in fasta format; see also \code{\link{getRNA}}),
 \item \code{type = "rm"} :
 (for Repeat Masker output file retrieval; see also
 \code{\link{getRepeatMasker}}),
 \item \code{type = "assemblystats"} :
 (for genome assembly quality stats file retrieval;
 see also \code{\link{getAssemblyStats}}).
 }}

\item{restart_at_last}{a logical value indicating whether or not \code{meta.retrieval} should pick up at the last species when re-running the function.
\itemize{
\item If \code{restart_at_last = TRUE} (Default) then \code{meta.retrieval} will skip all organisms that are already present in the folder
and will start downloading all remaining species. However, this way \code{meta.wretrieval} will not be able to check whether
already downloaded organism files are corrupted or not by checking the md5 checksum.
\item If \code{restart_at_last = FALSE} then \code{meta.retrieval} will start from the beginning and crawl through already downloaded
organism files and check whether already downloaded organism files are corrupted or not by checking the md5 checksum.
After checking existing files the function will start downloading all remaining organisms.
}}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database
as either a reference genome or a representative genome. Options are:
\itemize{
\item \code{reference = FALSE} (Default): all organisms (reference, representative, and non-representative genomes) are downloaded.
\item \code{reference = TRUE}: organisms that are downloaded must be either a reference or representative genome. Thus, most genomes which are usually non-reference genomes
will not be downloaded.
}}

\item{combine}{just in case \code{type = "assemblystats"} is specified, shall
assemby stats of individual species be imported and combined to a
\code{\link{data.frame}}?}

\item{path}{path to the folder in which downloaded genomes shall be stored.
By default the kingdom name is used to name the output folder.}
}
\value{
a character vector storing the file paths of the retrieved files.
}
\description{
Download genomes, proteomes, cds, gff, rna, or assembly stats
files of all species within a kingdom of life.
}
\details{
This function aims to perform bulk retrieval of the genomes,
proteomes, cds, etc. of species that belong to the same kingdom of life or
to the same subgroup.
}
\examples{
\dontrun{
# get all available kingdoms for refseq
getKingdoms(db = "refseq")
# download all vertebrate genomes from refseq
meta.retrieval(kingdom = "vertebrate_mammalian",
               db = "refseq",
               type = "genome")

# get all available kingdoms for genbank
getKingdoms(db = "genbank")
# download all vertebrate genomes from genbank
meta.retrieval(kingdom = "vertebrate_mammalian",
               db = "genbank",
               type = "genome")


# In case users do not wish to retrieve genomes from an entire kingdom,
# but rather from a subgoup (e.g. from species belonging to the
# Gammaproteobacteria class, a subgroup of the bacteria kingdom),
# they can use the following workflow"
# First, users can again consult the getKingdoms() function to retrieve
# kingdom information.
getKingdoms(db = "refseq")

# In this example, we will choose the bacteria kingdom.
# Now, the getGroups() function allows users to obtain available
# subgroups of the bacteria kingdom.
getGroups(db = "refseq", kingdom = "bacteria")

# Now we choose the group Gammaproteobacteria and specify
# the group argument in the meta.retrieval() function
meta.retrieval(kingdom = "bacteria",
   roup = "Gammaproteobacteria",
   db = "refseq",
   type = "genome")
}
}
\seealso{
Other meta_retrival: 
\code{\link{meta.retrieval.all}()}
}
\author{
Hajk-Georg Drost
}
\concept{meta_retrival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGENOMEREPORT.R
\name{getGENOMEREPORT}
\alias{getGENOMEREPORT}
\title{Retrieve NCBI GENOME_REPORTS file}
\usage{
getGENOMEREPORT(
  local_file = file.path(cachedir(), "_ncbi_downloads", "overview.txt")
)
}
\arguments{
\item{local_file}{character, file path, default: file.path(cachedir(), "_ncbi_downloads", "overview.txt")}
}
\value{
a tibble object with the report
}
\description{
Retrieves NCBI GENOME_REPORTS file from
ftp://ftp.ncbi.nlm.nih.gov/genomes/GENOME_REPORTS/overview.txt.
}
\examples{
\dontrun{
report <- getGENOMEREPORT()
report
}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_genome.R
\name{read_genome}
\alias{read_genome}
\title{Import Genome Assembly as Biostrings or data.table object}
\usage{
read_genome(file, format = "fasta", obj.type = "Biostrings", ...)
}
\arguments{
\item{file}{a character string specifying the path to the file
storing the genome.}

\item{format}{a character string specifying the file format used to store
the genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.}

\item{obj.type}{a character string specifying the object stype in which
the genomic sequence shall be represented. Either as
\code{obj.type = "Biostrings"} (default) or
as \code{obj.type = "data.table"}.}

\item{...}{additional arguments that are used by the
\code{\link[seqinr]{read.fasta}} function.}
}
\value{
Either a \code{Biostrings} or \code{data.table} object.
}
\description{
This function reads an organism specific genome stored in a
defined file format.
}
\details{
This function takes a string specifying the path to the genome file
of interest as first argument (e.g. the path returned by
\code{\link{getGenome}}).
}
\seealso{
Other genome: 
\code{\link{getGenomeSet}()},
\code{\link{getGenome}()}

Other readers: 
\code{\link{read_cds}()},
\code{\link{read_gff}()},
\code{\link{read_proteome}()},
\code{\link{read_rna}()}
}
\author{
Hajk-Georg Drost
}
\concept{genome}
\concept{readers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getENSEMBLGENOMES.R
\name{getENSEMBLGENOMESInfo}
\alias{getENSEMBLGENOMESInfo}
\title{Retrieve ENSEMBLGENOMES info file}
\usage{
getENSEMBLGENOMESInfo()
}
\description{
Retrieve species and genome information from
http://rest.ensemblgenomes.org/info/species?content-type=application/json/.
}
\examples{
\dontrun{
info.file <- getENSEMBLGENOMESInfo()
info.file
}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.ensembl.info.R
\name{get.ensembl.info}
\alias{get.ensembl.info}
\title{Helper function to retrieve species information from the ENSEMBL API}
\usage{
get.ensembl.info(update = FALSE, division)
}
\arguments{
\item{update}{logical, default FALSE. If TRUE, force re-download of info.}

\item{division}{the ENSEMBL database (division) for which information shall
be retrieved (available options can be obtained with \code{\link{ensembl_divisions}}).}
}
\description{
This function interfaces with the ENSEMBL API
(https://rest.ensembl.org/info/species?content-type=application/json)
and internally stores the output to use this information for subsequent
retrieval function calls.
}
\examples{
\dontrun{
# Look at available ENSEMBL division options
ensembl_divisions()
# Retrieve available information for EnsemblVertebrates
example <- get.ensembl.info(division = "EnsemblVertebrates")
example
# Update information file stored in the tempdir() folder.
example_update <- get.ensembl.info(division = "EnsemblVertebrates", update = TRUE)
example_update
}
}
\seealso{
\code{\link{ensembl_divisions}}, \code{\link{getKingdomAssemblySummary}}, \code{\link{getENSEMBLInfo}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is.genome.available.R
\name{is.genome.available}
\alias{is.genome.available}
\title{Check Genome Availability}
\usage{
is.genome.available(
  db = "refseq",
  organism,
  skip_bacteria = "TRUE",
  details = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
\item \code{db = "uniprot"}
}}

\item{organism}{there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{details}{a logical value specifying whether or not details on genome
size, kingdom, etc. shall be printed to the console intead of a
boolean value.}
}
\value{
a logical value specifing whether or not the genome of the input
organism is available. In case \code{details} = \code{TRUE} only a character
string specifying the genome details is being returned.
}
\description{
This function checks the availability of a given genome on the
NBCI servers specified as scientific name.
}
\details{
Internally this function calls the \code{\link{listGenomes}} function to
detect all available genomes and checks whether or not the specified organism
is available for download.
}
\examples{
\dontrun{
# checking whether the Homo sapiens genome is stored on NCBI
is.genome.available(organism = "Homo sapiens", db = "refseq")

# and printing details
is.genome.available(organism = "Homo sapiens", db = "refseq", details = TRUE)

# checking whether the Homo sapiens genome is stored on ENSEMBL
is.genome.available(organism = "Homo sapiens", db = "ensembl")

# and printing details
is.genome.available(organism = "Homo sapiens",
                    details = TRUE,
                    db = "ensembl")
}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_database_all.R
\name{download.database.all}
\alias{download.database.all}
\title{Download all elements of an NCBI databse}
\usage{
download.database.all(db, path = NULL)
}
\arguments{
\item{db}{a character string specifying the database that shall be downloaded
(selected from \code{\link{listDatabases}}).}

\item{path}{a character string specifying the location (a folder) in which
the corresponding
database shall be stored. In case this folder does not exist yet,
it will be created.}
}
\value{
A character vector storing the file paths of the downloaded databases.
}
\description{
The \code{\link{download.database}} functions allows users to
retrieve individual packages of a NCBI database. This function is designed to
retrieve the entire database selected by the users (hence all packages 
corresponding to this database).
}
\examples{
\dontrun{
# search for available NCBI databases
  listNCBIDatabases(db = "all")
# choose database NCBI nr and download compelete database
  download.database.all(db = "nr", path = "nr")
}
}
\seealso{
\code{\link{download.database}}, \code{\link{listNCBIDatabases}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/organismFilters.R
\name{organismFilters}
\alias{organismFilters}
\title{Retrieve Ensembl Biomart filters for a qyery organism}
\usage{
organismFilters(organism, update = FALSE, topic = NULL)
}
\arguments{
\item{organism}{a character string specifying the scientific name of a 
query organism.}

\item{update}{a logical value specifying whether or not the local 
listMart.txt, listDatasets.txt, and listFilters_organism.txt files shall be 
updated by remote access to BioMart.}

\item{topic}{a character string specifying a topic (category) of filters, 
e.g. \code{topic} = \code{"id"}.}
}
\value{
a data.frame storing corresponding filter names, description, 
datasets, and marts.
}
\description{
In addition to the \code{\link{organismBM}} and 
\code{\link{organismAttributes}} functions, this function
returns all available filters that can be accessed through different marts 
and datasets for a given query organism.
}
\details{
For a given query organism, this function retrieves all available 
filters that can be accessed through different marts and datasets.

Sometimes the same filter names correspond to different datasets and 
marts causing problems when using \code{\link{getMarts}}. 
The approach introduced by this function provides (again) a organism centric 
way of accessing organism specific filters.

The \code{topic} argument allows the user to search for specific filters 
topics/categories for faster selection.
}
\note{
When you run this function for the first time, the data retrieval procedure 
will take some time, due to the remote access to BioMart. The corresponding 
result is then saved in a *.txt file within the \code{\link{tempdir}}
directory  named "_biomart/listMarts.txt","_biomart/listDatasets.txt", and 
"_biomart/listFilters_organism.txt", allowing subsequent queries to perform 
much faster.
}
\examples{
\dontrun{ 
# search for filter topic "id" 
head(organismFilters("Homo sapiens", topic = "id"), 20)
}
}
\references{
\url{http://biomart.org/}

Mapping identifiers for the integration of genomic datasets with the
R/Bioconductor package biomaRt. Steffen Durinck, Paul T. Spellman, Ewan
Birney and Wolfgang Huber, Nature Protocols 4, 1184-1191 (2009).

BioMart and Bioconductor: a powerful link between biological databases and
microarray data analysis. Steffen Durinck, Yves Moreau, Arek Kasprzyk, Sean
Davis, Bart De Moor, Alvis Brazma and Wolfgang Huber, Bioinformatics 21,
3439-3440 (2005).
}
\seealso{
\code{\link{organismBM}}, \code{\link{organismAttributes}}, 
\code{\link{getAttributes}}, \code{\link{getDatasets}}, 
\code{\link{getMarts}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMetaGenomeAnnotations.R
\name{getMetaGenomeAnnotations}
\alias{getMetaGenomeAnnotations}
\title{Retrieve annotation *.gff files for metagenomes from NCBI Genbank}
\usage{
getMetaGenomeAnnotations(
  name,
  path = file.path("_ncbi_downloads", "metagenome", "annotations")
)
}
\arguments{
\item{name}{metagenome name retrieved by \code{\link{listMetaGenomes}}.}

\item{path}{a character string specifying the location (a folder) 
in which the corresponding metagenome annotations shall be stored. 
Default is
\code{path} = \code{file.path("_ncbi_downloads","metagenome","annotations")}.}
}
\description{
Retrieve available annotation *.gff files for metagenomes 
from NCBI Genbank. NCBI Genbank allows users
to download entire metagenomes and their annotations of several metagenome 
projects. This function downloads available metagenomes that can then be 
downloaded via \code{\link{getMetaGenomes}}.
}
\examples{
\dontrun{
# Frist, retrieve a list of available metagenomes
listMetaGenomes()

# Now, retrieve the 'human gut metagenome'
getMetaGenomeAnnotations(name = "human gut metagenome")
}
}
\seealso{
\code{\link{getMetaGenomes}}, \code{\link{listMetaGenomes}}, 
\code{\link{getGFF}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_cds.R
\name{summary_cds}
\alias{summary_cds}
\title{Retrieve summary statistics for a coding sequence (CDS) file}
\usage{
summary_cds(file, organism)
}
\arguments{
\item{file}{file path to a CDS file in \code{fasta} format.}

\item{organism}{character string specifying the organism at hand.}
}
\description{
A summary statistics of specific CDS features is returned.
}
\details{
The summary statistics include:
\itemize{
\item \code{total_seqs}: 
\item \code{nnn_abs}: The total number of NNN's 
(over all chromosomes/scaffolds/contigs) in all coding sequences combined
\item \code{nnn_perc}: The percentage (relative frequency) of NNN's 
(over all chromosomes/scaffolds/contigs) compared to the total number of 
nucleotides of all coding sequences
}
}
\seealso{
\code{\link{getCollection}}, \code{\link{getCDS}}, \code{\link{read_cds}}, \code{\link{summary_genome}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listMetaGenomes.R
\name{listMetaGenomes}
\alias{listMetaGenomes}
\title{List available metagenomes on NCBI Genbank}
\usage{
listMetaGenomes(details = FALSE)
}
\arguments{
\item{details}{a boolean value specifying whether only the scientific names 
of stored metagenomes shall be returned (\code{details = FALSE}) or all 
information such as "organism_name","bioproject", 
etc (\code{details = TRUE}).}
}
\description{
List available metagenomes on NCBI genbank. NCBI genbank 
allows users to download entire metagenomes of several metagenome projects. 
This function lists all available metagenomes that can then be downloaded via
\code{\link{getMetaGenomes}}.
}
\examples{
\dontrun{
# retrieve available metagenome projects at NCBI Genbank
listMetaGenomes()
# retrieve detailed information on available metagenome projects 
# at NCBI Genbank
listMetaGenomes(details = TRUE)
}
}
\seealso{
\code{\link{getMetaGenomes}}, \code{\link{getMetaGenomeSummary}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getAttributes.R
\name{getAttributes}
\alias{getAttributes}
\title{Retrieve All Available Attributes for a Specific Dataset}
\usage{
getAttributes(mart, dataset, mute_citation = FALSE)
}
\arguments{
\item{mart}{a character string specifying the database (mart) 
for which datasets shall be listed.}

\item{dataset}{a character string specifying the dataset for which 
attributes shall be listed.}

\item{mute_citation}{logical value indicating whether citation message should be muted.}
}
\description{
This function queries the BioMart Interface and returns a table
storing all available attributes for a specific dataset.
}
\examples{
\dontrun{
# search for available datasets
getMarts()

# choose database (mart): ENSEMBL_MART_ENSEMBL
# and get a table of all available datasets from this BioMart database
head(getDatasets(mart = "ENSEMBL_MART_ENSEMBL"), 10)

# choose dataset: "hsapiens_gene_ensembl"
head(getAttributes(mart = "ENSEMBL_MART_ENSEMBL", 
                   dataset = "hsapiens_gene_ensembl") , 5)
}
}
\seealso{
\code{\link{getMarts}}, \code{\link{getDatasets}},
\code{\link{getFilters}}, \code{\link{organismBM}}, 
\code{\link{organismFilters}}, \code{\link{organismAttributes}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_proteome.R
\name{read_proteome}
\alias{read_proteome}
\title{Import Proteome as Biostrings or data.table object}
\usage{
read_proteome(file, format = "fasta", obj.type = "Biostrings", ...)
}
\arguments{
\item{file}{a character string specifying the path to the file storing
the proteome.}

\item{format}{a character string specifying the file format used to store the
genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.}

\item{obj.type}{a character string specifying the object stype in which the
genomic sequence shall be represented.
Either as \code{obj.type = "Biostrings"} (default) or as
\code{obj.type = "data.table"}.}

\item{...}{additional arguments that are used by
\code{\link[seqinr]{read.fasta}}.}
}
\value{
Either a \code{Biostrings} or \code{data.table} object.
}
\description{
This function reads an organism specific proteome stored in a
defined file format.
}
\details{
This function takes a string specifying the path to the
proteome file of interest as first argument.

It is possible to read in different proteome file standards such as
\emph{fasta} or \emph{genebank}.
}
\seealso{
Other readers: 
\code{\link{read_cds}()},
\code{\link{read_genome}()},
\code{\link{read_gff}()},
\code{\link{read_rna}()}

Other proteome: 
\code{\link{getProteomeSet}()},
\code{\link{getProteome}()}
}
\author{
Hajk-Georg Drost
}
\concept{proteome}
\concept{readers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMetaGenomeSummary.R
\name{getMetaGenomeSummary}
\alias{getMetaGenomeSummary}
\title{Retrieve the assembly_summary.txt file from NCBI genbank metagenomes}
\usage{
getMetaGenomeSummary(
  local_file = file.path(cachedir(), "assembly_summary_metagenomes_genbank.txt")
)
}
\arguments{
\item{local_file}{where to store this backend file, default:
file.path(cachedir(), "assembly_summary_metagenomes_genbank.txt")}
}
\description{
Retrieval function of the assembly_summary.txt file
from NCBI genbank metagenomes.
This files stores all available metagenome projects on NCBI Genbank.
}
\examples{
\dontrun{
meta.summary <- getMetaGenomeSummary()
meta.summary
}
}
\seealso{
\code{\link{getKingdomAssemblySummary}},
\code{\link{getSummaryFile}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getENSEMBLInfo.R
\name{getENSEMBLInfo}
\alias{getENSEMBLInfo}
\title{Retrieve ENSEMBL info file}
\usage{
getENSEMBLInfo(update = TRUE)
}
\arguments{
\item{update}{logical, default TRUE. Update cached list, if FALSE use existing
(if it exists)}
}
\value{
a tibble table storing info for all available ENSEMBL divisions.
}
\description{
Retrieve species and genome information from
http://rest.ensembl.org/info/species?content-type=application/json/.
}
\examples{
\dontrun{
# look at available divisions
ensembl_divisions()
# retrieve information for all ENSEMBL divisions at once
test <- getENSEMBLInfo()
test
# retrieve information for a particular ENSEMBL division (e.g. EnsemblVertebrates)
test_vertebrates <- get.ensembl.info(update = TRUE, division = "EnsemblVertebrates")
test_vertebrates
}
}
\seealso{
\code{\link{ensembl_divisions}}, \code{\link{get.ensembl.info}}, \code{\link{getKingdomAssemblySummary}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getReleases.R
\name{getReleases}
\alias{getReleases}
\title{Retrieve available database releases or versions
of ENSEMBL}
\usage{
getReleases(db = "ensembl")
}
\arguments{
\item{db}{a character string specifying the database from which
available resease versions shall be retrieved:
\itemize{
\item \code{db = "ensembl"}
}}
}
\description{
Retrieve available database releases or versions of
ENSEMBL.
}
\examples{
\dontrun{
# retrieve available resease versions of ENSEMBL
getReleases("ensembl")
}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getUniProtInfo.R
\name{getUniProtInfo}
\alias{getUniProtInfo}
\title{Get uniprot info from organism}
\usage{
getUniProtInfo(organism, path = cachedir(), update = TRUE)
}
\arguments{
\item{organism}{character, name of organism}

\item{path}{path at which the info file shall be stored locally.}

\item{update}{shall the internal \code{\link{cachedir}} file be deleted and the info
file freshly downloaded from the UniProt API?}
}
\description{
Get uniprot info from organism
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGO.R
\name{getGO}
\alias{getGO}
\title{Gene Ontology Query}
\usage{
getGO(organism, genes, filters, ...)
}
\arguments{
\item{organism}{a character string specifying the scientific name 
of a query organism.}

\item{genes}{a character vector storing the gene ids of a organisms 
of interest to be queried against Ensembl Biomart.}

\item{filters}{a character vector specifying the filter (query key) for 
the Ensembl Biomart query, e.g. \code{filter} = \code{"ensembl_gene_id"}.}

\item{...}{additional parameters that can be passed to the 
\code{\link{biomart}} function.}
}
\description{
This function takes a gene id as character vector from a given 
query organism and returns the corresponding GO terms and additional GO 
information.
}
\details{
This function takes the scientific name of a query organism, a set of genes 
for which GO terms and additional information shall be retrieved, and a 
filter argument that specifies the attribute for the query genes.
}
\examples{
\dontrun{ 
GO_tbl <- getGO(organism = "Arabidopsis thaliana", 
                genes    = c("AT1G06090", "AT1G06100"),
                filters  = "ensembl_gene_id")

# look at the result
head(GO_tbl)
}
}
\seealso{
\code{\link{biomart}}, \code{\link{organismFilters}}, 
\code{\link{organismBM}}, \code{\link[biomaRt]{getBM}}, 
\code{\link{getMarts}}, \code{\link{getDatasets}}, 
\code{\link{getFilters}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getDatasets.R
\name{getDatasets}
\alias{getDatasets}
\title{Retrieve All Available Datasets for a BioMart Database}
\usage{
getDatasets(mart, mute_citation = FALSE)
}
\arguments{
\item{mart}{a character string specifying the database (mart) for 
which datasets shall be listed.}

\item{mute_citation}{logical value indicating whether citation message should be muted.}
}
\description{
This funcion queries the BioMart API and returns a table
storing all available datasets for a selected BioMart databases.
}
\examples{
\dontrun{
# search for available datasets
# getMarts()
# choose database: "ENSEMBL_MART_ENSEMBL"
head(getDatasets("ENSEMBL_MART_ENSEMBL"), 10)
}
}
\seealso{
\code{\link{getMarts}}, \code{\link{getAttributes}}, 
\code{\link{getFilters}}, \code{\link{organismBM}}, 
\code{\link{organismFilters}}, \code{\link{organismAttributes}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSet.R
\name{getProteomeSet}
\alias{getProteomeSet}
\title{Proteome retrieval of multiple species}
\usage{
getProteomeSet(
  db = "refseq",
  organisms,
  reference = FALSE,
  release = NULL,
  skip_bacteria = TRUE,
  gunzip = TRUE,
  update = FALSE,
  path = "set_proteomes",
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organisms}{a character vector storing the names of the organisms than shall be retrieved.
There are three available options to characterize an organism:}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{gunzip}{a logical, indicating whether or not files should be unzipped.}

\item{update}{logical, default FALSE. Updated backend cached files needed.
Usually keep this false, to make ut run much faster. Only set to TRUE, if you
believe you cache is outdated (Species only exist in newest release etc)}

\item{path}{a character string specifying the location (a folder) in which
the corresponding proteomes shall be stored. Default is \code{path} = \code{"set_proteomes"}.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}
}
\value{
File path to downloaded genomes
(names are identifiers: 'new' (file was downloaded now),
'old' files did already exist)
}
\description{
Main proteome retrieval function for a set of organism of interest.
By specifying the scientific names of the organisms of interest the corresponding fasta-files storing the proteome of the organisms of interest
will be downloaded and stored locally. proteome files can be retrieved from several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory 'set_CDSs' to store
the CDSs of interest as fasta files for future processing.
In case the corresponding fasta file already exists within the
'set_CDSs' folder and is accessible within the workspace,
no download process will be performed.
}
\examples{
\dontrun{
# download the proteomes of three different species at the same time
#### Database: NCBI RefSeq
file_paths <- getProteomeSet(db = "refseq", organisms = c("Arabidopsis thaliana",
                                      "Arabidopsis lyrata",
                                       "Capsella rubella"))
# look at file paths
file_paths

#### Database: NCBI Genbank
file_paths <- getProteomeSet(db = "genbank", organisms = c("Arabidopsis thaliana",
                                      "Arabidopsis lyrata",
                                       "Capsella rubella"))
# look at file paths
file_paths

# download the proteomes of three different species at the same time
#### Database: ENSEMBL
file_paths <- getProteomeSet(db = "ensembl", organisms = c("Homo sapiens",
                                      "Mus musculus",
                                       "Caenorhabditis elegans"))
# look at file paths
file_paths

# download the proteomes of three different species at the same time
#### Database: UniProt
file_paths <- getProteomeSet(db = "uniprot", organisms = c("Homo sapiens",
                                      "Mus musculus",
                                       "Caenorhabditis elegans"))
# look at file paths
file_paths
}

}
\seealso{
Other getBioSet: 
\code{\link{getBioSet}()},
\code{\link{getCDSSet}()},
\code{\link{getCollectionSet}()},
\code{\link{getGFFSet}()},
\code{\link{getGenomeSet}()},
\code{\link{getRNASet}()}

Other proteome: 
\code{\link{getProteome}()},
\code{\link{read_proteome}()}
}
\author{
Hajk-Georg Drost
}
\concept{getBioSet}
\concept{proteome}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/summary_genome.R
\name{summary_genome}
\alias{summary_genome}
\title{Retrieve summary statistics for a genome assembly file}
\usage{
summary_genome(file, organism)
}
\arguments{
\item{file}{file path to a genome assembly file in \code{fasta} format.}

\item{organism}{character string specifying the organism at hand.}
}
\description{
A summary statistics of specific genome features is generated.
These statistics are useful to assess the genome quality of retrieved genome assemblies
when performing comparative genomics tasks. This way, users can assess whether or not
patterns found based on genome comparisons aren't just a technical artifact of
differences in genome assembly quality.
}
\details{
The summary statistics include:
\itemize{
\item \code{genome_size_mbp}: Genome size in mega base pairs
\item \code{n50_mbp}: The N50 contig size of the genome assembly in mega base pairs
\item \code{n_seqs}: The number of chromosomes/scaffolds/contigs of the genome assembly file
\item \code{n_nnn}: The absolute number of NNNs (over all chromosomes or scaffolds or contigs) in the genome assembly file
\item \code{rel_nnn}: The percentage (relative frequency) of NNNs (over all chromosomes or scaffolds or contigs) compared to the total number of
nucleotides in the genome assembly file
\item \code{genome_entropy}: The \code{Shannon Entropy} of the genome assembly file (median entropy over all individual chromosome entropies)
\item \code{n_gc}: The total number of GCs
(over all chromosomes or scaffolds or contigs) in the genome assembly file
\item \code{rel_gc}: The (relative frequency) of GCs
(over all chromosomes or scaffolds or contigs) compared to the total number of
nucleotides in the genome assembly file
}
}
\examples{
\dontrun{
# retrieve genome from NCBI RefSeq
Sc <- biomartr::getGenome(db = "refseq", organism = "Saccharomyces cerevisiae")
# compute genome assembly summary statistics
Sc_genome_summary <- summary_genome(file = Sc, organism = "Saccharomyces cerevisiae")
# look at results
Sc_genome_summary
}
}
\seealso{
\code{\link{summary_cds}}, \code{\link{getCollection}}, \code{\link{getGenome}}, \code{\link{read_genome}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getUniProtSTATS.R
\name{getUniProtSTATS}
\alias{getUniProtSTATS}
\title{Retrieve UniProt Database Information File (STATS)}
\usage{
getUniProtSTATS(update = FALSE)
}
\arguments{
\item{update}{shall the internal \code{\link{cachedir}} file be deleted and the \code{STATS}
file freshly downloaded from the UniProt FTP servers?}
}
\description{
The UniProt stores a \code{STATS} file to summarise all available information for their reference proteomes.
Users can now download this file and process it with \code{biomartr}.
}
\examples{
\dontrun{
# retrieve STATS file from UniProt
uniprot_info <- getUniProtSTATS(update = TRUE)
# look at results
uniprot_info
}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/organismBM.R
\name{organismBM}
\alias{organismBM}
\title{Retrieve Ensembl Biomart marts and datasets for a query organism}
\usage{
organismBM(organism = NULL, update = FALSE, mute_citation = TRUE)
}
\arguments{
\item{organism}{a character string specifying the scientific name of a 
query organism. Default is \code{organism} = \code{NULL}. In this case all 
available biomart connections are returned.}

\item{update}{a logical value specifying whether or not the local 
listMart.txt and listDatasets.txt files shall be updated by remote access
 to BioMart.}

\item{mute_citation}{logical value indicating whether citation message should be muted.}
}
\description{
This function returns either all available biomart connections 
for all available organisms for which biomart access is possible, or 
(when specified) returns all organism specific biomart connections.
}
\details{
This function collects all available biomart connections and returns a table 
storing the organism for which biomart connections are available as well as 
the corresponding mart and database.
}
\note{
When you run this function for the first time, the data retrieval procedure 
will take some time, due to the remote access to BioMart. The corresponding 
result is then saved in a *.txt file named "_biomart/listDatasets.txt" in the
\code{\link{tempdir}} directory, allowing subsequent queries to perform
 much faster.
}
\examples{
\dontrun{ 
# returning all available biomart connections
head(organismBM(), 20)
# retrieving all available datasets and biomart connections for
# a specific query organism (scientific name)
organismBM(organism = "Homo sapiens")
# you can also update the downloaded version using 
# the "update = TRUE" argument
head(organismBM(update = TRUE), 20)
}
}
\references{
\url{http://biomart.org/}

Mapping identifiers for the integration of genomic datasets with the
R/Bioconductor package biomaRt. Steffen Durinck, Paul T. Spellman, Ewan
Birney and Wolfgang Huber, Nature Protocols 4, 1184-1191 (2009).

BioMart and Bioconductor: a powerful link between biological databases and
microarray data analysis. Steffen Durinck, Yves Moreau, Arek Kasprzyk, Sean
Davis, Bart De Moor, Alvis Brazma and Wolfgang Huber, Bioinformatics 21,
3439-3440 (2005).
}
\seealso{
\code{\link{getMarts}}, \code{\link{getDatasets}}, 
\code{\link{biomart}}, \code{\link{organismFilters}}, 
\code{\link{organismAttributes}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listGroups.R
\name{listGroups}
\alias{listGroups}
\title{List number of available genomes in each taxonomic group}
\usage{
listGroups(db = "refseq", kingdom = "all", details = FALSE)
}
\arguments{
\item{db}{a character string specifying the database for which genome 
availability shall be checked. Available options are:
\itemize{
\item \code{db = "refseq"} 
\item \code{db = "genbank"}
}}

\item{kingdom}{a kingdom specification retrieved by 
\code{\link{getKingdoms}}.}

\item{details}{shall all species corresponding to the specified 
\code{kingdom} be returned? Default is \code{details = FALSE}.}
}
\description{
Users can retrieve the available number of sequenced 
genomes per group. Only available for \code{db = "refseq"} and 
\code{db = "genbank"}.
}
\examples{
\dontrun{
# example for refseq
listGroups(db = "refseq")
# example for genbank
listGroups(db = "genbank")
### in case groups should be specified by kingdom
# first, retrieve available kingdom names
listKingdoms()
# now we choose kingdom "bacteria"
listGroups(db = "refseq", kingdom = "bacteria")
# or
listGroups(db = "genbank", kingdom = "bacteria")
}
}
\seealso{
\code{\link{listGenomes}}, \code{\link{is.genome.available}}, 
\code{\link{listKingdoms}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getKingdoms.R
\name{getKingdoms}
\alias{getKingdoms}
\title{Retrieve available kingdoms of life}
\usage{
getKingdoms(db = "refseq")
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved: \code{db = "refseq"}, \code{db = "genbank"},
\code{db = "ensembl"}, \code{db = "ensemblgenomes"}.
Default is \code{db = "refseq"}.}
}
\description{
A short list of available kingdoms of life
}
\examples{
# retrieve kingdoms available from refseq
getKingdoms(db = "refseq")

# retrieve kingdoms available from genbank
getKingdoms(db = "genbank")
}
\seealso{
\code{\link{meta.retrieval}}, \code{\link{getGenome}},
\code{\link{getProteome}}, \code{\link{getCDS}}, \code{\link{getGroups}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/refseqOrganisms.R
\name{refseqOrganisms}
\alias{refseqOrganisms}
\title{Retrieve All Organism Names Stored on refseq}
\usage{
refseqOrganisms()
}
\description{
This function extracts all organism names (scientific names)
for which genomes, proteomes, and CDS files are stored on 
the NCBI refseq server.
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSummaryFile.R
\name{getSummaryFile}
\alias{getSummaryFile}
\title{Helper function to retrieve the assembly_summary.txt file from NCBI}
\usage{
getSummaryFile(db, kingdom, file = assemblies_info_path(db, kingdom))
}
\arguments{
\item{db}{database name. E.g. \code{refseq} or \code{genbank}.}

\item{kingdom}{kingdom for which assembly_summary.txt file shall be
retrieved. See also \code{\link{getKingdoms}}.}

\item{file}{path, local path to total summary file, default is in tmp folder.}
}
\description{
Retrieval function of the assembly_summary.txt file from NCBI.
}
\examples{
\dontrun{
test <- getSummaryFile("refseq","plant")
test
}
}
\seealso{
\code{\link{getKingdomAssemblySummary}},
\code{\link{getMetaGenomeSummary}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getKingdomAssemblySummary.R
\name{getKingdomAssemblySummary}
\alias{getKingdomAssemblySummary}
\title{Retrieve and summarise the assembly_summary.txt files from
NCBI for all kingdoms}
\usage{
getKingdomAssemblySummary(
  db,
  skip_bacteria = TRUE,
  file = assemblies_info_path(db)
)
}
\arguments{
\item{db}{database name. E.g. \code{refseq} or \code{genbank}.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{file}{path, local path to total summary file, default is in tmp folder.}
}
\description{
Retrieval function of the assembly_summary.txt file
from NCBI for all kingdoms.
The assembly_summary.txt files store available species on NCBI.
}
\examples{
\dontrun{
# This example will run the default version of this function
# whereby information for Bacteria are not downloaded
test <- getKingdomAssemblySummary(db = "genbank", skip_bacteria = TRUE)
test
# Users can then retrieve information for Bacteria by skip_bacteria = FALSE
test2 <- getKingdomAssemblySummary(db = "genbank", skip_bacteria = FALSE)
test2
}
}
\seealso{
\code{\link{getSummaryFile}}, \code{\link{getMetaGenomeSummary}}, \code{\link{get.ensembl.info}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_assemblystats.R
\name{read_assemblystats}
\alias{read_assemblystats}
\title{Import Genome Assembly Stats File}
\usage{
read_assemblystats(file, type = "raw")
}
\arguments{
\item{file}{a character string specifying the path to the file storing
the Genome Assembly Stats file.}

\item{type}{a tibble object, either \code{type = "raw"} to import the entire genome assembly
stats file or \code{type = "stats"} to import overall statistics including
all chromosomes, mitochondria and plastids.}
}
\description{
This function reads an organism specific Genome Assembly
Stats file that was retrieved with \code{\link{getAssemblyStats}}.
}
\details{
This function takes a string specifying the path to the Genome
Assembly Stats file of interest (e.g. the path returned by
\code{\link{getAssemblyStats}}) and imports it.
}
\seealso{
\code{\link{getAssemblyStats}}, \code{\link{read_genome}},
\code{\link{read_proteome}}, \code{\link{read_cds}}, \code{\link{read_gff}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_cds.R
\name{read_cds}
\alias{read_cds}
\title{Import CDS as Biostrings or data.table object}
\usage{
read_cds(
  file,
  format = "fasta",
  obj.type = "Biostrings",
  delete_corrupt = FALSE,
  ...
)
}
\arguments{
\item{file}{a character string specifying the path to the file storing
the CDS.}

\item{format}{a character string specifying the file format used to store
the genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.}

\item{obj.type}{a character string specifying the object stype in which the
genomic sequence shall be represented.
Either as \code{obj.type = "Biostrings"} (default) or as
\code{obj.type = "data.table"}.}

\item{delete_corrupt}{a logical value specifying whether potential CDS
sequences that cannot be divided by 3 shall be
be excluded from the the dataset. Default is \code{delete_corrupt = FALSE}.}

\item{...}{additional arguments that are used by
\code{\link[seqinr]{read.fasta}}.}
}
\value{
A data.table storing the gene id in the first column and the
corresponding sequence as string in the second column.
}
\description{
This function reads an organism specific CDS stored in a
defined file format.
}
\details{
The \code{read.cds} function takes a string specifying the path
to the cds file of interest as first argument.

It is possible to read in different proteome file standards such as
\emph{fasta} or \emph{genebank}.

CDS stored in fasta files can be downloaded from
http://www.ensembl.org/info/data/ftp/index.html.
}
\seealso{
Other cds: 
\code{\link{getCDSSet}()},
\code{\link{getCDS}()}

Other readers: 
\code{\link{read_genome}()},
\code{\link{read_gff}()},
\code{\link{read_proteome}()},
\code{\link{read_rna}()}
}
\author{
Hajk-Georg Drost
}
\concept{cds}
\concept{readers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listDatabases.R, R/listNCBIDatabases.R
\name{listDatabases}
\alias{listDatabases}
\alias{listNCBIDatabases}
\title{Retrieve a List of Available NCBI Databases for Download}
\usage{
listDatabases(db = "nr", update = FALSE)

listNCBIDatabases(db = "nr", update = FALSE)
}
\arguments{
\item{db}{a character string specifying the name of the database that shall 
be searched for.}

\item{update}{a logical value specifying whether or not the local 
listDatabases.txt file shall be updated by remote access to NCBI.}
}
\description{
This function allows you to retrieve a list of database names 
and versions that can be downloaded from correspondning servers.

Database retrieval is crucial for most biological studies and analyses.
There is a vast diversity of databases that can be accessed remotely or that 
can be downloaded to your local machine. This function provides an interface 
to databases that can be downloaded from NCBI servers and lists all available
databases and their database version to be able to select an appropriate 
database for download with \code{\link{download.database}}.
}
\examples{
\dontrun{
# retrieve all versions of the NCBI 'nr' database that can be downloaded
listNCBIDatabases(db = "nr")

# analogous:
# listNCBIDatabases(db = "cdd")
# listNCBIDatabases(db = "nt")
# listNCBIDatabases(db = "gss")
# listNCBIDatabases(db = "refseq_protein")
}
}
\seealso{
\code{\link{download.database}}, \code{\link{download.database.all}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMarts.R
\name{getMarts}
\alias{getMarts}
\title{Retrieve information about available Ensembl Biomart databases}
\usage{
getMarts()
}
\description{
This funcion queries the Ensembl Biomart API and returns a table
storing information about all available Ensembl Biomart databases.
}
\examples{
\dontrun{
# get a table of all available databases from Ensembl Biomart
getMarts()
 }
}
\seealso{
\code{\link{getDatasets}}, \code{\link{getAttributes}},
\code{\link{getFilters}}, \code{\link{organismBM}},
\code{\link{organismFilters}}, \code{\link{organismAttributes}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getSet.R
\name{getCollectionSet}
\alias{getCollectionSet}
\title{Retrieve a Collection: Genome, Proteome, CDS, RNA, GFF, Repeat Masker, AssemblyStats of multiple species}
\usage{
getCollectionSet(
  db = "refseq",
  organisms,
  reference = FALSE,
  release = NULL,
  skip_bacteria = TRUE,
  gunzip = TRUE,
  update = FALSE,
  remove_annotation_outliers = TRUE,
  path = "set_collections",
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organisms}{a character vector storing the names of the organisms than shall be retrieved.
There are three available options to characterize an organism:}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{gunzip}{a logical, indicating whether or not files should be unzipped.}

\item{update}{logical, default FALSE. Updated backend cached files needed.
Usually keep this false, to make ut run much faster. Only set to TRUE, if you
believe you cache is outdated (Species only exist in newest release etc)}

\item{remove_annotation_outliers}{shall outlier lines be removed from the input \code{annotation_file}?
If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
for further exploration.}

\item{path}{a character string specifying the location (a folder) in which
the corresponding collection shall be stored. Default is
\code{path} = \code{file.path("_db_downloads","collections")}.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}
}
\value{
File path to downloaded genomes
(names are identifiers: 'new' (file was downloaded now),
'old' files did already exist)
}
\description{
Main collection retrieval function for an organism of interest.
By specifying the scientific name of an organism of interest a collection consisting of
the genome file, proteome file, CDS file, RNA file, GFF file, Repeat Masker file, AssemblyStats
file of the organism of interest
can be downloaded and stored locally. Collections can be retrieved from
several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory 'set_CDSs' to store
the CDSs of interest as fasta files for future processing.
In case the corresponding fasta file already exists within the
'set_CDSs' folder and is accessible within the workspace,
no download process will be performed.
}
\examples{
\dontrun{
getBioSet("refseq", organisms = c("Arabidopsis thaliana",
                                  "Arabidopsis lyrata",
                                  "Capsella rubella"), 
                                  set_type = "cds")
}
}
\seealso{
Other getBioSet: 
\code{\link{getBioSet}()},
\code{\link{getCDSSet}()},
\code{\link{getGFFSet}()},
\code{\link{getGenomeSet}()},
\code{\link{getProteomeSet}()},
\code{\link{getRNASet}()}

Other collection: 
\code{\link{getCollection}()}
}
\author{
Hajk-Georg Drost
}
\concept{collection}
\concept{getBioSet}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_rna.R
\name{read_rna}
\alias{read_rna}
\title{Import RNA as Biostrings or data.table object}
\usage{
read_rna(file, format = "fasta", obj.type = "Biostrings", ...)
}
\arguments{
\item{file}{a character string specifying the path to the file
storing the RNA.}

\item{format}{a character string specifying the file format used to store the
genome, e.g. \code{format = "fasta"} (default) or \code{format = "gbk"}.}

\item{obj.type}{a character string specifying the object stype in which the
genomic sequence shall be represented.
Either as \code{obj.type = "Biostrings"} (default) or
as \code{obj.type = "data.table"}.}

\item{...}{additional arguments that are used by
\code{\link[seqinr]{read.fasta}}.}
}
\value{
A data.table storing the gene id in the first column and the
corresponding sequence as string in the second column.
}
\description{
This function reads an organism specific RNA stored in a
defined file format.
}
\details{
This function takes a string specifying the path to the RNA file
of interest as first argument. It is possible to read in different proteome
file standards such as \emph{fasta} or \emph{genebank}.
}
\seealso{
Other rna: 
\code{\link{getRNASet}()},
\code{\link{getRNA}()}

Other readers: 
\code{\link{read_cds}()},
\code{\link{read_genome}()},
\code{\link{read_gff}()},
\code{\link{read_proteome}()}
}
\author{
Hajk-Georg Drost
}
\concept{readers}
\concept{rna}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getCollection.R
\name{getCollection}
\alias{getCollection}
\title{Retrieve a Collection: Genome, Proteome, CDS, RNA, GFF, Repeat Masker, AssemblyStats}
\usage{
getCollection(
  db = "refseq",
  organism,
  reference = TRUE,
  skip_bacteria = TRUE,
  release = NULL,
  gunzip = FALSE,
  remove_annotation_outliers = FALSE,
  path = file.path("_db_downloads", "collections"),
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organism}{Organism selector id,
there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{gunzip}{a logical, indicating whether or not files should be unzipped.}

\item{remove_annotation_outliers}{shall outlier lines be removed from the input \code{annotation_file}?
If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
for further exploration.}

\item{path}{a character string specifying the location (a folder) in which
the corresponding collection shall be stored. Default is
\code{path} = \code{file.path("_db_downloads","collections")}.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}
}
\value{
File path to downloaded genome.
}
\description{
Main collection retrieval function for an organism of interest.
By specifying the scientific name of an organism of interest a collection consisting of
the genome file, proteome file, CDS file, RNA file, GFF file, Repeat Masker file, AssemblyStats
file of the organism of interest
can be downloaded and stored locally. Collections can be retrieved from
several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory relative to file type, if you get fasta genomes it will be
_ncbi_downloads/genomes'.
In case the corresponding fasta file already exists within the
'_ncbi_downloads/genomes' folder and is accessible within the workspace,
no download process will be performed.
For other file types the same rule applies.
}
\examples{
\dontrun{
# download the collection of Homo sapiens from refseq
# and store the corresponding genome file in '_ncbi_downloads/collection'
 Hsap_collection <- getCollection( db       = "refseq",
             organism = "Homo sapiens",
             path = file.path("_db_downloads","collections"))
# download the collection of Homo sapiens from genbank
# and store the corresponding genome file in '_ncbi_downloads/collection'
 Hsap_collection <- getCollection( db       = "genbank",
             organism = "Homo sapiens",
             path = file.path("_db_downloads","collections"))
# download the collection of Homo sapiens from ensembl
# and store the corresponding genome file in '_ncbi_downloads/collection'
 Hsap_collection <- getCollection( db       = "ensembl",
             organism = "Homo sapiens",
             path = file.path("_db_downloads","collections"))
}

}
\seealso{
Other getBio: 
\code{\link{getBio}()},
\code{\link{getCDS}()},
\code{\link{getGFF}()},
\code{\link{getGenome}()},
\code{\link{getProteome}()},
\code{\link{getRNA}()}

Other collection: 
\code{\link{getCollectionSet}()}
}
\author{
Hajk-Georg Drost
}
\concept{collection}
\concept{getBio}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomart.R
\name{biomart}
\alias{biomart}
\title{Main BioMart Query Function}
\usage{
biomart(genes, mart, dataset, attributes, filters, mute_citation = FALSE, ...)
}
\arguments{
\item{genes}{a character vector storing the gene ids of a organisms
of interest to be queried against BioMart.}

\item{mart}{a character string specifying the mart to be used. Users
can obtain available marts using \code{\link{getMarts}}.}

\item{dataset}{a character string specifying the dataset within the mart to
be used, e.g. \code{dataset} = \code{"hsapiens_gene_ensembl"}.}

\item{attributes}{a character vector specifying the attributes that shall be
used, e.g. \code{attributes} = 
\code{c("start_position","end_position","description")}.}

\item{filters}{a character vector specifying the filter (query key) for the 
BioMart query, e.g. \code{filter} = \code{"ensembl_gene_id"}.}

\item{mute_citation}{logical value indicating whether citation message should be muted.}

\item{...}{additional parameters for the
\code{\link[biomaRt]{getBM}} function.}
}
\value{
A data.table storing the initial query gene vector in 
the first column, the output
gene vector in the second column, and all attributes in
the following columns.
}
\description{
This function takes a set of gene ids and the biomart
specifications and performs a biomart query for the given set of gene ids.
}
\details{
This function is the main query function of the biomartr package.

It enables to fastly access annotations of a given gene set based 
on the \pkg{biomaRt} package
implemented by Steffen Durinck et al.
}
\examples{
\dontrun{
# 1) select a mart
getMarts()

# we will select mart 'plants_mart' and search for available datasets
getDatasets(mart = "plants_mart")

# we choose dataset 'athaliana_eg_gene' and run biomart()
# using mart: 'plants_mart', dataset: "athaliana_eg_gene"
# attributes: c("start_position","end_position","description")
# for an example gene set of Arabidopsis thaliana:
# c("AT1G06090", "AT1G06100", "AT1G06110", "AT1G06120",
#    "AT1G06130", "AT1G06200")

biomart(genes      = c("AT1G06090", "AT1G06100",
                       "AT1G06110", "AT1G06120",
                       "AT1G06130", "AT1G06200"),
        mart       = "plants_mart",
        dataset    = "athaliana_eg_gene",
        attributes = c("start_position","end_position","description"),
        filters    = "ensembl_gene_id")
}

}
\seealso{
\code{\link{organismFilters}}, \code{\link{organismBM}},
\code{\link[biomaRt]{listAttributes}}, \code{\link[biomaRt]{getBM}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getProteome.R
\name{getProteome}
\alias{getProteome}
\title{Proteome Retrieval}
\usage{
getProteome(
  db = "refseq",
  organism,
  reference = TRUE,
  skip_bacteria = TRUE,
  release = NULL,
  gunzip = FALSE,
  update = TRUE,
  path = file.path("_ncbi_downloads", "proteomes"),
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organism}{Organism selector id,
there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{gunzip}{a logical, indicating whether or not files should be unzipped.}

\item{update}{logical, default TRUE. (Uniprot only for now!)
If species info file exists already,
do not re download, makes it faster but the file can be old, i.e. no longer
as complete as it could be.}

\item{path}{a character string specifying the location (a folder) in which
the corresponding proteome shall be stored. Default is
\code{path} = \code{file.path("_ncbi_downloads","proteomes")}.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}
}
\value{
File path to downloaded proteome.
}
\description{
Main proteome retrieval function for an organism of interest.
By specifying the scientific name of an organism of interest the
corresponding fasta-file storing the proteome of the organism of interest
can be downloaded and stored locally. Proteome files can be retrieved from
several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory '_ncbi_downloads/proteomes' to store
the proteome of interest as fasta file for future processing.
}
\examples{
\dontrun{
# download the proteome of Arabidopsis thaliana from NCBI RefSeq
# and store the corresponding proteome file in '_ncbi_downloads/refseq/proteomes'
file_path <- getProteome( db       = "refseq",
             organism = "Arabidopsis thaliana",
             path     = file.path("_ncbi_downloads","refseq","proteomes") )
# import proteome into R session
Ath_proteome <- read_proteome(file_path, format = "fasta")

# download the proteome of Arabidopsis thaliana from NCBI Genbank
# and store the corresponding proteome file in '_ncbi_downloads/genbank/proteomes'
file_path <- getProteome( db       = "genbank",
             organism = "Arabidopsis thaliana",
             path     = file.path("_ncbi_downloads","genbank","proteomes") )
# import proteome into R session
Ath_proteome <- read_proteome(file_path, format = "fasta")

# and store the corresponding proteome file in '_downloads/uniprot/proteomes'
file_path <- getProteome( db       = "uniprot",
             organism = "Arabidopsis thaliana",
             path     = file.path("_downloads","uniprot","proteomes") )
# import proteome into R session
Ath_proteome <- read_proteome(file_path, format = "fasta")

# download the proteome of Arabidopsis thaliana from ENSEMBL
# and store the corresponding proteome file in '_downloads/ensembl/proteomes'
file_path <- getProteome( db       = "ensembl",
             organism = "Arabidopsis thaliana",
             path     = file.path("_downloads","ensembl","proteomes") )
# import proteome into R session
Ath_proteome <- read_proteome(file_path, format = "fasta")
}
}
\seealso{
Other getBio: 
\code{\link{getBio}()},
\code{\link{getCDS}()},
\code{\link{getCollection}()},
\code{\link{getGFF}()},
\code{\link{getGenome}()},
\code{\link{getRNA}()}

Other proteome: 
\code{\link{getProteomeSet}()},
\code{\link{read_proteome}()}
}
\author{
Hajk-Georg Drost
}
\concept{getBio}
\concept{proteome}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_rm.R
\name{read_rm}
\alias{read_rm}
\title{Import Repeat Masker output file}
\usage{
read_rm(file)
}
\arguments{
\item{file}{a character string specifying the path to the file 
storing the Repeat Masker output (e.g. retrieved with \code{\link{getRepeatMasker}}).}
}
\description{
This function reads an organism specific 
Repeat Masker output file.
}
\details{
This function takes a string specifying the path to the 
Repeat Masker output file of interest as first argument.
}
\seealso{
\code{\link{getRepeatMasker}}, \code{\link{read_genome}}, 
\code{\link{read_proteome}}, \code{\link{read_gff}}, \code{\link{read_rna}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cachedir}
\alias{cachedir}
\title{Get directory to store back end files like kingdom summaries etc}
\usage{
cachedir(non_temp_cache = "~/.biomartr_cache_dir.rds")
}
\arguments{
\item{non_temp_cache}{"~/.biomartr_cache_dir.rds",}
}
\value{
reads the rds file, and returns the path for local cache, if not
existing, use tempdir().
}
\description{
Get directory to store back end files like kingdom summaries etc
}
\examples{
cachedir()
}
\seealso{
Other cachedir: 
\code{\link{cachedir_set}()}
}
\concept{cachedir}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get.ensembl.info.R
\name{ensembl_divisions}
\alias{ensembl_divisions}
\title{List all available ENSEMBL divisions}
\usage{
ensembl_divisions()
}
\description{
Retrieve a list of available databases on ENSEMBL for which \code{get.ensembl.info} can be retrieved.
}
\examples{
ensembl_divisions()
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_annotation_biomartr.R
\name{check_annotation_biomartr}
\alias{check_annotation_biomartr}
\title{Check whether an annotation file contains outlier lines}
\usage{
check_annotation_biomartr(annotation_file, remove_annotation_outliers = FALSE)
}
\arguments{
\item{annotation_file}{a file path to the annotation file.}

\item{remove_annotation_outliers}{shall outlier lines be removed from the input \code{annotation_file}? 
If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
for further exploration.}
}
\description{
Some annotation files include lines with character lengths greater than 65000. 
This causes problems when trying to import such annotation files into R using \code{import}.
To overcome this issue, this function screens for such lines
in a given annotation file and removes these lines so that
\code{import} can handle the file.
}
\examples{
\dontrun{
# download an example annotation file from NCBI RefSeq
Ath_path <- biomartr::getGFF(organism = "Arabidopsis thaliana")
# run annotation file check on the downloaded file
biomartr::check_annotation_biomartr(Ath_path)
# several outlier lines were detected, thus we re-run the
# function using 'remove_annotation_outliers = TRUE'
# to remove the outliers and overwrite the file
biomartr::check_annotation_biomartr(Ath_path, remove_annotation_outliers = TRUE)
}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRNA.R
\name{getRNA}
\alias{getRNA}
\title{RNA Sequence Retrieval}
\usage{
getRNA(
  db = "refseq",
  organism,
  reference = FALSE,
  skip_bacteria = TRUE,
  release = NULL,
  assembly_type = "toplevel",
  path = file.path("_ncbi_downloads", "RNA"),
  gunzip = FALSE,
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organism}{Organism selector id,
there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{assembly_type}{a character, default "toplevel". id type of assembly, either "toplevel" or "primary_assembly" usually.}

\item{path}{a character string specifying the location (a folder) in which
the corresponding
CDS file shall be stored. Default is
\code{path} = \code{file.path("_ncbi_downloads","RNA")}.}

\item{gunzip}{a logical, indicating whether or not files should be unzipped.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}
}
\value{
File path to downloaded genome.
}
\description{
Main retrieval function for RNA sequences of an organism
of interest. By specifying the scientific name of an organism of interest the
corresponding fasta-file storing the RNA information for the organism
of interest can be downloaded and stored locally.
RNA files can be retrieved from several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory relative to file type, if you get fasta genomes it will be
_ncbi_downloads/genomes'.
In case the corresponding fasta file already exists within the
'_ncbi_downloads/genomes' folder and is accessible within the workspace,
no download process will be performed.
For other file types the same rule applies.
}
\examples{
\dontrun{
# download the RNA of Arabidopsis thaliana from refseq
# and store the corresponding RNA file in '_ncbi_downloads/RNA'
file_path <- getRNA( db       = "refseq",
             organism = "Arabidopsis thaliana",
             path     = file.path("_ncbi_downloads","RNA"))

Ath_RNA <- read_rna(file_path, format = "fasta")
}
}
\seealso{
Other getBio: 
\code{\link{getBio}()},
\code{\link{getCDS}()},
\code{\link{getCollection}()},
\code{\link{getGFF}()},
\code{\link{getGenome}()},
\code{\link{getProteome}()}

Other rna: 
\code{\link{getRNASet}()},
\code{\link{read_rna}()}
}
\author{
Hajk-Georg Drost
}
\concept{getBio}
\concept{rna}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/meta.retrieval.all.R
\name{meta.retrieval.all}
\alias{meta.retrieval.all}
\title{Perform Meta-Genome Retrieval of all organisms in all kingdoms of life}
\usage{
meta.retrieval.all(db = "refseq", type = "genome", reference = FALSE)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "emsembl"}
\item \code{db = "ensemblgenomes"}
}}

\item{type}{type of sequences that shall be retrieved. Options are:
\itemize{
 \item \code{type = "genome"} :
 for genome assembly retrieval; see also \code{\link{getGenome}}),
 \item \code{type = "proteome"} :
 (for proteome retrieval; see also \code{\link{getProteome}}),
 \item \code{type = "cds"} :
 (for coding sequence retrieval; see also \code{\link{getCDS}}),
 \item \code{type = "gff"} :
(for annotation file retrieval in gff format; see also \code{\link{getGFF}}),
\item \code{type = "gtf"} :
(for annotation file retrieval in gtf format
(only for ensembl and ensemblgenomes); see also \code{\link{getGTF}}),
 \item \code{type = "rna"} :
 (for RNA file retrieval in fasta format; see also \code{\link{getRNA}}),
 \item \code{type = "rm"} :
 (for Repeat Masker output file retrieval; see also
 \code{\link{getRepeatMasker}}),
 \item \code{type = "assemblystats"} (for genome assembly quality stats
 file retrieval; see also \code{\link{getAssemblyStats}}).
 }}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database
as either a reference genome or a representative genome. Options are:
\itemize{
\item \code{reference = FALSE} (Default): all organisms (reference, representative, and non-representative genomes) are downloaded.
\item \code{reference = TRUE}: organisms that are downloaded must be either a reference or representative genome. Thus, most genomes which are usually non-reference genomes
will not be downloaded.
}}
}
\value{
a character vector storing the file paths of the retrieved files.
}
\description{
Download genomes, proteomes, cds, gff, rna, or assembly stats
files of individual species of all kingdoms of life.
}
\details{
This function aims to perform bulk retrieval of all genomes
of species for all kingdoms of life.
}
\examples{
\dontrun{
# download all genomes from refseq
meta.retrieval.all(db = "refseq", type = "genome")
# download all vertebrate genomes from genbank
meta.retrieval.all(db = "genbank", type = "genome")
# download all vertebrate genomes from ensemblgenomes
meta.retrieval.all(db = "genbank", type = "ensemblgenomes")
}
}
\seealso{
Other meta_retrival: 
\code{\link{meta.retrieval}()}
}
\author{
Hajk-Georg Drost
}
\concept{meta_retrival}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getBioSet.R
\name{getBio}
\alias{getBio}
\title{A wrapper to all bio getters, selected with 'type' argument}
\usage{
getBio(
  db = "refseq",
  organism,
  type,
  reference = FALSE,
  release = NULL,
  gunzip = FALSE,
  update = FALSE,
  skip_bacteria = TRUE,
  path = paste0("set_", toupper(type)),
  remove_annotation_outliers = FALSE,
  analyse_genome = FALSE,
  assembly_type = "toplevel",
  format = "gff3",
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organism}{Organism selector id,
there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{type}{biological sequence type. (alternatives are: genome, gff, cds, rna, proteome, collection (all the others))}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{gunzip}{a logical, indicating whether or not files should be unzipped.}

\item{update}{logical, default FALSE. Updated backend cached files needed.
Usually keep this false, to make ut run much faster. Only set to TRUE, if you
believe you cache is outdated (Species only exist in newest release etc)}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{path}{character, default location is paste0("set_", toupper(type))}

\item{remove_annotation_outliers}{shall outlier lines be removed from the input \code{annotation_file}?
If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
for further exploration.}

\item{analyse_genome}{logical, default FALSE. If TRUE, get general genome statistics like
gc content etc. For more details, see ?summary_genome}

\item{assembly_type}{a character, default "toplevel". id type of assembly, either "toplevel" or "primary_assembly" usually.}

\item{format}{"gff3", alternative "gtf" for ensembl.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}
}
\value{
File path to downloaded genome.
}
\description{
A wrapper to all bio getters, selected with 'type' argument
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory relative to file type, if you get fasta genomes it will be
_ncbi_downloads/genomes'.
In case the corresponding fasta file already exists within the
'_ncbi_downloads/genomes' folder and is accessible within the workspace,
no download process will be performed.
For other file types the same rule applies.
}
\seealso{
Other getBio: 
\code{\link{getCDS}()},
\code{\link{getCollection}()},
\code{\link{getGFF}()},
\code{\link{getGenome}()},
\code{\link{getProteome}()},
\code{\link{getRNA}()}
}
\author{
Hajk-Georg Drost
}
\concept{getBio}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listGenomes.R
\name{listGenomes}
\alias{listGenomes}
\title{List All Available Genomes either by kingdom, group, or subgroup}
\usage{
listGenomes(
  db = "refseq",
  type = "all",
  subset = NULL,
  details = FALSE,
  skip_bacteria = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database for which genome
availability shall be checked. Available options are:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{type}{a character string specifying a potential filter of available
genomes. Available options are:
\itemize{
\item \code{type = "all"}
\item \code{type = "kingdom"}
\item \code{type = "group"}
\item \code{type = "subgroup"}
}}

\item{subset}{a character string or character vector specifying a subset of
\code{type}. E.g. if users are interested in retrieving all
\code{Eukaryota} species, they can specify: \code{type = "kingdom"} and
\code{subset = "Eukaryota"}.}

\item{details}{a boolean value specifying whether only the scientific names
of stored genomes shall be returned (details = FALSE) or all information such
as
\itemize{
\item \code{organism_name}
\item \code{kingdoms}
\item \code{group}
\item \code{subgroup}
\item \code{file_size_MB}, etc.
}}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}
}
\description{
This function retrieves the names of all genomes available on
the NCBI ftp:// server and stores the results in a file named 'overview.txt'
inside the directory _ncbi_downloads' that is built inside the workspace.
}
\details{
Internally this function loads the the overview.txt file from NCBI
and creates a directory '_ncbi_downloads' in the \code{temdir()}
folder to store the overview.txt file for future processing. In case the
overview.txt file already exists within the '_ncbi_downloads' folder and is
accessible within the workspace, no download process will be performed again.
}
\note{
Please note that the ftp:// connection relies on the NCBI or ENSEMBL server
and cannot be accurately accessed via a proxy.
}
\examples{
\dontrun{
# print details for refseq
listGenomes(db = "refseq")
# print details for all plants in refseq
listGenomes(db = "refseq", type = "kingdom")
# print details for all plant groups in refseq
listGenomes(db = "refseq", type = "group")
# print details for all plant subgroups in refseq
listGenomes(db = "refseq", type = "subgroup")
# Ensembl
listGenomes(db = "ensembl", type = "kingdom", subset = "EnsemblVertebrates")
}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGFF.R
\name{getGFF}
\alias{getGFF}
\title{Genome Annotation Retrieval (GFF3)}
\usage{
getGFF(
  db = "refseq",
  organism,
  reference = FALSE,
  skip_bacteria = TRUE,
  release = NULL,
  gunzip = FALSE,
  remove_annotation_outliers = FALSE,
  path = file.path("_ncbi_downloads", "annotation"),
  mute_citation = FALSE,
  format = "gff3"
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organism}{Organism selector id,
there are three options to characterize an organism:
\itemize{
\item by \code{scientific name}: e.g. \code{organism = "Homo sapiens"}
\item by \code{database specific accession identifier}: e.g. \code{organism = "GCF_000001405.37"} (= NCBI RefSeq identifier for \code{Homo sapiens})
\item by \code{taxonomic identifier from NCBI Taxonomy}: e.g. \code{organism = "9606"} (= taxid of \code{Homo sapiens})
}}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{release}{a numeric, the database release version of ENSEMBL (\code{db = "ensembl"}). Default is \code{release = NULL} meaning
that the most recent database version is used. \code{release = 75} would for human would give the stable
GRCh37 release in ensembl. Value must be > 46, since ensembl did not structure their data
if the standard format before that.}

\item{gunzip}{a logical, indicating whether or not files should be unzipped.}

\item{remove_annotation_outliers}{shall outlier lines be removed from the input \code{annotation_file}?
If yes, then the initial \code{annotation_file} will be overwritten and the removed outlier lines will be stored at \code{\link{tempdir}}
for further exploration.}

\item{path}{a character string specifying the location (a folder) in which
the corresponding annotation file shall be stored. Default is
\code{path = file.path("_ncbi_downloads","annotation")}.}

\item{mute_citation}{logical, default FALSE, indicating whether citation message should be muted.}

\item{format}{"gff3", alternative "gtf" for ensembl.}
}
\value{
File path to downloaded genome.
}
\description{
Main retrieval function for GFF files of an
organism of interest. By specifying the scientific name of an organism of
interest the corresponding gff file storing the annotation  for the organism
of interest can be downloaded and stored locally. GFF files can be retrieved
from several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

and creates a directory relative to file type, if you get fasta genomes it will be
_ncbi_downloads/genomes'.
In case the corresponding fasta file already exists within the
'_ncbi_downloads/genomes' folder and is accessible within the workspace,
no download process will be performed.
For other file types the same rule applies.
}
\examples{
\dontrun{
# download the annotation of Arabidopsis thaliana from refseq
# and store the corresponding genome file in '_ncbi_downloads/annotation'
Athal_gff <- getGFF( db       = "refseq",
               organism = "Arabidopsis thaliana",
               path = file.path("_ncbi_downloads","annotation"),
               remove_annotation_outliers = TRUE)
Athal_gff_import <- read_gff(Athal_gff)


# download the genome of Arabidopsis thaliana from genbank
# and store the corresponding genome file in '_ncbi_downloads/annotation'
Athal_gff <- getGFF( db       = "genbank",
               organism = "Arabidopsis thaliana",
               path = file.path("_ncbi_downloads","annotation"),
               remove_annotation_outliers = TRUE)
Athal_gff_import <- read_gff(Athal_gff)

# download the genome of Homo sapiens from ensembl
# and store the corresponding genome file in '_ncbi_downloads/annotation'
Hsap_gff <- getGFF( db       = "ensembl",
               organism = "Homo sapiens",
               path = file.path("_ncbi_downloads","annotation"),
               remove_annotation_outliers = TRUE)
Hsap_gff_import <- read_gff(Hsap_gff)

}

}
\seealso{
Other getBio: 
\code{\link{getBio}()},
\code{\link{getCDS}()},
\code{\link{getCollection}()},
\code{\link{getGenome}()},
\code{\link{getProteome}()},
\code{\link{getRNA}()}

Other gff: 
\code{\link{getGFFSet}()},
\code{\link{read_gff}()}
}
\author{
Hajk-Georg Drost
}
\concept{getBio}
\concept{gff}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRepeatMasker.R
\name{getRepeatMasker}
\alias{getRepeatMasker}
\title{Repeat Masker Retrieval}
\usage{
getRepeatMasker(
  db = "refseq",
  organism,
  reference = FALSE,
  skip_bacteria = TRUE,
  path = file.path("_ncbi_downloads", "repeatmasker"),
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome 
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
}}

\item{organism}{a character string specifying the scientific name of the 
organism of interest, e.g. \code{organism = "Homo sapiens"}.}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023), 
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the 
bacterial summary file will be downloaded.}

\item{path}{a character string specifying the location (a folder) in which 
the corresponding file shall be stored. Default is 
\code{path} = \code{file.path("_ncbi_downloads","repeatmasker")}.}

\item{mute_citation}{logical value indicating whether citation message should be muted.}
}
\value{
File path to downloaded Repeat Masker output file.
}
\description{
Main Repeat Masker output retrieval function for an 
organism of interest.
By specifying the scientific name of an organism of interest the 
corresponding Repeat Masker file storing the genome of the organism of 
interest can be downloaded and stored locally. 
Repeat Masker files can be retrieved from several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/
 
and creates a directory '_ncbi_downloads/repeatmasker' to store
the files of interest as fasta file for future processing.
In case the corresponding fasta file already exists within the
'_ncbi_downloads/repeatmasker' folder and is accessible within the workspace,
no download process will be performed.
}
\examples{
\dontrun{

# download the Repeat Masker output file of Homo sapiens from refseq
# and store the corresponding genome file in '_ncbi_downloads/genomes'
file_path <- getRepeatMasker( db       = "refseq", 
             organism = "Homo sapiens", 
             path = file.path("_ncbi_downloads","repeatmasker"))

Hsap_repeatmasker <- read_rm(file_path)

}

}
\seealso{
\code{\link{getGenome}}, \code{\link{getProteome}}, \code{\link{getCDS}}, 
\code{\link{getGFF}}, \code{\link{getRNA}}, \code{\link{getCollection}}, \code{\link{meta.retrieval}}, 
\code{\link{read_rm}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGroups.R
\name{getGroups}
\alias{getGroups}
\title{Retrieve available groups for a kingdom of life (only available for NCBI RefSeq and NCBI Genbank)}
\usage{
getGroups(db = "refseq", kingdom)
}
\arguments{
\item{db}{a character string specifying the database from which the genome 
shall be retrieved: 
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
}

Default is \code{db = "refseq"}.}

\item{kingdom}{a character string specifying for which kingdom of life 
groups shall be retrieved. See \code{\link{getKingdoms}} for details.}
}
\description{
A short list of available groups for a kingdom of life.
}
\examples{
# get possible kigdom names
getKingdoms(db = "refseq")
\dontrun{
# retrieve subgroups for vertebrate_mammalian available from refseq
getGroups(db = "refseq", kingdom = "vertebrate_mammalian")

# get possible kigdom names
getKingdoms(db = "genbank")
# retrieve subgroups for vertebrate_mammalian available from genbank
getGroups(db = "genbank", kingdom = "vertebrate_mammalian")
}
}
\seealso{
\code{\link{meta.retrieval}}, \code{\link{getGenome}}, 
\code{\link{getProteome}}, \code{\link{getCDS}}, \code{\link{getKingdoms}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/biomartr-package.R
\docType{package}
\name{biomartr-package}
\alias{biomartr-package}
\alias{biomartr}
\title{Genomic Data Retrieval}
\description{
This package interacts with a suite of web Application
Programming Interfaces and FTP sites to perform automated genomic data
retieval and annotation information retrieval.
}
\section{About}{

To automate the retrieval process on a meta-genomic scale, this package
 provides useful interface functions for genomic sequence retrieval and
 functional annotation retrieval.
 The major aim of \code{biomartr} is to facilitate computational
 reproducibility and large-scale handling of genomic data for
 (meta-)genomic analyses.

In detail, \code{biomartr} aims to provide users with an easy to use
framework to obtain genome, proteome, CDS, GFF (annotation), genome
assembly quality, and metagenome project data. Furthermore, an interface to
the Ensembl Biomart database allows users to retrieve functional annotation
for genomic loci.
Users can download entire databases
such as
\itemize{
\item \code{NCBI RefSeq}
\item \code{NCBI nr}
\item \code{NCBI nt}
\item \code{NCBI Genbank}
\item \code{NCBI nt}
\item \code{Ensembl}
\item \code{Ensembl Genomes}
\item \code{UniProt}
}
}

\author{
Hajk-Georg Drost \email{hajk-georg.drost@tuebingen.mpg.de}
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getAssemblyStats.R
\name{getAssemblyStats}
\alias{getAssemblyStats}
\title{Genome Assembly Stats Retrieval}
\usage{
getAssemblyStats(
  db = "refseq",
  organism,
  reference = FALSE,
  skip_bacteria = TRUE,
  type = "download",
  path = file.path("_ncbi_downloads", "genomeassembly_stats"),
  mute_citation = FALSE
)
}
\arguments{
\item{db}{a character string specifying the database from which the genome
shall be retrieved:
\itemize{
\item \code{db = "refseq"}
\item \code{db = "genbank"}
\item \code{db = "ensembl"}
}}

\item{organism}{a character string specifying the scientific name of the
organism of interest, e.g. \code{organism = "Homo sapiens"}.}

\item{reference}{a logical value indicating whether or not a genome shall be downloaded if it isn't marked in the database as either a reference genome or a representative genome.}

\item{skip_bacteria}{Due to its enormous dataset size (> 700MB as of July 2023),
the bacterial summary file will not be loaded by default anymore. If users
wish to gain insights for the bacterial kingdom they needs to actively specify \code{skip_bacteria = FALSE}. When \code{skip_bacteria = FALSE} is set then the
bacterial summary file will be downloaded.}

\item{type}{shall only the file be retrieved (default)
\code{type = "download"} or should the corresponding file be downloaded and
subsequently be imported \code{type = "import"}.}

\item{path}{a character string specifying the location (a folder) in
which the corresponding file shall be stored. Default is
\code{path} = \code{file.path("_ncbi_downloads","genomeassembly_stats")}.}

\item{mute_citation}{logical value indicating whether citation message should be muted.}
}
\value{
File path to downloaded genome assembly stats file.
}
\description{
Main genome assembly stats retrieval function for an organism
of interest. By specifying the scientific name of an organism of interest the
corresponding  genome assembly stats file storing the assembly statistics of
the organism of interest can be downloaded and stored locally.
Genome assembly stats files can be retrieved from several databases.
}
\details{
Internally this function loads the the overview.txt file from NCBI:

 refseq: ftp://ftp.ncbi.nlm.nih.gov/genomes/refseq/

 genbank: ftp://ftp.ncbi.nlm.nih.gov/genomes/genbank/

to retrieve available scientific names of organisms and creates a directory
'_ncbi_downloads/genomeassembly_stats' to store
the Genome Assembly Stats of interest as text file for future processing.
In case the corresponding fasta file already exists within the
'_ncbi_downloads/genomeassembly_stats' folder and is
accessible within the workspace, no download process will be performed.

An example genome assembly stats file can be found here:
ftp://ftp.ncbi.nlm.nih.gov/genomes/all/GCF/000/001/405/
GCF_000001405.36_GRCh38.p10/GCF_000001405.36_GRCh38.p10_assembly_stats.txt.
}
\examples{
\dontrun{
# download the genome assembly stats file of Saccharomyces cerevisiae
# from NCBI RefSeq
# and store the corresponding genome file in
# '_ncbi_downloads/genomeassembly_stats'
file_path <- getAssemblyStats( db = "refseq",
                 organism = "Saccharomyces cerevisiae",
                 path = file.path("_ncbi_downloads","genomeassembly_stats"))
# import the raw file as it is downloaded
Scerevisiae.stats <- read_assemblystats(file_path, type = "raw")

# download the genome assembly stats file of Saccharomyces cerevisiae
# from NCBI RefSeq
# and import overall statistics of the genome assembly
Scerevisiae.stats.import <- getAssemblyStats( db = "refseq",
                 organism = "Saccharomyces cerevisiae",
                 type = "import",
                 path = file.path("_ncbi_downloads","genomeassembly_stats"))
}

}
\seealso{
\code{\link{getGenome}}, \code{\link{getProteome}}, \code{\link{getCDS}},
\code{\link{getGFF}}, \code{\link{getRNA}}, \code{\link{getCollection}}, \code{\link{meta.retrieval}},
\code{\link{read_assemblystats}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMetaGenomes.R
\name{getMetaGenomes}
\alias{getMetaGenomes}
\title{Retrieve metagenomes from NCBI Genbank}
\usage{
getMetaGenomes(name, path = file.path("_ncbi_downloads", "metagenome"))
}
\arguments{
\item{name}{metagenome name retrieved by \code{\link{listMetaGenomes}}.}

\item{path}{a character string specifying the location (a folder) in 
which the corresponding metagenome shall be stored. 
Default is \code{path} = \code{file.path("_ncbi_downloads","metagenome")}.}
}
\description{
Retrieve available metagenomes from NCBI Genbank. 
NCBI Genbank allows users to download entire metagenomes of several 
metagenome projects. This function downloads available metagenomes that can 
then be downloaded via \code{\link{getMetaGenomes}}.
}
\examples{
\dontrun{
# Frist, retrieve a list of available metagenomes
listMetaGenomes()

# Now, retrieve the 'human gut metagenome'
getMetaGenomes(name = "human gut metagenome")
}
}
\seealso{
\code{\link{getMetaGenomeAnnotations}}, 
\code{\link{listMetaGenomes}}
}
\author{
Hajk-Georg Drost
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_gff.R
\name{read_gff}
\alias{read_gff}
\title{Import GFF File}
\usage{
read_gff(file)
}
\arguments{
\item{file}{a character string specifying the path to the file
storing the CDS.}
}
\value{
Either a \code{Biostrings} or \code{data.table} object.
}
\description{
This function reads an organism specific CDS stored
in a defined file format.
}
\details{
This function takes a string specifying the path to the GFF file
of interest (e.g. the path returned by \code{\link{getGFF}}).
}
\seealso{
Other gff: 
\code{\link{getGFFSet}()},
\code{\link{getGFF}()}

Other readers: 
\code{\link{read_cds}()},
\code{\link{read_genome}()},
\code{\link{read_proteome}()},
\code{\link{read_rna}()}
}
\author{
Hajk-Georg Drost
}
\concept{gff}
\concept{readers}
